/* cpl-nodes-action.h - handlers and definitions for CPL action tags. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_NODES_ACTION_H_
#define CPL_NODES_ACTION_H_

#include "cpl-nodes-internal.h"

/*
 * A <redirect> node.
 */
class redirect_node : public cpl_node {
private:
  bool permanent;

public:
  redirect_node();

  // XXX WRITEME virtual cpl_run_result run();
  virtual ~redirect_node() { }

  friend class redirect_handler;
};


/*
 * The handler for a <redirect> node.
 */
class redirect_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  redirect_handler();

  static void set_permanent(cpl_parser*, const std::string&);
};


/*
 * A <proxy> node.
 */
class proxy_node : public cpl_node {
public:
  typedef enum {parallel, sequential, first_only} order;

private:
  int timeout;
  bool recurse;
  proxy_node::order ordering;

  cpl_node_ptr busy;
  cpl_node_ptr noanswer;
  cpl_node_ptr redirection;
  cpl_node_ptr failure;
  cpl_node_ptr default_output; // "default" is a reserved word

public:
  proxy_node();
  // XXX WRITEME virtual cpl_run_result run();
  virtual ~proxy_node() { }

  friend class proxy_handler;
  FRIEND_SIMPLE_OUTPUT_HANDLER(proxy, busy);
  FRIEND_SIMPLE_OUTPUT_HANDLER(proxy, noanswer);
  FRIEND_SIMPLE_OUTPUT_HANDLER(proxy, redirection);
  FRIEND_SIMPLE_OUTPUT_HANDLER(proxy, failure);
  FRIEND_SIMPLE_OUTPUT_HANDLER(proxy, default);
};


/*
 * The handler for a <proxy> node.
 */
class proxy_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  proxy_handler();

  static void set_timeout(cpl_parser*, const std::string&);
  static void set_recurse(cpl_parser*, const std::string&);
  static void set_ordering(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};

/* Handlers for outputs of a <proxy> node. */
DECLARE_SIMPLE_OUTPUT_HANDLER(proxy, busy);
DECLARE_SIMPLE_OUTPUT_HANDLER(proxy, noanswer);
DECLARE_SIMPLE_OUTPUT_HANDLER(proxy, redirection);
DECLARE_SIMPLE_OUTPUT_HANDLER(proxy, failure);
DECLARE_SIMPLE_OUTPUT_HANDLER(proxy, default);


/* Value maps for a <proxy> node. */
extern std::map<std::string, proxy_node::order> proxy_order_map;
extern void init_proxy_order_map();


/*
 * A <reject> node.
 */
class reject_node : public cpl_node {
public:
  typedef enum {unknown, symbolic, numeric} reject_status;
  typedef enum {busy, notfound, reject, error} symbolic_reject_status;

private:
  reject_status status_type;
  symbolic_reject_status symbolic_status;
  int numeric_status;

  std::string reason;

public:
  reject_node();

  virtual ~reject_node() { }

  friend class reject_handler;
};


/*
 * A handler for a <reject> node.
 */
class reject_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  reject_handler();

  static void set_status(cpl_parser*, const std::string&);
  static void set_reason(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};

/* Value maps for a <reject> node. */
extern std::map<std::string, reject_node::symbolic_reject_status>
reject_status_map;
extern void init_reject_status_map();



/*
 * Singleton handlers.
 */

extern redirect_handler the_redirect_handler;

extern proxy_handler the_proxy_handler;
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(proxy, busy);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(proxy, noanswer);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(proxy, redirection);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(proxy, failure);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(proxy, default);

extern reject_handler the_reject_handler;


#endif /* CPL_NODES_ACTION_H_ */
