/* cpl-nodes-location.h - handlers and definitions for CPL location tags. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_NODES_LOCATION_H_
#define CPL_NODES_LOCATION_H_

#include "cpl-nodes-internal.h"

/*
 * The handler for an explicit <location> node.
 */
class location_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  location_handler();

  static void set_url(cpl_parser*, const std::string&);
  static void set_priority(cpl_parser*, const std::string&);
  static void set_clear(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};


/*
 * An explicit <location> node.
 */
class location_node : public cpl_node {
private:
  std::string url;
  float priority;
  bool clear;

  cpl_node_ptr output;

public:
  location_node();
  virtual cpl_run_result run();
  virtual ~location_node() { }

  friend class location_handler;
};


/*
 * The handler for a <lookup> node.
 */
class lookup_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  lookup_handler();

  static void set_source(cpl_parser*, const std::string&);
  static void set_timeout(cpl_parser*, const std::string&);
  static void set_use(cpl_parser*, const std::string&);
  static void set_ignore(cpl_parser*, const std::string&);
  static void set_clear(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};


/*
 * A <lookup> node.
 */
class lookup_node : public cpl_node {
private:
  std::string source;
  int timeout; /* Or should this be float? */
  std::string use; /* XXX Change this to a proper set */
  std::string ignore; /* XXX Change this to a proper set */
  bool clear;

  cpl_node_ptr success;
  cpl_node_ptr notfound;
  cpl_node_ptr failure;
public:
  lookup_node();
  // XXX WRITEME virtual cpl_run_result run();
  virtual ~lookup_node() { }

  friend class lookup_handler;
  FRIEND_SIMPLE_OUTPUT_HANDLER(lookup, success);
  FRIEND_SIMPLE_OUTPUT_HANDLER(lookup, notfound);
  FRIEND_SIMPLE_OUTPUT_HANDLER(lookup, failure);
};


/* Handlers for outputs of a <lookup> node. */
DECLARE_SIMPLE_OUTPUT_HANDLER(lookup, success);
DECLARE_SIMPLE_OUTPUT_HANDLER(lookup, notfound);
DECLARE_SIMPLE_OUTPUT_HANDLER(lookup, failure);

/*
 * The handler for a <remove-location> node.
 */
class remove_location_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  remove_location_handler();

  static void set_location(cpl_parser*, const std::string&);
  static void set_param(cpl_parser*, const std::string&);
  static void set_value(cpl_parser*, const std::string&);
  
  virtual void end(cpl_parser *) const;
};


/*
 * A <remove-location> node.
 */
class remove_location_node : public cpl_node {
private:
  std::string location;
  std::string param; // XXX replace these two with attr/value pairs
  std::string value;

  cpl_node_ptr output;

public:
  // XXX WRITEME virtual cpl_run_result run();
  virtual ~remove_location_node() { }

  friend class remove_location_handler;
};


/*
 * Singleton handlers.
 */

extern location_handler the_location_handler;

extern lookup_handler the_lookup_handler;
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(lookup, success);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(lookup, notfound);
DECLARE_EXTERN_SINGLETON_SIMPLE_OUTPUT_HANDLER(lookup, failure);

extern remove_location_handler the_remove_location_handler;


#endif /* CPL_NODES_LOCATION_H_ */
