/* cpl-nodes-nca.h - handlers and definitions for CPL non-signalling
 * operation tags.   ("NCA" stands for "non-call actions", the old name for
 * this class of tags.)
 */ 
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_NODES_NCA_H_
#define CPL_NODES_NCA_H_

#include "cpl-nodes-internal.h"

/*
 * The handler for a <mail> node.
 */
class mail_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  mail_handler();

  static void set_url(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};


/*
 * A <mail> node.
 */
class mail_node : public cpl_node {
private:
  std::string url;

  cpl_node_ptr output;

public:
  mail_node() : cpl_node() { }

  virtual ~mail_node() { }

  friend class mail_handler;
};


/*
 * The handler for a <log> node.
 */
class log_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  log_handler();

  static void set_name(cpl_parser*, const std::string&);
  static void set_comment(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};


/*
 * An explicit <log> node.
 */
class log_node : public cpl_node {
private:
  std::string name;
  std::string comment;

  cpl_node_ptr output;

public:
  log_node() : cpl_node() { }

  virtual ~log_node() { }

  friend class log_handler;
};


/*
 * Singleton handlers.
 */

extern mail_handler the_mail_handler;

extern log_handler the_log_handler;


#endif /* CPL_NODES_NCA_H_ */
