/* cpl-nodes-toplevel.h - handlers and definitions for top-level CPL tags. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_NODES_TOPLEVEL_H_
#define CPL_NODES_TOPLEVEL_H_

#include "cpl-nodes-internal.h"

/* class cpl_toplevel_handler is declared in cpl-nodes.h, since it's public. */

/*
 * The handler for the <cpl> node.
 */
class cpl_cpl_handler : public element_handler {
public:
  cpl_cpl_handler();

  virtual void start(cpl_parser*) const;
  virtual void end(cpl_parser*) const;
};


/*
 * The handler for an <ancillary> output of <cpl>
 */
class cpl_ancillary_handler : public element_handler {
public:
  cpl_ancillary_handler();
};
  

/*
 * The handler for a <subaction> output of <cpl>
 */
class cpl_subaction_handler : public cpl_output_handler {
private:
  static void set_id(cpl_parser*, const std::string&);

public:
  cpl_subaction_handler();
  
  virtual cpl_parser_stackelement_ptr new_stack_frame() const;

  virtual void end(cpl_parser*) const;
};
  

/*
 * A stack frame for a subaction.  Contains information about the action's
 * ID.
 */
class subaction_stackelement : public cpl_parser_stackelement {
private:
  std::string id;

public:
  subaction_stackelement(const element_handler *h) : 
    cpl_parser_stackelement(h) { }

  ~subaction_stackelement() { }

  friend class cpl_subaction_handler;
};


/*
 * The handler for the <outgoing> output of <cpl>
 */
class cpl_outgoing_handler : public cpl_output_handler {
public:
  cpl_outgoing_handler() : cpl_output_handler(NULL, "outgoing") { }

  virtual void end(cpl_parser*) const;
};


/*
 * The handler for the <incoming> output of <cpl>
 */
class cpl_incoming_handler : public cpl_output_handler {
public:
  cpl_incoming_handler() : cpl_output_handler(NULL, "incoming") { }

  virtual void end(cpl_parser*) const;
};


/*
 * The handler for a <sub> node.
 * (This isn't actually a toplevel node, but it's closely enough associated
 * with <subaction> that it can go in this file.)
 */
class sub_handler : public cpl_node_handler {
protected:
  virtual cpl_node_ptr allocate_node() const;

public:
  sub_handler();

  static void set_ref(cpl_parser*, const std::string&);

  virtual void end(cpl_parser*) const;
};


/*
 * A <sub> node.
 * (This isn't actually a toplevel node, but it's closely enough associated
 * with <subaction> that it can go in this file.)
 */
class sub_node : public cpl_node {
private:
  std::string ref_name;
  cpl_node_ptr ref;

public:
  // XXX WRITEME virtual cpl_run_result run();
  virtual ~sub_node() { }

  friend class sub_handler;
};



/*
 * Singleton handlers.
 */

/* the_cpl_toplevel_handler is declared in cpl-nodes.h, since it's public. */
extern cpl_cpl_handler the_cpl_cpl_handler;

extern cpl_ancillary_handler the_cpl_ancillary_handler;
extern cpl_subaction_handler the_cpl_subaction_handler;
extern cpl_outgoing_handler the_cpl_outgoing_handler;
extern cpl_incoming_handler the_cpl_incoming_handler;

extern sub_handler the_sub_handler;


#endif /* CPL_NODES_TOPLEVEL_H_ */
