/* cpl-nodes.cpp - generic functions and definitions for CPL nodes. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#include <iostream>

#include "cpl-exception.h"

#include "cpl-nodes.h"
#include "cpl-nodes-internal.h"

#include "cpl-nodes-toplevel.h"
#include "cpl-nodes-switch.h"
#include "cpl-nodes-location.h"
#include "cpl-nodes-action.h"
#include "cpl-nodes-nca.h"

using namespace std;

/*
 * cpl_base_handler::output_all_cpl_nodes()
 * Fill this cpl base handler's subnode map with all the CPL node handlers.
 */
void cpl_base_handler::output_all_cpl_nodes()
{
  insert_subnode_map("address-switch", &the_address_switch_handler);
  insert_subnode_map("string-switch", &the_string_switch_handler);
  insert_subnode_map("time-switch", &the_time_switch_handler);
  insert_subnode_map("priority-switch", &the_priority_switch_handler);

  insert_subnode_map("location", &the_location_handler);
  insert_subnode_map("lookup", &the_lookup_handler);
  insert_subnode_map("remove-location", &the_remove_location_handler);

  insert_subnode_map("redirect", &the_redirect_handler);
  insert_subnode_map("proxy", &the_proxy_handler);
  insert_subnode_map("reject", &the_reject_handler);

  insert_subnode_map("mail", &the_mail_handler);
  insert_subnode_map("log", &the_log_handler);

  insert_subnode_map("sub", &the_sub_handler);
} /* cpl_base_handler::output_all_cpl_nodes() */


/*
 * cpl_node_handler::start(cpl_parser*) const
 * Allocate a CPL node, and push it onto the parse stack.
 */
void cpl_node_handler::start(cpl_parser* parser) const
{
  cpl_node_ptr p = allocate_node();
  
  parser->setParentCurrentOutput(p);
  parser->setCurrentNode(p);
} /* cpl_node_handler::start(cpl_parser*) const */


/*
 * cpl_output_handler::cpl_output_handler(const char *const,
 *                                        const char *const)
 * Create a CPL output handler: fill its subnode_map with all the CPL node
 * handlers.
 */
cpl_output_handler::cpl_output_handler(const char *const space,
                                       const char *const name) :
  cpl_base_handler(space, name)
{
  output_all_cpl_nodes();
} /* cpl_output_handler::cpl_output_handler(const char *const,
   *                                        const char *const) */


/*
 * cpl_node::run()
 * Dummy function so we don't have to define all the run() methods
 * too soon.
 * XXX: cpl_node::run() should be pure virtual once we've implemented
 * everything.  Delete me!
 */
cpl_run_result cpl_node::run()
{
  cerr << "cpl_node::run() called -- override me!" << endl;

  cpl_run_result res;
  res.result_type = cpl_run_result::result_execution_result;
  res.execution_result = cpl_execution::cpl_default;

  return res;
} /* cpl_node::run() */


/*
 * init_cpl_node_data()
 * Initialize the CPL node data: value map tables.
 * XXX todo: also initialize the node handlers -- eliminate static
 * constructors.
 */
void init_cpl_node_data()
{
  init_yes_no_map();

  init_string_field_map();
  init_address_field_map();
  init_address_subfield_map();
  init_priority_value_map();

  init_proxy_order_map();
  init_reject_status_map();
} /* init_cpl_node_data() */


/*
 * free_cpl_node_data()
 * Free dynamically-allocated CPL node data.
 * XXX todo: once we're dynamically allocating node handlers, delete them here.
 */
void free_cpl_node_data()
{ 
  // Nothing yet...
} /* free_cpl_node_data() */


std::map<std::string, bool> yes_no_map;

/*
 * init_yes_no_map()
 * Initialize a generically-useful map: from "yes"/"no" to boolean.
 */
void init_yes_no_map()
{
  yes_no_map["yes"] = true;
  yes_no_map["no"] = false;
} /* init_true_false_map() */
