/* cpl-nodes.h - class definitions for externally visible properties
 * of CPL nodes. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_NODES_H_
#define CPL_NODES_H_

#include <string>

#include "cpl.h"
#include "cpl-parse.h"

class cpl_run_result;

/*
 * Base class for generic CPL nodes.
 */
class cpl_node {
public:
  virtual cpl_run_result run() /* XXX = 0 */;

  virtual ~cpl_node() { };
};


/*
 * The result of calling cpl_node::run()
 */
class cpl_run_result {
public:
  enum { result_next_node, result_execution_result } result_type;
  cpl_node_ptr next_node;
  cpl_execution::cpl_execution_result execution_result;
};


/*
 * The CPL toplevel handler.  The element handler when we're outside
 * any elements.
 */
class cpl_toplevel_handler : public element_handler {
public:
  cpl_toplevel_handler();

  virtual void end(cpl_parser*) const;
};


/*
 * The singleton toplevel handler.
 */
extern cpl_toplevel_handler the_cpl_toplevel_handler;

/*
 * Initialize/cleanup CPL nodes.
 */
extern void init_cpl_node_data();
extern void free_cpl_node_data();


#endif /* CPL_NODES_H_ */
