/* cpl.h - external definitions for a parsed CPL. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CPL_H_
#define CPL_H_

#include <map>
#include <string>
#include <boost/smart_ptr.hpp>

class cpl_execution;
class cpl_node;
class cpl;
typedef boost::shared_ptr<cpl_node> cpl_node_ptr;
typedef boost::shared_ptr<cpl> cpl_ptr;

class InputSource;

class cpl {
private:
  std::map<std::string, cpl_node_ptr> subs;
  cpl_node_ptr incoming;
  cpl_node_ptr outgoing;

  static bool initialized;

protected:
  void internal_parse(const char *src);
  void internal_parse(const unsigned char *buf, size_t len,
                      const char* bufname);
  void internal_parse(InputSource& source);

public:
  cpl();
  cpl(const char *src);
  cpl(const unsigned char *buf, size_t len, const char* bufname);
  virtual ~cpl();

  void clear();
  
  void parse(const char* src);
  void parse(const unsigned char* buf, size_t len, const char* bufname);

  static void initialize();
  static void cleanup();

  friend class cpl_execution;

  // XXX hacks?
  friend class cpl_cpl_handler;
  friend class cpl_outgoing_handler;
  friend class cpl_incoming_handler;
  friend class cpl_subaction_handler;
  friend class sub_handler;

  static const char default_cpl_xml_public_id[] =
    "-//IETF//DTD RFCxxxx CPL 1.0//EN";

  static const char default_cpl_xml_namespace[] =
    "http://www.rfc-editor.org/rfc/rfcXXXX.txt";
};

class cpl_execution {
private:
  const cpl_ptr the_cpl;
  cpl_node_ptr current_node;

public:
  typedef enum { cpl_proxy, cpl_redirect, cpl_reject, cpl_default }
  cpl_execution_result;

  cpl_execution(const cpl& the_cpl);
  cpl_execution_result go();
};


#endif /* CPL_H_ */
