# Columbia SIP server makefile fragment for cpl
# Copyright 1998-2001 by Columbia University; all rights reserved 

cpl_dir := $(topdir)/cpl

CPL_MODULES := recur check

CPL_SRC := \
  $(cpl_dir)/cpl.cpp \
  $(cpl_dir)/cpl-exception.cpp \
  $(cpl_dir)/cpl-parse.cpp \
  $(cpl_dir)/cpl-nodes.cpp \
  $(cpl_dir)/cpl-nodes-toplevel.cpp \
  $(cpl_dir)/cpl-nodes-location.cpp \
  $(cpl_dir)/cpl-nodes-action.cpp \
  $(cpl_dir)/cpl-nodes-switch.cpp \
  $(cpl_dir)/cpl-nodes-nca.cpp \
  $(cpl_dir)/cpl-utils.cpp

CPL_HEADERS := \
  $(cpl_dir)/cpl.h \
  $(cpl_dir)/cpl-exception.h \
  $(cpl_dir)/cpl-parse.h \
  $(cpl_dir)/cpl-nodes.h \
  $(cpl_dir)/cpl-nodes-internal.h \
  $(cpl_dir)/cpl-nodes-toplevel.h \
  $(cpl_dir)/cpl-nodes-location.h \
  $(cpl_dir)/cpl-nodes-action.h \
  $(cpl_dir)/cpl-nodes-switch.h \
  $(cpl_dir)/cpl-nodes-nca.h \
  $(cpl_dir)/cpl-templatefns.h \
  $(cpl_dir)/cpl-utils.h
 
CPL_MISC := \
  $(cpl_dir)/module.mk

CPL_MISCCLEAN :=

CPL_OBJ_C := \
  $(patsubst %.c,%.o,$(filter %.c,$(CPL_SRC)))
CPL_OBJ_CPP := \
  $(patsubst %.cpp,%.o,$(filter %.cpp,$(CPL_SRC)))
CPL_OBJ := $(CPL_OBJ_C) $(CPL_OBJ_CPP)

CPL_DEP_C := \
  $(CPL_OBJ_C:.o=.d)
CPL_DEP_CPP := \
  $(CPL_OBJ_CPP:.o=.d)
CPL_DEP := \
  $(CPL_DEP_C) $(CPL_DEP_CPP)

# How other parts of the build find us:
# Target for make:
LIBCPL := $(cpl_dir)/libcpl.a

# Incantations for compile and build:
LIBCPL_INC := -I$(topsrcdir)/$(cpl_dir)
LIBCPL_LIB := -L$(cpl_dir) -lcpl

# Include files required locally.
CPL_INCS := $(XERCES_C_INC) $(LIBCINE_INC)
CPL_REQS := need-xerces-c

$(CPL_OBJ_C): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(CPL_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(CPL_DEP_C): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(cpl_dir) -o $@ $(CPL_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

$(CPL_OBJ_CPP): %.o: %.cpp
	@echo "compiling $<..."
	$(CXX) $(CPL_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(CPL_DEP_C): %.d: %.cpp
	@echo "dependencies for $<..."
	$(DEPEND) -d $(cpl_dir) -o $@ $(CPL_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

$(LIBCPL): $(CPL_OBJ) $(CPL_REQS)
	@echo "creating library $@..."
	$(AR) rc $@ $(CPL_OBJ)
	$(RANLIB) $@

.PHONY: cpl
cpl: $(LIBCPL)

SRC       += $(CPL_SRC)
HEADERS   += $(CPL_HEADERS)
MISCCLEAN += $(CPL_MISCCLEAN)

# Include the description for sub-modules
include $(patsubst %,$(topsrcdir)/$(cpl_dir)/%/module.mk,$(CPL_MODULES))

CPL_DIST := $(CPL_SRC) $(CPL_MAYBESRC) $(CPL_HEADERS) \
	$(CPL_MISC) $(CPL_MISCDIST) \
	$(CPL_CHECK_DIST)

CPL_BINDIST := $(CPL_BIN) $(CPL_MISCDIST) \
	$(CPL_CHECK_DIST)
