/* dayandposition.h - Represent a day-of-the-week and its position within
 * a larger unit (month or year) */
/* Copyright 2001 Columbia University.  All rights reserved. */

/* This is a manual translation of the Java Cal-code package. */

#ifndef DAYANDPOSITION_H_
#define DAYANDPOSITION_H_

#include <string>
#include <iosfwd>

#include "calendar.h"

class DayAndPosition {
private:
  Calendar::Weekday day;
  int position;
  
public:
  DayAndPosition();
  DayAndPosition(Calendar::Weekday d, int p);
  DayAndPosition(const std::string&);
  
  Calendar::Weekday getDayOfWeek() const;
  void setDayOfWeek(Calendar::Weekday d);
  
  std::string getString() const;
  void setString(const std::string& s);

  int getDayPosition() const;
  void setDayPosition(int p);
  
  bool operator==(const DayAndPosition& other) const;
  
  static bool isValidDayOfWeek(Calendar::Weekday d);
  static bool isValidDayPosition(int p);
  
  static Calendar::Weekday parseDayOfWeek(const std::string&);
  static std::string generateDayOfWeek(Calendar::Weekday);

  std::string toString() const;

  friend std::ostream& operator<<(std::ostream&, const DayAndPosition&);
};
                   

#endif /* DAYANDPOSITION_H_ */
