/* duration.h - Represent an iCalendar duration. */
/* Copyright 2001 Columbia University.  All rights reserved. */

/* This is a manual translation of the Java Cal-code package. */

#ifndef DURATION_H_
#define DURATION_H_

#include <time.h>
#include <string>

class Duration
{
protected:
  int weeks;
  int days;
  int hours;
  int minutes;
  int seconds;

public:
  static const int SECONDS_PER_MINUTE = 60;
  static const int MINUTES_PER_HOUR   = 60;
  static const int HOURS_PER_DAY      = 24;
  static const int DAYS_PER_WEEK      = 7;
  
  static const int SECONDS_PER_HOUR   =
    MINUTES_PER_HOUR * SECONDS_PER_MINUTE;
  static const int SECONDS_PER_DAY    =
    HOURS_PER_DAY * SECONDS_PER_HOUR;
  static const int SECONDS_PER_WEEK   =
    DAYS_PER_WEEK * SECONDS_PER_DAY;

  Duration();
  Duration(int d, int h, int m, int s);
  Duration(int h, int m, int s);
  Duration(int w);
  Duration(const std::string& str);
  
  void clear();
  
  std::string getString() const;
  void        setString(const std::string& str);

  int  getWeeks() const;
  void setWeeks(int w);
  
  int  getDays() const;
  void setDays(int d);
  
  int  getHours() const;
  void setHours(int h);
  
  int  getMinutes() const;
  void setMinutes(int m);
  
  int  getSeconds() const;
  void setSeconds(int s);
  
  time_t getInterval() const;
  
  void setInterval(time_t interval);
  
protected:
  void normalize();
  void checkWeeksOkay(int f);
  void checkNonWeeksOkay(int f);
  
public:
  std::string toString() const;
};

#endif /* DURATION_H_ */
