// cand-instant-test.h -- test candidate instants.

#include <iostream>

#include "calendar.h"
#include "recur.h"

using namespace std;

#define ARRAYSIZE(a) (sizeof(a) / sizeof((a)[0]))

#define FORMAT "%A, %B %e, %Y %l:%M:%S %p %Z"

int main(int argc, char * argv[])
{
  const time_t starts[] = 
  {
    105376260, // May 4, 1973, 11:11:00 EDT
    825614108, // February 29, 1996, 17:15:08 UTC
    917647014, // January 29, 1999, 21:56:54 UTC
    946684799  // December 31, 1999, 23:59:59 UTC
  };
      
  const time_t nows[] = 
  {
    970609319, // October 3, 2000, 21:41:59 UTC
    // time(), // now
    986184000, // April 2, 2001, 04:00:00 UTC
    1041825609 // January 6, 2003, 04:00:09 UTC
  };

  const Recurrence::Frequency types[] = 
  {
    Recurrence::NO_RECURRENCE,
    Recurrence::SECONDLY,
    Recurrence::MINUTELY,
    Recurrence::HOURLY,
    Recurrence::DAILY,
    Recurrence::WEEKLY,
    Recurrence::MONTHLY,
    Recurrence::YEARLY
  };

  unsigned int i, j, k;
  
  for (j = 0; j < ARRAYSIZE(types); j++) {
    for (i = 0; i < ARRAYSIZE(starts); i++) {
      for (k = 0; k < ARRAYSIZE(nows); k++) {
        try {
          Calendar start(starts[i], "GMT");
          
          Calendar now(nows[k], "GMT");
          
          Recurrence rule(start, Duration(), types[j]);
          
          Calendar* back = rule.getCandidateStartTime(now, false, false);
          Calendar* forward = rule.getCandidateStartTime(now, true, false);
          
          cout << "Start:      " << start.format(FORMAT) << endl;
          cout << "Interval:   " << types[j] << endl;
          cout << "Now:        " << now.format(FORMAT) << endl;
          cout << "Cand(back): " << 
            (back != NULL ? back->format(FORMAT) :
             "NULL") << endl;
          cout << "Cand(forw): " <<
            (forward != NULL ? forward->format(FORMAT) :
             "NULL") << endl;
          cout << endl;

          delete back;
          delete forward;
        }
        catch (calendar_exception& e) {
          cout.flush();
          cerr << "CAUGHT EXCEPTION: " << e.what() << endl;
          cerr << "\t(types[" << j << "] = " << types[j] <<
            ", starts[" << i << "] = " << starts[i] <<
            ", nows[" << k << "] = " << nows[k] << ")" << endl;
        }
      }
      cout << endl;
    }
    cout << endl;
  }
}
