# Makefile fragment for cinema recurrence tests
# Copyright 1998-2001 by Columbia University; all rights reserved 

cpl_recur_test_dir := $(cpl_recur_dir)/test

DIRS += $(cpl_recur_test_dir)


CPL_RECUR_TEST_BIN := \
	$(cpl_recur_test_dir)/woy-test \
	$(cpl_recur_test_dir)/cand-instant-test \
	$(cpl_recur_test_dir)/dst-test \
	$(cpl_recur_test_dir)/recur-test \
	$(cpl_recur_test_dir)/recur-string-test


CPL_RECUR_TEST_SRC := \
	$(cpl_recur_test_dir)/woy-test.cpp \
	$(cpl_recur_test_dir)/cand-instant-test.cpp \
	$(cpl_recur_test_dir)/dst-test.cpp \
	$(cpl_recur_test_dir)/recur-test.cpp \
	$(cpl_recur_test_dir)/tests.cpp \
	$(cpl_recur_test_dir)/tests-string.cpp


CPL_RECUR_TEST_MISC := \
	$(cpl_recur_test_dir)/module.mk


CPL_RECUR_TEST_OBJ_C := \
	$(patsubst %.c,%.o,$(filter %.c,$(CPL_RECUR_TEST_SRC)))
CPL_RECUR_TEST_OBJ_CPP := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(CPL_RECUR_TEST_SRC)))
CPL_RECUR_TEST_OBJ := \
	$(CPL_RECUR_TEST_OBJ_C) $(CPL_RECUR_TEST_OBJ_CPP)

CPL_RECUR_TEST_DEP_C := \
        $(CPL_RECUR_TEST_OBJ_C:.o=.d)
CPL_RECUR_TEST_DEP_CPP := \
        $(CPL_RECUR_TEST_OBJ_CPP:.o=.d)
CPL_RECUR_TEST_DEP := \
        $(CPL_RECUR_TEST_DEP_C) $(CPL_RECUR_TEST_DEP_CPP)


WOY_TEST_OBJS := \
	$(cpl_recur_test_dir)/woy-test.o

CAND_INSTANT_TEST_OBJS := \
	$(cpl_recur_test_dir)/cand-instant-test.o

DST_TEST_OBJS := \
	$(cpl_recur_test_dir)/dst-test.o

RECUR_TEST_OBJS := \
	$(cpl_recur_test_dir)/recur-test.o \
	$(cpl_recur_test_dir)/tests.o

RECUR_STRING_TEST_OBJS := \
	$(cpl_recur_test_dir)/recur-test.o \
	$(cpl_recur_test_dir)/tests-string.o


CPL_RECUR_TEST_LIBS := $(LIBCPLRECUR_LIB) $(LIBCINE_LIB)

CPL_RECUR_TEST_INCS := $(LIBCPLRECUR_INC) $(LIBCINE_INC)

$(CPL_RECUR_TEST_OBJ_C): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(CPL_RECUR_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(CPL_RECUR_TEST_DEP_C): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(cpl_recur_test_dir) -o $@ $(CPL_RECUR_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# Order of -c and -o matters.
$(CPL_RECUR_TEST_OBJ_CPP): %.o: %.cpp
	@echo "compiling $<..."
	$(CXX) $(CPL_RECUR_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -o $@ -c $<

$(CPL_RECUR_TEST_DEP_CPP): %.d: %.cpp
	@echo "dependencies for $<..."
	$(DEPEND) -d $(cpl_recur_test_dir) -o $@ $(CPL_RECUR_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<


test: $(CPL_RECUR_TEST_BIN)

$(cpl_recur_test_dir)/woy-test: $(WOY_TEST_OBJS) $(LIBCPLRECUR)
	@echo "linking $@..."
	$(CXX) -o $@ $(CFLAGS) $(LDFLAGS) $(WOY_TEST_OBJS) $(CPL_RECUR_TEST_LIBS) $(LIBS)

$(cpl_recur_test_dir)/cand-instant-test: $(CAND_INSTANT_TEST_OBJS) $(LIBCPLRECUR)
	@echo "linking $@..."
	$(CXX) -o $@ $(CFLAGS) $(LDFLAGS) $(CAND_INSTANT_TEST_OBJS) $(CPL_RECUR_TEST_LIBS) $(LIBS)

$(cpl_recur_test_dir)/dst-test: $(DST_TEST_OBJS) $(LIBCPLRECUR)
	@echo "linking $@..."
	$(CXX) -o $@ $(CFLAGS) $(LDFLAGS) $(DST_TEST_OBJS) $(CPL_RECUR_TEST_LIBS) $(LIBS)

$(cpl_recur_test_dir)/recur-test: $(RECUR_TEST_OBJS) $(LIBCPLRECUR)
	@echo "linking $@..."
	$(CXX) -o $@ $(CFLAGS) $(LDFLAGS) $(RECUR_TEST_OBJS) $(CPL_RECUR_TEST_LIBS) $(LIBS)

$(cpl_recur_test_dir)/recur-string-test: $(RECUR_STRING_TEST_OBJS) $(LIBCPLRECUR)
	@echo "linking $@..."
	$(CXX) -o $@ $(CFLAGS) $(LDFLAGS) $(RECUR_STRING_TEST_OBJS) $(CPL_RECUR_TEST_LIBS) $(LIBS)

BIN += $(CPL_RECUR_TEST_BIN)
SRC += $(CPL_RECUR_TEST_SRC)
HEADERS += $(CPL_RECUR_TEST_HEADERS)

CPL_RECUR_TEST_DIST := $(CPL_RECUR_TEST_SRC) $(CPL_RECUR_TEST_HEADERS) $(CPL_RECUR_TEST_MISC) \
	$(CPL_RECUR_TEST_MISCDIST)

# These test binaries don't need to go in the bindist
CPL_RECUR_TEST_BINDIST := $(CPL_RECUR_TEST_MISCDIST)
