/*
 * Copyright (c) 2000, 2001, Columbia University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution. 
 *
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.  
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "recur.h"

#include "tests.h"

using namespace std;

static void exploreRecur(RecurHolder& r);

#define FORMAT "%A, %B %e, %Y %l:%M:%S %p %Z"

int main(int argc, char *argv[]) {
  vector<RecurHolder> tests;
  try {
    tests = getTests();
  }
  catch (exception& e) {
    cerr << "Caught exception setting up tests: " << e.what() << endl;
    exit(1);
  }
  unsigned int i;

  for (i = 0; i < tests.size(); i++) {
    exploreRecur(tests[i]);
  }
}

static void exploreRecur(RecurHolder& r)
{
  int i;

  Calendar now = r.start;

  cout << r.name << endl;

  try {
    for (i = 0; i < r.runCount && now.getTime() < r.end; ) {
      time_t old_time = now.getTime();
      try {
        if (r.recurrence.isInRecurrence(now)) {
          cout << now.format(FORMAT) << endl;
          i++;
        }
      }
      catch (exception& e) {
        cerr << "Caught exception for " << now.format(FORMAT);
        cerr << ": " << e.what() << endl;
      }
      now.add(r.period, r.step);
      if (now.getTime() < old_time) {
        /* time_t wrapped */
        break;
      }
    }
  }
  catch (exception& e) {
    cerr << "Caught exception for " << r.name;
    cerr << ": " << e.what() << endl;
  }
  /* if i < runcount, this means that we ran over 'end', which probably
   * means we exhausted the set.  Otherwise, it means the set probably
   * continues. */
  if (i >= r.runCount) {
    cout << "..." << endl;
  }
  cout << endl;
}
