// timezone.h -- object wrapper around struct tzinfo

#ifndef TIMEZONE_H_
#define TIMEZONE_H_

#include "tztimeext.h"
#include <iosfwd>

class TimeZone {
private:
  struct tzinfo * tz;
  char * tzname;

  static char * default_tzname;

private:
  /* Forbid copying and assignment.  It's too expensive. */
  TimeZone(const TimeZone&);
  TimeZone& operator=(const TimeZone&);

  void init(const char *);
  static const char * getDefault();

public:
  TimeZone();
  TimeZone(const char *);
  ~TimeZone();

  const char * const getName() const { return tzname; }
  const struct tzinfo * getTZ() const { return tz; }

  operator const struct tzinfo* () const { return tz; }

  friend std::ostream& operator<<(std::ostream& os, const TimeZone& z) {
    os << (z.tzname != NULL ? z.tzname : "NULL");
    return os;
  }
  
  static void setDefault(const char *);
  static void clearDefault(const char *);
};
  


#endif /* TIMEZONE_H_ */
