#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# Above lines are more portable than hardcoding the tclsh path.

# Installation script for cinema system for
# binary distribution.

# Various directories in the distribution.
set applications [list "sipd" "rtspd" "sip323" "sipconf" "sipum" "web" "tools" "doc" "mysql" "license"]

set reportfooter "  Please report any installation problem to 
    \"Kundan Singh\" <kns10@cs.columbia.edu> (preferred), 
    \"Jonathan Lennox\" <lennox@cs.columbia.edu> or 
    \"Henning Schulzrinne\" <hgs@cs.columbia.edu>"

proc waitforuser {} {
    puts -nonewline "\nPress <return> to continue ..."
    gets stdin dummy
}

proc getpath {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    while { [gets stdin dummy] > 0} {
        if { [file exists "$dummy"] == 1} {
            return $dummy
        } else {
            puts "This path does not exists. Enter another."
            puts -nonewline "> "
            flush stdout
        }
    }
    if { $def != "" } {
      puts "Assuming default value as\n $def"
    }
    return $def
}

proc getstring {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    if { [gets stdin dummy] > 0} {
        return $dummy
    } else {
        puts "Assuming default value as $def"
        return $def    
    }
}

proc checkmodulepath {def {name ""} {url ""}} {
  global tcl_platform
  set thispath  $def
  set binary [file tail $def]
  if { $name == "" } {
      set name $binary
  }
  if { $url == "" } {
      set url "README"
  }
  if { [info exists tcl_platform(platform)] &&
       $tcl_platform(platform) == "windows" } {
    set thispath [getpath "Enter the path for $name, e.g., /progra~1/$name/bin/$binary.exe." "$def"]
    if {![file exists $thispath]} {
      puts "$thispath does not exists. Please try again."
      exit 1
    }
  } else {
   if { [catch {set thispath [exec which $binary]}] == 0 && 
        [lindex [split $thispath " "] 0] != "no" } {
       puts "$name is detected at $thispath.
       If this is incorrect terminate the script now."
   } else {
       puts "$name is not installed. Please install it (see $url),
       add it in your PATH environment variable
       and re-run this installation script."
       exit 1
   }
  }
  return $thispath
}

proc ExecuteSQL {host user passwd database command} {
    global mysqlpath
    set e 0
    if { $host == "localhost" && $passwd == "NULL" } {
        set e [catch {exec $mysqlpath -u$user -e "$command" $database} msg]
    } elseif { $host == "localhost" && $passwd != "NULL" } {
        set e [catch {exec $mysqlpath -u$user -p$passwd -e "$command" $database} msg]
    } elseif { $host != "localhost" && $passwd == "NULL" } {
        set e [catch {exec $mysqlpath -u$user -h$host -e "$command" $database} msg]
    } else { 
        set e [catch {exec $mysqlpath -u$user -h$host -p$passwd -e "$command" $database} msg]
    }
    if { $e == 0 } {
      puts "$command"
    } else {
      puts "Mysql error in $command with $host,$user,$passwd: $msg\n"
    }
}

puts "CINEMA, (c) 2001, Columbia University, installation script."
puts "  If the installation gets terminated abmormally, please
  uncompress the binary distribution again and then retry.

  $reportfooter
"

if { $argv0 != "./install.tcl" } {
    puts "Change your current directory to the directory where
  install.tcl is present and invoke the script as 
    $ ./install.tcl         -- on Unix
    $ tclsh ./installtcl    -- on Windows"
    exit 1;
}

# Get the current directory and set it as install directory.
set installdir [pwd]

# Initialize present array if the directory is present
foreach app $applications {
    if {[file isdirectory "$installdir/$app"]} {
        set present($app) 1 
    } else {
        set present($app) 0
    }
}
if { $present(sipd) || $present(sipum) || $present(sipconf) } {
    set present(mysql)  1
}
if { $present(sipd) || $present(sipum) || $present(sipconf) || $present(sip323) } {
    set present(license) 1
}

puts -nonewline "Available applications: "
foreach app $applications {
    if {[file isdirectory "$installdir/$app"]} {
        puts -nonewline "$app,"
    }
}
puts ""

puts "Installation directory is $installdir"

# Other applications...
if { [info exists tcl_platform(platform)] && $tcl_platform(platform) == "windows" } {
    set addsipuser "tools/addsipuser/Release/addsipuser.exe"
    set md5string  "tools/md5string/Release/md5string.exe"
    set random32   "tools/random32/Release/random32.exe"
    set fbsql      "tools/fbsql/Release/fbsql.dll"
    set canonicalize "../tools/canonicalize/Release/canonicalize -d -D ../tools/canonicalize/dialplan.sample"
} else {
    set addsipuser "tools/addsipuser/addsipuser"
    set md5string  "tools/md5string/md5string"
    set random32   "tools/random32/random32"
    set fbsql      "tools/fbsql/libfbsql.so"
    set canonicalize "../tools/canonicalize/canonicalize -a -d -n -D ../tools/canonicalize/dialplan.sample"
}


set hostname "localhost"

# Try getting the local host name.
puts "Trying to find local host name..."
catch {set hostname [exec uname -n]}
if { $hostname == "localhost" } {
  catch {set hostname "[hostname]"}
}
puts "Host name is $hostname"

# We assume that the $hostname is of the form host.domain
# where domain is used for default value of DOMAINS.
set dummylist [split $hostname .]
set dummylist [lrange $dummylist 1 [llength $dummylist]]
set domain    [join $dummylist .]
set dummylist [linsert $dummylist 0 www]

if {$domain == "" } {
  set domain "wonderland.com"
  set domain [getstring "
Enter the domain of your organization.
default is $domain" $domain]
  puts ""
}

# Check for tclsh
if { $present(web) == 1 } {
  set tclpath [checkmodulepath "/usr/bin/tclsh" "Tcl" "http://dev.scriptics.com"]
}

# Check if mysql is present in the path.
if {$present(mysql) == 1 } {
  set mysqlpath [checkmodulepath "/usr/local/bin/mysql" "MySQL" "http://www.mysql.com"]
}

# Add the sipd directory to the LD_LIBRARY_PATH, for the mysql shared lib
# (and the libraries it depends on).
if { $present(sipd) == 1 } {
  if {[info exists env(LD_LIBRARY_PATH)]} {
    set env(LD_LIBRARY_PATH) "$env(LD_LIBRARY_PATH):$installdir/sipd"
  } else {
    set env(LD_LIBRARY_PATH) "$installdir/sipd"
  }
}

# Email domain defaults to domain
if { $present(web) || $present(mysql) || $present(rtspd) } {
  set emaildomain $domain
  set emaildomain [getstring "
Enter the email domain of your organization.
default is $emaildomain" $emaildomain]
  puts ""

  set LOGNAME "root"
  catch {set LOGNAME [exec whoami]}
  puts "Administrator's login id is $LOGNAME"

  set adminemail "$LOGNAME@$emaildomain"
  puts "Administrator's email is $adminemail"
}

if { $present(mysql) == 1 } {
  # Get the password and host name for mysql.
  set mysqluser "root"
  set mysqluser [getstring "
Enter the user name used for accessing the MySQL database.
Default is $mysqluser" $mysqluser]
  puts ""

  set mysqlhost "localhost"
  set mysqlhost [getstring "
Enter the host name on which MySQL server is running.
Default is $mysqlhost" $mysqlhost]
  puts ""

  set mysqlpasswd "NULL"
  set mysqlpasswd [getstring "
Enter the user password for accessing mysql for user $mysqluser.
Default is $mysqlpasswd" $mysqlpasswd]
  puts ""
}

if { $present(web) } {
  # Get the host names for various applications :web server, sipd.
  set webhost $hostname
  set webhost [getstring "
Enter the host name on which the HTTP server is running.
Default is $webhost" $webhost]
  puts ""
}

if { $present(sipd) == 1 } {
  set sipdhost $hostname
  set sipdhost [getstring "
Enter the hostname on which the SIP server (sipd) will be running.
Default is $sipdhost" $sipdhost]
  puts ""
}

if { $present(sipum) == 1 } {
  set sipumhost $hostname
  set sipumhost [getstring "
Enter the hostname on which the SIP unified messaging (sipum) will be running.
Default is $sipumhost" $sipumhost]
  puts ""
}

if { $present(rtspd) == 1 } {
  set rtspdhost "$hostname:8554"
  set rtspdhost [getstring "
Enter the hostname and port number on which 
the RTSP media server (rtspd) will be running.
Default is $rtspdhost" $rtspdhost]
  puts ""
}

if { $present(sipconf) == 1 } {
  set sipconfhost $hostname
  set sipconfhost [getstring "
Enter the hostname on which the SIP conferencin server (sipconf) will be running.
Default is $sipconfhost" $sipconfhost]
  puts ""
}

if { $present(sip323) == 1 } {
  set sip323host $hostname
  set sip323host [getstring "
Enter the hostname on which the SIP-H.323 converter (siph323) will be running.
Default is $sip323host" $sip323host]
  puts ""
}

# Update the mysql database
# Try setting privileges for different users on different hosts.
if { $present(mysql) == 1 } {
  if { $mysqlpasswd != "NULL" } {
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
    "UPDATE user SET Password=PASSWORD('$mysqlpasswd') WHERE user='$mysqluser'"
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
    "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$hostname IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
    "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$mysqlhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    if { $present(web) == 1} {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    }
    if { $present(sipd) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipdhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    }
    if { $present(sipum) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipumhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    }
    if { $present(sipconf) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipconfhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"
    }
  } else {
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
    "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$hostname  WITH GRANT OPTION"
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
    "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$mysqlhost  WITH GRANT OPTION"
    if { $present(web) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhost  WITH GRANT OPTION"
    }
    if { $present(sipd) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipdhost  WITH GRANT OPTION"
    }
    if { $present(sipum) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipumhost  WITH GRANT OPTION"
    }
    if { $present(sipconf) == 1 } {
      ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
      "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipconfhost  WITH GRANT OPTION"
    }
  }

  ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "mysql" \
  "FLUSH PRIVILEGES"
}

# Creating the SIP database
if { $present(mysql) } {
  if { $mysqlpasswd != "NULL" && \
  [catch {exec $mysqlpath -u$mysqluser -p$mysqlpasswd -h$mysqlhost < "scripts/createsip.sql"} msg] != 0 || \
  $mysqlpasswd == "NULL" && \
  [catch {exec $mysqlpath -u$mysqluser -h$mysqlhost < "scripts/createsip.sql"} msg] != 0 \
  } {
    puts "Could not create sip database, ignoring: $msg"
  } else {
    puts "Created empty SIP database.\n"
  }

  # Adding this user as the administrator.
  set adminpasswd $mysqlpasswd
  if { $adminpasswd == "NULL" } {
    set adminpasswd $mysqluser
    if { $adminpasswd == "NULL" || $adminpasswd == "" } {
      set adminpasswd "none"
    }
  }

  puts "*IMPORTANT*: Adding administrator $adminemail with password $adminpasswd"

  catch {exec $addsipuser -d sql://$mysqluser:$mysqlpasswd@$mysqlhost/sip -u $adminemail -p $adminpasswd -r $domain -a -f} msg
  puts $msg

  if { $present(sipum) == 1} {
    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "ALTER TABLE put ALTER COLUMN sip_groups SET DEFAULT 'cgi voicemail'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET sip_groups='cgi voicemail' where user='default@$domain'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET sip_groups='cgi admin voicemail' where user='$adminemail'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET busy='rtsp://SERVER/audio/welcome.au' where user='default@$domain' OR  user='$adminemail'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET noresponse='rtsp://SERVER/audio/welcome.au' where user='default@$domain' OR  user='$adminemail'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET um_timeout='15' where user='default@$domain' OR  user='$adminemail'"

    ExecuteSQL $mysqlhost $mysqluser $mysqlpasswd "sip" \
    "UPDATE put SET message_template='To: \[email \$to\]
Reply-To: \[email \$from\]
Subject: voice mail - \$subject
Priority: \$priority

Dear \$name,

Voice mail from \[dataurl \$from \$from\] has arrived. 
You can play the message by using QuickTime using   
url \$rtspurl or
by going to the web page at \$httpurl.

- \$administrator

--
This mail is auto generated by the vmail script.
' where user='default@$domain' OR  user='$adminemail'"

    }
    # If present sipum (and mysql)
  }
  # If present mysql

if { $present(web) == 1 } {
  set webpath  "$installdir/web"
  set webpath [getpath "
Enter the absolute directory path where you would like to install
the web files. This must be accessible from your HTTP server.
A subdirectory \"cinema\" is created under this directory.
Default is $webpath" $webpath]
  puts ""

  if { $webpath != "$installdir/web" } {
    set webpath "$webpath/cinema"

    if { [file isdirectory $webpath] } {
        puts "Web path exists. Renaming to cinema_old."
        catch {file rename -force $webpath [file dirname $webpath]/cinema_old}
    } else {
        if {[catch {file mkdir $webpath} msg] != 0} {
            puts "Could not create $webpath: $msg"
        }
    }
    puts "Copying web files ... please wait."

    # Set the tclpath
    set newtclpath $tclpath
    catch {regsub -all / $tclpath \\/ newtclpath}

    if {[catch {set files [glob -- "$installdir/web/*"]} msg] == 0} {
      foreach filen $files {
        puts " [file tail $filen]..."
        if {[file isdirectory $filen]} {
          if {[catch {
             file mkdir "$webpath/[file tail $filen]"
             set subfiles [glob -- "$installdir/web/[file tail $filen]/*"]
             foreach subfile $subfiles {
               puts " [file tail $filen]/[file tail $subfile]"
               if {[catch {
                   file copy -force "$subfile" "$webpath/[file tail $filen]/[file tail $subfile]"} msg ] != 0 } {
                       puts "    Error in copy: $msg"
                   }
             }
          } msg] != 0 } {
             puts "    Error in creating dir: $msg"
          }
        } elseif {[file isfile $filen]} {
          if { [file extension $filen] == ".cgi" || [file extension $filen] == ".tcl"} {                
            if {[catch {
              set fp1 [open "$filen" "r"]
              set fp2 [open "$webpath/[file tail $filen]" "w"]
              
              puts $fp2 "#! $tclpath"
              
              fcopy $fp1 $fp2
              close $fp1
              close $fp2
            } msg] != 0 } {
                puts "    Error: $msg"
            }
        } else {
          if {[catch {file copy -force "$filen" "$webpath/[file tail $filen]"} msg] != 0} {
              puts "    Error: $msg"
          }
        }         
      } else {
        puts "   Neither dir nor file; ignored."
      }
    } 
    # For all files

  } else {
    puts "Could not list the web directory $installdir/*: $msg"
  }
} else {
    puts "Web path already exists in installation directory."
}

  if {[catch {file copy -force "$installdir/$md5string" "$webpath"} msg] != 0} {
    puts "Could not copy [file tail $md5string]: $msg"
  }
  if {[catch {file copy -force "$installdir/$random32" "$webpath"} msg] != 0} {
    puts "Could not copy [file tail $random32]: $msg"
  }
  if {[catch {file copy -force "$installdir/$fbsql" "$webpath"} msg] != 0} {
    puts "Could not copy [file tail $fbsql]: $msg"
  }

  if {[info exists tcl_platform(platform)] && $tcl_platform(platform) == "windows"} {
    puts "Make sure that you have enough permissions for $webpath"
  } else {
    if {[catch {exec chmod -R 755 "$webpath"} msg] != 0} {
      puts "Could not change permissions for $webpath: $msg"
    }
  }
}

# Set voicemail path if needed.
if { $present(sipum) == 1 } {
  set vmailuserdir  "/user/local/sipum/users"
  set vmailuserdir [getpath "
Enter the absolute directory path where you would like to create 
per-user directory. Default is $vmailuserdir" $vmailuserdir]
  puts ""

  if {![file isdirectory "$vmailuserdir/audio"]} {
    if {[catch {file mkdir "$vmailuserdir/audio"} err] !=0 } {
      puts "Could not created $vmailuserdir/audio: $err"
    }
  }

  if {[catch {file copy -force "$installdir/sipum/audio/welcome.au" "$vmailuserdir/audio/welcome.au"} err] != 0 } {
    puts "Could not copy welcome.au: $err"
  }
}

# Creating cinema.conf file.
if { $present(web) == 1} {
  if {[catch {set fp [open "$webpath/cinema.conf" "w"]} msg ] == 0} {
     puts $fp "
# CINEMA configuration file
# SQL database server
SQL sql://$mysqluser:$mysqlpasswd@$mysqlhost/sip

# domains this server is handling
DOMAINS $domain

# E-mail address of the maintainer of the web pages (to whom errors are sent)
ADMINISTRATOR $adminemail

# Directory for the sipd configuration file
SERVERROOT    $installdir/sipd

# Mode is local or remote
MODE          local

# Default realm is same as the domain name.
REALM  $domain
"

    if { $present(sipum) != 0 } {
      puts $fp "
# Top level directory, under which all user accounts are maintained.
VMAILUSERDIR $vmailuserdir
"
      puts $fp "
# Host name of thr voice mail server. Default is \"vmail.$domain\"
VMAILHOST $sipumhost
"
    }

    catch {close $fp}
    puts "Created $webpath/cinema.conf..."
  } else {
     puts "Could not open $webpath/cinema.conf for writing: $msg"
  }
}

# Creating cinema.conf file.
if { $present(sipd) == 1 } {
  if {[catch {set fp [open "sipd/sipd.conf" "w"]} msg ] == 0} {
    puts $fp "
ServerRoot    .
ErrorLog      stderr
# The canonical name of this domain is 'example.com'.
ServerName    $domain
# We accept registrations for any host *.example.com, or our own IP address.
Domain        $domain
Canonicalize  \"$canonicalize\"
#AuthMethod    Digest
#AuthMethod    Basic
AuthMethod    none

PrivateKey    1
NonceLifeTime 60
ClockSkew 300
LogFormat     \"%h %u %t \\\"%r\\\" %s To=%{To}i From=%{From}i Call-ID=%{Call-ID}i %T\"
RequestLog    logs/access_log
RequestLog    \"|tee /tmp/sipoutput.\$USER\"
DisplayAmbiguous true
#HelpResolve    true
ProxyRecursion  true
ThirdPartyReg   true
ForeignDomain   proxy
UserConfig      sql://$mysqluser:$mysqlpasswd@$mysqlhost/sip
DefaultReg      proxy
GatewayMap      gateways.sample
EnumRoot        trial.e164.com
AgentxSocketPath 127.0.0.1:5062
StartSnmp       false
TryStateless    false
"
    catch {close $fp}
    puts "Created sipd/sipd.conf...."

  } else {
    puts "Could not create sipd/sipd.conf: $msg"
  }
}
 
if { $present(rtspd) } {
  if { $present(sipum) } {
      set docroot "$vmailuserdir"
  } else {
      set docroot "$installdir/rtspd"
  }

  if {[catch {set fp [open "rtspd/rtspd.conf" "w"]} msg ] == 0} {
    puts $fp \
"DocumentRoot  $docroot
CNAME         $adminemail
ServerRoot    $installdir/rtspd
"
    catch {close $fp}
    puts "Created rtspd/rtspd.conf...."

  } else {
    puts "Could not create rtspd/rtspd.conf: $msg"
  }
}

if { $present(sipum) } {
  if {[catch {set fp [open "sipum/script/config" "w"]} msg ] == 0} {
    puts $fp "
vmailserver $sipumhost:10022
rtspserver  $rtspdhost
httpserver  $webhost
emaildomain $domain

onbusy      rtsp://SERVER/audio/welcome.au
noresponse  rtsp://SERVER/audio/welcome.au
topdir      $vmailuserdir/LOGNAME/
msgfile     $vmailuserdir/LOGNAME/inbox/MSGID.au
rtspmsgurl  rtsp://SERVER/LOGNAME/inbox/MSGID.au
httpmsgurl  http://$webhost/cinema/sipum/script/vmail.cgi
vmailhome   http://$webhost/cinema/sipum/script/vmail.cgi
register    LOGNAME@DOMAIN default
waittimeout 15
emailto     LOGNAME@DOMAIN
randomdigits 6
totalsize_KB  2000
max_msgsize_kb 1000
"
    catch {close $fp}
    puts "Created sipum/script/config...."

  } else {
    puts "Could not create sipum/script/config: $msg"
  }
}

puts "Installation done..."

if { $present(license) == 1} { 
  puts "Please insert the license string provided to you in
appropriate license file."
}

if { $present(sipd) == 1} {
  puts "You can run following commands to see if sipd works or not:
 \$ cd $installdir/sipd
 \$ ./start-sipd -v -X -s .
"
}

if { $present(rtspd) == 1} {
  puts "You can run following commands to see if rtspd works or not:
 \$ cd $installdir/rtspd
 \$ ./rtspd -v -X -p 8554 .
"
}

if { $present(sip323) == 1} {
  puts "You can run following commands to see if siph323 works or not:
 \$ cd $installdir/sip323
 \$ ./siph323 -h
"
}

if { $present(sipum) == 1} {
  puts "You can run following commands to see if sipum works or not:
 \$ cd $installdir/sipum
 \$ ./sipconf -v -X -r -D$mysqlhost -P$mysqlpasswd -U$mysqluser
"
}

if { $present(sipconf) == 1} {
  puts "You can run following commands to see if sipconf works or not:
 \$ cd $installdir/sipconf
 \$ ./sipconf -h
"
} 

puts "You add new user or to change your profile visit appropriate
URL corresponding to $webpath.
Make sure that you have followed the instructions (1) through
(4) of binary installation in doc/install.html.

Once again, $reportfooter"
