#ifndef PARSER_SIP_H
#define PARSER_SIP_H
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "http.h"
#include "types.h"
#include "uri.h"
#include "dstring.h"

typedef struct sip_via {              /* SIP Via header */
  int type;                           /* SOCK_DGRAM or SOCK_STREAM */
  struct sockaddr_in sin;             /* host address */
  //sock_t sin;             /* host address */
  struct in_addr received;            /* real address received from,
                                       * if different from sin->sin_addr */
  char *hostname;                     /* hostname */
  int islocal;                        /* this host is us! */
  struct {
    char *str;                        /* branch ID, as string */
    unsigned long hash;               /* hash part of branch ID */
    int id;                           /* numeric branch identifier */
  } branch;
} sip_via_t;

typedef struct sip_cseq {
  u_int32 seq;                        /* sequence number */
  char method[21];                    /* INVITE, CANCEL, ... */
} sip_cseq_t;

typedef struct sip_addr_t {
  char *display_name;                 /* (dyn) */
  uri_t uri;
} sip_addr_t;

typedef enum {L_unspecified, L_proxy, L_redirect, L_reject} sip_action_t;

typedef struct sip_contact_t {
  time_t expires;                     /* needed for registration */
  sip_action_t action;
  time_t last_modified;               /* last updated */
  float q;
  uri_t uri;
  char *nameaddr;
  char *comment;
  char *methods;                      /* which methods can be proxied */
} sip_contact_t;

extern sip_addr_t sip_addr(char *v);
extern int             SIP_NameAddr(char *v, DString *name, DString *addr);
extern sip_via_t       SIP_Via     (parser_t *, int order, int resolve,
                                    char *localhost, unsigned short localport);
extern sip_cseq_t      SIP_CSeq    (parser_t *);
extern char           *SIP_CallID  (parser_t *);
extern time_t          SIP_Expires (parser_t *);
extern sip_addr_t      SIP_To      (parser_t *);
extern sip_addr_t      SIP_From    (parser_t *);
extern const char     *SIP_Subject (parser_t *);
extern sip_contact_t  *SIP_Contact (parser_t *);
extern int             SIP_Has_Mandatory_Headers (parser_t *);

extern int SIP_ViaEqual (const sip_via_t *via1, const sip_via_t *via2);


extern char           *SIP_BuildContact (const sip_contact_t *);
extern void            SIP_FreeContact (sip_contact_t *);

extern char           *SIP_BuildNameAddr (const char *, const uri_t *);

extern const int default_sip_port;

extern const uri_defaults_t sip_uri_defaults[];

#endif
