/*
* E6181 Advanced Internet Services
* Li Liao
* Project: SSL/TSL for SIPD server
* ssl_tcp.c - handle incoming SSL TCP connections and requests
*/

#include <stdio.h>     /* FILE */
#include <string.h>    /* strerror */
#include <pthread.h>   /* pthread_ */
#include <stdlib.h>    /* malloc/free */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>    /* close */
#include "request-common.h"
#include "error.h"
#include "pthread-util.h"
#include "sysdep.h"
#include "thread_pool.h"
#include "ssl_common.h"

#include "ssl_tcp.h"

/*
 * ReceiveSSLTCPRequests()
 * Process incoming SSL TCP requests and send out responses.
 * Messages without Content-Length are assumed to have no body. (This
 * differs from HTTP/1.0 conventions.)
 */
void *ReceiveSSLTCPRequests(void *arg)
{
  struct ssl_socket_info *si = (struct ssl_socket_info *)arg;
  enum {Header, CR, CRNL, CRNLCR, NL, Body} state = Header;
  int content_length;      /* Content-Length field */
  int cl = 0;              /* bytes read in content (body) */
  int c;                   /* current character */
  char ch;                 /* same, as char */
  DString header, body;    /* header and body */
  FILE *f;                 /* socket, as FILE */
  request_common_t *r;     /* request parameters */
  char buf[1024];  
  int bytes_read;
  sock_t dum_s;

  if ( si == NULL ) {
    error("ReceiveSSLTCPRequests", "param is NULL\n");
    return NULL;
  }


/*
  f = fdopen_socket(si->socket, "r");
  if (f == NULL) {
    error ("ReceiveSSLTCPRequests", "Couldn't fdopen socket %d for reading",
           si->socket);
    free(si);
    return NULL;
  }
*/


  c = EOF; /* To avoid warning */  
  f = fopen("tmp1", "rw");
  while((bytes_read = SSL_read(si->ssl, buf, 1024)) >0 ) {
    //bytes_read = SSL_read(si->ssl, buf, 1024);
    printf("bytes_read=%d\n", bytes_read);
    printf("content=%s", buf);
    fwrite(buf, 1, bytes_read, f);


  /* Once around the loop for each message. */
  do {
    DStringInit(&header);
    DStringInit(&body);
    state          = Header;
    content_length = INFINITE_CONTENT_LENGTH;
    cl             = 0;
    r              = NULL;

    while ((cl < content_length) && ((c = getc(f)) != EOF)) {
      ch = c;
      DStringAppend(state == Body ? &body : &header, &ch, 1);
      switch (state) {
      case Header:
        if (c == '\r') state = CR;
        else if (c == '\n') state = NL;
        break;
      case CR:
        if (c == '\n') state = CRNL;
        else state = Header;
	break;
      case CRNL:
        if (c == '\r') state = CRNLCR;
        else state = Header;
        break;
      case NL:
      case CRNLCR:
        if (c == '\n') {
          if (header.length > 4) {
            /*
            debug("ReceiveSSLTCPRequests",
                  "Incoming TCP message on socket %d from %s:%d:\n%s\n",
                  si->socket,
                  inet_ntoa(si->sin.sin_addr), ntohs(si->sin.sin_port),
                  DStringValue(&header));
            */
            //dum_s.ssl = si->ssl;
            dum_s.ssl = si->ssl;
            
            r = (request_common_t *) RequestCreate(SOCK_SSL, si->socket, dum_s);
            r->server = si->server; /* To know original server */

            if (HTTP_Parse(DStringValue(&header), &r->p) < 0) {
	      /* If this was a request whose headers didn't parse, we want
	       * to try to send a 400 response to it.  On the other hand,
	       * there's not much we can do with responses that didn't parse,
	       * so just mark them as the unknown message type so that the
	       * server ignores them and frees their memory.
	       */
	      if (r->p.type == http_request) {
                ResponseSetStatus((request_t *)r, 400, NULL);
	      } else {
		r->p.type = http_unknown_type;
	      }
            }
            content_length = HTTP_ContentLength(&r->p);
            if (missing_content_length_means_zero) { /* RTSP */
              /* INFINITE_CONTENT_LENGTH is the "default" value returned by
                 the parser, implying content length was not specified */
              if (content_length == INFINITE_CONTENT_LENGTH) {
		     content_length = 0;
              }
            }
          }
          else content_length = -1;
          DStringFree(&header);
          state = Body;
        }
        else state = Header;
        break; 
      case Body:
        cl++;
        break;
      }
    }
    /* We now have a complete message, with header and content. */
    if (r) {
      r->request_body.content = DStringValue(&body);
      r->request_body.content_length = content_length;
      RequestProcess((request_t *)r);
    }
  } while (c != EOF);
  }
  fclose(f);

  /*
  debug("ReceiveTCPRequests", "Connection from %s:%d closed (socket %d)\n",
    inet_ntoa(si->sin.sin_addr), ntohs(si->sin.sin_port), si->socket);
  */

  free(si);

  return NULL;
} /* ReceiveSSLTCPRequests */


/*
* SSL TCP request receiver. 
*/
void *ReceiveSSLTCP(void *arg)
{
  struct ssl_tcp_thread_arg *ptr = (struct ssl_tcp_thread_arg *)arg; 
  void *server = ptr->server;
  unsigned short port = (ptr->port);  /* SSL port. This is int and not short */
  struct ssl_socket_info *si;  /* accept socket information */
  struct sockaddr_in sin;
  SSL *ssl;  /* bind socket address */
  SSL_CTX *ctx;
  BIO *sbio;

  int sinlen;              /* length of address */
  int s;                   /* socket for incoming TCP connection requests */
  int thr_err;             /* thread error condition */
  int opt;                 /* opt for setting setsockopt */


  /* build ssl context */
  ctx = initialize_ctx(KEYFILE, PASSWORD);
  load_dh_params(ctx, DHFILE);
  generate_eph_rsa_key(ctx);

  /* build socket. */
  if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    error("ReceiveSSLTCP", "socket\n");
    return NULL;
  }

  /* Bind to server address. */
  memset(&sin, 0, sizeof(sin));
  sin.sin_addr.s_addr=INADDR_ANY;
  sin.sin_family=AF_INET;
  sin.sin_port=htons(port);

  /* Set "reuseaddr" (and "reuseport", if it exists) so that we don't
   * have to wait for TIME_WAIT to expire if we crash the server while
   * connections are open. */
  opt = 1;
  if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(opt)) < 0) {
    error("RecieveSSLTCP", "setsockopt(SO_REUSEADDR)");
    return NULL;
  }

  if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
    error("ReceiveSSLTCP", "bind on %hu\n", port);
    return NULL;
  }
  if (listen(s, 1000) < 0) {
    return NULL;
  }


  /* Block waiting for connection. */
  while (1) {
    si = (struct ssl_socket_info *) malloc(sizeof(struct ssl_socket_info));
    if (si == NULL) {
      error("ReceiveSLLTCP", "malloc\n");
      return NULL;
    }
    
    si->server  = server;
      
    sinlen = sizeof(si->ssl);
    if ((si->socket = accept(s, 0, &sinlen)) < 0) {
    //if ((si->socket = accept(s, 0, 0)) < 0) {
      error("ReceiveSSLTCP", "accept\n");
      free(si);
      return NULL;
    }
    /*
    debug("ReceiveSSLTCP", "Connection from %s:%d (socket %d)\n",
      inet_ntoa(si->sin.sin_addr), ntohs(si->sin.sin_port), si->socket);
    */

    /* build ssl socket */
    sbio = BIO_new_socket(si->socket, BIO_NOCLOSE);
    ssl = SSL_new(ctx);
    SSL_set_bio(ssl, sbio, sbio);
    si->ssl = ssl;

    /* accept ssl socket */
    if(SSL_accept(ssl)<=0) {
      error("Problem with", "SSL accept");
      return NULL;
    }

    /* Create a new thread for each TCP connection. */
    if ((thr_err = pool_schedule(ReceiveSSLTCPRequests, (void *)si))) {
      error("ReceiveTCP", "unable to get thread from pool\n");
      closesocket(si->socket);
      free(si);
    }
    
  }

  return NULL;
} /* ReceiveSSLTCP */

