/* Replacement for getutent() on systems that don't have it */

#include <sys/types.h>
#include <utmp.h>
#include <unistd.h>
#include <fcntl.h>
#include "getutent.h"

static int utmp_file_fd = -1;


static int get_utmp_file_fd(void)
{
  if (utmp_file_fd == -1) {
    utmp_file_fd = open(_PATH_UTMP, O_RDONLY);
  }

  return utmp_file_fd;
}


struct utmp *getutent(void)
{
  int utmp_fd;
  static struct utmp ut;

  utmp_fd = get_utmp_file_fd();
  if (utmp_fd == -1)
    return NULL;

  if (read(utmp_fd, &ut, sizeof(ut)) != sizeof(ut))
    return NULL;

  return &ut;
}


void endutent(void)
{
  close(utmp_file_fd);
  utmp_file_fd = -1;
}
