#ifndef HEADERS_H
#define HEADERS_H
#include <time.h>
#include "dstring.h"
#include "rfc822.h"

typedef struct {
  struct header_t {
    char *name;               /* dynamic string */
    char *value;              /* dynamic string */
    struct header_t *next; 
  } *headers;
  struct header_t *last;      /* last response header */
} headers_t;

#ifdef __GNUC__
/* This function uses printf-style formats, so tell GNUC to do printf
 * format checking. */
#define ATTRIBUTE_PRINTF(fmt, args) \
   __attribute__ ((format (printf, fmt, args)))
#else
#define ATTRIBUTE_PRINTF(fmt, args)
#endif

extern void HeaderAppend(headers_t *hh, const char *name, const char *value);
extern void HeaderReplaceOrAppend(headers_t *hh, const char *name,
                                  const char *value);
extern void HeaderDeleteCore(headers_t *hh, const char *name,
                             struct header_t *start, int occurrence);
extern void HeaderDelete(headers_t *hh, const char *name);
extern void HeaderDeleteFirst(headers_t *hh, const char *name);
extern void HeaderDeleteSet(headers_t *hh, headers_t *del);
extern void HeaderPrepend(headers_t *hh, const char *name, const char *value);
extern void HeaderMerge(headers_t *dst, const headers_t *src);

extern void HeaderPrintf(headers_t *hh, int replace, const char *name,
                         const char *fmt, ...)
     ATTRIBUTE_PRINTF(4, 5);

extern void HeaderCftime(headers_t *hh, const char *name, const char *format,
                         time_t t, int localtime);

extern char *HeaderGet(headers_t *hh, const char *name, int occurance,
                       void **hp);

extern void HeaderCopyAllBut(headers_t *dest, const headers_t *src,
                             const char *name);
extern void HeaderCopyParsedHeaders(headers_t *dest,
                                    parser_header_t *src);
extern void HeaderCopyOneParsedHeader(headers_t *dest,
                                      parser_header_t *src,
                                      const char *name);

extern void HeaderMessage(DString *msg, const headers_t *hh);
extern void HeaderFree(headers_t *hh);
#undef ATTRIBUTE_PRINTF
#endif /* HEADERS_H */
