/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : msgflow.c
#
#  Module       : Message Flow Display for Program Trace.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 27 Nov 1999
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include "msgflow.h" /* For prototypes */

/*---------MACROS--------------*/

#define MAX_MSGCLASS_NAME_SIZE   10
#define MAX_MSGCLASS_COUNT       10
#define MAX_MSG_NAME_SIZE        30
#define MAX_ENDPOINT_NAME_SIZE   20
#define MAX_MSG_DESC_SIZE        100
#define MAX_ENDPOINT_COUNT       5
#define MAX_PAGE_WIDTH           60

/*---------TYPES---------------*/

/* Description of a message class */
typedef struct 
{
	char name[MAX_MSGCLASS_NAME_SIZE];  /* Name of the message class */
	int  endpoint_count;                /* Number of modules */
	char endpoint_name[MAX_ENDPOINT_COUNT][MAX_ENDPOINT_NAME_SIZE]; /* Names of endpoints */
	int  location[MAX_ENDPOINT_COUNT];  /* Internal variable */
} structMsgClassInfo;

/* Description of a message */
typedef struct
{
	char name[MAX_MSGCLASS_NAME_SIZE];   /* Message name */
	char description[MAX_MSG_DESC_SIZE]; /* Message Description */
} structMsgInfo;

/*---------GLOBALS-------------*/

/* List of message classes registered */
int                gnMsgClassCount = 0;  
structMsgClassInfo gasMsgClass[MAX_MSGCLASS_COUNT];

/* Max width of the output text */
int                gnMaxPageWidth = MAX_PAGE_WIDTH;

/* Name of the message class to which last drawn msg belongs */
char               gszLastMsgClass[MAX_MSGCLASS_NAME_SIZE];

/* Output file ptr */
FILE *fileptr = NULL;

/*---------FUNCTION------------*/

int MessageFlowFile(FILE *fp)
{
  /* Set the file ptr */
  fileptr = fp;
  return 0;
}


int RegisterMsgClass(char *name, int count, ... )
{
	va_list endpoint;
	int j;
	int endpoint_count = 0;
	int i=gnMsgClassCount;
	gnMsgClassCount++;

	strcpy(gasMsgClass[i].name,name);
	gasMsgClass[i].endpoint_count = 0;

	va_start(endpoint, count);
	while(endpoint_count < count)
	{
		strcpy(gasMsgClass[i].endpoint_name[
			gasMsgClass[i].endpoint_count],
			va_arg(endpoint, char *));
		gasMsgClass[i].endpoint_count ++;
		endpoint_count++;
	}

	va_end(endpoint);

	for ( j=0; j<gasMsgClass[i].endpoint_count; j++)
	{
		if ( j== gasMsgClass[i].endpoint_count - 1)
			gasMsgClass[i].location[j] = gnMaxPageWidth-1;
		else
			gasMsgClass[i].location[j] =
				gnMaxPageWidth/(gasMsgClass[i].endpoint_count-1)*j;
	}
	return 0;
}

int DrawMessageFlow(char *msgclass,char *src,char *dest,char *msg)
{
	char szLine[MAX_PAGE_WIDTH + 1];
	char szMsgLine[MAX_PAGE_WIDTH + 1];
	char szMsgName[MAX_PAGE_WIDTH + 1];
	int i,j;
	int srcind= -1; 
	int destind= -1;
	int startat = -1;

  FILE *myfile = fileptr;

  if ( fileptr == NULL )
    return 0;

	for ( i=0; i<gnMsgClassCount; i++)
	{
		if ( strcmp(gasMsgClass[i].name,msgclass) == 0)
		{
			break;
		}
	}
	if ( i == MAX_MSGCLASS_COUNT )
	{
		fprintf(myfile, "Can not find the msg class (%s) or class not registered\n", msgclass);
    return -1;
	}

	memset(szLine,' ',MAX_PAGE_WIDTH);
	szLine[MAX_PAGE_WIDTH] = '\0';

	for ( j=0; j<gasMsgClass[i].endpoint_count; j++)
	{
		szLine[gasMsgClass[i].location[j]] = '|';
	}

	if ( strcmp(gszLastMsgClass,msgclass) != 0)
	{
		/* Draw the endpoint names also */
		strcpy(gszLastMsgClass,msgclass);

		strcpy(szMsgLine,szLine);
		fprintf(myfile,"\n");
		for (j=0; j<gasMsgClass[i].endpoint_count; j++)
		{
			if ( j==0 ) startat = gasMsgClass[i].location[j];
			else
			if ( j==gasMsgClass[i].endpoint_count-1)
				startat = gasMsgClass[i].location[j]
					 - strlen(gasMsgClass[i].
					endpoint_name[j]) + 1;
			else
			{
				startat = gasMsgClass[i].location[j] -
					strlen(gasMsgClass[i].
					endpoint_name[j])/2;
			}
			strncpy(&szMsgLine[startat],gasMsgClass[i].endpoint_name[j],
				strlen(gasMsgClass[i].endpoint_name[j]));
		}
		fprintf(myfile,"%s\n",szMsgLine);
	}
	strcpy(szMsgLine,szLine);
	strcpy(szMsgName,szLine);

	/* Find source and dest */
	for ( j=0; j<gasMsgClass[i].endpoint_count;j++)
	{
		if ( strcmp(gasMsgClass[i].endpoint_name[j],src) == 0)
			srcind = j;
		if ( strcmp(gasMsgClass[i].endpoint_name[j],dest) == 0)
			destind = j;
	}
	if ( srcind == -1 )
		fprintf(myfile,"Source not correct or not registered\n");
	
	if ( destind == -1 )
		fprintf(myfile,"Dest not correct or not registered\n");
	
	if ( srcind == destind )
		fprintf(myfile,"Msg from src to src not allowed\n");

	srcind = gasMsgClass[i].location[srcind];
	destind = gasMsgClass[i].location[destind];
	

	strncpy(&szMsgName[(srcind+destind)/2-strlen(msg)/2],msg,
		strlen(msg));

	if ( srcind > destind )
	{
		srcind--; destind++;
		szMsgLine[destind] = '<';
		destind++;
		memset(&szMsgLine[destind],'-',srcind-destind+1);
	}
	else
	{
		srcind ++; destind--;
		szMsgLine[destind] = '>';
		destind--;
		memset(&szMsgLine[srcind],'-',destind-srcind+1);
	}
	fprintf(myfile,"%s\n%s\n%s\n%s\n",szLine,szMsgName,szMsgLine,szLine);

  fflush(myfile);

	return 0;
}

/************************** END OF FILE ********************************
*/
