#ifndef REQUEST_COMMON_H_
#define REQUEST_COMMON_H_

#include <limits.h>
#include <netinet/in.h>
#include "rfc822.h"
#include "http.h"
#include "uri.h"
#include "dstring.h"
#include "error.h"
#include "headers.h"

/*
#include <openssl/ssl.h> 
*/
#include "/n/ober/u/class/ll272/openssl/openssl-0.9.6a/include/openssl/ssl.h"


#define SOCK_SSL SOCK_STREAM + SOCK_DGRAM +1

/* This is the common part of a request_t, the parts that libcine uses. */
/* IF YOU CHANGE THIS, UPDATE libsip/request.h AND rtspd/request.h
   AS WELL!!! */

typedef struct body_t {
  char *content_type;  /* Content-Type header */
  int content_length;  /* Content-Length header */
  char *content;
} body_t;

typedef union sock_t {
  struct sockaddr_in tcp_s;
  SSL *ssl;
} sock_t;

typedef struct request_common_t {
  /* Basic request information */
  double time_received;          /* time request was received */

  /* Where is the response going to? */
  int type;                      /* SOCK_DGRAM, SOCK_STREAM, SOCK_SSL */
  int socket;                    /* socket id for response */
  sock_t source_sin; /* socket address request came from */
  sock_t sin;        /* socket address for response */

  /* Information about the request */
  parser_t p;                    /* header for this request */
  body_t request_body;           /* The body of the request; */

  http_authorization_t authorization;  /* Authorization header */

  /* Information about the response */
  int status;
  char *reason;
  headers_t headers;                  /* Headers of response */
  body_t response_body;               /* Body to send with respnse */
  
  void *server;   /* pointer to the server object */
} request_common_t;

typedef struct request_t request_t;


/* Functions the sip/rtsp request.c must provide (used by tcp.c and udp.c) */

extern request_t *RequestCreate(int type, int socket, sock_t sin); 
extern void RequestProcess(request_t *r);
extern void ResponseSetStatus(request_t *r, int status, const char *reason);

extern const int missing_content_length_means_zero;

#endif /* REQUEST_COMMON_H_ */
