# Makefile fragment for libcine tests
# Copyright 1998-1999 by Columbia University; all rights reserved 

cine_test_dir := $(libcine_dir)/test

DIRS += $(cine_test_dir)


CINE_TEST_BIN := \
	$(cine_test_dir)/gmatch-test \
	$(cine_test_dir)/servers-test \
	$(cine_test_dir)/uri-test


CINE_TEST_SRC := \
	$(cine_test_dir)/gmatch-test.c \
	$(cine_test_dir)/servers-test.c \
	$(cine_test_dir)/uri-test.c


CINE_TEST_HEADERS := 

#	$(cine_test_dir)/copyright.h

CINE_TEST_MISC := \
	$(cine_test_dir)/module.mk

CINE_TEST_MISCDIST := 

#	$(cine_test_dir)/cine_test.html

CINE_TEST_OBJ := \
	$(patsubst %.c,%.o,$(filter %.c,$(CINE_TEST_SRC)))

CINE_TEST_DEP := \
	$(CINE_TEST_OBJ:.o=.d)


GMATCH_TEST_OBJS := \
	$(cine_test_dir)/gmatch-test.o

GMATCH_TEST_LIBS := $(LIBCINE_LIB)

URI_TEST_OBJS := \
	$(cine_test_dir)/uri-test.o

URI_TEST_LIBS := $(LIBCINE_LIB) $(NISLIBS) $(SOCKLIBS)

SERVERS_TEST_OBJS := \
	$(cine_test_dir)/servers-test.o

SERVERS_TEST_LIBS := $(LIBCINE_LIB) $(LIBRESPARSE_LIB) \
	$(RESOLVLIBS) $(SOCKLIBS)

CINE_TEST_INCS := $(LIBCINE_INC)

$(CINE_TEST_OBJ): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(CINE_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(CINE_TEST_DEP): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(cine_test_dir) -o $@ $(CINE_TEST_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

test: $(cine_test_dir)/gmatch-test $(cine_test_dir)/servers-test \
		$(cine_test_dir)/uri-test

$(cine_test_dir)/gmatch-test: $(GMATCH_TEST_OBJS) $(LIBCINE)
	@echo "linking $@..."
	$(CC) -o $@ $(GMATCH_TEST_OBJS) $(GMATCH_TEST_LIBS)

# Note that servers uses threads, so servers-test needs the pthread libraries.
$(cine_test_dir)/servers-test: $(SERVERS_TEST_OBJS) \
				$(LIBCINE) $(LIBRESPARSE)
	@echo "linking $@..."
	$(CC) -o $@ $(LDFLAGS) $(SERVERS_TEST_OBJS) $(SERVERS_TEST_LIBS) $(LIBS)

$(cine_test_dir)/uri-test: $(URI_TEST_OBJS) $(LIBCINE)
	@echo "linking $@..."
	$(CC) -o $@ $(URI_TEST_OBJS) $(URI_TEST_LIBS)

BIN += $(CINE_TEST_BIN)
SRC += $(CINE_TEST_SRC)
HEADERS += $(CINE_TEST_HEADERS)

CINE_TEST_DIST := $(CINE_TEST_SRC) $(CINE_TEST_HEADERS) $(CINE_TEST_MISC) \
	$(CINE_TEST_MISCDIST)

# These test binaries don't need to go in the bindist
CINE_TEST_BINDIST := $(CINE_TEST_MISCDIST)
