#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "servers.h"

void usage(char *prog)
{
  fprintf(stderr, 
          "Usage: %s [-t udp|tcp] [-p default-port] addr service\n", 
          prog);
  exit(2);
} /* usage */


int main(int argc, char *argv[])
{
  char c;
  int transport = 0;
  int port = 5060;

  server_t *s;
  int i;

  extern char *optarg;
  extern int optind;

  while ((c = getopt(argc, argv, "p:t:")) != EOF) {
    switch (c) {
    case 'p':
      port = atoi(optarg);
      break;
    case 't':
      if (strcasecmp(optarg, "udp") == 0) 
        transport = SOCK_DGRAM;
      else if (strcasecmp(optarg, "tcp") == 0)
        transport = SOCK_STREAM;
      else
        usage(argv[0]);
      break;
    case '?':
      usage(argv[0]);
      break;
    }
  }

  if (argc < optind + 2) {
    usage(argv[0]);
  }
  
  s = servers(argv[optind], argv[optind+1], transport, -port);
  for (i = 0; s[i].host; i++) {
    printf("%6d: %s = %s %s port %d\n",
      s[i].preference, s[i].host, 
      inet_ntoa(s[i].sin.sin_addr),
      s[i].type == SOCK_DGRAM ? "UDP" : "TCP",
      ntohs(s[i].sin.sin_port)); 
  }
  return 0;
} /* main */
