/*
 *thread_pool.h - Implementation of a thread pool, with a 
 * G/G/T/N queue.
 *
 *Copyright 2000 by Columbia University; all rights reserved 
 */
#ifndef THREAD_POOL_H__
#define THREAD_POOL_H__

/*
 * InitThreadPool
 *
 * maxthreads is the total threads to create.
 * the maximum queue size will by default be set to 2*maxthreads.
 * use InitThreadPool2 if you want a smaller queue.
*/
int InitThreadPool(int maxthreads);

/*
 * InitThreadPool2
 * 
 *  maxthreads is the total threads to create, maxqueuesize is
 * the maximum allowed queue size of the G/G/T/N queue, where
 * T = maxthreads, and N=maxqueuesize.
 */
extern int InitThreadPool2(int maxthreads, int maxqueuesize);

/*
 * Executes the function start_routine in a separate thread.
 *
 * Returns 0 if successfull. Returns non-zero if queue is full
 */
extern int pool_schedule(void *(*start_routine)(void *),
				void *arg);

/*
 * Returns the current thread pool status. 
 *
 * Value assignment is done only to non-null params.
 * 
 */
extern int pool_get_status(int *max_threads, int *max_queuesize, int *current_queuesize);

/*
 * Destroy the pool and cleanup resources.
 */
extern int DestroyThreadPool();


#endif   /* thread_pool.h */


