/*
* Add delay (in seconds) to starting time.
*/
#include "sysdep.h"
#include <time.h>
#include <sys/time.h>
#include "timespec.h"
#define NSEC 1000000000
#define USEC 1000000

struct timespec timespec_delay(struct timeval now, double delay)
{
  struct timespec next;

  next.tv_sec = now.tv_sec;
  next.tv_nsec = now.tv_usec * 1000 + (delay - (long)delay) * NSEC;
  if (next.tv_nsec > NSEC) {
    next.tv_sec++;
    next.tv_nsec -= NSEC;
  }
  next.tv_sec += (long)delay;
  return next;
} /* timespec_delay */


struct timeval timeval_until(struct timeval timeout)
{
  struct timeval now, remaining;

  gettimeofday(&now, NULL);

  if (timeout.tv_sec < now.tv_sec ||
      (timeout.tv_sec == now.tv_sec && timeout.tv_usec <= now.tv_usec)) {
    remaining.tv_sec = 0;
    remaining.tv_usec = 0;
    return remaining;
  }

  while (timeout.tv_usec < now.tv_usec) {
    timeout.tv_usec += USEC;
    timeout.tv_sec -= 1;
  }
  remaining.tv_usec = timeout.tv_usec - now.tv_usec;
  remaining.tv_sec = timeout.tv_sec - now.tv_sec;

  return remaining;
} /* timeval_until */
