/* -*- C++ -*- */
/* cinema_file_table.h -- A file database implementation, intended as a
 * interface to file based database. */
   
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CINEMA_FILE_TABLE_H_
#define CINEMA_FILE_TABLE_H_

#include "cinema_table.h"
#include "cinema_dummy_table.h"

/* A file based CINEMA table. */
class cinema_file_table : public cinema_dummy_table
{
private:
  uri_t table_uri;
  put_table_entry *deserialize_put_row(const char *row);

public:
  cinema_file_table(std::string uri);
  cinema_file_table(uri_t& uri);

  virtual ~cinema_file_table();

  virtual char* get_key(const uri_t *uri) const;
  virtual uri_t get_uri(const char *key) const;

  virtual alias_table_entry *get_aliases_by_alias(const char* alias);

  virtual put_table_entry *get_put_by_user(const char* user);
  

  virtual sub_table_entry *get_sub_by_fromto(const char* from, const char* to);
  
  virtual int num_subscriptions(const char *to);
  
  virtual sub_table_entry **get_all_subscriptions(const char* to, int *count);
  
  virtual void insert_sub(const sub_table_entry *entry);

  virtual void delete_sub_by_fromto(const char* from, const char *to);
  
  TABLE_SCHEME("file")
};

#endif /* CINEMA_FILE_TABLE_H_ */
