/* -*- C++ -*- */
/* cinema_mysql_table.h -- MySQL interfaces to the CINEMA database tables. */
   
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CINEMA_MYSQL_TABLE_H_
#define CINEMA_MYSQL_TABLE_H_

#include <ctime>  

#include "stack_lock.h"

#include "cinema_table.h"
#include "mysqlconn.h"


/* MySQL implementation of CINEMA tables. */
class cinema_mysql_table : virtual public cinema_table
{
private:
  uri_t table_uri;

  void init_connection();

protected:
  MysqlConnection* conn;
  pthread_mutex_obj conn_lock;

  void sanity_check_result(int, bool, unsigned int, const char*, const char*,
			   const char*);

  alias_table_entry *deserialize_next_alias_row();
  put_table_entry *deserialize_next_put_row();
  contact_table_entry* deserialize_next_contact_row();
  sub_table_entry* deserialize_next_sub_row();

public:
  cinema_mysql_table(std::string uri);
  cinema_mysql_table(uri_t& uri);

  virtual ~cinema_mysql_table();

  virtual char* get_key(const uri_t *uri) const;
  virtual uri_t get_uri(const char *key) const;


  virtual alias_table_entry *get_aliases_by_alias(const char* alias);

  virtual put_table_entry *get_put_by_user(const char* user);


  virtual contact_table_entry** get_contacts_by_user(const char *user,
                                                     int *count = NULL);

  virtual void insert_contact(const contact_table_entry *entry);

  virtual void delete_contacts_by_user(const char *user);

  virtual void delete_contact_by_contact(const char *user,
                                         const char *contact);

  virtual void delete_expired_contacts_by_user(const char *user, 
                                               time_t expire_time);

  virtual sub_table_entry *get_sub_by_fromto(const char* from, const char* to);
  
  virtual int num_subscriptions(const char *to);
  
  virtual sub_table_entry **get_all_subscriptions(const char* to, int *count);
  
  virtual void insert_sub(const sub_table_entry *entry);

  virtual void delete_sub_by_fromto(const char* from, const char *to);
  
  TABLE_SCHEME("sql")

  static time_t sql_datetime_to_time(const char *);
  static time_t sql_timestamp14_to_time(const char *);

  static void time_to_sql_datetime(time_t, char *);
  
  static int parse_sip_groups(const char*);
  static int parse_sip_methods(const char*);
  static put_auth parse_auth(const char*);
  static contact_sip_action parse_action(const char*);
        
};

#endif /* CINEMA_MYSQL_TABLE_H_ */
