#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dbaccess.h"

extern int debug_flag;

int main(int argc, char* argv[])
{
  cinema_table *table;
  alias_table_entry *alias_entry;
  put_table_entry *put_entry;
  contact_table_entry **contact_entries;
  db_err err;
  char *reason;
  char lastmodbuf[256];
  char *user;
  int count;

  debug_flag = 1;

  if (argc < 3) {
    fprintf(stderr, "Usage: %s URL alias\n", argv[0]);
    exit(2);
  }

  if ((table = new_cinema_table(argv[1], &err, &reason)) == NULL) {
    fprintf(stderr, "Could not create table: %s\n",
            reason);
    exit(1);
  }

  if ((alias_entry = alias_get_by_alias(table, argv[2], &err, &reason))
      == NULL) {
    if (err == DB_NOTFOUND) {
      printf("Not an alias: %s\n", argv[2]);
      user = strdup(argv[2]);
    }
    else {
      fprintf(stderr, "Could not get alias table entry: %s\n",
              reason);
      exit(1);
    }
  }
  else {
    strftime(lastmodbuf, sizeof(lastmodbuf), "%a, %b %d %Y %T GMT",
             gmtime(&alias_entry->last_modified));
    
    printf("%s: %s (%ld = %s)\n",
           alias_entry->alias, alias_entry->primary_user,
           (long)alias_entry->last_modified, lastmodbuf);
    
    user = strdup(alias_entry->primary_user);

    free_alias_table_entry(alias_entry);
  }

  if ((put_entry = put_get_by_user(table, user, &err, &reason)) == NULL) {
    fprintf(stderr, "Could not get PUT entry: %s\n",
            reason);
    exit(1);
  }

  printf("%s: found\n", user);

  free_put_table_entry(put_entry);

  if ((contact_entries = contacts_get_by_user(table, user, &count,
                                              &err, &reason))
      == NULL) {
    if (err == DB_NOTFOUND) {
      printf("Not registered: %s\n", user);
    }
    else {
      fprintf(stderr, "Could not get contact table entries: %s\n",
              reason);
      exit(1);
    }
  }
  else {
    int i;

    for (i = 0; i < count; i++) {
      printf("%s[%i] = %s\n", contact_entries[i]->user, i,
             contact_entries[i]->contact);
    }

    free_contact_table_entries(contact_entries);
  }

  free(user);
  free_table(table);

  exit(0);
}
