/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : buffer.h
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#                 Gautam Nair  (gnair@cs.columbia.edu)
#
#  Created      : 28 Apr 2000
#
#  Description  : Prototypes for buffer management.
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#ifndef BUFFER_H
#define BUFFER_H


/* Buffer management for media packets */
typedef struct media_buffer_t {
  char *data; /* pointer to buffer */
  int  size; /* size of data in buffer */
  int  maxsize; /* size of buffer using malloc */
  struct media_buffer_t *next; /* next in the list */
  unsigned long param;   /* some user param */
  int oRemove;  /* Whether to remove this or not */
} media_buffer_t;

/* Delay buffer calculation */
typedef struct delay_calc_t {
  double di;
  double vi;
  unsigned long ti;
  unsigned long pi;
  int    first_time;
} delay_calc_t;

#define DELAY_ALPHA  (0.95)
#define DELAY_MU     (4.0)
#define DELAY_INITIAL (0)  /* XXX (changed to 0 from 1600, for 200 millisec) */

media_buffer_t *NewBuffer(int size);
void FreeBuffer(media_buffer_t *p);
void BufferEnqueue(media_buffer_t **q, media_buffer_t *buf);
void DeleteQueue(media_buffer_t **q);

#endif

/************************** END OF FILE ********************************
*/

