/***********************************************************************
  SIP Phone, $Id: list.c,v 1.1 2000/09/18 19:03:37 kns10 Exp $
  Copyright (c) 1999-2000 by Columbia University. All rights reserved.
*********************************************************************/

#include "list.h"   /* list_t, ... */
#include <stdio.h>  /* NULL, ... */
#include <string.h> /* strcpy, strncmp */
#include <stdlib.h> /* malloc, ... */

//#define DEBUG           // uncomment this line to get debug output
#ifdef DEBUG 
#define dprintf printf
#else
#define dprintf
#endif

/* refer list.h for documentation */

int ListCreate (list_t *l, int initialsize, int increment) 
{

  if ( initialsize > 0 ) {
    if ((l->data = (char**) calloc (initialsize, (sizeof(char*)))) == NULL) {
      dprintf("Error -- ListCreate, calloc failed\n");
      return -1;
    }  
  }
  else {
    l->data = NULL;
  }
  if ( increment <= 0 ) {
    increment = 1;
  }

  l->count = 0;
  l->max_count = initialsize;
  l->increment = increment;
  strcpy (l->MAGIC, "LIST_MAGIC");
  return 0;
} /* ListCreate */


int ListAddElement (list_t *l, void *ptr, int size)
{
  char **tempstorage;
  char *tempcontainer;
  int i, index = -1;

  if ( l==NULL || ptr == NULL ) {
    dprintf("Error -- ListAddElement, %s is NULL\n", l==NULL ? "l" : "ptr");
    return -1;
  }

  if ( size <= 0 ) {
    dprintf("Error -- ListAddElement, size is %d\n", size);
    return -1;
  }

  dprintf ("in ListAddElement\n");
  if (strncmp (l->MAGIC, "LIST_MAGIC",11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return -1;
  }

  for ( i=0; i<l->count; i++) {
    if ( *(l->data + i) == NULL ){
      dprintf("Free slot found at %d, total = %d\n", i, l->count);
      index = i;
    }
  }
  if (index == -1 ) {
    if ( l->count == l->max_count ) {
      /* The list is full of elements. Allocating more space in the list
       * to accomodate the new elements. This is done by allocating an
       * with 5 more elements and using this for the list
       */
      dprintf ("need to alloc more space\n");
      if ((tempstorage = (char**) calloc ((l->count + 
					   l->increment),sizeof(char*))) == NULL) {
	dprintf ("malloc error");
	return -1;
      }

      /* updating the maxcount to reflect the maximum number of elements
       * possible 
       */
      l->max_count += l->increment;
    
      /* copying the data from the old area to the new one */
      memcpy ((char*) tempstorage, (char*) l->data, 
	      (l->count * sizeof(char*)));
      free (l->data); /* freeing old area */
      l->data = tempstorage;  /* assigning the data pointer to the new
				 area */    
    }
    index= l->count;

    l->count++;
    dprintf("Incrementing count %d\n", l->count);

  }
  else {
    /* nothing*/
  }
  
  dprintf("11 count %d, size %d\n", l->count, size);

  /* allocating memory for the container of the element */
  if ((tempcontainer = (char*)malloc(size)) == NULL) {
    dprintf ("malloc error\n");
    return (-1);
  }
  dprintf("ListAddElement -- %p\n", tempcontainer);

  /* copying the element into the allocated memory */
  dprintf("12 count %d\n", l->count);
  memcpy (tempcontainer, ptr, size);

  dprintf("copying at %d, count %d\n", index, l->count);

  /* inserting the pointer into the array and incrementing the count */
  *(l->data + index) = tempcontainer;
  dprintf ("1 count is : %d\n", (index));
  dprintf ("6 count is after increment: %d\n", (index));
  dprintf ("2 returning : %d\n", l->count - 1);
  return (index);
} /* ListAddElement */


void* ListElement (list_t *l, int index)
{
  if ( l == NULL ) {
    dprintf ("Error-- ListElement, NULL\n");
    return NULL;
  }

  //dprintf ("in ListElement \n");
  if (strncmp (l->MAGIC, "LIST_MAGIC",11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return NULL;
  }
  if (((l->count) <= index) || (index < 0)) {
    dprintf ("Error-- ListElement, outof range %d\n", index);
    return NULL;
  }
  //dprintf("ListElement -- %p\n", *(l->data+index));
  return ( *(l->data + index));  
} /* ListElement */


int ListDeleteElement (list_t *l, int index)
{
  if ( l == NULL ) {
    dprintf ("Error-- ListDeleteElement, NULL\n");
    return -1;
  }

  dprintf ("7 in ListDeleteElement\n");
  if (strncmp (l->MAGIC, "LIST_MAGIC", 11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return -1;
  }
  if ((l->count <= index) || (index < 0)) {
    dprintf ("Error-- ListDeleteElement, outof range %d\n", index);
    return -1;
  }
  free (*(l->data +index));
  *(l->data+index) = NULL;
  /* XXX do not reduce count */
  dprintf ("8 returning out of ListDeleteItem\n");
  return 0;
} /* ListDeleteElement */

int ListDelete (list_t *l)
{
  int count;
  
  if ( l == NULL ) {
    dprintf ("Error-- ListDelete, NULL\n");
    return -1;
  }

  if (strncmp (l->MAGIC, "LIST_MAGIC", 11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return -1;
  } 
  
  /* freeing the elements in the array */
  for (count = 0; count < l->count; count ++) {
    if (*(l->data+count) != NULL ) {
      free(*(l->data+count));
    }
  }
  /* freeing the array of element pointers */
  free (l->data);
  l->data = NULL;
  l->count = 0;

  return 0;
} /* ListDelete */


int ListCount (list_t *l)
{
  if ( l == NULL ) {
    dprintf ("Error-- ListCount, NULL\n");
    return -1;
  }
  if (strncmp (l->MAGIC, "LIST_MAGIC", 11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return -1;
  }
  return (l->count);  
}

int ListSize(list_t *l)
{
  int count,i;

  if ( l == NULL ) {
    dprintf ("Error-- ListIsEmpty, NULL\n");
    return -1;
  }
  if (strncmp (l->MAGIC, "LIST_MAGIC", 11) != 0) {
    /* something wrong with the pointer being returned */
    dprintf (" ERROR --> corrupt list_t pointer\n");
    return -1;
  }

  count = 0;
  for (i = 0; i < l->count; i ++) {
    if (*(l->data+i) != NULL ) {
      //dprintf("ListSize -- iterator %p\n", *(l->data+i));
      count++;
    }
  }
  dprintf("ListSize -- %d\n", count);
  return (count);  
}



