/***********************************************************************
  SIP Phone, $Id: list.h,v 1.1 2000/09/18 19:03:38 kns10 Exp $
  Copyright (c) 1999-2000 by Columbia University. All rights reserved.
*********************************************************************/


#ifndef LIST_H
#define LIST_H

typedef struct {
  char **data; /* array of pointers */
  unsigned int  count; /* count. Current number of elements */
  unsigned int  max_count; /* Size of array: data */
  unsigned int  increment; /* number of elements by which to increase
			    * the size of the list */ 
  char          MAGIC[11];
} list_t;


/*
 * ListCreate: this initialises a list item. 
 * Input: l: pointer to a list_t structure, 
 *        elem_size: size of the element to be put in the list
 *        initialsize: initial size of the list
 *        increment: the number of elements by which to increase the
 *                   size of the list if the present size is
 *                   inadequate
 * Output: 0 on success; -1 on failure
 */
int ListCreate (list_t *l, int initialsize, int increment);

/*
 * ListAddElement: This adds an element to the list
 * Input: l: pointer to initialized list structure
 *        ptr: pointer to the data to be inserted into the list
 * Output: On success, the index of the newly inserted element is
 *         returned; on failure, -1 is returned
 */
int ListAddElement (list_t *l, void *ptr, int size);

/*
 * ListElement: Returns a pointer to an element in the list
 * Input:  l: pointer to the list
 *         index: index of element in the list
 *         size: size of element
 * Output: on success a pointer to the element is returned; on failure
 *         NULL is returned
 */
void* ListElement (list_t *l, int index);

/* ListDeleteElement: Deletes an element from the list
 * Input: l: pointer to the list
 *        index: the index of the element to delete
 * Output: 0 on success; -1 on failure
 */
int ListDeleteElement (list_t *l, int index);

/* ListDelete: Frees up memory allocated for the list
 * input: l: pointer to list
 * output : 0 on success; -1 on failure
 */
int ListDelete (list_t *l);

/* ListCount: Returns the number of elements in the list
 * input : l: pointer to list_t structure
 * output: number of elements in the list on success; -1 on failure
 */
int ListCount (list_t *l);

/* ListSize: Returns the number of elements valid in the list.
 */
int ListSize (list_t *l);

#endif



