/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : sdp.h
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#                 Gautam Nair  (gnair@cs.columbia.edu)
#
#  Created      : 28 Apr 2000
#
#  Description  : SDP prototypes.
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#ifndef SDP_H
#define SDP_H

#include <sys/socket.h>
#include <netinet/in.h>

/* Payload type definition */
typedef struct payload_type_t {
  int pt;  /* payload type */
  char * description;  /* what goes in SDP, e.g., pcmu/8000 */
} payload_type_t;

/* Media type */
typedef enum {
  RTPM_MEDIA_AUDIO, 
  RTPM_MEDIA_VIDEO,
  RTPM_MEDIA_DATA,
  RTPM_MEDIA_UNKNOWN = -1
} media_type_t;

/* m= line in SDP */
typedef struct media_line_t {
  struct sockaddr_in sin; /* IPv4 address and port in network order */
  int ttl;                /* only for multicast */
  list_t payloads;        /* list of payload_type_t */
  media_type_t type;
} media_line_t;

/* Session description: SDP */
typedef struct session_description_t {
  list_t media_line_t;    /* list of media_line_t */
} session_description_t;

int SDPInit(session_description_t *sdp);
int SDPCopy(session_description_t *dest,
            session_description_t *src);
int SDPFree(session_description_t *sdp);
int SDPPrint(session_description_t *sdp);
int SDPDefault(session_description_t *sdp);
session_description_t *SDPParse(const char *str, int size);
char *GetPayloadTypeString(int pt);
char *GetPayloadType(int *pt, session_description_t *sdp);
int GetPayloadTypeFromDescr(char *pt, session_description_t *sdp);

#endif

/************************** END OF FILE ********************************
*/

