/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : mixer.c
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 25 Apr 2000
#
#  Description  : RTP Audio mixer test application, main file.
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>
#include <unistd.h>

#include <rtpmapi.h>

extern int debug_flag;

/*
 * usage()
 * Usage information. Exits program with status code 1.
 */
void usage(char *prog) 
{
  fprintf(stderr,"Usage: %s "
	  "[-p port]\n"
	  "p port: port number to listen for commands, default is 8000\n"
	  , prog);
  exit(1);
} /* usage */

/* Thread to process the request */

int main()
{
  int len;
  char buffer[1500];
  char *lasts;
  char *cptr;
  FILE *fp;
  int oReturn;

  debug_flag = 1;

  while ( 1 ) {
    if ( fgets(buffer, sizeof(buffer), stdin) == NULL ) {
      printf("fgets returns NULL\n");
      break;
    }
    len = strlen(buffer);

    //len = recv(sock, buffer, sizeof(buffer));
    if ( len > 0 ) {
      buffer[len] = '\0';
      //printf("Received %d bytes\n%s\n", len, buffer);
      lasts = NULL;
      cptr = strtok_r(buffer, " \r\n", &lasts); 
      if ( cptr != NULL ) {
        if ( !strcasecmp(cptr, "add" ) ) {
          char *conf = strtok_r(NULL, " \r\n", &lasts);
          char *user = strtok_r(NULL, " \r\n", &lasts);
          conf = (conf == NULL) ? "<none>" : conf;
          user = (user == NULL) ? "<none>" : user;

          printf("add %s %s\n", conf, user);
          oReturn = RTPMAddCallLeg(conf, user);
          fprintf(stdout,"returned %d\n", oReturn);
        }
        else if ( !strcasecmp(cptr, "remove")) {
          char *nstr = strtok_r(NULL, " \r\n", &lasts);
          int  cid = (nstr == NULL) ? -1 : atoi(nstr);

          printf("remove %d\n", cid);
          oReturn = RTPMRemoveCallLeg(cid);
          fprintf(stdout, "returned %d\n", oReturn);
        }
        else if ( !strcasecmp(cptr, "setsdp")) {
          char *nstr = strtok_r(NULL, " \r\n", &lasts);
          int  cid = (nstr == NULL) ? -1 : atoi(nstr);
          session_description_t *sdp;
          char sdpbuf[1500];
          strcpy(sdpbuf, "");

          printf("setsdp %d\n", cid);

          while (fgets(buffer,sizeof(buffer), stdin)){
            //printf("%s\n", buffer);
            if ( strlen(buffer) <= 2 ) {
              break;
            }
            strcat(sdpbuf, buffer);
          }
 
          //printf("%s\n", sdpbuf);
          sdp = SDPParse(sdpbuf, strlen(sdpbuf));
          oReturn = RTPMSetSelfSessionDescription(cid, *sdp);
          fprintf(stdout, "returned %d\n", oReturn);
        }
        else if ( !strcasecmp(cptr, "getsdp")) {
          char *nstr = strtok_r(NULL, " \r\n", &lasts);
          int  cid = (nstr == NULL) ? -1 : atoi(nstr);
          session_description_t sdp;

          printf("getsdp %d\n", cid);
          sdp = RTPMGetRemoteSessionDescription(cid);
          SDPPrint(&sdp);
        }
        else {
          fprintf(stderr,"Command (%s) not found\n", cptr);
        }
        fflush(fp);
      }
    }
    else {
      break;
    }
  }

  /* Wait loop */
  while ( 1 ) {
    sleep(10000);
  }
  return 0;
}


/************************** END OF FILE ********************************
*/

