# Columbia SIP makefile fragment for sipapi
# Copyright 1999-2000 by Columbia University; all rights reserved 

sipapi_dir := $(topdir)/libsip++

SIPAPI_SRC := \
  $(sipapi_dir)/user.cpp \
  $(sipapi_dir)/iptelapi.cpp \
  $(sipapi_dir)/sipapi.cpp \
  $(sipapi_dir)/sipintf.cpp \
  $(sipapi_dir)/sipcall.cpp \
  $(sipapi_dir)/ads.cpp \
  $(sipapi_dir)/sipsdp.cpp \
  $(sipapi_dir)/sipreg.cpp

SIPAPI_HEADERS := \
  $(sipapi_dir)/iptelapi.h \
  $(sipapi_dir)/sipapi.h  \
  $(sipapi_dir)/sipcall.h \
  $(sipapi_dir)/sipintf.h \
  $(sipapi_dir)/sipreg.h \
  $(sipapi_dir)/ads.h

SIPAPI_MISC := \
  $(sipapi_dir)/module.mk \
  $(sipapi_dir)/libsipapi.dsp

SIPAPI_MISCDIST := \
  $(sipapi_dir)/README

SIPAPI_OBJ_C := \
	$(patsubst %.c,%.o,$(filter %.c,$(SIPAPI_SRC)))
SIPAPI_OBJ_CPP := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(SIPAPI_SRC)))
SIPAPI_OBJ := \
	$(SIPAPI_OBJ_C) $(SIPAPI_OBJ_CPP)

SIPAPI_DEP_C := \
        $(SIPAPI_OBJ_C:.o=.d)
SIPAPI_DEP_CPP := \
        $(SIPAPI_OBJ_CPP:.o=.d)
SIPAPI_DEP := \
        $(SIPAPI_DEP_C) $(SIPAPI_DEP_CPP)

# How other parts of the build find us.
# Target to make:
SIPAPI := $(sipapi_dir)/libsip++.a

# Add a target for building sip++.
.PHONY: sip++
sip++ : $(SIPAPI)

# Incantations for compile and build:
SIPAPI_INC := -I$(topsrcdir)/$(sipapi_dir)
SIPAPI_LIB := -L$(sipapi_dir) -lsip++

# Include files required locally.
SIPAPI_INCS := $(LIBSIP_INC) $(LIBCINE_INC) $(DBAPI_INC)
SIPAPI_REQS := need-ldap need-db

$(SIPAPI_OBJ_C): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(SIPAPI_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(SIPAPI_DEP_C): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(sipapi_dir) -o $@ $(SIPAPI_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# Order of -c and -o matters.
$(SIPAPI_OBJ_CPP): %.o: %.cpp
	@echo "compiling $<..."
	$(CXX) $(SIPAPI_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -o $@ -c $<

$(SIPAPI_DEP_CPP): %.d: %.cpp
	@echo "dependencies for $<..."
	$(DEPEND) -d $(sipapi_dir) -o $@ $(SIPAPI_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# Do not include it in all:
# all: $(SIPAPI)

# SIPAPI combines libsip,libresparse and libcine.
$(SIPAPI): $(SIPAPI_OBJ) $(LIBSIP) $(DBAPI) $(LIBRESPARSE) $(LIBCINE) $(LIBDICT)
	@echo "creating library $@..."
	@$(AR) rcv $@ $^
	@$(RANLIB) $@

SRC       += $(SIPAPI_SRC) $(SIPAPI_OTHER_SRC)
HEADERS   += $(SIPAPI_HEADERS)
MISC      += $(SIPAPI_MISC) $(SIPAPI_MAYBESRC)
MISCDIST  += $(SIPAPI_MISCDIST)
MISCCLEAN += $(SIPAPI_OTHER_MISCCLEAN)

# Various distributions.

# SIPAPI source distribution.

SIPAPI_DIST := $(SIPAPI_SRC) $(SIPAPI_MAYBESRC) $(SIPAPI_HEADERS) \
             $(SIPAPI_MISC) $(SIPAPI_MISCDIST)

.PHONY: sipapi-dist

SIPAPI_DIST_NAME := sip++-$(VERSION)-$(DATE)

sipapi-dist:
	$(MKDIST) $(topsrcdir) $(SIPAPI_DIST_NAME) \
	$(TOP_DIST) \
	$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
	$(LIBDICT_DIST) \
	$(WIN32_DIST) \
	$(SIPAPI_DIST)

# SIPAPI binary distribution
# SIPUA_LIBS := $(SIPAPI_LIB) $(LIBSIP_LIB) $(LIBCINE_LIB) \
#	$(LIBRESPARSE) $(LIBDICT_LIB) \
#	$(LDAPLIBS) $(DBLIBS) $(RESOLVLIBS) $(NISLIBS) $(SOCKLIBS)

SIPAPI_EXAMPLE_SRC := \
  $(topdir)/sipua/main.cpp \
  $(topdir)/sipua/command.cpp \
  $(topdir)/sipua/radio.cpp

SIPAPI_EXAMPLE_HEADERS := \
  $(topdir)/sipua/radio.h \
  $(topdir)/sipua/rtp.h \
  $(topdir)/sipua/pcmu_l.h \
  $(topdir)/sipua/types.h \
  $(topdir)/sipua/command.h


SIPAPI_EXAMPLE_MISC := \
  $(topdir)/sipua/Makefile

SIPAPI_EXAMPLE := \
  $(SIPAPI_EXAMPLE_SRC) $(SIPAPI_EXAMPLE_HEADERS) $(SIPAPI_EXAMPLE_MISC) \
  $(topdir)/sipua/sipua \
  $(topdir)/sipua/debug/sipua.exe $(topdir)/sipua/release/sipua.exe

SIPAPI_APIHEADERS := \
  $(topdir)/libsip++/iptelapi.h \
  $(topdir)/libsip++/sipapi.h  \
  $(topdir)/libsip++/ads.h \
  $(topdir)/libcine/sysdep.h


SIPAPI_BINDIST := $(SIPAPI_MISCDIST) $(SIPAPI_APIHEADERS) \
        $(SIPAPI) $(LIBSIP) $(LIBCINE) $(LIBRESPARSE) $(LIBDICT) \
	$(topdir)/config.h \
        $(SIPAPI_EXAMPLE)

.PHONY: sipapi-bindist

SIPAPI_BINDIST_NAME += sip++-$(VERSION)-$(DATE)-$(SYSNAME)

sipapi-bindist:: $(SIPAPI)
	$(MKDIST) $(topsrcdir) $(SIPAPI_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(SIPAPI_BINDIST)


# SIPAPI Win32 binary distribution.

SIPAPI_BINDIST_WIN32 := $(SIPAPI_MISCDIST) \
		$(topdir)/libsip++/debug/libsip++.lib \
		$(topdir)/libsip++/debug/libsip.lib \
		$(topdir)/libsip++/debug/libcine.lib \
		$(topdir)/libsip++/debug/libdict.lib \
		$(topdir)/libsip++/debug/ntutils.lib \
		$(topdir)/libsip++/debug/resparse.lib \
		$(topdir)/libsip++/release/libsip++.lib \
		$(topdir)/libsip++/release/libsip.lib \
		$(topdir)/libsip++/release/libcine.lib \
		$(topdir)/libsip++/release/libdict.lib \
		$(topdir)/libsip++/release/ntutils.lib \
		$(topdir)/libsip++/release/resparse.lib \
  $(SIPAPI_EXAMPLE_SRC) $(SIPAPI_EXAMPLE_HEADERS) $(SIPAPI_EXAMPLE_MISC) \
  $(topdir)/sipua/debug/sipua.exe $(topdir)/sipua/release/sipua.exe \
  $(SIPAPI_APIHEADERS)


.PHONY: sipapi-bindist-win32

SIPAPI_BINDIST_WIN32_NAME := sip++-$(VERSION)-$(DATE)-WinNT

sipapi-bindist-win32::
	$(MKDIST_WIN32) $(topsrcdir) $(SIPAPI_BINDIST_WIN32_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(SIPAPI_BINDIST_WIN32)

# Create the doc++ documentation.

.PHONY: sipapi-doc

sipapi-doc:
	cd $(topsrcdir); cd libsip++; \
        doc++ -f -B footer.html -d docs ads.h iptelapi.h sipapi.h 
