#include "sysdep.h"
#include "policy.h"

/* Failure Policy: handle "generic" failed requests */
/* XXX: doesn't support "transitioning" to a failure policy from
 * some other one, yet.
 */

/* failure_init()
 * Called when a request which failed for some reason first arrives.
 * This does all the work of the failure, since it is
 * simple request/response.
 */
static int failure_init(request_t *r, policy_info_t *reg, int reg_status) {
  /* Make sure we're not "failing" with a success or intermediate response */
  if (reg_status < 300)
    return 1;

  policy_send_new_response(r, reg_status, NULL);
  /* Wait for the timeout period */
  policy_set_timeout(r, 30);

  return 0;
} /* failure_init */


/* failure_handle_request()
 * Handle a subsequent request after a failure.  This shouldn't happen
 * -- we haven't sent a 2xx, so we won't see ACK, and we've sent a final
 * response, so we won't see CANCEL.
 */
static int failure_handle_request(request_t *r, message_t *m)
{
  error("failure_handle_request", "Shouldn't be called");
  return 1;
} /* failure_handle_request */


/* failure_handle_response()
 * handle a subsequent response after a failure.  This definitely
 * shouldn't happen, since we don't have any branches.
 */
static int failure_handle_response(request_t *r, message_t *m, branch_t *b)
{
  error("failure_handle_request", "Shouldn't be called");
  return 1;
} /* failure_handle_response */


/* failure_timeout_expired()
 * Called when the final timeout expires.  When this happens, clean up.
 */
static int failure_timeout_expired(request_t *r)
{
  policy_done(r);
  return 0;
} /* failure_timeout_expired */
     

/* failure_cleanup()
 * failure doesn't keep any local information, so do nothing.
 */
static int failure_cleanup(request_t *r)
{
  return 0;
}


policy_t failure_policy =
{
  "failure",
  &failure_init,
  &failure_handle_request,
  &failure_handle_response,
  &failure_timeout_expired,
  &failure_cleanup
};
