/* policy.h - definition and calls for a policy handler. */

#ifndef POLICY_H_
#define POLICY_H_

/* Information passed from execute_policy() to the individual policy
   callback. */

#include "request-sip.h"

typedef struct policy_info_t policy_info_t;


/* The definition of a policy */
typedef struct policy_t {
  char *name;
  int (*init)(request_t *r, policy_info_t *reg, int reg_status);
  int (*handle_request)(request_t *r, message_t *m);
  int (*handle_response)(request_t *r, message_t *m, branch_t *branch);
  int (*timeout_expired)(request_t *r);
  int (*cleanup)(request_t *r);
} policy_t;

/* Functions policies can call */
void policy_send_new_response(request_t *r, int status, char *reason);
void policy_send_modified_new_response(request_t *r, int status, char *reason,
                                       headers_t *new_headers,
                                       headers_t *del_headers,
                                       body_t *body);
void policy_forward_response(request_t *r, message_t *m);
void policy_forward_user_response(request_t *r, message_t *m);
void policy_proxy_original_request_to(request_t *r, uri_t *uri,
                                      message_t *m, void *policy_info);
void policy_proxy_subsequent_request_to(request_t *r, branch_t *b,
                                        message_t *m);
void policy_cancel_branch(request_t *r, branch_t *b);
void policy_set_timeout(request_t *r, double delay);
void policy_done(request_t *r);

/* Execute a policy */
void execute_policy(request_t *r, policy_info_t *reg, int reg_status);

/* Libsip's policies */
extern policy_t failure_policy;

#endif /* POLICY_H_ */
