/*
 * rr_support.h - Record-Route support. RFC 2543bis03 semantics.
 *   
 */
#ifndef RR_SUPPORT_H__
#define RR_SUPPORT_H__
#include "message.h"


/*
 * Add a Record-Route header, referring to the local machine,
 * to the request described by the message m. The uri is
 * derived from the Contact or From headers of message_t. 
 * 
 * hash, and branchid help get uniqueness when forking happens.
 */
extern void 
AddRecordRoute_Request(message_t *m, unsigned long hash, int branchId);


/*
 * Based on the given uri build a Record-Route header.
 * hash, and branch id are given for uniqueness as mandated 
 * by bis03 draft, and can be 0.
 */
extern parser_element_t*
BuildRRElement(const uri_t *u, unsigned long hash, int branchId);


/*
 * When a 200 response is received from a UAS, change the Record-Route header
 * inserted by us, if present, to the Contact/From of UAS (callee instead
 * of caller). 
*/
extern void ModifyRRElement(message_t *m, unsigned long hash);

/*
 * Given a set of Record-Route headers, builds a route set for a UAS.
 * 
 * Used by the presence agent, which behaves as a UAS.
 * 
 * Free retval after using.
 */
extern char* BuildUASRouteSet(parser_t *p);


#endif
