/* sipsnmp_interface.h - the interface of a snmp support provider. */
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef SIPSNMP_INTERFACE_H_
#define SIPSNMP_INTERFACE_H_
/*
 * This file defines the interface of a snmp support provider. 
 *
 * the interface needs to be defined/used in sip entities. 
 *
 * SnmpProcessMessage is a callback function set at run-time 
 * (we basically use the libsipsnmp services).
 * If there are snmp support providers (thank msft) that are 
 * capable of  processng these calls ,
 * they will hook-into this when their sipsnmp_main init function is invoked.
 * 
 * the code for invoking this should look like
 * 
 *  if(SnmpProcessMessage)
 *    (*SnmpProcessMessage)(....)
 *
 *
 * Note that i have kept two copies of this file, one in the libsip 
 * directory and another in libsipsnmp directory to reduce 
 * makefile dependencies.
 * i dont see this changing so, this should work fine. 
*/

#include "request-sip.h"

typedef struct snmp_data {
  
#define SUB_AGENT      1   /* attaches to main snmp agent through agentx */
#define MASTER_AGENT   0   /* runs on port 161, and serves requests */
  int sub_agent;
 
  const char* agentx_socket_path; /* The AF_UNIX path to use to talk to snmpd*/
				     
  /* others to be added as needed */

} SnmpData;

#define SNMP_TYPE_REQ_IN        0       /* incoming request message */
#define SNMP_TYPE_RES_OUT       1       /* outgoing response */
#define SNMP_TYPE_REQ_OUT       2       /* outgoing request */
#define SNMP_TYPE_RES_IN        3       /* incoming response */
#define SNMP_TYPE_ADD_TRANS     4       /* a new transaction is happening */
#define SNMP_TYPE_DEL_TRANS     5       /* an existing transaction is ending */


typedef struct {

  char request_type[16];        /* when called from branch (for example, 
				   Sending a proxy request, use this to store
				   the message type that this branch is sending
				   out - as the data items of the parent
				   request_t struct may not necessarily
				   have the same state as the branch
				*/
  
  /* others to be added as needed */
}sipsnmp_extra_data;

/*
 * This is the init function for the snmp support provider. Call this in a 
 * separate thread. It goes into an infinite loop and serves snmp requests.
 *
 */
extern void* sipsnmp_main(void* arg);

/*
 * The first parameter is the request_t struct corresponding to this
 * session (if it is a branch, send the parent struct).
 *
 * The second param is one out of the 6 #defs defined above.
 *
 * The last param can be NULL. if it is not null, it contains data elems
 * pertaining to this request being sent out/ or response received from 
 * downstream server's /client.
 */
extern void (*SnmpProcessMessage)
(struct request_t *p_req, int type, const sipsnmp_extra_data *p_extra_data); 


#endif /* SIPSNMP_INTERFACE_H_ */
