#ifdef __SIPSNMP_MIB__
/*
 * This file implements the code for get/set methods of Sip summary 
 * statistics objects.
 */
/*
 * commonstats.c
 *
 *  This file implements the get/set interface functions for SipCommonStats Objects.
 *
 * Configuration parameters are sent out/set in the functions here.
 *
 * For a discussion on general design used through out this module, see the 
 * design doc in the libsipsnmp root directory.
 *
 * Note that this file is intended to be included inside sipMIB.c and hence
 * will not be compiled directly.
 */

/*
 * var_sipSummaryStatsTable(): Get fns for SipSummaryStatsTable, see
 * SipCommonSummaryStats data structure for members handled here.
 */
GET_FUNCTION(var_sipSummaryStatsTable)
{
  
  const SipCommonSummaryStats *p_stats;
  p_stats = &p_sipEntity->sipCommonSummStatsTable.sipCommonSummaryStats;
  
  /* we have only one entity: last param is hence 1 */
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  /* 
   * this is where we do the value assignments for the mib results.
   */
  switch(vp->magic) {
    
  case SIPSUMMARYINREQUESTS:
    long_ret = p_stats->sipSummaryInRequests;
    break;
    
  case SIPSUMMARYOUTREQUESTS:
    long_ret = p_stats->sipSummaryOutRequests;
    break;
    
  case SIPSUMMARYINRESPONSES:
    long_ret = p_stats->sipSummaryInResponses;
    break;
    
  case SIPSUMMARYOUTRESPONSES:
    long_ret = p_stats->sipSummaryOutResponses;
    break;
    
  case SIPSUMMARYTOTALTRANSACTIONS:
    long_ret = p_stats->sipSummaryTotalTransactions;
    break;
    
  default:
    ERROR_MSG(DEF_ERR);
    return NULL;
  }
  return (unsigned char*)&long_ret;

} /* var_sipSummaryStatsTable */


/* var_sipMethodStatsTable : see SipCommonMethodStats data structure inside
 * SipCommonSummStatsTable.
 */
GET_FUNCTION(var_sipMethodStatsTable)
{
  
  const SipCommonMethodStats *p_stats;
  
  int first_index = vp->magic - SIPSTATSINVITEINS;
  int half_first_index = first_index / 2;

  p_stats = &p_sipEntity->sipCommonSummStatsTable.sipCommonMethodStats;

  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    
    return NULL;
  
  if(half_first_index < STATS_MAX) {
    if(first_index % 2) 
      long_ret = p_stats->sipStats[half_first_index][STATS_IN];
    else
      long_ret = p_stats->sipStats[half_first_index][STATS_OUT];
  }
  else {
    ERROR_MSG(DEF_ERR);
    return NULL;
  }
  
  return (unsigned char*)&long_ret;

} /* var_sipMethodStatsTable */

/*
 * var_sipStatsExtMethodTable() : Extension methods. See 
 * SipStatsExtMethodTable and its associated accessor functions for more.
 */
GET_FUNCTION(var_sipStatsExtMethodTable)
{
  
  int  valid_oid = 1;
  
  SipCommonSummStatsTable *p_summ;
  SipCommonExtMethodStats *p_head, *p_toserve; 

  int oidlen = SIPMIB_OIDLEN + 
    sipMIB_variables[SIPSTATSEXTMETHODNAME].namelen;
  
  p_summ = (SipCommonSummStatsTable*)
    &p_sipEntity->sipCommonSummStatsTable;
  
  LOCK();
  
  p_head = p_toserve = p_summ->pSCEMS_head;
  
  if(!exact) {
    
    if(vp->namelen != oidlen) {
      valid_oid = 0;
      goto ext_errgoto;
    } 
    
    if(*length <= oidlen) { /* concat applnIndex and extMethodIndex */
      
      memcpy(name,vp->name,vp->namelen * sizeof(oid)); 

      name[vp->namelen] = 1;
      name[vp->namelen + 1] = 1;

      p_toserve = p_head;
      *length = vp->namelen + 2;

      goto extgoto;

    }
   
    if(*length != oidlen+2) {
      valid_oid = 0;
      goto ext_errgoto;
    }
    
    if(name[*length - 3] != vp->name[*length - 3]) {
      name[*length - 3] = vp->name[*length - 3];
      name[*length - 2] = 1;
      name[*length - 1] = 1;
      p_toserve = p_head;
    }
    else {
      
      int index = name[*length - 1];

      /* find it */
      p_head = (SipCommonExtMethodStats*)getExtMethodEntry(p_head,index);
      
      if(!p_head || !p_head->pNext)
	goto ext_errgoto;
      
      p_head = p_head->pNext;
      
      name[*length - 1]  = p_head->sipStatsExtMethodIndex;
      
      p_toserve = p_head; 
    }
    goto extgoto;
  }
  
  if(exact) { 
    
    int index1= name[*length -2];
    int index = name[*length - 1];
    
    if(*length != oidlen+2 || index1!=1) {
      valid_oid = 0;
      goto ext_errgoto;
    }
    
    p_head = (SipCommonExtMethodStats*)getExtMethodEntry(p_head,index);
    
    if(!p_head)
      goto ext_errgoto;
    
    p_toserve = p_head;
  }
  
 extgoto: 
  if(!p_toserve) 
    goto ext_errgoto;
  
  *var_len = sizeof(long);
  name[*length - 1] = p_toserve->sipStatsExtMethodIndex;
  
  switch(vp->magic) {
    
    case SIPSTATSEXTMETHODNAME:
        *write_method = write_sipStatsExtMethodName;
        strncpy(string,p_toserve->sipStatsExtMethodName,SPRINT_MAX_LEN);
        *var_len = strlen(string);
	goto ext_strgoto;

    case SIPSTATSEXTMETHODINS:
        long_ret = p_toserve->sipStatsExtMethodIns;
	goto ext_longgoto;

    case SIPSTATSEXTMETHODOUTS:
        
        long_ret = p_toserve->sipStatsExtMethodOuts;
	goto ext_longgoto;

    case SIPSTATSEXTMETHODSTATUS:
        *write_method = write_sipStatsExtMethodStatus;
        long_ret = p_toserve->sipStatsExtMethodRowStatus;
	goto ext_longgoto;
	
  default:
    valid_oid = 0;
    ERROR_MSG(DEF_ERR);
  }
 
 ext_errgoto:

  UNLOCK(); 
  
  if(valid_oid) {
    if(vp->magic == SIPSTATSEXTMETHODNAME)
      *write_method = write_sipStatsExtMethodName;
    else
      *write_method = write_sipStatsExtMethodStatus;
  }

  return NULL;

 ext_strgoto:
  UNLOCK();
  return (unsigned char*)string;

 ext_longgoto:
  UNLOCK();
  return (unsigned char*)&long_ret;
}

/*
 * var_sipStatusCodesTable() : SipStatusCodeTable.
 */
GET_FUNCTION(var_sipStatusCodesTable)
{
  
  int first_index = vp->magic - SIPSTATSINFOCLASSINS;
  int half_first_index = first_index /2;
  
  const SipCommonStatusCodeStats *p_stats;
  p_stats = &p_sipEntity->sipCommonSummStatsTable.sipCommonStatusCodeStats;
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  
  if(half_first_index < CLASS_MAX) {
    if(first_index % 2) 
      long_ret = p_stats->sipStatsInfoClass[half_first_index][STATS_IN];
    else
      long_ret = p_stats->sipStatsInfoClass[half_first_index][STATS_OUT];
  }
  else {
    ERROR_MSG(DEF_ERR);
    return NULL;
  }
  
  return (unsigned char*)&long_ret;
  
} /* var_sipStatusCodeTable */


/*
 * var_sipCommonStatusCodeTable() : Common Status Codes, extensions.
 */
GET_FUNCTION(var_sipCommonStatusCodeTable)
{

  SipCommonSummStatsTable *p_summ;
  SipCommonSCodeSpecial *p_head, *p_toserve; 
  
  int oidlen = SIPMIB_OIDLEN + 
    sipMIB_variables[SIPCOMMONSTATUSCODEINS].namelen;
  
  int valid_oid = 1;
  
  p_summ = (SipCommonSummStatsTable*)
    &p_sipEntity->sipCommonSummStatsTable;
  
  LOCK();
  
  p_head = p_toserve = p_summ->pSCSCS_head;
  
  if(!exact) {

    if(vp->namelen != oidlen) {
      valid_oid = 0;
      goto scode_errgoto;
    } 
    
    if(*length <= oidlen) { /* concat applnIndex and uriIndex */
      
      memcpy(name,vp->name,vp->namelen * sizeof(oid)); 
      name[vp->namelen] = 1;
      name[vp->namelen + 1] = 1;
      *length = vp->namelen + 2;

      p_toserve = p_head;
     
      goto scodegoto;
    }
    
    if(*length != oidlen+2) {
      valid_oid = 0;
      goto scode_errgoto;
    }

    if(name[*length - 3] != vp->name[*length - 3]) {
      name[*length - 3] = vp->name[*length - 3];
      name[*length - 2] = 1;
      name[*length - 1] = 1;
      p_toserve = p_head;
    }
    else {
      
      int index = name[*length - 1];
      /* find it */
      p_head = (SipCommonSCodeSpecial*)getSCodeEntry(p_head,index);
      
      if(!p_head || !p_head->pNext)
	goto scode_errgoto;
      
      p_head = p_head->pNext;
      
      name[*length - 1]  = p_head->sipCommonStatusCodeValue;
      
      p_toserve = p_head; 
    }
    goto scodegoto;
  }
  
  if(exact) { 
    
    int index1= name[*length -2];
    int index = name[*length - 1];
    
    if(*length != oidlen+2 || index1!=1) {
      valid_oid = 0;
      goto scode_errgoto;
    }

    p_head = (SipCommonSCodeSpecial*)getSCodeEntry(p_head,index);

    if(!p_head) 
      goto scode_errgoto;
    
    p_toserve = p_head;
  }

 scodegoto:

  if(!p_toserve) 
    goto scode_errgoto;
  
  *var_len = sizeof(long);
  
  name[*length - 1] = p_toserve->sipCommonStatusCodeValue;
  
  switch(vp->magic) {
    
  case SIPCOMMONSTATUSCODEINS:
    long_ret = p_toserve->sipCommonStatusCodeIns;
    goto scode_longret;

  case SIPCOMMONSTATUSCODEOUTS:
    long_ret = p_toserve->sipCommonStatusCodeOuts;
    goto scode_longret;

  case SIPCOMMONSTATUSCODESENDNOTIF:
    *write_method = write_sipCommonStatusCodeSendNotif;
    long_ret = p_toserve->sipCommonStatusCodeSendNotif;
    goto scode_longret;

  case SIPCOMMONSTATUSCODESTATUS:
    *write_method = write_sipCommonStatusCodeStatus;
    long_ret = p_toserve->sipCommonStatusCodeStatus;
    goto scode_longret;
    
  default:
    ERROR_MSG(DEF_ERR);
  }

 scode_errgoto:
  UNLOCK();

  if(valid_oid) {
    if(vp->magic == SIPCOMMONSTATUSCODESENDNOTIF)
      *write_method = write_sipCommonStatusCodeSendNotif;
    else
      *write_method = write_sipCommonStatusCodeStatus;
  }    

  return NULL;

 scode_longret:
  UNLOCK();
  return (unsigned char*)&long_ret;
  
} /* var_sipCommonStatusCodeTable */


/*
 * var_sipOtherStatsTable() : Other counter objects.
 */
GET_FUNCTION(var_sipOtherStatsTable)
{

  if (header_simple_table(vp,name,length,exact,var_len,   
	write_method, 1) == MATCH_FAILED )
    return NULL;
  
  if(vp->magic == SIPNUMUNSUPPORTEDURIS) {
        long_ret = p_sipEntity->sipNumUnsupportedUris;
        return (unsigned char *) &long_ret;
  }

  else 
        ERROR_MSG(DEF_ERR);

  return NULL;

} /* var_sipOtherStatsTable */


static SipCommonExtMethodStats new_stats;

SET_FUNCTION(write_sipStatsExtMethodName)
{

  int err = SNMP_ERR_NOERROR;

  if(statP != NULL) 
    return SNMP_ERR_BADVALUE;
  
  switch ( action ) {
    
  case RESERVE1:
    
    if (var_val_type != ASN_OCTET_STR) 
      return SNMP_ERR_WRONGTYPE;
     
    if (var_val_len > 12)    
      return SNMP_ERR_WRONGLENGTH;
   
    break;
    
    
  case RESERVE2:
    {
      char *p = (char*) var_val;
    
      const SipCommonSummStatsTable *p_summ = 
	&p_sipEntity->sipCommonSummStatsTable;
      
      if(!strcasecmp(p,"REGISTER") ||
	 !strcasecmp(p,"OPTIONS") ||
	 !strcasecmp(p,"BYE") ||
	 !strcasecmp(p,"INVITE") ||
	 !strcasecmp(p,"ACK") ||
	 !strcasecmp(p,"CANCEL")) 
	return SNMP_ERR_BADVALUE;
      
      strncpy(new_stats.sipStatsExtMethodName,(char*)var_val,12);
      new_stats.sipStatsExtMethodRowStatus = SNMP_ROWSTATUS_NOTREADY;

      new_stats.sipStatsExtMethodIndex = name[name_len -1];

      LOCK();
      if(getExtMethodEntryByName(p_summ->pSCEMS_head,p) != NULL)
	err = SNMP_ERR_WRONGVALUE;
      UNLOCK();
    }
  
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
  case COMMIT:
    
    {
      SipCommonSummStatsTable *p_summ = (SipCommonSummStatsTable*)
	&p_sipEntity->sipCommonSummStatsTable;
      
      printf("commit getting called...\n");
      fflush(stdout);
      
      LOCK();
      if(p_summ->addMethodToMonitor(p_summ,&new_stats) < 0)
	  err = SNMP_ERR_COMMITFAILED;
      UNLOCK();
    }
  }
  
  return err;
}


SET_FUNCTION(write_sipStatsExtMethodStatus)
{
  int err = SNMP_ERR_NOERROR;

  int rowstatus = 0;
  unsigned int index = 0;


  printf("inside extmethodstatus");
  fflush(stdout);

  if(statP == NULL) /* we dont expect new elements here */
    return SNMP_ERR_BADVALUE;
  
  index = name[name_len -1];
  
  rowstatus = (unsigned int)*(long*)var_val;
      
  if(!index 
     || rowstatus < SNMP_ROWSTATUS_CREATEANDGO
     || rowstatus>SNMP_ROWSTATUS_DESTROY) {
    printf("inconsistent rowstatus");
    fflush(stdout);
    return SNMP_ERR_INCONSISTENTVALUE;
  }
  
  switch ( action ) {
    
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)  
      return SNMP_ERR_WRONGTYPE;
    
    if (var_val_len != sizeof(new_stats.sipStatsExtMethodRowStatus))
      return SNMP_ERR_WRONGLENGTH;
      
  case RESERVE2:
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
  case COMMIT:
    {
      SipCommonSummStatsTable *p_summ = (SipCommonSummStatsTable*)
	&p_sipEntity->sipCommonSummStatsTable;
      
      LOCK();
      if(rowstatus == SNMP_ROWSTATUS_DESTROY) 
	p_summ->delMethodToMonitor(p_summ,index);
      else 
	{
	  SipCommonExtMethodStats *p_elem = 
	    (SipCommonExtMethodStats*) 
	    getExtMethodEntry(p_summ->pSCEMS_head,index);
	  
	  if(p_elem) {
	    if(rowstatus == SNMP_ROWSTATUS_CREATEANDGO)
	      p_elem->sipStatsExtMethodRowStatus = SNMP_ROWSTATUS_ACTIVE;
	    else
	      p_elem->sipStatsExtMethodRowStatus = SNMP_ROWSTATUS_NOTREADY;
	  }
	  else {
	    error("write_sipStatsExtMethodRowStatus",
		  "Buggy code...expected element not found");
	    err = SNMP_ERR_GENERR; /* this should never happen */
	  }
	}
      UNLOCK();
    }
  }  
  return err;
}

static SipCommonSCodeSpecial temp_element; 
  
SET_FUNCTION(write_sipCommonStatusCodeSendNotif)
{
  int scode;
  unsigned int sendNotif;
  SipCommonSummStatsTable *p_summ;
   int err = SNMP_ERR_NOERROR;

  scode = name[name_len - 1];

  sendNotif = (unsigned int)*(long*)var_val;

  if(statP != NULL) /* we allow only new elements */
    return SNMP_ERR_INCONSISTENTVALUE;

  switch ( action ) {
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)
      return SNMP_ERR_WRONGTYPE;
    
    if (var_val_len != sizeof(long_ret))
      return SNMP_ERR_WRONGLENGTH;
    break;
    
  case RESERVE2:
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
    
  case COMMIT:
    
    p_summ = (SipCommonSummStatsTable*)&p_sipEntity->sipCommonSummStatsTable;
    
    temp_element.sipCommonStatusCodeValue = scode;
    temp_element.sipCommonStatusCodeSendNotif = sendNotif;
    temp_element.sipCommonStatusCodeStatus = SNMP_ROWSTATUS_NOTREADY;

    LOCK();
    p_summ->addSCodeToMonitor(p_summ,&temp_element);
    UNLOCK();
  }
  return err;
}


SET_FUNCTION(write_sipCommonStatusCodeStatus)
{
  
  int scode;
  unsigned int rowstatus;

  SipCommonSummStatsTable *p_summ;
  SipCommonSCodeSpecial *p_temp;

  int err = SNMP_ERR_NOERROR;

  scode = name[name_len - 1];

  rowstatus = (unsigned int)*(long*)var_val;

  if(statP == NULL) /* we allow only existing elements */
    return SNMP_ERR_INCONSISTENTVALUE;  
  
  
  switch ( action ) {
  
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)
      return SNMP_ERR_WRONGTYPE;
         
    if (var_val_len != sizeof(long_ret))
      return SNMP_ERR_WRONGLENGTH;
          
    break;


  case RESERVE2:
    
    if(rowstatus < SNMP_ROWSTATUS_CREATEANDGO ||
       rowstatus > SNMP_ROWSTATUS_DESTROY)
      err = SNMP_ERR_BADVALUE;

  case FREE:
  case ACTION:
  case UNDO:
    break;
    
  case COMMIT:
    p_summ = (SipCommonSummStatsTable*)
      &p_sipEntity->sipCommonSummStatsTable;
    
    LOCK();
    
    if(rowstatus == SNMP_ROWSTATUS_DESTROY) 
      p_summ->delSCodeToMonitor(p_summ,scode);
    else 
      {
	 
	p_temp = (SipCommonSCodeSpecial*)
	  getSCodeEntry(p_summ->pSCSCS_head,scode);
	
	if(p_temp) 
	  {
	    if(rowstatus == SNMP_ROWSTATUS_CREATEANDGO)
	      p_temp->sipCommonStatusCodeStatus = SNMP_ROWSTATUS_ACTIVE;
	    else
	      p_temp->sipCommonStatusCodeStatus = SNMP_ROWSTATUS_NOTREADY;
	  }
	else {
	  error("write_sipCommonStatusCodeStatus",
		"Buggy code...expected element not found");
	  err = SNMP_ERR_GENERR; /* this should never happen */
	}
      }
    UNLOCK();
    
  }
  return err;
}


#endif
