/*
 *
 * sipentity_registrards.h - data structure defs for sip registrars
 *
 */

#ifndef SIPENTITY_REGISTRARDS_H__
#define SIPENTITY_REGISTRARDS_H__

/*
 *
 * Registrar configuration object
 *
 */
typedef struct sip_regcfg_entry {
  
  u_int sipRegAllowThirdParty;
  int (*setSipRegAllowThirdParty)(struct sip_regcfg_entry* p_cfg,TRUTHVALUE);
  
  u_int sipRegContactDfltExpiryDuration; /* default expiry time in seconds */
  int (*setSipRegContactDfltExpiryDuration)(struct sip_regcfg_entry* p_cfg,
					    u_int timeSecs);
  
  u_int sipRegMaxContactExpiryDate; /* Maximum expiry time in seconds */
  int (*setSipRegMaxContactExpiryDate)(struct sip_regcfg_entry* p_cfg,
				       u_int durationSecs);  
  
  u_int sipRegRespHasContacts;  /* send contact list with register response? */
  int (*setSipRegRespHasContacts)(struct sip_regcfg_entry* p_cfg,TRUTHVALUE);  
  
  u_int sipRegMaxUsers;         /* maximum users supported */
  
} SipRegCfgEntry;


/*
 *
 * Contact data structure
 *
 * Every registered user can have several contacts
 *
 */
typedef struct sip_regcontact_entry {
  
  u_int sipContactIndex;                      /* index, assigned at constru */
  
  char sipContactDisplayName[STANDARDSTRLEN]; 
  
  char sipContactURI[STANDARDSTRLEN];
  
  char sipContactExpiry[32] ;                 /*"2d-1d-1d,1d:1d:1d.1d,1a1d:1d"*/

  char sipContactPreference[12];              /* 0.0 to 1.0 in string rep, ASN.1 synt */
  
  u_int sipContactAction;                     /* SIPSERVERACTION_PROXY / REDIRECT */
  
  time_t sipContactLastUpdated;               /* see definition of time_t */

  u_int sipContactRetryAfter;                 /* not implemented now */

  struct sip_regcontact_entry *pNext;         /* link list */

} SipRegContactEntry;


/*
 *
 * Per user data structure for registration info
 *
 */
typedef struct sip_reguser_entry {

  u_int sipUserIndex;                         /* unique index, assigned at construction time*/
  
  char sipUserUri[256]; 

  u_int sipUserAuthFailures;                  /* number of auth fails, since start */  

  u_int isValid;                              /* 1 if valid, 0 if should not be used */

  struct sip_reguser_entry* pNext;            /* linked list */

} SipRegUserEntry;


/*
 *
 * This is the Registrar object.
 *
 */
typedef struct sip_registrar {
 
  SipRegCfgEntry sipRegCfgEntry;              /* Registrar config */
   
  u_int sipRegAcceptedRegistrations;          /* Accepted registrations */ 

  u_int sipRegRejectedRegistrations;          /* Rejected registrations */

  SipRegUserEntry *p_userList;                /* linked list of active users */

} SipRegistrar;


/*
 *
 * utility functions 
 *
 */

/*
 *
 * search the list p_UserList and returns the entry which has
 * the same sipUserIndex as second param. Returns NULL if not found.
 *
 * The returned pointer must be treated const. 
 * see also 
 *  the quote in sipentity_commonds.h
*/
extern const SipRegUserEntry* 
getUser(const SipRegistrar* p_registrar, 
	int sipUserIndex);

/*
 *
 * search the contact list for a given contact index
 *
 */
extern const SipRegContactEntry* 
getContact(const SipRegContactEntry* p_contactList, 
	   int contactIndex);

/* 
 *
 * returns the number of registered users from the sip database.
 * [1..getNumRegisteredUsers] elements  are then made available in the 
 * p_UserList pointer.
 *
 */
extern int getNumRegisteredUsers(SipRegistrar*);

/*
 *
 * For a given p_user from the p_UserList, returns all the contact entries
 * as a linked-list. *p_rows gives the number of non-expired contacts 
 * contained in the list.
 * 
 * Once usage is finished, free using a call to freeContactList.
 *
 */
extern SipRegContactEntry* getContactList(const SipRegUserEntry* p_user, 
					  int* p_rows);

/*
 *
 * frees all members of the contact list
 *
 */
extern void 
freeContactList(SipRegContactEntry* p_contactListHead);


#endif      /* sipentity_registrar_ds.h */

