/*
 *
 * sipentity_serverds.h - data structure definitions for sip proxy and redirect
 * servers.
 *
 */
#ifndef SIPENTITY_SERVERDS_H__
#define SIPENTITY_SERVERDS_H__

/*
 *
 * Configuration object for sip proxy/redirect servers.
 *
 */
typedef struct  sip_server_cfg {
  
  char *sipServerHost;  
  int (*setSipServerHost)(struct sip_server_cfg*,const char*);
 
  /*
   * If this object is true, the server sends 100 Trying when a 
   * request is processed. 
   *
   */ 
  u_int sipInformationAlwaysSendTrying;   
  int (*setSipInformationAlwaysSendTrying)(struct sip_server_cfg*,TRUTHVALUE);
  
  u_int sipInformationTryingIntervalMsecs;
  int (*setSipInformationTryingIntervalMsecs)(struct sip_server_cfg*,u_int milliSecs);
  
  /*
   *
   * The pgp stuff is almost obsolete.
   *
   */
  char sipPgpVersion[12];
  
#define DFLT_REDIRECT   1
#define DFLT_PROXY      2
  u_int sipServerDfltAction;
  int (*setSipServerDfltAction)(struct sip_server_cfg*,u_int action);
  
  u_int sipServerRespectUAAction;
  int (*setSipServerRespectUAAction)(struct sip_server_cfg*,TRUTHVALUE);
  
} SipServerCfgEntry;

/*
 *
 * Proxy specific configuration. 
 *
 */
typedef struct sip_proxy_cfg_entry {
  
  u_int sipRequestMaxExpires;
  int (*setSipRequestMaxExpires)(struct sip_proxy_cfg_entry*,u_int secs);
  
  u_int sipProxyStateful;
  int (*setSipProxyStateful)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxySendsCancel;
  int (*setSipProxySendsCancel)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxyForwardAll1xx;
  int (*setSipProxyForwardAll1xx)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxyRecursion;
  int (*setSipProxyRecursion)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxyProvideAlternatives;
  int (*setSipProxyProvideAlternatives)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxyRecordRoute;
  int (*setSipProxyRecordRoute)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

  u_int sipProxyStateTimeout;
  int (*setSipProxyStateTimeout)(struct sip_proxy_cfg_entry*,u_int secs);
                          
#define PROXY_AUTH_NONE   1
#define PROXY_AUTH_BASIC  2
#define PROXY_AUTH_DIGEST 3
#define PROXY_AUTH_PGP    4
  
  u_int sipProxyAuthMethod;
  int (*setSipProxyAuthMethod)(struct sip_proxy_cfg_entry*,u_int value);

#define PROXY_AUTH_ALGOOTHER  1
#define PROXY_AUTH_ALGOMD5    2
#define PROXY_AUTH_ALGOSHA1   3
  u_int sipProxyAuthPgpAlgorithm;
  int (*setSipProxyAuthPgpAlgorithm)(struct sip_proxy_cfg_entry*,u_int algo);

  char *sipProxyAuthRealm;
  int (*setSipProxyAuthRealm)(struct sip_proxy_cfg_entry*,const char* newRealm);

  u_int sipProxyNonceLifeTimeMsecs;
  int (*setSipProxyNonceLifeTimeMsecs)(struct sip_proxy_cfg_entry*,u_int msecsValue);

  char* sipPgpPrivateKey;
  int (*setSipPgpPrivateKey)(struct sip_proxy_cfg_entry*,const unsigned char*);

  u_int sipHideRespect;
  int (*setSipHideRespect)(struct sip_proxy_cfg_entry*,TRUTHVALUE);

} SipProxyCfgEntry;

/*
 * 
 * Interdomain trust relationships between multiple sip proxy 
 * servers. not implemented right now.
 *
 */
typedef struct sip_rxproxy_auth_entry {
  u_int sipRxProxyAuthIndex;
  char sipRxProxyAuthRealm[STANDARDSTRLEN];
  unsigned char sipRxProxyAuthPassword[STANDARDSTRLEN];
  u_int sipRxProxyAuthStatus;
} SipRxProxyAuthEntry;


/*
 *
 * SipProxyServer objects for proxy and redirect servers.
 *
 */
typedef struct sip_proxy_server_entry {
  
  SipProxyCfgEntry  sipProxyCfgEntry;
  
  int nSipRxProxyAuthEntries;
  SipRxProxyAuthEntry* sipRxProxyAuthEntry;
  
  int (*addProxyAuthEntry)(const SipRxProxyAuthEntry*);
  int (*delProxyAuthEntry)(SipRxProxyAuthEntry*);

  u_int sipNumProxyRequireFailures;
  
} SipProxyServerEntry;


typedef struct sip_server_entry {
  
  SipServerCfgEntry sipServerCfgEntry;
  
  SipProxyServerEntry sipProxyServerEntry;
  
} SipServerEntry;

#endif      /* sipentity_serverds.h */
