#ifdef __SIPSNMP_MIB__
/*
 * Implementation of the SIP Proxy and Redirect MIB GET functions.
 */

/*
 * var_sipServerCfgTable():
 */
GET_FUNCTION(var_sipServerCfgTable)
{
  const SipServerCfgEntry *p_cfg;
  
  p_cfg = &p_sipEntity->sipServer.sipServerCfgEntry;
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  switch(vp->magic) {
    
  case SIPSERVERHOST:
        *write_method = write_sipServerHost;
        strncpy(string,p_cfg->sipServerHost,SPRINT_MAX_LEN);
        *var_len = strlen(string);
        return (unsigned char *) string;
	
  case SIPINFORMATIONALWAYSSENDTRYING:
    *write_method = write_sipInformationAlwaysSendTrying;
      long_ret = p_cfg->sipInformationAlwaysSendTrying;
      return (unsigned char *) &long_ret;
      
    case SIPINFORMATIONTRYINGINTERVAL:
        *write_method = write_sipInformationTryingInterval;
        long_ret = p_cfg->sipInformationTryingIntervalMsecs;
        return (unsigned char *) &long_ret;

    case SIPPGPVERSION:
        
        strncpy(string,p_cfg->sipPgpVersion,SPRINT_MAX_LEN);
        *var_len = strlen(string);
        return (unsigned char *) string;

    case SIPSERVERDFLTACTION:
        *write_method = write_sipServerDfltAction;
        long_ret = p_cfg->sipServerDfltAction;
        return (unsigned char *) &long_ret;

    case SIPSERVERRESPECTUAACTION:
        *write_method = write_sipServerRespectUAAction;
        long_ret = p_cfg->sipServerRespectUAAction;
        return (unsigned char *) &long_ret;

    default:
      ERROR_MSG(DEF_ERR);
  }
  return NULL;

}

/*
 * var_sipProxyCfgTable():
 */
GET_FUNCTION(var_sipProxyCfgTable)
{

    const SipProxyCfgEntry *p_cfg;
    p_cfg = &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;

    if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
                                                == MATCH_FAILED )
    return NULL;

  switch(vp->magic) {

    case SIPREQUESTMAXEXPIRES:
        *write_method = write_sipRequestMaxExpires;
        long_ret = p_cfg->sipRequestMaxExpires;
        return (unsigned char *) &long_ret;

    case SIPPROXYSTATEFUL:
        *write_method = write_sipProxyStateful;
        long_ret = p_cfg->sipProxyStateful;
        return (unsigned char *) &long_ret;

    case SIPPROXYSENDSCANCEL:
        *write_method = write_sipProxySendsCancel;
        long_ret = p_cfg->sipProxySendsCancel;
        return (unsigned char *) &long_ret;

    case SIPPROXYFORWARDALL1XX:
        *write_method = write_sipProxyForwardAll1xx;
        long_ret = p_cfg->sipProxyForwardAll1xx;
        return (unsigned char *) &long_ret;

    case SIPPROXYRECURSION:
        *write_method = write_sipProxyRecursion;
        long_ret = p_cfg->sipProxyRecursion;
        return (unsigned char *) &long_ret;

    case SIPPROXYPROVIDEALTERNATIVES:
        *write_method = write_sipProxyProvideAlternatives;
        long_ret = p_cfg->sipProxyProvideAlternatives;
        return (unsigned char *) &long_ret;

    case SIPPROXYRECORDROUTE:
        *write_method = write_sipProxyRecordRoute;
        long_ret = p_cfg->sipProxyRecordRoute;
        return (unsigned char *) &long_ret;

    case SIPPROXYSTATETIMEOUT:
        *write_method = write_sipProxyStateTimeout;
        long_ret = p_cfg->sipProxyStateTimeout;
        return (unsigned char *) &long_ret;

    case SIPPROXYAUTHMETHOD:
        *write_method = write_sipProxyAuthMethod;
        long_ret = p_cfg->sipProxyAuthMethod;
        return (unsigned char *) &long_ret;

    case SIPPROXYAUTHPGPALGORITHM:
        *write_method = write_sipProxyAuthPgpAlgorithm;
        long_ret = p_cfg->sipProxyAuthPgpAlgorithm;
        return (unsigned char *) &long_ret;

    case SIPPROXYAUTHREALM:
        *write_method = write_sipProxyAuthRealm;
	if(p_cfg->sipProxyAuthRealm)
	  strncpy(string,p_cfg->sipProxyAuthRealm,SPRINT_MAX_LEN);
        else
	  strcpy(string,"");
	*var_len = strlen(string);
        return (unsigned char *) string;

    case SIPPROXYNONCELIFETIME:
        *write_method = write_sipProxyNonceLifeTime;
        long_ret = p_cfg->sipProxyNonceLifeTimeMsecs;
        return (unsigned char *) &long_ret;

    case SIPPGPPRIVATEKEY:
      *write_method = write_sipPgpPrivateKey;
      *var_len = 0;
      return NULL;
      
  case SIPHIDERESPECT:
    *write_method = write_sipHideRespect;
    long_ret = p_cfg->sipHideRespect;
    return (unsigned char *) &long_ret;

  default:
    ERROR_MSG(DEF_ERR);
  }
  return NULL;

}


/*
 * var_sipRxProxyAuthTable(): trust relationships, not implemented.
 */
GET_FUNCTION(var_sipRxProxyAuthTable)
{

  return NULL;

  /*  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  
  switch(vp->magic) {
    

    case SIPRXPROXYAUTHREALM:
        *write_method = write_sipRxProxyAuthRealm;
        *string = 0;
        *var_len = strlen(string);
        return (unsigned char *) string;

    case SIPRXPROXYAUTHPASSWORD:
        *write_method = write_sipRxProxyAuthPassword;
        *string = 0;
        *var_len = strlen(string);
        return (unsigned char *) string;

    case SIPRXPROXYAUTHSTATUS:
        *write_method = write_sipRxProxyAuthStatus;
        long_ret = 0;
        return (unsigned char *) &long_ret;


    default:
      ERROR_MSG(DEF_ERR);
  }
  return NULL;
  */ 
}

/*
 * var_sipProxyStatsTable():
 */
GET_FUNCTION(var_sipProxyStatsTable)
{
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  switch(vp->magic) {
    
  case SIPNUMPROXYREQUIREFAILURES:
    {
      const SipProxyServerEntry *ps;
        ps = &p_sipEntity->sipServer.sipProxyServerEntry;
        long_ret = ps->sipNumProxyRequireFailures;
        return (unsigned char *) &long_ret;
    }

  default:
    ERROR_MSG(DEF_ERR);
    
  }
  return NULL;
}

SET_FUNCTION(write_sipServerHost)
{
  return SNMP_ERR_NOCREATION;
}

typedef int(*PFN)(SipServerCfgEntry*, u_int);

static int
write_sipServerCfgEntry_int(int  action,
			    u_char   *var_val,
			    u_char   var_val_type,
			    size_t   var_val_len,
			    u_char   *statP,
			    oid      *name,
			    size_t   name_len, 
			    SipServerCfgEntry* p_cfg, PFN pfn)
{
    
  switch ( action ) {
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)
      return SNMP_ERR_WRONGTYPE;
    
    if (var_val_len > sizeof(u_int))
      return SNMP_ERR_WRONGLENGTH;
    break;
    
    
  case RESERVE2:
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
    
  case COMMIT:
    {
      u_int save_val;
      save_val = (unsigned int) (*(long *) var_val);
      (*pfn)(p_cfg,save_val);
    }
    break;
  }
  return SNMP_ERR_NOERROR;
}

SET_FUNCTION(write_sipInformationAlwaysSendTrying)
{

   SipServerCfgEntry *p_cfg;
   
   p_cfg = (SipServerCfgEntry *)
     &p_sipEntity->sipServer.sipServerCfgEntry;
   
   return   write_sipServerCfgEntry_int(action, var_val,
					var_val_type, var_val_len, 
					statP,
					name, name_len,
					p_cfg,
					p_cfg->setSipInformationAlwaysSendTrying);
   
}

SET_FUNCTION(write_sipInformationTryingInterval)
{
 
   SipServerCfgEntry *p_cfg;

   p_cfg = (SipServerCfgEntry *)
     &p_sipEntity->sipServer.sipServerCfgEntry;
   
   return   write_sipServerCfgEntry_int(action, var_val,
					var_val_type,var_val_len, 
					statP,
					name,name_len,
					p_cfg,
					p_cfg->setSipInformationTryingIntervalMsecs);
          
}

SET_FUNCTION(write_sipServerDfltAction)
{
  
  SipServerCfgEntry *p_cfg;
  p_cfg = (SipServerCfgEntry *)
    &p_sipEntity->sipServer.sipServerCfgEntry;
   
  return write_sipServerCfgEntry_int(action, var_val,
				     var_val_type,var_val_len, 
				     statP,
				     name,name_len,
				     p_cfg,
				     p_cfg->setSipServerDfltAction);
}

SET_FUNCTION(write_sipServerRespectUAAction)
{
  SipServerCfgEntry *p_cfg;
  p_cfg = (SipServerCfgEntry *)
    &p_sipEntity->sipServer.sipServerCfgEntry;
   
  return   write_sipServerCfgEntry_int(action, var_val,
				       var_val_type,var_val_len, 
				       statP,
				       name,name_len,
				       p_cfg,
				       p_cfg->setSipServerDfltAction);
}

typedef int (*SPFN)(SipProxyCfgEntry*, u_int);

static int write_sipProxyCfgEntry_int(int      action,
				      u_char   *var_val,
				      u_char   var_val_type,
				      size_t   var_val_len,
				      u_char   *statP,
				      oid      *name,
				      size_t   name_len, 
				      SipProxyCfgEntry *p_cfg, 
				      SPFN spfn)
     
{
  static u_int save_val;
  
  switch ( action ) {
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)
      return SNMP_ERR_WRONGTYPE;
    
    if (var_val_len > sizeof(save_val))
      return SNMP_ERR_WRONGLENGTH;
    
    break;
    
    
  case RESERVE2:
    save_val = (u_int)(*(unsigned long*)var_val);
    break;
    
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
  case COMMIT:
    (*spfn)(p_cfg,save_val);
    break;
  }
  return SNMP_ERR_NOERROR;
}


SET_FUNCTION(write_sipRequestMaxExpires)
{
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipRequestMaxExpires);
}


SET_FUNCTION(write_sipProxyStateful)
{
  
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyStateful);
}

SET_FUNCTION(write_sipProxySendsCancel)
{
  
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxySendsCancel);
}

SET_FUNCTION(write_sipProxyForwardAll1xx)
{
  SipProxyCfgEntry *p_cfg;
  
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyForwardAll1xx);
}


SET_FUNCTION(write_sipProxyRecursion)
{
  SipProxyCfgEntry *p_cfg;

  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyRecursion);
}


SET_FUNCTION(write_sipProxyProvideAlternatives)
{
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyProvideAlternatives);
}




SET_FUNCTION(write_sipProxyRecordRoute)
{
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyRecordRoute);
}




SET_FUNCTION(write_sipProxyStateTimeout)
{
  SipProxyCfgEntry *p_cfg;

  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;

  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyStateTimeout);
}


static int
write_sipProxyAuthMethod(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)&p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;

  return write_sipProxyCfgEntry_int(action,
            var_val,var_val_type,
            var_val_len,
            statP,name,
            name_len,p_cfg, p_cfg->setSipProxyAuthMethod);
}




static int
write_sipProxyAuthPgpAlgorithm(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
  SipProxyCfgEntry *p_cfg;
  p_cfg = (SipProxyCfgEntry *)&p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;

  return write_sipProxyCfgEntry_int(action,
            var_val,var_val_type,
            var_val_len,
            statP,name,
            name_len,p_cfg, p_cfg->setSipProxyAuthPgpAlgorithm);
}


SET_FUNCTION(write_sipHideRespect)
{

 SipProxyCfgEntry *p_cfg;
 p_cfg = (SipProxyCfgEntry *)
   &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
 
 return write_sipProxyCfgEntry_int(action,
				   var_val,var_val_type,
				   var_val_len,
				   statP,name,
				   name_len,
				   p_cfg, p_cfg->setSipHideRespect);
}



SET_FUNCTION(write_sipProxyAuthRealm)
{
  return SNMP_ERR_NOCREATION;
}




SET_FUNCTION(write_sipProxyNonceLifeTime)
{
  
  SipProxyCfgEntry *p_cfg;
  
  p_cfg = (SipProxyCfgEntry *)
    &p_sipEntity->sipServer.sipProxyServerEntry.sipProxyCfgEntry;
  
  return write_sipProxyCfgEntry_int(action,
				    var_val,var_val_type,
				    var_val_len,
				    statP,name,
				    name_len,
				    p_cfg, 
				    p_cfg->setSipProxyNonceLifeTimeMsecs);

}



SET_FUNCTION(write_sipPgpPrivateKey)
{
  return SNMP_ERR_NOCREATION;

}






#endif
