#ifdef __SIPSNMP_MIB__

/*
 * This file implements the get/set functions that will be called
 * by the ucd-snmp agent in response to extern requests for the 
 * SIP-REGISTRAR-MIB
 */

/*
 * var_sipRegCfgTable(): SipRegCfgTable and its sub-elements
 */
GET_FUNCTION(var_sipRegCfgTable)
{
  const SipRegCfgEntry *p_reg;
  const SipRegistrar *p_registrar;
  
  p_registrar = &p_sipEntity->sipRegistrar;
  p_reg = &p_registrar->sipRegCfgEntry;
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  REG_LOCK();
  
  switch(vp->magic) {
    
    
  case SIPREGALLOWTHIRDPARTY:
    *write_method = write_sipRegAllowThirdParty;
    long_ret = p_reg->sipRegAllowThirdParty;
    break;
    
  case SIPREGCONTACTDFLTEXPIRYDURATION:
    *write_method = write_sipRegContactDfltExpiryDuration;
    long_ret = p_reg->sipRegContactDfltExpiryDuration;
    break;
    
  case SIPREGCONTACTDFLTEXPIRYDATE:
    
    *write_method = write_sipRegContactDfltExpiryDate;
    strcpy(string,"9999-12-31 23:59:59");
    *var_len = strlen(string);
    REG_UNLOCK();
    return (unsigned char *) string;
    
  case SIPREGMAXCONTACTEXPIRYDATE:
    *write_method = write_sipRegMaxContactExpiryDate;
    long_ret = p_reg->sipRegMaxContactExpiryDate;
    break;
    
  case SIPREGRESPHASCONTACTS:
    *write_method = write_sipRegRespHasContacts;
    long_ret = p_reg->sipRegRespHasContacts;
    break;
    
  case SIPREGMAXUSERS: 
    long_ret = p_reg->sipRegMaxUsers;
    break;
    
  case SIPREGCURRENTUSERS:
    
    long_ret = getNumRegisteredUsers((SipRegistrar*)p_registrar);
    break;
    
  default:
    ERROR_MSG(DEF_ERR);
    REG_UNLOCK();
    return NULL;
  }
  REG_UNLOCK();
  return (unsigned char *) &long_ret;
}

/*
 * var_sipRegUserTable(): User table, list users
 */
GET_FUNCTION(var_sipRegUserTable)
{
  const SipRegistrar *p_registrar;
  const SipRegUserEntry *p_entry;
  
  int numUsers,uindex=1,rettype=RETTYPE_STR;
  int oidlen = 0;
  
  oidlen = 5 + SIPMIB_OIDLEN;
  
  p_registrar = &p_sipEntity->sipRegistrar; 
  
  REG_LOCK(); 
  
  if((numUsers = getNumRegisteredUsers((SipRegistrar*)p_registrar)) <= 0) {
    error("var_sipRegUserTable","%d users were found...\n",numUsers);
    goto user_done;
  }
  
  if(!exact) {
    
    if(vp->namelen != oidlen) goto user_done; 
    
    if(*length <= oidlen) { /* concat applnIndex and userIndex */
      
      memcpy(name,vp->name,vp->namelen * sizeof(oid)); 
      name[vp->namelen] = 1;
      name[vp->namelen + 1] = 1;
      uindex = 1;
      *length = vp->namelen + 2;
      goto user_data;
    }
    
    if(*length != oidlen+2) goto user_done;
    
    if(name[*length - 3] != vp->name[*length - 3]) {
      name[*length - 3] = vp->name[*length - 3];
      name[*length - 2] = 1;
      name[*length - 1] = 1;
      uindex=1;
    }
    else {
      
      uindex = name[*length - 1];
      
      if(uindex<numUsers) 
	name[*length - 1]  = ++uindex;
      else
	goto user_done;
       
    }
    
    goto user_data;
  
  }
  
  if(exact) { 
    
    int index1= name[*length -2];
    uindex = name[*length - 1];
     
    if(*length != oidlen+2 || index1!=1 || uindex>numUsers) goto user_done;
    
  }

 user_data:
  
  if(name[*length - 2] != 1) goto user_done;
  
  *var_len = sizeof(long);
  
  if(!(p_entry = getUser(p_registrar, uindex))) 
    goto user_done;
  
  /* 
   * this is where we do the value assignments for the mib results.
   */
  switch(vp->magic) {
    
  case SIPUSERURI:
    *write_method = write_sipUserUri;
    strncpy(string,p_entry->sipUserUri,SPRINT_MAX_LEN);
    *var_len = strlen(string);
     break; 
     
  case SIPUSERPASSWORD:
    *write_method = write_sipUserPassword;
    strcpy(string,"N/A");
    *var_len = strlen(string);
    break;
    
  case SIPUSERAUTHENTICATIONFAILURES:
    
    long_ret = p_entry->sipUserAuthFailures;
    rettype = RETTYPE_LONG;
    break;
    
  case SIPUSERTABLEROWSTATUS:
    *write_method = write_sipUserTableRowStatus;
    long_ret = 1;
    rettype = RETTYPE_LONG;
    
  default:
    ERROR_MSG(DEF_ERR);
    REG_UNLOCK();
    return NULL;
  }
  
  REG_UNLOCK();
  if(rettype == RETTYPE_STR) 
    return (unsigned char*)string;
  else
    return (unsigned char*)&long_ret;
  
 user_done: 
  REG_UNLOCK();
  return NULL;

} /* var_sipUserTable */


/*
 * var_sipContactTable() : Contact entry and its sub elements.
 */
GET_FUNCTION(var_sipContactTable)
{
  SipRegContactEntry *p_entry=NULL, *p_tofree=NULL;

  int  userIndex=0, contactIndex=0;
  
  const SipRegistrar *p_registrar;
  const SipRegUserEntry *p_user=NULL;  
  const SipRegContactEntry *p_temp=NULL;
  
  int numUsers,rettype=RETTYPE_STR,rows,u;
  int oidlen = SIPMIB_OIDLEN + 5;
  
 
  p_registrar = &p_sipEntity->sipRegistrar; 
  
  
  REG_LOCK(); 
  
  
  numUsers = getNumRegisteredUsers((SipRegistrar*)p_registrar);
  
  if(numUsers<=0) goto done;
  
 
  if(!exact) {   
    
    if(vp->namelen == oidlen && *length <= oidlen) { /*first invocation */ 
      
      memcpy(name,vp->name,vp->namelen * sizeof(oid));
      
      u = 1;
      
      while(u<=numUsers) {

	if(p_tofree) {
	  freeContactList(p_tofree);
	  p_tofree = NULL;
	}

	p_user = getUser(p_registrar,u);
	
	if(p_user) {
	  
	  userIndex = p_user->sipUserIndex;
	  
	  rows = 0;
	  
	  p_tofree = p_entry = getContactList(p_user,&rows);
	  
	  if(rows> 0) {
	    p_temp = getContact(p_entry,1);
	    contactIndex = p_temp->sipContactIndex;
	    break;
	  }
	}
	u++;
      }
      
      if(u>numUsers) goto done; /* no user has contact entries */
      
      name[vp->namelen] = 1;
      
      name[vp->namelen+1] = userIndex;
      
      name[vp->namelen + 2] = contactIndex;
      
      *length = vp->namelen + 3;
      
      goto ready;
    }
     
    if(vp->namelen == oidlen && *length == oidlen+3) { /* second */
      
      if(vp->name[vp->namelen - 1] == name[vp->namelen - 1]) {
	
	if(name[*length - 3] != 1) goto done;
	
	/* test if contact list has enough people */
	contactIndex = name[*length -1];
	userIndex = name[*length -2];
	
	p_user = getUser(p_registrar,userIndex);
	
	if(p_user) {
	  
	  p_tofree = p_entry = getContactList(p_user,&rows);
	  
	  p_temp = getContact(p_entry,contactIndex + 1);
	  
	  if(p_temp) {
	    name[*length - 1] = p_temp->sipContactIndex;
	    goto ready;
	  }
	  else {
	    
	    u = userIndex + 1;

       	    while(u<=numUsers) {
       
	      p_user = getUser(p_registrar,u);
	      
	      if(p_user) {  
	
		rows = 0;
		
		if(p_tofree) {
		  freeContactList(p_tofree);
		  p_tofree = NULL;
		}

		p_tofree = p_entry = getContactList(p_user,&rows);
		
		if(rows> 0) {
		  p_temp = getContact(p_entry,1);
		  contactIndex = p_temp->sipContactIndex;
		  break;
		}
	      }
	      
	      u++;
	    }
	    
	    if(u>numUsers) goto done;
	    
	    name[*length - 1] = contactIndex;
	    name[*length - 2] = u;
	    
	    goto ready;
	  }
	}
	goto done;
      }	 
      
      if(vp->name[vp->namelen - 1] > name[vp->namelen -1]) {
	
	memcpy(name,vp->name,vp->namelen * sizeof(oid));
	
	u = 1;
	
	while(u<=numUsers) {
	   
	  p_user = getUser(p_registrar,u);
	  
	  if(p_user) {
	    rows = 0;
	    
	    if(p_tofree) {
	      freeContactList(p_tofree);
	      p_tofree = NULL;
	    }

	    p_tofree = p_entry = getContactList(p_user,&rows);
	    
	    userIndex = p_user->sipUserIndex;
	    
	    if(rows> 0) {
	      p_temp = getContact(p_entry,1);
	      contactIndex = p_temp->sipContactIndex;
	      break;
	    }
	  }
	  u++;
	}
	
	if(u>numUsers) goto done; /* no user has contact entries */
              
	name[vp->namelen] = 1;
	name[vp->namelen+1] = userIndex;
	name[vp->namelen + 2] = contactIndex;
	*length = vp->namelen + 3;
	
	goto ready;
	
      }
    }
    goto done;
  }
  
  if(exact) {
   
    if(*length != oidlen+3 || 
       name[*length-3]!= 1 || 
       name[*length - 2] > numUsers) 
      goto done;
    
    userIndex = name[*length - 2];
    contactIndex = name[*length - 1];
    
    p_user = getUser(p_registrar,userIndex);
    
    if(p_user) {
      p_tofree = p_entry = getContactList(p_user,&rows);
      
      if(rows>0) {
	p_temp  = getContact(p_entry,contactIndex);
	if(p_temp) goto ready;       
      }
    }

    goto done;
  }
  
 ready:
  *var_len = sizeof(long);
  
  switch(vp->magic) {
    
  case SIPCONTACTDISPLAYNAME:
    
    if(p_temp->sipContactDisplayName != NULL &&
       strlen(p_temp->sipContactDisplayName) > 0) 
      strncpy(string,p_temp->sipContactDisplayName,SPRINT_MAX_LEN);
    else
      strncpy(string,p_temp->sipContactURI,SPRINT_MAX_LEN);
    
    *var_len = strlen(string);
    break;
    
  case SIPCONTACTURI:
    
    strncpy(string,p_temp->sipContactURI,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    break;
    
  case SIPCONTACTLASTUPDATED:
    
    {
      const SipCommonCfgInterface *p_cfgIntf;
  
      p_cfgIntf = &p_sipEntity->sipCommonCfgTable.sipCommonCfgInterface;
      long_ret = labs(p_cfgIntf->sipServiceLastChange - p_temp->sipContactLastUpdated) * 100;
      
    }
    *var_len = sizeof(long);
    rettype = RETTYPE_LONG;
    break;
    
  case SIPCONTACTEXPIRY:
    {
      char *p, *p2;
      int temp,values[6] = { 9999, 12, 31, 23, 59, 59 }, max = 6, i = 0;
      unsigned short *t;
      unsigned char *p1;
      
      strncpy(string,p_temp->sipContactExpiry,SPRINT_MAX_LEN);
      p = string;
      while((p = (char*)strtok_r(p,(const char*)"-:,",&p2)) != NULL && 
	    i < max) 
	{
	  if((temp = atoi(p)) > 0)
	    values[i++ ] = temp;
	  p = NULL;
	}
      
      t = (unsigned short*)string;
      *t = (unsigned short)values[0];
      p1 = (unsigned char*)string;
      p1 += 2;
      for(i=1;i<max;i++) {
	*p1 = (unsigned char)values[i];
	p1++;
      }
      
      *var_len = 7;
    }
    break;
    
  case SIPCONTACTPREFERENCE:
    
    strncpy(string,p_temp->sipContactPreference,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    break;
     
  case SIPCONTACTACTION:
    
    long_ret = p_temp->sipContactAction;
    *var_len = sizeof(long);
    rettype = RETTYPE_LONG;
    break;
    
    
  case SIPCONTACTRETRYAFTER:
    
    *string = 0;
    *var_len = strlen(string);
    break;
    
    
  default:
    ERROR_MSG(DEF_ERR);
    goto done;
  }
  
  REG_UNLOCK();
  
  if(p_tofree) 
    freeContactList(p_tofree);
  
  if(rettype==RETTYPE_STR)
    return (unsigned char*)string;
  else
    return (unsigned char*)&long_ret; 
  
 done: 
  
  REG_UNLOCK();
   
  if(p_tofree)
    freeContactList(p_tofree);
  
  return NULL;

} /* var_sipRegContactTable */


/*
 * var_sipRegStatsTable(): Statistic objects.
 */
GET_FUNCTION(var_sipRegStatsTable)
{
  
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)
      == MATCH_FAILED )
    return NULL;
  
  switch(vp->magic) {
    
  case SIPREGACCEPTEDREGISTRATIONS:
    
    long_ret = p_sipEntity->sipRegistrar.sipRegAcceptedRegistrations;
    return (unsigned char *) &long_ret;
    
  case SIPREGREJECTEDREGISTRATIONS:
    
    long_ret = p_sipEntity->sipRegistrar.sipRegRejectedRegistrations;
    return (unsigned char *) &long_ret;
    
  default:
    ERROR_MSG(DEF_ERR);
  }
  return NULL;
}


typedef int(*FN)(SipRegCfgEntry*,u_int);

static int
write_sipRegCfg_int(int  action,
		    u_char   *var_val,
		    u_char   var_val_type,
		    size_t   var_val_len,
		    u_char   *statP,
		    oid      *name,
		    size_t   name_len,FN pfn,SipRegCfgEntry *p_this)
{
  static unsigned int regcfg_saveint;
  
  switch ( action ) {
  case RESERVE1:
    if (var_val_type != ASN_INTEGER)
      return SNMP_ERR_WRONGTYPE;
    
    if (var_val_len > sizeof(regcfg_saveint))
      return SNMP_ERR_WRONGLENGTH;
    
    break;
    
    
  case RESERVE2:
    regcfg_saveint = (unsigned int)(*(long *) var_val);
    break;
    
  case FREE:
  case ACTION:
  case UNDO:
    break;
    
    
  case COMMIT:
    (*pfn)(p_this,regcfg_saveint);
    break;
  }
  return SNMP_ERR_NOERROR;
}

/* 
 * toggle the allowThirdParty field.
 */
SET_FUNCTION(write_sipRegAllowThirdParty)
{
  SipRegCfgEntry *p_cfg =  (SipRegCfgEntry*)
    &p_sipEntity->sipRegistrar.sipRegCfgEntry;
  
  unsigned int i = (unsigned int)(*(long*)var_val);
  
  if(i!= 1 && i!= 2) 
    return SNMP_ERR_BADVALUE;
  
  return write_sipRegCfg_int(action,
			     var_val,var_val_type,var_val_len,
			     statP,
			     name,name_len,
			     p_cfg->setSipRegAllowThirdParty,
			     p_cfg);

} /* write_sipRegAllowThirdParty */


SET_FUNCTION(write_sipRegContactDfltExpiryDuration)
{

  SipRegCfgEntry *p_cfg = (SipRegCfgEntry*)
    &p_sipEntity->sipRegistrar.sipRegCfgEntry;
  
  return write_sipRegCfg_int(action,
			     var_val,var_val_type,var_val_len,
			     statP,
			     name,name_len,
			     p_cfg->setSipRegContactDfltExpiryDuration,
			     p_cfg);

} /* write_sipRegContactDfltExpiryDuration */


SET_FUNCTION(write_sipRegContactDfltExpiryDate)
{
  return SNMP_ERR_BADVALUE;
} 

SET_FUNCTION(write_sipRegMaxContactExpiryDate)
{
  SipRegCfgEntry *p_cfg = (SipRegCfgEntry*)
    &p_sipEntity->sipRegistrar.sipRegCfgEntry;
  
  return 
    write_sipRegCfg_int(action,
			var_val,var_val_type,var_val_len,
			statP,
			name,name_len,
			p_cfg->setSipRegMaxContactExpiryDate,
			p_cfg); 
}


SET_FUNCTION(write_sipRegRespHasContacts)
{
  SipRegCfgEntry *p_cfg = (SipRegCfgEntry*)
    &p_sipEntity->sipRegistrar.sipRegCfgEntry;

  unsigned int i = (unsigned int)(*(long*)var_val);
  
  if(i!= 1 && i!= 2) 
    return SNMP_ERR_BADVALUE;
 
 return 
   write_sipRegCfg_int(action,
		       var_val,var_val_type,var_val_len,
		       statP,
		       name,name_len,
		       p_cfg->setSipRegRespHasContacts,p_cfg);

}

SET_FUNCTION(write_sipUserUri) 
{
  if(statP == NULL)
    return SNMP_ERR_NOCREATION;
  else
    return SNMP_ERR_READONLY;
}

SET_FUNCTION(write_sipUserPassword)
{
  
  if(statP == NULL)
    return SNMP_ERR_NOCREATION;
  else
    return SNMP_ERR_READONLY;
}

SET_FUNCTION(write_sipUserTableRowStatus)
{

  if(statP == NULL)
    return SNMP_ERR_NOCREATION;
  else
    return SNMP_ERR_READONLY;
}

#endif
