#ifndef SIPSNMP_UTILS_H__
#define SIPSNMP_UTILS_H__

/**********************************************************************
 *
 *               libsipsnmp - snmp support provider.
 *
 *       sipsnmp_utils.h - Helper functions
 *
 *
 *      Copyright(c) 2000-2001 Columbia University ; all rights reserved
 *
 *           Initial version by: Sankaran N, February 2001.
 *
 **********************************************************************/  

/*
 *Max time_t vaule for infinite time
 */
#define MAX_TIME_T  ((time_t) 0x7fffffff)    
#define MAX_TIME_STR  "9999-12-31,23:59:59"

#ifndef HAVE_TIMEGM
#include "timegm.h"
#endif 


/*
 *
 * time2String()
 * Convert the time_t to a string representation
 * YYYY-MM-DD HH:MM:SS (adapted from code in register.c)
 * Infinite time (MAX_TIME_T) corresponds to
 * 0000-00-00 00:00:00
 * All others are converted as per GMT  
 * On input - buflen > 20 
 *
 */
extern int time2string(time_t time, char* resultBuf, int buflen);
   
/*
 *
 * Convert string representation of time to time_t.
 * db string: YYYY-MM-DD HH:MM:SS
 * Infinite time (MAX_TIME_T) corresponds to
 * 0000-00-00 00:00:00
 * All others are converted as per GMT
 *
 */
extern time_t string2time(const char *string);


/*
 *
 * returns the ip address (or domain name) of the primary
 * name server by querying the /etc/resolv.conf file. 
 *
 * the first parameter is a buffer to contain the result,
 * the second param is the length of buffer (must be >24)
 *
 */
extern void get_primary_ns(char *buf, int len);



#endif   /* sipsnmp_utils.h */
