#ifdef __SIPSNMP_MIB__
/*
 * var_sipCurrentTransTable
 *
 * SIP Transaction mapper for get functions.
*/

/*
 * var_sipCurrentTransTable(): Return the current active transaction count.
 */
GET_FUNCTION(var_sipCurrentTransTable)
{
  
  if (header_simple_table(vp,name,length,exact,var_len,write_method, 1)  
      == MATCH_FAILED )
    return NULL;
  
  if(vp->magic == SIPCURRENTTRANSACTIONS) {
    long_ret = p_sipEntity->nSipCurrentTransactions;
    return (unsigned char *) &long_ret;
  }
  
  ERROR_MSG(DEF_ERR);
  return NULL;
}

/*
 * var_sipTransactionTable(): Return transaction data
 */

static int index_to_transact_map[1024];
static struct timeval last_cleared;

GET_FUNCTION(var_sipTransactionTable)
{
  
  const SipCommonTransactionMib *p_head, *p_toserve= NULL;
  
  struct timeval acc;
  
  int final_index=1;
  
  int oidlen = SIPMIB_OIDLEN +  6;
  
  gettimeofday(&acc,NULL);
  
  if(acc.tv_sec - last_cleared.tv_sec > 300) {
    memset(index_to_transact_map,0,sizeof(index_to_transact_map));
    last_cleared.tv_sec = acc.tv_sec;
  }

  TRANS_LOCK();
  
  p_head = p_sipEntity->p_transactList;
  
  
  if(p_sipEntity->nSipCurrentTransactions <= 0 || !p_head)
    goto tdone;
  
  if(!exact) {
    
    if(vp->namelen != oidlen) goto tdone;
    
    if(*length <= oidlen) { /* concat applnIndex and uriIndex */
      
      memcpy(name,vp->name,vp->namelen * sizeof(oid)); 
      
      name[vp->namelen] = 1;
      name[vp->namelen + 1] = 1;
      
      if(!index_to_transact_map[1]) {
	
	p_toserve = getTransactionByIndex(p_head,1);
	
	if(p_toserve)
	  index_to_transact_map[1] = p_toserve->transaction_id;
	
      }
      else {
	p_toserve = getTransactionById(p_head,index_to_transact_map[1]);
	
	if(!p_toserve) 
	  p_toserve = NULL;
      }

      *length = vp->namelen + 2;
      goto tgoto;
    }

    if(*length != oidlen+2) goto tdone;
    
    if(name[*length - 3] != vp->name[*length - 3]) {
      name[*length - 3] = vp->name[*length - 3];
      name[*length - 2] = 1;
      name[*length - 1] = 1;

      if(!index_to_transact_map[1]) {
	p_toserve = getTransactionByIndex(p_head,1);
	index_to_transact_map[1] = p_toserve->transaction_id;
      }
      else { 
	p_toserve = getTransactionById(p_head,index_to_transact_map[1]);
	goto tgoto;
      }
    }
    else {
      
      int index = name[*length - 1];
      /* find it */
      if(index <= 0 || index>=1024) goto tgoto;
     
      final_index = index+1;

      name[*length - 1]  = index+1;
      
      if(index_to_transact_map[index+1]) 
	p_toserve = getTransactionById(p_head,index_to_transact_map[index+1]);
      else 
	{
	  p_toserve = getTransactionByIndex(p_head,index+1);
	  if(p_toserve) 
	    index_to_transact_map[index+1] = p_toserve->transaction_id;
	  else
	    goto tdone;
	}
    }
    goto tgoto;
  }
  
  if(exact) { 
   
    int index1= name[*length -2];
    int index = name[*length - 1];
    
    if(*length != oidlen+2 || index1!=1) goto tdone;
    
    if(index<=0 || index>=1024) goto tdone;
    
    if(index_to_transact_map[index]) {
      
      p_toserve = getTransactionById(p_head,index_to_transact_map[index+1]);
      
      final_index = index;
      
    }
    else {
      
      p_toserve = getTransactionByIndex(p_head,index);
      
      if(!p_toserve) goto tdone;
      else 
	index_to_transact_map[index] = p_toserve->transaction_id;
    }
  }
  
 tgoto: 
  
  *var_len = sizeof(long);

  if(!p_toserve) 
    goto tdone;

  /* 
   * this is where we do the value assignments for the mib results.
   */
  switch(vp->magic) {
    
  case SIPTRANSTO:
    
    strncpy(string,p_toserve->sipTransTo,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    goto tsucc;
    
  case SIPTRANSFROM:
    
    strncpy(string,p_toserve->sipTransFrom,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    goto tsucc;
    
  case SIPTRANSCALLID:
    
    strncpy(string,p_toserve->sipTransCallId,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    goto tsucc;
    
  case SIPTRANSCSEQ:
    
    long_ret = p_toserve->sipTransCSeq;
    goto tlongret;
    
  case SIPTRANSSTATE:
    
    {
      u_int temp;
      long_ret = getTransactionState(p_toserve,&temp);
    }
    goto tlongret;
    
  case SIPTRANSNUMOUTSTANDINGBRANCHES:
    {
      u_int temp;
      long_ret = getTransactionState(p_toserve,&temp);
      long_ret = temp;
    }
    goto tlongret;
    
  case SIPTRANSEXPIRY:
    
    {
      char *p, *p2;
      int temp,values[6] = { 9999, 12, 31, 23, 59, 59 }, max = 6, i = 0;
      unsigned short *t;
      unsigned char *p1;
      
      strncpy(string,p_toserve->sipTransExpiry,SPRINT_MAX_LEN);
      
      p = string;
      
      while((p = (char*)strtok_r(p,(const char*)"-:, ",&p2)) != NULL && 
	    i < max) 
	{
	  if((temp = atoi(p)) > 0)
	    values[i++ ] = temp;
	  p = NULL;
	}
      
      t = (unsigned short*)string;
      *t = (unsigned short)values[0];
      p1 = (unsigned char*)string;
      p1 += 2;
      for(i=1;i<max;i++) {
	*p1 = (unsigned char)values[i];
	p1++;
      }
      
      *var_len = 7;
      goto tsucc;
    }
    
  case SIPTRANSCALLINGPARTYCONTENTTYPE:
    
    strncpy(string,p_toserve->sipTransCallingPartyContentType,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    goto tsucc;
    
  case SIPTRANSCALLEDPARTYCONTENTTYPE:
    
    strncpy(string,p_toserve->sipTransCalledPartyContentType,SPRINT_MAX_LEN);
    *var_len = strlen(string);
    
    if(final_index>0 && final_index<=1024) 
      index_to_transact_map[final_index] = 0;

    goto tsucc;
    
  default:
    ERROR_MSG("");
  }

 tdone:
  TRANS_UNLOCK();
  return NULL;

 tsucc:
  TRANS_UNLOCK();
  return (unsigned char*) string;

 tlongret:
  TRANS_UNLOCK();
  return (unsigned char*)&long_ret;
}

#endif

