#ifdef __SIPSNMP_MIB__
/*
 * trapgen.c
 *
 * Implementation of trap sending module.
 *
 * Gets included as part of sipMIB.c, so dont compile directly.
 * 
 */

typedef struct {
  oid objid[MAX_OID_LEN];
  size_t oidlen;
  u_char type;
} oid_data;

/*
  The mib oid's can be obtained by running snmptranslate command as below:
  
  bash-2.04$ snmptranslate sipMIB.sipCommonMIB.sipCommonMIBNotifPrefix.
  sipCommonMIBNotif.sipStatusCodeNotifValue
  .1.3.6.1.2.1.9998.2.7.0.2
*/


static oid_data od[5] = {
  { { 1, 3, 6, 1, 2, 1, 9998, 2, 7, 0, 2} , 11 , ASN_INTEGER },
  { { 1, 3, 6, 1, 2, 1, 9998, 2, 7, 0, 3}, 11, ASN_OCTET_STR } ,
  { { 1, 3, 6, 1, 2, 1, 9998, 2, 7, 0, 4}, 11, ASN_OCTET_STR },
  { { 1, 3, 6, 1, 2, 1, 9998, 2, 7, 0, 5}, 11, ASN_OCTET_STR } ,
  { { 1, 3, 6, 1, 2, 1, 9998, 2, 7, 0, 6}, 11, ASN_COUNTER }
};


/*
 * Send a trap to configured destinations with the data given in p_notifData
 */
void send_trap(const SipCommonNotif* p_notifData) 
{
  
  /* one each for elements of SipCommonNotif */ 
  
  struct variable_list var_list[5]; 
  
  int i = 0;
  
  if(!p_notifData) {
    error("send_trap","bad call to send_trap, arg null");
    return;
  }

  memset(var_list,0,sizeof(var_list));
  
  for(i=0;i<5;i++) {
    if(i<4)  
      var_list[i].next_variable = &var_list[i+1];
    /* for the fifth elem, next_var is null */

    var_list[i].name = od[i].objid;
    var_list[i].name_length = od[i].oidlen;
    var_list[i].type = od[i].type;
  
  }
  
  var_list[0].val.integer = (long*)&p_notifData->sipStatusCodeNotifValue;
  var_list[0].val_len = sizeof(long);

  var_list[1].val.string = (unsigned char*)p_notifData->sipStatusCodeNotifTo;
  var_list[1].val_len = strlen(p_notifData->sipStatusCodeNotifTo);
  
  var_list[2].val.string = (unsigned char*)p_notifData->sipStatusCodeNotifFrom;
  var_list[2].val_len = strlen(p_notifData->sipStatusCodeNotifFrom);
  
  
  var_list[3].val.string = (unsigned char*)p_notifData->sipStatusCodeNotifCallId;
  var_list[3].val_len = strlen(p_notifData->sipStatusCodeNotifCallId);
  
  var_list[4].val.integer = (long*)&p_notifData->sipStatusCodeNotifCSeq;
  var_list[4].val_len = sizeof(long);
  
  /* call the ucd-snmp trap sending function */

  send_v2trap(&var_list[0]);
  
  return;

} /* send_trap */

#endif
