#ifndef AUTHEN_H
#define AUTHEN_H

#include "md5.h"
#include "uri.h"
#include "stream.h"

#define MAX_PATH    1024

#define MAXHOSTLIST     255
#define MAXUSERLIST     255
#define MAXGROUPLIST    255

#define HOSTTYPE        1
#define USERTYPE        2
#define GROUPTYPE       3

#define DENY_THEN_ALLOW     1
#define ALLOW_THEN_DENY     2

#define ALLOWED         1
#define DENIED          0

#define AUTHEN_BASIC    1
#define AUTHEN_DIGEST   2


typedef struct check {
    int type;                   /* type of check, host/user/group */
    int permission;             /* allowed or denied */
    char *who;                  /* name of the host/user/group */
} check_t;

typedef struct accessdb {
    int type;                   /* flatfile or databse */
    char *passwdfile;           /* where is the password file */
    char *groupfile;            /* where is the group file */
} accessdb_t;

/* 
 * realm structure
 *
 * contains lists of allowed hosts/groups/users
 * and the order in which they are checked in.
 * the access list contains information of the
 * actual passwd/group file, whether flatfile
 * or db
 */
typedef struct realm {
    long time;                  /* init time */
    char *title;                /* title of the realm */
    char *path;                 /* where is the realm */
    char *rootpath;             /* the full path of the realm */
    int override;               /* can children override */
    struct accessdb *access;    /* password/group file */
    int hostorder;              /* allow/deny or deny/allow */
    int userorder;
    int grouporder;
    struct check *hostallowed[MAXHOSTLIST];
    struct check *hostdenied[MAXHOSTLIST];
    struct check *userallowed[MAXUSERLIST];
    struct check *userdenied[MAXUSERLIST];
    struct check *groupallowed[MAXGROUPLIST];
    struct check *groupdenied[MAXGROUPLIST];
    struct realm *parent;
    struct realm *next;
} realm_t;



/* prototypes */

realm_t *Authen_Init(char *);
void Authen_Show_Realm(realm_t *);
realm_t *Authen_ReadAccess(request_t *, realm_t *);
int Authen_CheckPoint(request_t *, realm_t *, int);

#endif
