/* getuserid.c -- get user ID */
/* Copyright 2000 Columbia University */

#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include "getusername.h"

#if defined(__sparc__) && defined(__svr4__)
/* Use final Posix semantics for getpwuid_r, not draft/Solaris semantics */
#define _POSIX_PTHREAD_SEMANTICS
#endif

#include <pwd.h>

#include "sysdep.h"

#ifndef HAVE_GETPWUID_R
#include <pthread.h>
static pthread_mutex_t pw_lock = PTHREAD_MUTEX_INITIALIZER;
#endif

/*
 * getusername() - get the userid controlling the current process.  If it
 * can't be determined, set name[0] to '\0', and return NULL.
 *
 * Unfortunately, getlogin(), cuserid(), etc., are all either not portable
 * or not thread-safe.  And we can't even do get getenv("LOGNAME")
 * etc. since it's not reliable. So we have this function to substitute.
 *
 * NOTE:  Do not call non-reentrant getpw*() from any other thread while
 * using this function!
 */
extern char* getusername(char *name, int namelen)
{
  struct passwd pwd;
  struct passwd *result;
#ifdef HAVE_GETPWUID_R
  char buffer[1024];
  int ret;
#endif
  uid_t uid;

  uid = getuid();

#ifdef HAVE_GETPWUID_R
  ret = getpwuid_r(uid, &pwd, buffer, sizeof(buffer), &result);
  if (ret != 0) {
    result = NULL;
  }
#else
  pthread_mutex_lock(&pw_lock);
  result = getpwuid(uid);
  if (result != NULL) {
    memcpy(&pwd, result, sizeof(struct passwd));
    result = &pwd;
  }
  pthread_mutex_unlock(&pw_lock);
#endif

  if (result == NULL) {
    name[0] = '\0';
    return NULL;
  }

  strncpy(name, result->pw_name, namelen);
  return name;
} /* getusername */

