# Columbia server makefile fragment for rtspd
# Copyright 1998-2000 by Columbia University; all rights reserved 

rtspd_dir := $(topdir)/rtspd

RTSPD_MODULES := docs

# Include the description for each sub-module
include $(patsubst %,$(topsrcdir)/$(rtspd_dir)/%/module.mk,$(RTSPD_MODULES))


RTSPD_BIN := \
  $(rtspd_dir)/rtspd

RTSPD_SRC = \
  $(rtspd_dir)/config.c \
  $(rtspd_dir)/announce.c \
  $(rtspd_dir)/delete.c \
  $(rtspd_dir)/describe.c \
  $(rtspd_dir)/getusername.c \
  $(rtspd_dir)/mime.c \
  $(rtspd_dir)/pause.c \
  $(rtspd_dir)/play.c \
  $(rtspd_dir)/playservice.c \
  $(rtspd_dir)/qt.c \
  $(rtspd_dir)/recordservice.c \
  $(rtspd_dir)/request.c \
  $(rtspd_dir)/response.c \
  $(rtspd_dir)/rtsp.c \
  $(rtspd_dir)/rtspd.c \
  $(rtspd_dir)/service.c \
  $(rtspd_dir)/session.c \
  $(rtspd_dir)/setup.c \
  $(rtspd_dir)/snd.c \
  $(rtspd_dir)/stream.c \
  $(rtspd_dir)/teardown.c \
  $(rtspd_dir)/mp3.c \
  $(rtspd_dir)/rtpfile.c \
  $(rtspd_dir)/wav.c

RTSPD_CPPSRC := \
  $(rtspd_dir)/mpv.cpp \
  $(rtspd_dir)/mparse.cpp

RTSPD_MAYBESRC := \
  $(rtspd_dir)/authen.c \
  $(rtspd_dir)/authen_test.c \
  $(rtspd_dir)/parser_test.c \
  $(rtspd_dir)/realmedia.c


RTSPD_HEADERS := \
  $(rtspd_dir)/audio.h \
  $(rtspd_dir)/authen.h \
  $(rtspd_dir)/config.h \
  $(rtspd_dir)/getusername.h \
  $(rtspd_dir)/qt.h \
  $(rtspd_dir)/queue.h \
  $(rtspd_dir)/realmedia.h \
  $(rtspd_dir)/request-rtsp.h \
  $(rtspd_dir)/rtp.h \
  $(rtspd_dir)/rtp_api.h \
  $(rtspd_dir)/rtp_pt.h \
  $(rtspd_dir)/rtsp.h \
  $(rtspd_dir)/rtspd.h \
  $(rtspd_dir)/snd.h \
  $(rtspd_dir)/stream.h \
  $(rtspd_dir)/wav.h  \
  $(rtspd_dir)/mime.h \
  $(rtspd_dir)/mp3.h \
  $(rtspd_dir)/rtpfile.h \
  $(rtspd_dir)/mpv.h \
  $(rtspd_dir)/mparse.h

RTSPD_MISC := \
  $(rtspd_dir)/module.mk \
  $(rtspd_dir)/rtspd.dsp

RTSPD_MISCDIST := \
  $(rtspd_dir)/TODO \
  $(rtspd_dir)/BUGS \
  $(rtspd_dir)/README \
  $(rtspd_dir)/rtspd.conf \
  $(rtspd_dir)/mime.types

# Not working currently.
#  $(rtspd_dir)/authen.conf
#  $(rtspd_dir)/rtspclient.tcl


RTSPD_OBJ := \
	$(patsubst %.c,%.o,$(filter %.c,$(RTSPD_SRC)))

RTSPD_CPPOBJ := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(RTSPD_CPPSRC)))  

RTSPD_DEP := \
	$(RTSPD_OBJ:.o=.d)

RTSPD_CPPDEP := \
	$(RTSPD_CPPOBJ:.o=.d)

RTSPD_INCS := -I. $(LIBCINE_INC) $(LIBDICT_INC) -DYULIYA # XXX
RTSPD_LIBS := $(LIBCINE_LIB) $(LIBDICT_LIB) \
	$(NISLIBS) $(SOCKLIBS)
RTSPD_DEPENDENCIES := $(LIBCINE) $(LIBDICT)
RTSPD_REQS := 

$(RTSPD_OBJ): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(RTSPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(RTSPD_CPPOBJ): %.o: %.cpp
	@echo "compiling $<..."
	$(CXX) $(RTSPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(RTSPD_DEP): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(rtspd_dir) -o $@ $(RTSPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

$(RTSPD_CPPDEP): %.d: %.cpp
	@echo "dependencies for $<..."
	$(DEPEND) -d $(rtspd_dir) -o $@ $(RTSPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# all: $(rtspd_dir)/rtspd

# Note: we must link with $(CXX) (the C++ compiler) since rtspd includes some
# C++ code.
$(rtspd_dir)/rtspd:  $(RTSPD_OBJ) $(RTSPD_CPPOBJ) $(RTSPD_DEPENDENCIES)
	@echo "linking $@..."
	$(CXX) -o $@ $(RTSPD_OBJ) $(RTSPD_CPPOBJ) $(RTSPD_LIBS) $(LDFLAGS) $(LIBS)


.PHONY: rtspd
all: rtspd
rtspd: $(RTSPD_REQS) $(RTSPD_BIN)

BIN       += $(RTSPD_BIN) $(RTSPD_OTHER_BIN)
SRC       += $(RTSPD_SRC) $(RTSPD_CPPSRC) $(RTSPD_OTHER_SRC)
HEADERS   += $(RTSPD_HEADERS)
MISC      += $(RTSPD_MISC) $(RTSPD_MAYBESRC)
MISCDIST  += $(RTSPD_MISCDIST)
MISCCLEAN += $(RTSPD_OTHER_MISCCLEAN)

RTSPD_DIST := $(RTSPD_SRC) $(RTSPD_CPPSRC) \
        $(RTSPD_MAYBESRC) $(RTSPD_HEADERS) \
	$(RTSPD_MISC) $(RTSPD_MISCDIST)

RTSPD_BINDIST := $(RTSPD_BIN) $(RTSPD_MISCDIST)

.PHONY: rtspd-dist rtspd-bindist

RTSPD_DIST_NAME := rtspd-$(VERSION)-$(DATE)

rtspd-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(RTSPD_DIST_NAME) \
		$(TOP_DIST) \
		$(LIBCINE_DIST) \
		$(LIBDICT_DIST) \
		$(RTSPD_DIST) \
		$(WIN32_DIST) \
		$(RTSPD_DOCS_DIST)

RTSPD_BINDIST_NAME := rtspd-$(VERSION)-$(DATE)-$(SYSNAME)

rtspd-bindist: rtspd
	$(MKDIST) $(topsrcdir) $(RTSPD_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(RTSPD_BINDIST) \
		$(RTSPD_DOCS_BINDIST)

RTSPD_BINDIST_WIN32 := $(RTSPD_MISCDIST) \
		$(rtspd_dir)/rtspd.exe

RTSPD_BINDIST_WIN32_NAME := rtspd-$(VERSION)-$(DATE)-WinNT

rtspd-bindist-win32:
	$(MKDIST) $(topsrcdir) $(RTSPD_BINDIST_WIN32_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(RTSPD_BINDIST_WIN32) \
		$(RTSPD_DOCS_BINDIST)

