#include "../libcine/dstring.h"
#include "rtp_pt.h"
#include <stdio.h>
#include <unistd.h>        /* lseek() */
#include <string.h>        /* strcmp() */
#include <unistd.h>        /* open() */
#include <stdlib.h>        /* calloc() */
#include <fcntl.h>         /* O_RDONLY, O_RDWR, O_WRONLY */
#include <sys/types.h>
#include <netinet/in.h>    /* ntohl() */
#include <sys/stat.h>
#include <sys/time.h>
#include "mp3.h"
#define MAX_BUF_SIZE 2000
#define USERNAME_MAX 33 

const int bitrate[3][3][16] =
{
  {
    { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0 },
    { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, 0 },
    { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 }
  }, {
    { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 },
    { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
    { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }
  }, {
    { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0 },
    { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 },
    { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, 0 }
  }
};

const int samplerate[3][4] =
{
	{ 44100, 48000, 32000, 0 },
	{ 22050, 24000, 16000, 0 },
	{ 11025, 8000, 8000, 0 }
};


long long timing_get_time()
{
	struct timeval mtv;
	gettimeofday(&mtv, NULL);
	return (long long)(mtv.tv_sec) * 1000 + (long long)(mtv.tv_usec) / 1000;
}

void show_mp3FileHeader(mp3FileHeader *mfh)
	{
	int i,j;
	printf("syncword = %d\t layer = %d\n", mfh->syncword, mfh->layer);
	printf("version = %d\t bitrate_index = %d\n",mfh->version, mfh->bitrate_index);
	printf("samplerate_index = %d\t,padding = %d\n",mfh->samplerate_index, mfh->padding);
	printf("bitrate = %d\t samplerate = %d\n",mfh->bitrate, mfh->samplerate);
	printf("samples = %d\t framesize = %d\n",mfh->samples, mfh->framesize);
	printf("\n\n");
	printf("Samplerate table\n");
	printf("================\n\n");
	for(i=0;i<3;i++)
		{
		for(j=0;j<4;j++)
			printf("%d \t",samplerate[i][j]);
		printf("\n");
		}
	printf("\n");	
	}


void parseMP3header(mp3FileHeader *mfh, unsigned long header)
{
	mfh->syncword = (header >> 21) & 0x0fff;
	mfh->version = 3 - ((header >> 19) & 0x03) ;
	if(mfh->version == 3) mfh->version = 2;
	mfh->layer = 3 - ((header >> 17) & 0x03);
	mfh->error_protection = ((header >> 16) & 0x01) ? 0 : 1;
	mfh->bitrate_index = (header >> 12) & 0x0F;
	mfh->samplerate_index = (header >> 10) & 0x03;
	mfh->padding = (header >> 9) & 0x01;
	mfh->extension = (header >> 8) & 0x01;
	mfh->mode = (header >> 6) & 0x03;
	mfh->mode_ext = (header >> 4) & 0x03;
	mfh->copyright = (header >> 3) & 0x01;
	mfh->original = (header >> 2) & 0x01;
	mfh->emphasis = header & 0x03;
	
	mfh->stereo = (mfh->mode == MPEG_MODE_MONO) ? 1 : 2;
	mfh->bitrate = bitrate[mfh->version][mfh->layer][mfh->bitrate_index];
	mfh->samplerate = samplerate[mfh->version][mfh->samplerate_index];
	if (mfh->version == 0)
		mfh->samples = 1152;
	else
		mfh->samples = 576;
	if(!mfh->samplerate) return;
	if(mfh->layer)
		{
		/* for layer II and III */
		mfh->framesize = (144 * mfh->bitrate * 1000)/ mfh->samplerate + mfh->padding;
		}
	else
		{
		/* for layer I */
		
		mfh->framesize = ((12 * mfh->bitrate * 1000)/ mfh->samplerate + mfh->padding)*4;
		}
	/*show_mp3FileHeader(mfh);*/
	}

/* mp3 frame parsing stuff */
int is_mp3_header(unsigned long head, mp3FileHeader *mfh)
{
	/* print out the first three bytes in hex frm the header */
	int byte1;
	int byte2,byte3;
	byte1 = (head >> 24);
	byte2 = ((head>>16) & 255 );
	byte3 = (head >> 8) & 255; 

	/* fill out the header struct */
	parseMP3header(mfh, head);

	/* check for syncword */
	/* 255*8+7 has the lowest 11 bits set to 1 */
	if (mfh->syncword != (255*8+7))
		{
	return 0;
	}
	
	/* check for the right layer */
	if (mfh->layer != 2)
		{
		printf("not layer two\n");
		return 0;
		}

	/* make sure bitrate is sane */
	if (mfh->bitrate == 0)
		{
		printf("bitrate 0!!!\n");
		return 0;
		}

	/* make sure samplerate is sane */
	if (mfh->samplerate == 0)
		{
		printf("sample rate 0!!!\n");
		return 0;
		}
	return 1;
}

int read_next_header(FILE *fp, mp3Frame *pframe, mp3FileStatus *status, mp3FileHeader *mfh)
        {
        int i=0, header = 0,nextChar, tmpHeader;

        while(1)
                {
		i++;
                if((nextChar = fgetc(fp)) == EOF)
			{
			if(ferror(fp))
				{
				return -1;
				}
			else
				{
                        	return 0;
				}
			}
                header = (header << 8)+ nextChar;
		if(i<4) continue;
                if(is_mp3_header(header, mfh))
                        break;
                }
        if (status->_starttime <= 0)
                status->_starttime = timing_get_time();
        status->_frame_samples = mfh->samples;
        status->_frame_samplerate = mfh->samplerate;

        /* try to read the frame now */
	pframe->byte_string = (char *)calloc(mfh->framesize, sizeof(char));
	tmpHeader = header;
	for(i=3;i>=0;i--)
		{
		pframe->byte_string[i] = (char)(header&255);
		header = header>>8;
		}
	header = tmpHeader;
        for(i=4;i<mfh->framesize;i++)
                {
                nextChar = fgetc(fp);
                if(nextChar == EOF)
			{
			if(ferror(fp))
				{
				return -1;
				}
			else
				{
                        	return 0;
				}
			}
                pframe->byte_string[i] = (char)nextChar;
                }

        pframe->byte_count = mfh->framesize;
        status->_senttime += ((double)status->_frame_samples / (double)status->_frame_samplerate * 1000000);
        status->frames++;
        return 1;
        }
/* returns the duration of file in seconds */
double durationMp3(FILE *mp3File)
	{
	int origpos;
	mp3FileHeader mfh;
	mp3FileStatus status;
	mp3Frame frame;
	double duration;

	origpos = ftell(mp3File);
	fseek(mp3File, 0, SEEK_SET);

	while(read_next_header(mp3File, &frame, &status, &mfh))
		{
		if(frame.byte_string) free(frame.byte_string);
		}

	fseek(mp3File, origpos, SEEK_SET);
	return duration = (double)status._senttime/1000.0;
	} 

double positionMp3(FILE *mp3File, double npt)
        {
        mp3FileHeader mfh;
        mp3FileStatus status;
        mp3Frame frame;

        fseek(mp3File, 0, SEEK_SET);
        status._senttime = 0;
        while(read_next_header(mp3File, &frame, &status, &mfh))
                {
                if(frame.byte_string) free(frame.byte_string);
                if(status._senttime > npt*1000) break;
                }
        return (double) status._senttime/1000.0;
        }

/* 
This function performs the same functions as SndDescribe in snd.c except 
for the following changes :-
1. Does not handle payload types other than mp3->pt (whichis maintained at 14)
2. Does not store track information
3. Bitrate information may be set to 44.1 if no headers have been rate as mp3 
supports bitrate switching and bitrate makes sense only if a header has been read.
*/
char *Mp3Describe( void *mp, char *type)
{
  Mp3 *mp3 = (Mp3 *)mp;
  DString d;
  char username[USERNAME_MAX];
  if( mp3 == NULL)
    {
      return NULL;
    }
  
  getusername(username, sizeof(username));
  DStringInit(&d);
  if (!strcmp(type, "application/sdp")) 
    {
      DStringPrintf(&d, "v=0\r\n");
      DStringPrintf(&d, "o=%s %lu %lu IN IP4 %s\r\n",
                    username[0] != '\0' ? username : "-", time(0), time(0), "127.0.0.1");
      DStringPrintf(&d, "s=No information\r\n");
      if(mp3->header)
        DStringPrintf(&d, "b=CT:%.1f\r\n", mp3->header->bitrate / 1000.);
      else 	DStringPrintf(&d, "b=CT:%.1f\r\n", 44.1);
      DStringPrintf(&d, "t=0 0\r\n");
      DStringPrintf(&d, "a=length:npt=%.3lf\r\n", (double)(mp3->duration/1000));
      DStringPrintf(&d, "m=audio %d RTP/AVP %d\r\n", 0, mp3->pt);
    }
  return d.string;
} /* Mp3Describe */

/*
  This function ignores the track information and doesnot support writing.
  Returns NULL if it doesnt succeed in opening the file
*/
void *Mp3Open(char *path, char *track, int rw)
{
  Mp3 *mp3;
  if(rw != O_RDONLY)
    {
      /* no modes except read supported so return NULL */
      return NULL;
    }
  
  mp3 = (Mp3 *)calloc(1,sizeof(Mp3));
  mp3->fp = fopen(path,"r");
  if(!mp3->fp) 
    {
      free(mp3);
      return NULL;
    }
  mp3->pt = 14;
  mp3->duration = durationMp3(mp3->fp);
  mp3->header = (mp3FileHeader *)calloc(1,sizeof(mp3FileHeader));
  mp3->fileStatus = (mp3FileStatus *)calloc(1,sizeof(mp3FileStatus));
  mp3->fileStatus->_senttime = 0;
  mp3->frame = (mp3Frame *)calloc(1,sizeof(mp3Frame));
  mp3->frame->byte_count = 0;
  return mp3;
}


/*
 * Mp3Read()
 * Read a block of mp3 data into 'buffer' and determine next playout
 * time.  Set 'pt' to RTP payload type and 'tsinc' to time stamp
 * increment.  Return number of bytes read.  Return 0 on EOF and -1 on
 * error.
 
 ####################################################################
 #########	FOR DEMONSTRATION PURPOSES ONLY		############
 ####################################################################
 #       							   #
 # this file aslo dumps the mp3 file portions of the payload into   #
 # a file called .mp3file where its being run. servicing more than  #
 # one client can lead to garbling of this .mp3file as Mp3Read calls#
 # for two clients can be interleaved.                              #
 #								   #
 ####################################################################
 
*/

int Mp3Read(void *mp, char *buffer, double *npt, int *pt, int *tsinc)
{
  Mp3 *mp3 = (Mp3 *)mp;
  int i,flag;
  FILE *mp3file;
  mp3FileHeader mfh;
  if(mp3->frame->byte_string) 
    {
      free(mp3->frame->byte_string);
      mp3->frame->byte_string = NULL;
      mp3->frame->byte_count = 0;
    }
  flag = read_next_header(mp3->fp, mp3->frame, mp3->fileStatus, &mfh); 	
  if(flag <= 0) return flag;
  
  if(!mp3->header)
    {
      mp3->header = (mp3FileHeader *)calloc(1,sizeof(mp3FileHeader ));
    }
  
  memcpy(mp3->header, &mfh, sizeof(mp3FileHeader));
  for(i=0;i<4;i++) buffer[i] = 0;
  memcpy(buffer+4, mp3->frame->byte_string, mp3->frame->byte_count);
  *tsinc += 1152;
  *pt = mp3->pt;
  *npt += (1152.0/mfh.samplerate);
  /* dump the mp3 portion into .mp3file */
  mp3file = fopen(".mp3file","a");
  if(mp3file)
    {
      for(i=4;i<mp3->frame->byte_count+4;i++)
        fprintf(mp3file, "%c", buffer[i]);
      fclose(mp3file);
    }
  return mp3->frame->byte_count+4;
}


/*
 * Mp3Write()
 * This function is not supported, simply returns the size pretending that
 * the write was successful.
 */
int Mp3Write(void *mp3, int pt, int ts, int seq, char *buffer,
             int size, double *npt)
{
  return size;
}
/*
* Position within mp3 file to given time value.
* Return the actual value positioned to.
* If npt < 0.0 just set the mp3File to 0.0   
*/
double Mp3Seek(void *mp, double npt)
{
  Mp3 *mp3 = (Mp3 *)mp;
  if(mp3 == NULL)
    return 0.0;
  
  if(npt < 0.0)
    npt = 0.0;
  
  return positionMp3(mp3->fp, npt);
}



int Mp3Close(void *mp)
{
  Mp3 *mp3 = (Mp3 *)mp;

  if(!mp3) return -1;
  fclose(mp3->fp);
  if(mp3->header) free(mp3->header);
  if(mp3->fileStatus) free(mp3->fileStatus);
  if(mp3->frame)
    {
      if(mp3->frame->byte_string) free(mp3->frame->byte_string);
      free(mp3->frame);
    }
  free(mp3);
  return 0;
}

