/* mp3 stuff */
#define MPEG_MODE_MONO 3

typedef struct mp3_header {
	int syncword;
	int layer;
	int version;
	int error_protection;
	int bitrate_index;
	int samplerate_index;
	int padding;
	int extension;
	int mode;
	int mode_ext;
	int copyright;
	int original;
	int emphasis;
	int stereo;
	int bitrate;
	int samplerate;
	int samples;
	int framesize;
} mp3FileHeader;


typedef struct {
	int bitrate;		/* bitrate of this stream */
	int ispublic;		/* is this stream private? */
	int error;
	int frames;		/* total frames broadcasted */

	long long _starttime;	/* start of this period's timeclock */
	long long _senttime;	/* amout of data we've sent (in milliseconds) */
	int _frame_samples;	/* the number of samples for the current frame */
	int _frame_samplerate;  /* the samplerate of the current frame */
	int _frame_left;	/* how many bytes for teh rest of this frame */
	int _header_bridges;	/* is the header bridged?? */
	unsigned char _header_bridge[3];	/* put part of header here if it spans a boundary */
} mp3FileStatus;


typedef struct {
	int start;
	int len;
	} range;

typedef struct {
	char *byte_string;
	int byte_count;
	} mp3Frame;

typedef struct 
	{
	mp3FileHeader *header;
	mp3FileStatus *fileStatus;
	mp3Frame *frame;
	FILE *fp;
	double duration;
	int pt;
	} Mp3;
	
/***** 
extracts a frame from the buffer 
range specifies the start and length of the buffer 
fourth arguement is the size of buffer and sexond arguement the buffer itself
first arguement is the mp3Frame, notice that the buffer consists of integers
while the mp3Frame has a pinter to the byte string

as a side effect of this function the first element in the buffer is the 
first character(int) after the frame just extracted

return value is the number of valid characters in the buffer, and u shudnt 
overwrite them. is useful to avoid seeks because of reading bytes from
mp3File into a buffer first. 
******/
int extract_frame(mp3Frame *, int *, range, int);

int is_mp3_header(unsigned long head, mp3FileHeader *mh);

extern char *Mp3Describe( void *mp3, char *type);
extern void *Mp3Open(char *path, char *track, int rw);
extern int Mp3Read(void *mp3, char *buffer, double *npt, int *pt, int *tsinc);
extern int Mp3Write(void *mp3, int pt, int ts, int seq, char *buffer,
  int size, double *npt);
extern double Mp3Seek(void *mp3, double npt);
extern int Mp3Close(void *mp3); 


