/* 
 * Copyright (c) 2000 by Columbia University
 *
 * file: mparse.cpp
 * description: mpeg video parser
 * author: Stephen Chou
 * email: schou@cs.columbia.edu
 */


/* Copyright (c) 1995 Oregon Graduate Institute of Science and Technology
 * P.O.Box 91000-1000, Portland, OR 97291, USA;
 * 
 * Permission to use, copy, modify, distribute, and sell this software and its 
 * documentation for any purpose is hereby granted without fee, provided that 
 * the above copyright notice appear in all copies and that both that 
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the name of O.G.I. not be used in advertising or 
 * publicity pertaining to distribution of the software without specific, 
 * written prior permission.  O.G.I. makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is" 
 * without express or implied warranty.
 * 
 * O.G.I. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING 
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL 
 * O.G.I. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY 
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN 
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Shanwei Cen
 *         Department of Computer Science and Engineering
 *         email: scen@cse.ogi.edu
 */

/* Following 3 function and related C macros are taken out of MPEG VCR's vs.c 
 *   int ReadInfoFromFile(void);
 *   void WriteInfoToFile(void);
 *   int init_MPEG1_video_file(void);
 * Global static variable are converted into c++ class member variable
 * support multiple instanciation. -- Stephen Chou
 */
#include "mparse.h"

double Mpeg1video::pictureRateTable[] = {25.976, 24, 25, 29.97, 30, 50, 59.94, 60};


/* #define PARSEDEBUG 1 */
/* #define STCDEBUG */

// USESLICE enable use of slice offset collected during open.
// if USESLICE is not enable then the whole frame is treat as single slice
// #define USESLICE


#define min(a, b) ((a) > (b) ? (b) : (a))
#define max(a, b) ((a) > (b) ? (a) : (b))

#define FileRead(position, buf, size) \
	{ \
	  if (fseek(fp, (position), 0) == -1) \
	  { \
	    perror("VS error on fseek VideoFile"); \
	    exit(1); \
	  } \
	  while (fread((buf), (size), 1, fp) == 0) \
	  { if (errno == EINTR) { errno = 0; continue;}\
	    perror("VS error on fread VideoFile"); \
	    exit(1); \
	  } \
	}

/*
 * ReadInfoFromFile:  this function is taken from Shanwei Cen's MPEG VCR
 * description: read mpeg seq, gop, picture info from .info
 */
int Mpeg1video::ReadInfoFromFile(void)
{
  int fd = -1;
  int i;
  int fnlen = strlen(videoFile);
  char infofile[PATH_SIZE];

  strcpy(infofile, videoFile);

  strcpy(&infofile[fnlen], ".Info");
  fd = open(infofile, O_RDONLY);
  if (fd == -1)
  {
    fprintf(stderr, "Reminder: VS fails to open %s for read, ", infofile);
    perror("try create one");
    goto fail_ReadInfoFromFile;
  }
  read_int(fd, &i);
  if (i != fileSize)
  {
    fprintf(stderr, "Warning: fileSize in Info: %d not the same as actual %d.\n",
	    i, fileSize);
    goto fail_ReadInfoFromFile;
  }

  read_int(fd, &maxS);
  read_int(fd, &maxG);
  read_int(fd, &maxI);
  read_int(fd, &maxP);
  read_int(fd, &maxB);
  read_int(fd, &minS);
  read_int(fd, &minG);
  read_int(fd, &minI);
  read_int(fd, &minP);
  read_int(fd, &minB);
  read_int(fd, &numS);
  read_int(fd, &numG);
  read_int(fd, &numF);
  read_int(fd, &numI);
  read_int(fd, &numP);
  read_int(fd, &numB);
  read_int(fd, &averageFrameSize);
  read_int(fd, &horizontalSize);
  read_int(fd, &verticalSize);
  read_int(fd, &pelAspectRatio);
  read_int(fd, &pictureRate);
  read_int(fd, &vbvBufferSize);
  read_int(fd, &patternSize);

  memset(pattern, 0, PATTERN_SIZE);
  read_bytes(fd, pattern, patternSize);

  systemHeader = (struct SystemHeader *)malloc(sizeof(struct SystemHeader) * numS);
  if (systemHeader == NULL)
  {
    perror("Error: VS error on malloc SystemHeader");
    exit(1);
  }
  gopTable = (struct GopTable *)malloc(sizeof(struct GopTable) * numG);
  if (gopTable == NULL)
  {
    perror("Error: VS error on malloc GopHeader");
    exit(1);
  }
  frameTable = (struct FrameTable *)malloc(sizeof(* frameTable) * numF);
  if (frameTable == NULL)
  {
    perror("Error: VS error on malloc frameTable");
    exit(1);
  }

  for (i = 0; i < numS; i ++)
  {
    read_int(fd, (int *)&systemHeader[i].offset);
    read_int(fd, &systemHeader[i].size);
  }
  for (i = 0; i < numG; i ++)
  {
    read_int(fd, &gopTable[i].systemHeader);
    read_int(fd, (int *)&gopTable[i].offset);
    read_int(fd, &gopTable[i].headerSize);
    read_int(fd, &gopTable[i].size);
    read_int(fd, &gopTable[i].totalFrames);
    read_int(fd, &gopTable[i].previousFrames);
    read_int(fd, (int *)&gopTable[i].firstIoffset);
  }
  for (i = 0; i < numF; i ++)
  {
    read_byte(fd, &frameTable[i].type);
    read_short(fd, (short *)&frameTable[i].size);
  }

  close(fd);

  return 0;

 fail_ReadInfoFromFile:
  if (fd >= 0) {
    close(fd);
  }

  return -1;
}

/*
 * WriteInfoFromFile:  this function is taken from Shanwei Cen's MPEG VCR
 * description: write mpeg seq, gop, picture info to .info
 */
void Mpeg1video::WriteInfoToFile(void)
{
  int fd = -1;
  int i;
  int fnlen = strlen(videoFile);
  char infofile[PATH_SIZE];

  strcpy(infofile, videoFile);

  strcpy(&infofile[fnlen], ".Info");
  fd = open(infofile, O_WRONLY|O_CREAT, 0444);
  if (fd == -1) 
  {
      fprintf(stderr, "VS fails to open %s for write", videoFile);
      perror("");

    goto fail_WriteInfoToFile;
  }
  write_int(fd, fileSize);
  write_int(fd, maxS);
  write_int(fd, maxG);
  write_int(fd, maxI);
  write_int(fd, maxP);
  write_int(fd, maxB);
  write_int(fd, minS);
  write_int(fd, minG);
  write_int(fd, minI);
  write_int(fd, minP);
  write_int(fd, minB);
  write_int(fd, numS);
  write_int(fd, numG);
  write_int(fd, numF);
  write_int(fd, numI);
  write_int(fd, numP);
  write_int(fd, numB);
  write_int(fd, averageFrameSize);
  write_int(fd, horizontalSize);
  write_int(fd, verticalSize);
  write_int(fd, pelAspectRatio);
  write_int(fd, pictureRate);
  write_int(fd, vbvBufferSize);
  write_int(fd, patternSize);

  write_bytes(fd, pattern, patternSize);
  
  for (i = 0; i < numS; i ++)
  {
    write_int(fd, systemHeader[i].offset);
    write_int(fd, systemHeader[i].size);
  }
  for (i = 0; i < numG; i ++)
  {
    write_int(fd, gopTable[i].systemHeader);
    write_int(fd, gopTable[i].offset);
    write_int(fd, gopTable[i].headerSize);
    write_int(fd, gopTable[i].size);
    write_int(fd, gopTable[i].totalFrames);
    write_int(fd, gopTable[i].previousFrames);
    write_int(fd, gopTable[i].firstIoffset);
  }
  for (i = 0; i < numF; i ++)
  {
    write_byte(fd, frameTable[i].type);
    write_short(fd, frameTable[i].size);
  }

  fail_WriteInfoToFile:
  if (fd >= 0) {
      close(fd);
  }

  return;
}

#define nextByte  {int val; fileptr ++; \
		   if ((val = getc(fp)) == EOF) \
		   {\
		     perror("Crossed EOF or error while scanning"); \
		     return 1; \
		   } nb = val;}

#define computePicSize \
	if (inpic) \
	{ \
	  if (pictype == 'I') \
	  { \
	    maxI = max(maxI, (int)(fileptr - picptr - 4)); \
	    minI = min(minI, (int)(fileptr - picptr - 4)); \
	  } \
	  else if (pictype == 'P') \
	  { \
	    maxP = max(maxP, (int)(fileptr - picptr - 4)); \
	    minP = min(minP, (int)(fileptr - picptr - 4)); \
	  } \
	  else \
	  { \
	    maxB = max(maxB, (int)(fileptr - picptr - 4)); \
	    minB = min(minB, (int)(fileptr - picptr - 4)); \
	  } \
	  frameTable[ftptr].type = pictype; \
	  frameTable[ftptr++].size = (int)(fileptr - picptr - 4); \
	  inpic = 0; \
	}

/*
 * Mpeg1video:  this function is taken from Shanwei Cen's MPEG VCR
 * description: try to locate .Info for the stream. If not found, 
 *              parse stream and gather offset and size info for
 *              frame, gop, and sequence
 */
int Mpeg1video::init_MPEG1_video_file(void)
{
  unsigned char nb;
  int state = 0;
  unsigned long fileptr = 0;
  int shptr, gopptr, ftptr;
  int inpic = 0;
  unsigned long picptr = 0;
  int pictype = 0;
  int first = 0;

  
  fp = fopen(videoFile, "r");
  if (fp == NULL)
  {
    fprintf(stderr, "error on opening video file %s", videoFile);
    perror("");
    return 2;
  }

  if (fseek(fp, 0, 2) == -1)
  {
    fprintf(stderr, "File %s not seekable", videoFile);
    perror("");
    return 3;
  }
  fileSize = ftell(fp);
  
  fseek(fp, 0, 0);

  if (ReadInfoFromFile())
  {
    for (;;)
    {
      nextByte;
      if (state >= 0 && nb == 0x00)
	state ++;
      else if (state >= 2 && nb == 0x01)
	state = -1;
      else if (state == -1)
      {
	if (!first) first ++;
	else if (first == 1) first ++;

	switch (nb)
	{
	case 0xb7:	/* seq_end_code */
	  goto exit_phase1;
	  break;
	case 0xb3:	/* seq_start_code */
	  if (first == 1) first = 3;
	  if (first != 3)
	  {
	    fprintf(stderr, "VS error: given file is not in MPEG format.\n");
	    return 4;
	  }
	  numS ++;
	  break;
	case 0xb8:	/* gop_start_code */
	  numG ++;
	  break;
	case 0x00:	/* picture_start_code */
	  nextByte;
	  nextByte;
	  nb &= 0x38;
	  if (nb == 0x08)
	  {
	    numI ++;
	    if (numG == 2)
	      pattern[patternSize++] = 'I';
	  }
	  else if (nb == 0x10)
	  {
	    numP ++;
	    if (numG == 2)
	      pattern[patternSize++] = 'P';
	  }
	  else if (nb == 0x18)
	  {
	    numB ++;
	    if (numG == 2)
	      pattern[patternSize++] = 'B';
	  }
	  /*
	  else
	    fprintf(stderr, "VS error: unkonw picture type %d\n", nb);
	  */
	  break;
	default:
	  break;
	}
	state = 0;
      }
      else
	state = 0;
    }
   exit_phase1:

    if (first != 3)
    {
      fprintf(stderr, "VS error: given file \"%s\" is not of MPEG format.\n", videoFile);
      return 4;
    }

    pattern[patternSize] = 0;
    // STC
    // memset(sendPattern, 1, PATTERN_SIZE);

    numF = numI + numP + numB;
    averageFrameSize = fileptr / (unsigned)numF;
#ifdef PARSEDEBUG
    fprintf(stderr, "Pass one finished, total bytes read: %u, average frame size %d\n",
	    fileptr, averageFrameSize);
    fprintf(stderr, "numS-%d, numG-%d, numF-%d, numI-%d, numP-%d, numB-%d\n",
	    numS, numG, numI, numI, numP, numB);
    fprintf(stderr, "Pattern detected: %s\n", pattern);
#endif /* PARSEDEBUG */
    if (numF > MAX_FRAMES)
    {
      fprintf(stderr, "VS error: Number of frames (%d) is bigger than MAX_FRAMES (%d).\n\
  you need to update the constant definition in common.h and recompile.\n",
	      numF, MAX_FRAMES);
      return 5;
    }

    systemHeader = (struct SystemHeader *)malloc(sizeof(struct SystemHeader) * numS);
    if (systemHeader == NULL)
    {
      perror("VS error on malloc SystemHeader");
      return 7;
    }
    gopTable = (struct GopTable *)malloc(sizeof(struct GopTable) * numG);
    if (gopTable == NULL)
    {
      perror("VS error on malloc GopHeader");
      return 8;
    }
    frameTable = (struct FrameTable *)malloc(sizeof(* frameTable) * numF);
    if (frameTable == NULL)
    {
      perror("VS error on malloc frameTable");
      return 9;
    }

    rewind(fp);
    fileptr = 0;
    state = 0;
    inpic = 0;
    shptr = -1;
    gopptr = -1;
    ftptr = 0;

    for (;;)
    {
      nextByte;
      if (state >= 0 && nb == 0x00)
	state ++;
      else if (state >= 2 && nb == 0x01)
	state = -1;
      else if (state == -1)
      {
	switch (nb)
	{
	case 0xb7:	/* seq_end_code */
	  if (gopptr >= 0 && gopTable[gopptr].size == 0)
	    gopTable[gopptr].size = fileptr - gopTable[gopptr].offset - 4;
	  computePicSize;
	  goto exit_phase2;
	  break;
	case 0xb3:	/* seq_start_code */
	  if (gopptr >= 0 && gopTable[gopptr].size == 0)
	    gopTable[gopptr].size = fileptr - gopTable[gopptr].offset - 4;
	  computePicSize;
	  shptr ++;
	  systemHeader[shptr].offset = fileptr - 4;
	  systemHeader[shptr].size = 0;
	  break;
	case 0xb8:	/* gop_start_code */
	  if (systemHeader[shptr].size == 0)
	     systemHeader[shptr].size =fileptr -  systemHeader[shptr].offset - 4;
	  if (gopptr >= 0 && gopTable[gopptr].size == 0)
	    gopTable[gopptr].size = fileptr - gopTable[gopptr].offset - 4;
	  computePicSize;
	  gopptr ++;
	  gopTable[gopptr].systemHeader = shptr;
	  gopTable[gopptr].offset = fileptr - 4;
	  gopTable[gopptr].headerSize = 0;
	  gopTable[gopptr].size = 0;
	  gopTable[gopptr].totalFrames = 0;
	  gopTable[gopptr].previousFrames = gopptr ?
	      (gopTable[gopptr - 1].totalFrames + gopTable[gopptr - 1].previousFrames) : 0;

	  break;
	case 0x00:	/* picture_start_code */
	  if (gopTable[gopptr].headerSize == 0)
	  {
	    gopTable[gopptr].headerSize = fileptr - gopTable[gopptr].offset - 4;
	    gopTable[gopptr].firstIoffset = fileptr - 4;
	  }
	  gopTable[gopptr].totalFrames ++;
	  computePicSize;
	  picptr = fileptr - 4;
	  nextByte;
	  nextByte;
	  nb &= 0x38;
	  if (nb == 0x08)
	  {
	    pictype = 'I';
	    inpic = 1;
	  }
	  else if (nb == 0x10)
	  {
	    pictype = 'P';
	    inpic = 1;
	  }
	  else if (nb == 0x18)
	  {
	    pictype = 'B';
	    inpic = 1;
	  }
	  break;
	default:

	  break;
	}
	state = 0;
      }
      else
	state = 0;
    }

  exit_phase2:
    for (shptr = 0; shptr<numS; shptr++)
    {
      maxS = max(maxS, systemHeader[shptr].size);
      minS = min(minS, systemHeader[shptr].size);
    }
    for (gopptr = 0; gopptr<numG; gopptr++)
    {
      maxG = max(maxG, gopTable[gopptr].headerSize);
      minG = min(minG, gopTable[gopptr].headerSize);
    }

#ifdef PARSEDEBUG
    fprintf(stderr, "Pass 2 finished.\n");
    fprintf(stderr, "maxS-%d, maxG-%d, maxI-%d, maxP-%d, maxB-%d.\n", maxS, maxG, maxI, maxP, maxB);
    fprintf(stderr, "minS-%d, minG-%d, minI-%d, minP-%d, minB-%d.\n", minS, minG, minI, minP, minB);
#endif
#ifdef PARSEDEBUG
    {
      int i;

      fprintf(stderr, "id: offset    size  -- system header table:\n");
      for (i=0; i<numS; i++)
	fprintf(stderr, "%-3d %-9u %d\n", i, systemHeader[i].offset, systemHeader[i].size);
      fprintf(stderr,
	      "id:  header   offset   hdsize   totSize  frames   preframs Ioffset  Isize -- GOP\n");
      for (i=0; i<numG; i++)
      {
	fprintf(stderr, "%-4d %-8d %-8u %-8d %-8d %-8d %-8d %-8u %d\n",
		i,
		gopTable[i].systemHeader,
		gopTable[i].offset,
		gopTable[i].headerSize,
		gopTable[i].size,
		gopTable[i].totalFrames,
		gopTable[i].previousFrames,
		gopTable[i].firstIoffset,
		frameTable[gopTable[i].previousFrames].size
		);
      }

     fprintf(stderr, "\nframe information:");
      for (i=0; i<numF; i++)
	fprintf(stderr, "%c%c%-8d", (i%10 ? '\0' : '\n'), frameTable[i].type, frameTable[i].size);
      fprintf(stderr, "\n");

    }
#endif /* PARSEDEBUG */
    fseek(fp, systemHeader[0].offset+4, 0);
    nextByte;
    horizontalSize = ((int)nb <<4) & 0xff0;
    nextByte;
    horizontalSize |= (nb >>4) & 0x0f;
    verticalSize = ((int)nb <<8) & 0xf00;
    nextByte;
    verticalSize |= (int)nb & 0xff;
    nextByte;
    pelAspectRatio = ((int)nb >> 4) & 0x0f;
    pictureRate = (int)nb & 0x0f;
    nextByte;
    nextByte;
    nextByte;
    vbvBufferSize = ((int)nb << 5) & 0x3e0;
    nextByte;
    vbvBufferSize |= ((int)nb >>3) & 0x1f;
#ifdef PARSEDEBUG
    fprintf(stderr, "SysHeader info: hsize-%d, vsize-%d, pelAspect-%d, rate-%d, vbv-%d.\n",
	    horizontalSize, verticalSize, pelAspectRatio, pictureRate, vbvBufferSize);
#endif /* PARSEDEBUG */
    WriteInfoToFile();
  }

  return 0;
}

/*
 * function: ParseSeqHead
 * description Parse Sequence header and write info into SeqHeaderInfo_t
 */
void Mpeg1video::ParseSeqHead(SeqHeaderInfo_t* ci, 
			      long offset, 
			      int size,
			      unsigned char* buffer)
{
    SeqHeaderInfo_t* info = (SeqHeaderInfo_t*)buffer;

    FileRead(offset, buffer, size);
    ci->startcode = ntohl(info->startcode);
    ci->h_size = ((buffer[4] << 4) & 0xff0) + (buffer[5] & 0x0f);
    ci->v_size = (buffer[5] << 8) + (buffer[6] & 0xff);
    ci->aspect_ratio = (buffer[7] >> 4) & 0x0f;
    ci->bit_rate = (buffer[7] & 0x0f);
    ci->vbv_buffersize = ((buffer[10] << 5) & 0x3e0) +
	((buffer[11] >> 3) & 0x1f);
#ifdef STCDEBUG
    printf("\tcode %x, hsize=%d, vsize=%d aratio=%d bitrate=%d vbv=%d\n",
	   ci->startcode, ci->h_size, ci->v_size,
	   ci->aspect_ratio, ci->bit_rate, ci->vbv_buffersize);
#endif
}

/*
 * function: ParseGOPHead
 * description Parse GOP header and write info into GOPHeaderInfo_t
 */
void Mpeg1video::ParseGOPHead(GOPHeaderInfo_t* gop, 
			      long offset, int size,
			      unsigned char* buffer)
{
    GOPHeaderInfo_t* info = (GOPHeaderInfo_t*)buffer;


    FileRead(offset, buffer, size);

    gop->startcode = ntohl(info->startcode);
    gop->drop_flag = (buffer[4] & 0x80)? 1: 0;
    gop->tc_hours = (buffer[4] >> 2) & 0x1f;
    gop->tc_minutes = ((buffer[4] & 0x3) << 4) | (buffer[5] >> 4);
    gop->tc_seconds = ((buffer[5] & 0x7) << 3) | (buffer[6] >> 5);
    gop->tc_pictures = ((buffer[6] & 0x1f) << 1) | (buffer[7] >> 7);
#ifdef STCDEBUG
    printf("\t%x %x %x %x\n",
	   buffer[4], buffer[5], buffer[6], buffer[7]);
    printf("\tcode=%x, tc=%d:%d:%d, pictures=%d\n",
	   gop->startcode, gop->tc_hours, gop->tc_minutes,
	   gop->tc_seconds, gop->tc_pictures);
#endif
}


/*
 * function: ParsePictureHead
 * description: Parse PIcture header and write info into PicHeaderInfo_t
 *              Part of frame that can be fit into the buffer is copied
 *              If the buffer can encapsulate buffer then return 0
 *              Otherwise return offset for the next read
 */
long Mpeg1video::ParsePictureHead(PicHeaderInfo_t* info, 
				  long offset, // offset to the stream
				  int size, // size of frame
				  unsigned char* buffer, // result buffer
				  int remsize) // size of buffer
{
    PicHeaderInfo_t* ci = (PicHeaderInfo_t*)buffer;
    int dataread;

#ifndef USESLICE
    dataread = min(size, remsize);
#else // USESLICE
    if (size <= remsize) {
	dataread = size;
    } else {
	int j;

	for (j = sliceTable[curFrame].numslices - 1;
	     sliceTable[curFrame].sliceoffset[j] > size;
	     j--) {
	    if (j  == 0) {
		// one big slice
		dataread = size;
		break;
	    } else {
		dataread = sliceTable[curFrame].sliceoffset[j];
		if (dataread < size) {
		    break;
		}
	    }
	}
    }
#endif // USESLICE

    FileRead(offset, buffer, dataread);
    offset += dataread;

    info->startcode = ntohl(ci->startcode);
    info->tempref = (buffer[4] << 2) + (buffer[5] >> 6);
    info->pictype = (buffer[5] >> 3) & 0x7;
    info->vbv_delay = ((buffer[5] & 0x7) << 13) + (buffer[6] << 5) 
	+ (buffer[7] >> 3);

#ifdef STCDEBUG
    printf("\tcode=%x tempref=%d pictype=%d, vbvdelay=%d\n",
	   info->startcode, info->tempref, info->pictype,
	   info->vbv_delay);
#endif // STCDEBUG

    if (info->pictype >= 2) { // B and P frame
	info->pel_fv = (buffer[7] & 0x4)? 1: 0;
	info->forw_rcode = ((buffer[7] & 0x3) << 2) + (buffer[8] >> 7);

#ifdef STCDEBUG
	printf("\tpel_fv=%d forward_rcode=%d\n",
	       info->pel_fv, info->forw_rcode);
#endif // STCDEBUG
	if (info->pictype == 3) { // B frame
	    info->pel_bv = (buffer[8] & 0x70)? 1: 0;
	    info->back_rcode = (buffer[8] & 0x38) >> 3;
#ifdef STCDEBUG
	    printf("\tpel_bv=%d back_rcode=%d\n",
		   info->pel_bv, info->back_rcode);
#endif // STCDEBUG
	} else {
	    info->pel_bv = 0;
	    info->back_rcode = 0;
	}
    } else {
	info->pel_fv = 0;
	info->forw_rcode = 0;
	info->pel_bv = 0;
	info->back_rcode = 0;
    }
    if (size <= remsize) {
	return 0; // end of frame
    } else {
	return offset;
    }
}

/*
 * function: PrintFrames
 * description: Print out offset and information associating with 
 *              sequence, gop, and picture header and size
 */
void Mpeg1video::PrintFrames()
{
    int i, j;
    int frame;
    int eframe;
    int gstartoffset;
    long offset = 0;
    SeqHeaderInfo_t seqheader;
    GOPHeaderInfo_t gopheader;
    PicHeaderInfo_t picheader;
    unsigned char* buffer = new unsigned char[maxI];

    for (i = 0; i < numS; i++) {
	printf("%6ld: %6ld %6d= system header %d\n", offset,
	       systemHeader[i].offset, systemHeader[i].size, i);
	ParseSeqHead(&seqheader, offset, systemHeader[i].size, buffer);
	offset += systemHeader[i].size;
	for (j = 0; j < numG; j++) {
	    gstartoffset = offset;
	    printf("%6ld: %6ld %6d= GOP header %d\n", offset,
		   gopTable[j].offset, gopTable[j].headerSize, j);
	    ParseGOPHead(&gopheader, offset, gopTable[j].headerSize, buffer);
	    offset += gopTable[j].headerSize;
	    eframe = gopTable[j].previousFrames + gopTable[j].totalFrames; 
	    for (frame = gopTable[j].previousFrames; 
		 frame < eframe; frame++) {
		printf("%6ld: %6ld %6d= frame %d %c\n", offset,
		       offset,
		       frameTable[frame].size, frame, 
		       frameTable[frame].type);
		ParsePictureHead(&picheader, offset, frameTable[frame].size, buffer, maxI);
		offset += frameTable[frame].size;
	    }
	    if ((offset - gstartoffset) != gopTable[j].size) {
		printf("gop total size mismatch!\n");
	    }
	}
    }
    delete [] buffer;
}

void Mpeg1video::CreateSliceTable()
{
    int i;
    int j;
    int frame;
    int lframe; // last frame of gop
    int eframe; // offset end frame
    long offset;
    unsigned int match;
    unsigned char* mptr = (unsigned char*)&match;
    int numslices;
    int sliceOffset[MAX_SLICES];
    unsigned char* ptr;
    unsigned char* buffer = new unsigned char[maxI];

    sliceTable = new SliceOffsetTable[numF];

    for (j = 0; j < numG; j++) {
	offset = gopTable[j].headerSize + gopTable[j].offset;
	lframe = gopTable[j].previousFrames + gopTable[j].totalFrames; 
	for (frame = gopTable[j].previousFrames; 
	     frame < lframe; frame++) {

	    FileRead(offset, buffer, frameTable[frame].size);

	    match = SLICE_MIN_START_CODE;
	    eframe = (int)buffer + frameTable[frame].size - 4;
	    numslices = 0;
	    
	    for (ptr = (unsigned char*)buffer;
		 (int)ptr <= eframe; ptr++) {
		if ((ptr[0] == mptr[0]) && 
		    (ptr[1] == mptr[1]) && 
		    (ptr[2] == mptr[2]) && 
		    (ptr[3] == mptr[3])) {
		    sliceOffset[numslices] = (int)ptr - (int)buffer;
		    numslices++;
		    match++;
		}
	    }
	    sliceTable[frame].numslices = numslices;

#ifdef STCDEBUG
	    fprintf(stderr, "frame %d, offset=%ld, nslice=%d\n",
		    frame, offset, sliceTable[frame].numslices);
#endif
	    for (i = 0; i < numslices; i++) {
		sliceTable[frame].sliceoffset[i] = sliceOffset[i];
#ifdef STCDEBUG
		fprintf(stderr, "\tslice %d at %ld\n",
			i, sliceTable[frame].sliceoffset[i]);
#endif
	    }
	    offset += frameTable[frame].size;
	}
    }

    delete [] buffer;
}

/*
 * function: DeleteSliceTable
 */
void Mpeg1video::DeleteSliceTable()
{
    delete [] sliceTable;
}


/*
 * function: Open
 * description: open a mpeg stream specified by streamname
 */
void Mpeg1video::Open(char* streamname)
{
    curFrameOffset = 0;
    strcpy(videoFile, streamname);
    init_MPEG1_video_file();

#ifdef USESLICE
    CreateSliceTable();
#endif

}

/*
 * function: Close
 * description: free allocated resources
 */
void Mpeg1video::Close()
{
    free(gopTable);
    free(frameTable);
#ifdef USESLICE
    DeleteSliceTable();
#endif

    systemHeader = NULL;
    gopTable = NULL;
    frameTable = NULL;

    fclose(fp);
}

/*
 * function: NextFrame
 * description: calculating next frame and GOP number
 */
inline void Mpeg1video::NextFrame()
{
    curFrameOffset = 0;
    if (curGroup < 0) {
	// seq header was sent last time
	curGroup = 0;
	curFrame = -1;
    } else {
	if (curFrame < 0) {
	    // GOP header was sent last time
	    curFrame = gopTable[curGroup].previousFrames;
	} else {
	    curFrame++;
	}
	curFrameOffset = 0;

	if ((gopTable[curGroup].previousFrames + gopTable[curGroup].totalFrames) 
	    <= curFrame) {
	    // move to next gop, send gop header next time
	    curGroup++;
	    curFrame = -1;

	    // this is a good time to send new seq header
	}
    }
}

/*
 * function: ReadFrame
 * description: read "partially" read frame to a buffer
 */
long Mpeg1video::ReadFrame(unsigned char*buf, // buffer
			   int size,    // buffer size
			   long offset, // last read offset
			   int remsize) // size of frame
{
    int dataread;
#ifndef USESLICE
    dataread = min(size, remsize);
#else
    if (size <= remsize) {
	dataread = size;
    } else {
	int j;
	int offsetread = curFrameOffset - curFrameStart;
	for (j = sliceTable[curFrame].numslices - 1;
	     sliceTable[curFrame].sliceoffset[j] > size;
	     j--) {
	    if (j  == 0 || sliceTable[curFrame].sliceoffset[j] <= offsetread) {
		// one big slice too big to fit into buffer
		dataread = size;
		break;
	    } else {
		dataread = sliceTable[curFrame].sliceoffset[j] - offsetread;
		if (dataread < size) {
		    break;
		}
	    }
	}
    }

#endif
    FileRead(offset, buf, dataread);
    offset += dataread;    

    // check if whole frame is read to buffer
    if (size >= remsize) { 
	return 0; // end of frame
    } else {
	return offset;
    }
}

/*
 * fuction ReadNext
 * description: Read next frame
 *              read end of seq if sendEndSeqHeader or end of seq is detected
 *              send start of seq if curGroup < 0 or sendSeqHeader is set
 *              read GOP if frame# < 0
 *              Call ParsePictureHead to read beginning of a frame
 *              Call ReadFrame to read remaining frame too large to fit
 *                   into a buffer.
 * return:      return true if end of frame is read, false otherwise
 *              size is 0 when end of string is encounter, non-zero otherwise
 *              gop and fr are set to gop and frame number just read
 */
bool Mpeg1video::ReadNext(unsigned char* buffer, int* size, 
			  int* gop, int* fr)
{
    int rsize;
    rtpmpgext_t* mpgext = (rtpmpgext_t*)buffer;


    if (endSeq && (curGroup >= numG)) {
	// end of picture
	*size = 0;
	*gop = -1;
	*fr = -1;
	return false;
    }

    *gop = curGroup;
    *fr = curFrame;

    if (sendEndSeqHeader || (curGroup >= numG)) {
	// if there is end of sequence request or 
	// end of play-out (curGroup >= numG)
	long* endseq = (long*)&buffer[4];
	rsize = sizeof(long) + sizeof(rtpmpgext_t);
	if (rsize > *size) {
	    // error if buffer is smaller than headersize
	    *size = 0;
	    return false;
	}
	*size = rsize;

	mpgext->t = 0; // mpeg2 header extension not present
	mpgext->mbz = 0;
	SetTR(mpgext, 0);
	mpgext->p = 0;
	mpgext->s = 0; // no  seq header!!!
	mpgext->an = 0;
	mpgext->n = 0; // new picture header
	mpgext->b = 0; // beginning of slice

	mpgext->ffv = 0;
	mpgext->ffc = 0;
	mpgext->fbv = 0;
	mpgext->bfc = 0;

	*endseq = htonl(SEQ_END_CODE);

	sendEndSeqHeader = false;
	if (curGroup >= numG) {
	    endSeq = true;
	} else {
	    sendSeqHeader = true; 
	}

	return false;
    }

    if (sendSeqHeader || (curGroup < 0)) {
	// reading seq header
	rsize = systemHeader[0].size + sizeof(rtpmpgext_t);
	if (rsize > *size) {
	    // error if buffer is smaller than headersize
	    *size = 0;
	    return false;
	}
	*size = rsize;
	ParseSeqHead(&curSeqHeader, systemHeader[0].offset, 
		     systemHeader[0].size, 
		     &buffer[4]);


	mpgext->t = 0; // mpeg2 header extension not present
	mpgext->mbz = 0;
	SetTR(mpgext, 0);
	mpgext->p = 0;
	mpgext->s = 1; // new seq header!!!
	mpgext->an = 0;
	mpgext->n = 0; // no new picture header
	mpgext->b = 0; // beginning of slice

	mpgext->ffv = 0;
	mpgext->ffc = 0;
	mpgext->fbv = 0;
	mpgext->bfc = 0;

	sendSeqHeader = false; // reset sendSeqHeader flag
	if (curGroup < 0) {
	    NextFrame();
	}
	return false;
    } else {
	if (curFrame < 0) {
	    // reading gop header
	    rsize = gopTable[curGroup].headerSize + sizeof(rtpmpgext_t);
	    if (rsize > *size) {
		// error if buffer is smaller than headersize
		return false;
	    }
	    *size = rsize;
	    ParseGOPHead(&curGopHeader, gopTable[curGroup].offset, 
			 gopTable[curGroup].headerSize, 
			 &buffer[4]);

	    mpgext->t = 0; // mpeg2 header extension not present
	    mpgext->mbz = 0;
	    SetTR(mpgext, 0);
	    mpgext->p = 0;
	    mpgext->s = 0; // no new seq header
	    mpgext->an = 0;
	    mpgext->n = 0; // new picture header
	    mpgext->b = 0; // beginning of slice

	    mpgext->ffv = 0;
	    mpgext->ffc = 0;
	    mpgext->fbv = 0;
	    mpgext->bfc = 0;

	    NextFrame();
	    return false;
	} else {
	    // this is a picture
		    
	    if (curFrameOffset == 0) {
		// calculating starting offset of a frame
		int iframe;

		long foffset = gopTable[curGroup].offset + 
		    gopTable[curGroup].headerSize;
		for (iframe = gopTable[curGroup].previousFrames; 
		     iframe < curFrame; iframe++) {
		    foffset += frameTable[iframe].size;
		}
		curFrameStart = foffset;
		curFrameSize = frameTable[curFrame].size;
#ifdef STCDEBUG
		fprintf(stderr,
			"reading gop %d frame %d sz=%d start=%ld offset=%ld\n",
			curGroup, curFrame, curFrameSize, 
			curFrameStart, curFrameOffset);
#endif
		curFrameOffset = ParsePictureHead(&curPicHeader, 
						  curFrameStart,
						  curFrameSize,
						  &buffer[4], 
						  *size-4);

		mpgext->n = 1; // new picture header
		mpgext->b = 1; // beginning of slice


		if (curFrameOffset == 0) {
		    *size = curFrameSize + sizeof(rtpmpgext_t);
		} else {
		    *size = (curFrameOffset - curFrameStart) + 
			sizeof(rtpmpgext_t);
		}

	    } else {
		// the is part of a frame too large to fit into a buffer
		long lastFrameOffset = curFrameOffset;
#ifdef STCDEBUG
		fprintf(stderr,
			"reading gop %d frame %d sz=%d start=%ld offset=%ld rem=%ld\n",
			curGroup, curFrame, curFrameSize, 
			curFrameStart, curFrameOffset,
			curFrameSize - (curFrameOffset - curFrameStart));
#endif



		curFrameOffset = ReadFrame(&buffer[4], *size-4,
			  curFrameOffset, 
			  curFrameSize - (curFrameOffset - curFrameStart));

		if (curFrameOffset == 0) {
		    *size = (curFrameSize + curFrameStart - lastFrameOffset)
			+ sizeof(rtpmpgext_t);
		} else {
		    *size = (curFrameOffset - lastFrameOffset)
			+ sizeof(rtpmpgext_t);
		}

		mpgext->n = 0; // no picture header
		mpgext->b = 0; // not beginning of slice
	    }


	    // reading picture frames
	    mpgext->t = 0; // mpeg2 header extension not present
	    mpgext->mbz = 0;
	    SetTR(mpgext, curPicHeader.tempref);
	    mpgext->p = curPicHeader.pictype;
	    mpgext->s = 0; // no new seq header
	    mpgext->an = 0;

	    mpgext->ffv = curPicHeader.pel_fv;
	    mpgext->ffc = curPicHeader.forw_rcode;
	    mpgext->fbv = curPicHeader.pel_bv;
	    mpgext->bfc = curPicHeader.back_rcode;

	    if (curFrameOffset != 0) {
		mpgext->e = 0; // not end of slice
		return false;
	    } else {
		mpgext->e = 1; // end of slice
		NextFrame();
		return true;
	    }

	}
    }

}

/*
 * fuction SeekFrame
 * description: perform frame seek.  If the seek request is receive
 *              in the mid of play-out. Send end sequence header 
 *              and follow by start sequence header next.
 */
void Mpeg1video::SeekFrame(int frame)
{
    int j;
    bool found = false;

    endSeq = false;

    if (frame == 0) {
	// reset the sequence to the beginning of video stream
	sendSeqHeader = true;
	curFrame = -1;
	curGroup = -1;
	if ((curFrame == -1) && (curGroup == -1)) {
	    sendEndSeqHeader = false;
	} else {
	    // send sequence header when receiving seek request during
	    // mid of sequence
	    sendEndSeqHeader = true;
	}
	return;
    }

    if ((curFrame != -1) || (curGroup != -1)) {
	// detect frame reset during a playout
	// we want to issue an end of sequence follow by header transmission
	sendEndSeqHeader = true;
	sendSeqHeader = true;
    }

    curFrameOffset = 0;

    if (frame < 0) {
	// reset to the beginning of sequence
	curFrame = -1;
	curGroup = -1;
    } else {
	for (j = 0; j < numG; j++) {
	    if (gopTable[j].previousFrames <= frame &&
		(gopTable[j].previousFrames + gopTable[j].totalFrames) 
		> frame) {
		found = true;
		break;
	    }
	}
	if (found) {
	    curFrame = -1;
	    curGroup = j;
	} else {
	    fprintf(stderr, "Seek out of range\n");
	}
    }
}

/*
 * fuction Mpeg1video: class constructor
 * description: initialize member variables
 */
Mpeg1video::Mpeg1video() {
    fileSize = 0;
    maxS = 0; 
    maxG = 0;  
    maxI = 0; 
    maxP = 0; 
    maxB = 0;
    minS=0x7fffffff; 
    minG = 0x7fffffff;
    minI = 0x7fffffff; 
    minP = 0x7fffffff; 
    minB = 0x7fffffff;
    numS = 0; 
    numG = 0; 
    numF = 0; 
    numI = 0; 
    numP = 0; 
    numB = 0;
    patternSize = 0;
    curFrame = -1;
    curGroup = -1;
    sendSeqHeader = true;
    sendEndSeqHeader = false;
    endSeq = false;
}

#ifdef TEST
main(int argc, char* argv[])
{
    unsigned char buffer[1400];
    int size;
    Mpeg1video mpg;
    int gop;
    int frame;

    mpg.Open(argv[1]);
    // mpg.PrintFrames();
    do {
	size = 1400;
	mpg.ReadNext(buffer, &size, &gop, &frame);
	printf("gop %d frame %d: size=%d\n", gop, frame, size);
	if (size) {
	    // fwrite(&buffer[4], size-4, 1, stdout);
	}
    } while (size != 0);
    mpg.Close();
}

#endif
