#include <stdio.h>
#include <string.h>
#include "rfc822.h"
#include "http.h"
#include "rtsp.h"

static char *w(char *s)
{
  return s ? s : "(null)"; 
}

int main(int argc, char *argv[])
{
  char message[4096];
  int n = 0;
  parser_t P;
  parser_header_t    *h;
  parser_element_t   *e;
  parser_parameter_t *p;

  http_content_type_t content;
  http_server_t       server;
  http_agent_t        agent;
  struct tm tm;
  rtsp_transport_t    transport;
  rtsp_session_t      session;
  rtsp_range_t        range;
  rtsp_timestamp_t    timestamp;

  while (fgets(&message[n], sizeof(message), stdin)) {
    n += strlen(&message[n]);
  }
  printf("Input: %s\n---------------------------\n", message);

  P = HTTP_Parse(message);   
  printf("HTTP_Method   %s\n", w(HTTP_Method(&P)));
  printf("HTTP_URI      %s\n", w(HTTP_URI(&P)));
  printf("HTTP_Status   %d\n", HTTP_Status(&P)); 
  printf("HTTP_Reason   %s\n", w(HTTP_Reason(&P))); 
  printf("HTTP_Protocol %s\n", w(HTTP_Protocol(&P)));
  printf("HTTP_Version  %s\n", w(HTTP_Version(&P))); 
  printf("body          %s\n", w(P.body));

  for (h = P.headers; h; h = h->next) {
    printf("%s:\n", w(h->name));
    for (e = h->elements; e; e = e->next) {
      printf("  %s\n", w(e->value));
      for (p = e->parameters; p; p = p->next) {
        printf("    [%s]=%s\n", w(p->attribute), w(p->value));
      }
    }
  }

  printf("HTTP_ContentLength = %d\n", HTTP_ContentLength (&P));
  content = HTTP_ContentType(&P);
  printf("HTTP_ContentType   = %s/%s\n", content.type, content.subtype);
  server = HTTP_Server(&P);
  printf("HTTP_Server        = %s\n", w(server.type));
  agent = HTTP_UserAgent(&P);
  printf("HTTP_UserAgent     = %s\n", w(agent.type));

  tm = RFC822_Date(&P);
  printf("RFC822_Date        = %d\n", tm.tm_year);

  printf("RTSP_CSeq          = %d\n", RTSP_CSeq(&P));

  transport = RTSP_Transport(&P);
  printf("RTSP_Transport     = port=%d-%d\n",
    transport.port[0], transport.port[1]);
  printf("RTSP_Transport     = client=%d-%d\n",
    transport.client_port[0], transport.client_port[1]);
  printf("RTSP_Transport     = server=%d-%d\n",
    transport.server_port[0], transport.server_port[1]);
  printf("RTSP_Transport     = ttl=%d multicast=%d destination=%s\n",
    transport.ttl, transport.multicast, inet_ntoa(transport.destination));
  printf("RTSP_Transport     = compressed=%d\n", transport.compressed);
  session = RTSP_Session(&P);
  printf("RTSP_Session       = %s\n", w(session.id));
  range = RTSP_Range(&P);
  printf("RTSP_Range         = %f %f %f\n", range.begin, range.end, range.effective);
  printf("RTSP_Scale         = %f\n", RTSP_Scale(&P));
  timestamp = RTSP_Timestamp(&P);
  printf("RTSP_Timestamp     = %f %f\n", timestamp.timestamp, timestamp.delay);

  return 0;
}
