/*
* Pause()
* Insert pause request into request queue.  Match first pending request
* that spans the PAUSE time.  If the PAUSE contains no time, pause
* immediately.  If the PAUSE time matches no pending request, discard
* the request (probably, the request that matched it had already
* completed).
*/

#include "stream.h"
#include <pthread.h>

int Pause(Stream *st, request_t *r)
{
  request_t *rr;

  /* Lock stream state. */
  debug("Pause","mutex(st->lock) locking\n");
  pthread_mutex_lock(&st->lock);  
  debug("Pause","mutex(st->lock) locked\n");

  /* Immediate PAUSE. Ignore if there are no current requests pending. */
  if (r->range.begin < 0) {
    st->state = S_PAUSE;
    if (st->requests) {
      st->requests->range.end = 0.;
    }
    pthread_cond_signal(&st->pause);
  }
  /* Timed PAUSE for pending request. */
  else {
    for (rr = st->requests; rr; rr = r->next) {
      if (r->range.begin >= rr->range.begin &&
          r->range.begin <= rr->range.end) {
        rr->range.end = r->range.begin;
        break;
      }
    }
  }
  /* Unlock stream. */
  pthread_mutex_unlock(&st->lock);  
  debug("Pause","mutex(st->lock) unlocked\n");

  debug("Pause","Exiting");

  ResponseSetStatus(r, 200, NULL);
  return 200;
} /* Pause */
