/* QT.c*/
/* Copyright Columbia University */

#include <stdio.h>    /* fseek() */
#include <fcntl.h>    /* O_ */
#include <stdlib.h>   /* calloc(), free() */
#include <string.h>   /* strncmp() */
#include <math.h>     /* floor() */

#define  QT_INTERNAL

#include "qt.h"
#include "rtp_pt.h"
#include "dstring.h"  /* DString..() */
#include "getusername.h"   /* getusername() */

typedef unsigned short u_int16;
typedef unsigned long u_int32;

typedef  struct{
  u_int32   a;  /* */
  u_int32   b;
  u_int32   c;
} info_t;

typedef struct{
  u_int32   count;
  u_int32   duration;
}time_sample_t;

typedef struct{
  double   time;
  int      chunk;
  int      sample;
  u_int32  sample_offset;
  u_int32  chunk_offset;
  int      inchunk;
  u_int32  chunk_end;
  u_int32  size_to_read;
  int      next_chunk;
  
  
}seek_t;

typedef struct{
  char        type[5];
  
  u_int32     TrackID;
  u_int32     Duration;
  u_int32     Scale;
  char        format[4];
  u_int32     DurationSec;
  float       Width;
  float		  Height;
  u_int32     a;    /* samples_per_chunk */
  u_int32     b;    /* chunk_offset */
  u_int32     c;    /* sample_size */
  u_int32     d;    /* time_to_sample */
  info_t      *Samples_per_Chunk;
  u_int32     *Chunk_Offset;
  u_int32     *Sample_Size;
  time_sample_t *Time_to_Sample;
  u_int16     frame_countV;
  u_int32     temporal_qualityV;
  u_int32     spatial_qualityV;
  u_int16     channelsA;
  u_int16     sample_sizeA;
  int         size_seek;
  seek_t      *seek_array;
  int         chunks;
  int         samples;
  char        name[4];
  int         pt; /* payload type */
 
}track_t;
 
typedef struct{
  FILE        *f;
  char        path[80];
  u_int32     scale;
  u_int32     duration;
  float       rate;
  float       volume;
  track_t     *tracks_array;
  int         number_tracks;
  double      frames_per_sec;
  int         track;
}QT_t;



/*
* readl32()
* Read 32-bit integer 
*/
static u_int32 readl32(FILE *f)
{
  unsigned char u[4] = {0,0,0,0};

  if (fread(u, sizeof(char), 4, f) == 4) {
    return ((u_int32)u[0] << 24) | ((u_int32)u[1] << 16) |
           ((u_int32)u[2] <<  8) | ((u_int32)u[3]);
  }
  return 0;
} /* readl32 */


/*read in 32 bit long*/
static long readf32(FILE *f)
{
  unsigned char u[4] = {0,0,0,0};

  if (fread(u, sizeof(char), 4, f) == 4) {
    return (long)((((long)u[0] << 24) | ((long)u[1] << 16) |
           ((long)u[2] <<  8) | ((long)u[3]))/(float)16.0);
  }
  return 0.0;
} /* readf32 */


/* read in 16 bit unsigned int */
static u_int16 readl16(FILE *f)
{
  unsigned char u[2] = {0,0};

  if (fread(u, sizeof(char), 2, f) == 2) {
    return ((u_int16)u[0] <<  8) | ((u_int16)u[1]);
  }
  return 0;
} /* readl16 */

/* read in 16 bit signed int */
static int readf16(FILE *f)
{
  unsigned char u[2] = {0,0};

  if (fread(u, sizeof(char), 2, f) == 2) {
    return (int)( (((int)u[0] <<  8) | ((int)u[1]))/4.0);
  }
  return 0;
} /* readf16 */



int QTSeek (QT_t *qt, double npt) {
  double m_time;
  u_int32 i, j, sample, chunk_index, index, flag=0;
  u_int32 temp=0, temp1=0, temp2=0, chunk, inchunk;
  u_int32 offset, sample_offset, end_of_chunk, size_read;
 
  if (qt->track != -1) {
    index = qt->track;
  }
  else {
    return(-1);
  }
 
  m_time = npt * (double)qt->tracks_array[index].Scale;
  if (m_time >= qt->tracks_array[index].Duration)
    return (-1);
  /* find the sample to read */
  if (qt->tracks_array[index].d == 1) {
    sample = m_time / qt->tracks_array[index].Time_to_Sample[0].duration;
  }
  else {
    for (i = 0; i < qt->tracks_array[index].d; i++) {
      if ((temp + qt->tracks_array[index].Time_to_Sample[i].duration *
           qt->tracks_array[index].Time_to_Sample[i].count) > m_time) {
	temp2 = m_time - temp;
	sample = temp1 + 
          (u_int32)(temp2/qt->tracks_array[index].Time_to_Sample[i].duration);
	break;
      }
      else {
	temp += qt->tracks_array[index].Time_to_Sample[i].duration *
          qt->tracks_array[index].Time_to_Sample[i].count;
	temp1 += qt->tracks_array[index].Time_to_Sample[i].count;
      }
    }
  }
 
  temp = 0;
  temp1 = 0;
  temp2 = 0;

  /* find a chunk */
  for (i = 0; i < qt->tracks_array[index].a; i++) {
    if (i + 1 != qt->tracks_array[index].a) {
      if ((temp + (qt->tracks_array[index].Samples_per_Chunk[i+1].a -  
	qt->tracks_array[index].Samples_per_Chunk[i].a) *
           qt->tracks_array[index].Samples_per_Chunk[i].b) > sample) {
	chunk = qt->tracks_array[index].Samples_per_Chunk[i].a +
          (u_int32) ((sample - temp) /
                     qt->tracks_array[index].Samples_per_Chunk[i].b);
	temp2 = temp +
          (u_int32)(chunk - qt->tracks_array[index].Samples_per_Chunk[i].a) *
          qt->tracks_array[index].Samples_per_Chunk[i].b;
        inchunk = sample - temp2;
	chunk_index = i;
	break;
      } else {
	temp += (u_int32) ((qt->tracks_array[index].Samples_per_Chunk[i+1].a -
                            qt->tracks_array[index].Samples_per_Chunk[i].a) *
                           qt->tracks_array[index].Samples_per_Chunk[i].b) ;
	temp1 = qt->tracks_array[index].Samples_per_Chunk[i+1].a;
      }
    }
    else {
      chunk = (u_int32) (sample - temp) /
        qt->tracks_array[index].Samples_per_Chunk[i].b + temp1 ;
      temp2 = temp + (u_int32)(chunk - temp1) *
        qt->tracks_array[index].Samples_per_Chunk[i].b;
      inchunk = sample - temp2;
      chunk_index = i;
      break;
    }
  }

  
  /* find a chunk offset */
  offset = qt->tracks_array[index].Chunk_Offset[chunk-1];
  sample_offset = 0;
  /* find offset within the chunk and the size */
  if (qt->tracks_array[index].c == 1) {
    sample_offset = (inchunk )* qt->tracks_array[index].Sample_Size[0];
  }
  else {
    for (i = sample - inchunk; i < sample; i++) {
      sample_offset = sample_offset + qt->tracks_array[index].Sample_Size[i];
    }
  }

  temp  = 0;
  temp1 = 0;
  temp2 = 0;
  /* find end of chunk, size to read */
  temp = qt->tracks_array[index].Samples_per_Chunk[chunk_index].b;
  if (qt->tracks_array[index].c != 1) {
    for (i = sample - inchunk; i < sample - inchunk + temp+1; i++) {
      temp1 = temp1 + qt->tracks_array[index].Sample_Size[i];
    }
  }
  else {
    temp1 = temp * qt->tracks_array[index].Sample_Size[0];
  }
  end_of_chunk = offset + temp1;

  temp  = 0;
  temp1 = 0;
  temp2 = 0;
  temp = qt->tracks_array[index].Scale / qt->frames_per_sec;
  
  if (!strcmp(qt->tracks_array[index].type,"video")) {
    temp2 = qt->tracks_array[index].Sample_Size[sample-1];
  }
  else if (qt->tracks_array[index].samples != sample) {
    temp1 = temp / qt->tracks_array[index].Time_to_Sample[0].duration;
    if (qt->tracks_array[index].c != 1) {
      for (i=sample; i<sample + temp1; i++) {
        temp2  = temp2 + qt->tracks_array[index].Sample_Size[i];
      }
    }
    else {
      temp2 = temp1 * qt->tracks_array[index].Sample_Size[0];
    }
  }
  else {
    if (qt->tracks_array[index].c != 1) {
      temp2 = qt->tracks_array[index].Sample_Size[sample-1];
    }
    else {
      temp2 = qt->tracks_array[index].Sample_Size[0];
    }
  }
  
  size_read = temp2;
  temp  = 0;
  temp1 = 0;
  temp2 = 0;


  qt->tracks_array[index].seek_array =
    realloc(qt->tracks_array[index].seek_array, sizeof(seek_t));
  qt->tracks_array[index].seek_array[0].time = npt;
  qt->tracks_array[index].seek_array[0].chunk = chunk;
  qt->tracks_array[index].seek_array[0].next_chunk = 0;
  qt->tracks_array[index].seek_array[0].sample = sample;
  qt->tracks_array[index].seek_array[0].inchunk = inchunk;
  qt->tracks_array[index].seek_array[0].sample_offset = sample_offset;
  qt->tracks_array[index].seek_array[0].chunk_offset = offset;
  qt->tracks_array[index].seek_array[0].chunk_end = end_of_chunk;
  qt->tracks_array[index].seek_array[0].size_to_read = size_read;
  qt->tracks_array[index].size_seek = 1;

 
      

  i = 1;
  flag = 0;
  if (size_read > end_of_chunk - offset - sample_offset) {
   
    qt->tracks_array[index].seek_array[0].size_to_read = end_of_chunk - offset - sample_offset;
    size_read = size_read - qt->tracks_array[index].seek_array[0].size_to_read;
    while (flag != 1) {
      chunk = chunk + 1;
      qt->tracks_array[index].seek_array[i-1].next_chunk = chunk;
      sample = sample - inchunk + qt->tracks_array[index].Samples_per_Chunk[chunk_index].b;
     
      inchunk = 0;
      offset = qt->tracks_array[index].Chunk_Offset[chunk -1];
      sample_offset = 0;
      /* find chunk_index */
      for (j = 0;j<qt->tracks_array[index].a;j++) {
	if (j + 1 != qt->tracks_array[index].a) {
	  if (qt->tracks_array[index].Samples_per_Chunk[j+1].a > chunk){
	    chunk_index = j;
	   
	    break;
	  }
	} else {
	  chunk_index = j;
	  break;
	}
      }
      /* find end_of_chunk */
       temp = qt->tracks_array[index].Samples_per_Chunk[chunk_index].b;
       if (qt->tracks_array[index].c != 1) {
	 for (i = sample - inchunk ;i < sample - inchunk + temp;i++) {
	   temp1  = temp1 + qt->tracks_array[index].Sample_Size[i];
	 }
       } else {
	 temp1 = temp *  qt->tracks_array[index].Sample_Size[0];
	
       }
       end_of_chunk = offset + temp1;
       qt->tracks_array[index].size_seek++;
     qt->tracks_array[index].seek_array = realloc(  qt->tracks_array[index].seek_array ,sizeof(seek_t)*(i+1));
       qt->tracks_array[index].seek_array[i].time = npt;
       qt->tracks_array[index].seek_array[i].chunk = chunk;
       qt->tracks_array[index].seek_array[i].sample = sample;
       qt->tracks_array[index].seek_array[i].inchunk = inchunk;
       qt->tracks_array[index].seek_array[i].sample_offset = sample_offset;
       qt->tracks_array[index].seek_array[i].chunk_offset = offset;
       qt->tracks_array[index].seek_array[i].chunk_end = end_of_chunk;
       qt->tracks_array[index].seek_array[i].size_to_read = size_read;
       if (size_read > end_of_chunk - offset - sample_offset) {
	
	 qt->tracks_array[index].seek_array[i].size_to_read = end_of_chunk - offset - sample_offset;
	 size_read = size_read - qt->tracks_array[index].seek_array[i].size_to_read;
	 i++;
       }
       else {
	 flag = 1;
	 break;
       }
    }/* end while */
  }
  /* Leave it here for now, just in case  
   * for (j = 0;j<qt->tracks_array[index].size_seek;j++){
   *
   * printf("\n\nSample : %lu\n", qt->tracks_array[index].seek_array[j].sample);
   * printf("Inchunk  %i\n",qt->tracks_array[index].seek_array[j].inchunk);
   * printf("Chunk : %lu\n",qt->tracks_array[index].seek_array[j].chunk);
   * printf("Sample_Offset : %lu\n",qt->tracks_array[index].seek_array[j].sample_offset);
   * printf("Offset : %lu\n",qt->tracks_array[index].seek_array[j].chunk_offset);
   * printf("Time : %f\n",qt->tracks_array[index].seek_array[j].time);
   * printf("End of Chunkl : %lu\n",qt->tracks_array[index].seek_array[j].chunk_end);
   * printf("Size : %lu\n",qt->tracks_array[index].seek_array[j].size_to_read);
   * printf("Next chunk : %lu\n",qt->tracks_array[index].seek_array[j].next_chunk);
   * }
   */
      
	
    
    
  
  

  return (qt->tracks_array[index].seek_array[0].chunk_offset + 
	  qt->tracks_array[index].seek_array[0].sample_offset) ;
}


int QTRead(QT_t *qt,char *buffer,double *npt, int *pt, int *tsinc)
{
  int index,i,k,flag;
  double nnpt;
  double j;
  flag = 0;
  nnpt = *npt;
  if (qt->track != -1) {
    index = qt->track;
  }
  else return (-1);
  if (qt->tracks_array[index].size_seek > 0) {
    if (qt->tracks_array[index].seek_array[0].time == nnpt) {
      ; /* do nothing, just go read */
    }
    else if ((j =  QTSeek(qt,nnpt))<0)
      return -1;
  }
  else if ((j =  QTSeek(qt,nnpt))<0)
    return -1;

  /* read in the data */
  k = 0;
  *pt = qt->tracks_array[index].pt;
  rewind(qt->f);
  fseek(qt->f, (qt->tracks_array[index].seek_array[0].chunk_offset + 
	  qt->tracks_array[index].seek_array[0].sample_offset),SEEK_CUR);
  flag = 0;
  if (!strcmp(qt->tracks_array[index].type,"audio")) {
    flag = 1;
  }
  for (i = 0; i < qt->tracks_array[index].size_seek; i++) {
    /*read in the data */
    k = k + fread(&buffer[k],sizeof(char),qt->tracks_array[index].seek_array[i].size_to_read,qt->f);
    
  }
  if (flag) {
    *tsinc = j;
    *npt += *tsinc/qt->tracks_array[index].Scale;
  }
  else if (qt->tracks_array[index].size_seek == 1) {
    *tsinc =90000/qt->frames_per_sec;
    *npt += 1/qt->frames_per_sec; 
 }
 return k;

}/* QTRead */

int QTWrite(QT_t *qt,int pt,u_int32 ts,u_int16 seq, char *buffer,int size,double *npt)
{
  return (-1);
  /*impossible to implement , at least at this stage */
}


char  *QTDescribe(QT_t *qt,char *type)
{
  DString d;
  int	  i,index;
  char username[USERNAME_MAX];

  getusername(username, sizeof(username));

  index = -1;
  DStringInit(&d);
  if (!strcmp(type, "application/sdp")) {
    DStringPrintf(&d, "v=0\r\n");
    DStringPrintf(&d, "o=%s %lu %lu IN IP4 %s\r\n",
      username[0] != '\0' ? username : "-", (long)time(0), (long)time(0),
                  "127.0.0.1");
    
    DStringPrintf(&d, "t=0 0\r\n");
    DStringPrintf(&d, "a=length:npt=%.3f\r\n", (float)qt->duration/qt->scale);
	if (qt->track != -1) {
	  
	  index = qt->track;
	} 
	if (index != -1) {
		DStringPrintf(&d, "m=%s %d RTP/AVP %d\r\n",qt->tracks_array[index].type, 0, qt->tracks_array[index].pt);
		DStringPrintf(&d, "a=control: rtsp://127.0.0.1%s?%i\r\n",qt->path,qt->track);
        
		if (qt->tracks_array[index].pt > 95) {
		  DStringPrintf(&d, "a=ptime:%.3f\r\n", (1/ (qt->tracks_array[index].Scale /  qt->frames_per_sec)));
	
		  DStringPrintf(&d, "a=rtpmap:%d %s/%lu/%u\r\n", qt->tracks_array[index].pt,  qt->tracks_array[index].name,
				qt->tracks_array[index].Scale,  qt->tracks_array[index].channelsA);
		}
	}
	else {
		for (i = 0;i<qt->number_tracks;i++) {
		  DStringPrintf(&d, "m=%s %d RTP/AVP %d\r\n",qt->tracks_array[i].type, 0, qt->tracks_array[i].pt);
		  DStringPrintf(&d, "a=control: rtsp://127.0.0.1%s?%lu\r\n",qt->path,qt->tracks_array[i].TrackID);
		  
		  if (qt->tracks_array[index].pt > 95) {
		    DStringPrintf(&d, "a=ptime:%.3f\r\n", (1/ (qt->tracks_array[i].Scale /  qt->frames_per_sec)));
		    DStringPrintf(&d, "a=rtpmap:%d %s/%lu/%u\r\n", qt->tracks_array[i].pt,  qt->tracks_array[i].name,
				  qt->tracks_array[i].Scale,  qt->tracks_array[i].channelsA);
		  }
		}
	}
  }
  return d.string;


} /*end describe */

int  QTClose(QT_t *qt)
{
  int i;

  fclose(qt->f);
  for (i=0;i<qt->number_tracks;i++) {
    free(qt->tracks_array[i].Samples_per_Chunk);
    free(qt->tracks_array[i].Time_to_Sample);
    free(qt->tracks_array[i].Chunk_Offset);
    free(qt->tracks_array[i].Sample_Size);
    free(qt->tracks_array[i].seek_array);
    
  }
  free(qt->tracks_array);
  free(qt);
  return 0;
} /* QTClose */

QT_t *QTOpen(char *pathp,char *param,int rw)
{
	FILE	*f;
	char	magic[4];
	char    *mode;
	char    *path;
	char    *track;
	int     track_id;
	int     flag = 0;
	u_int32 temp,size,temporary,temp1,temp2,temp4,temp7;
	int		i=0,j,k,n,z,x;
	u_int32 sample_size;
	QT_t      *qt ;
	int		number_tracks = -1;
	

	static struct{
		char  description[4];
		int   pt;
		char  *name;
	} qt_pt [] = {
		/* aoudio only, all video goes to JPEG for now */
		{"raw ",96,"L8"},
		{"ima4",RTP_PT_DVI4_22,"DVI4"},
		{"twos",96,"L8"},
		{"MAC3",96,"MAC3"},
		{"MAC6",96,"MAC6"},
		{"mlaw",RTP_PT_PCMU,"PCMU"}
	};
/*	printf("SUPPLIED PATH: %s\n",pathp); */
	qt = calloc(1,sizeof(QT_t));
	qt->tracks_array = calloc(1,sizeof(track_t));
	qt->number_tracks = 0;
	
	if (rw == O_RDWR) mode = "r+";
	else if (rw == O_RDONLY) mode = "r";
	else if (rw == O_WRONLY) mode = "w";
	

	path = pathp;
        if ( strcmp (param,"")){
	  track = strtok(param,"?");
          flag = 1;
          track_id = atoi(track);
	}
	
	if ((f = fopen(path, mode)) ==NULL) {
	  free(qt);
	  printf("Error opening %s file \n QUITTING...\n",path);   
	  return NULL;
	}

	strcpy(qt->path,path);
	qt->f = f;
	qt->track = -1;
	

	while(!feof(f)){
      
	  temp = readl32(f);
	  temporary = temp;
	  if(feof(f)){
	    break;
	  }
	  if (fread(magic, 1, 4, f) != 4)  {
	    return NULL;
	  }
	  
	  if (!strncmp(magic,"moov",4)) {
	    while (temporary != 8) {
		  temp = readl32(f);
		  temp7 = temp;
		  temporary = temporary - temp;
		  if(feof(f)){
		    break;
		  }
		  if (fread(magic, 1, 4, f) != 4)  {
		   return NULL;
		  }
		
		  if (!strncmp(magic,"mvhd",4)) {
		    
			/* skip creation and modification time */	
		    fseek(f,12,SEEK_CUR);
		    /* time scale */
		    qt->scale = readl32(f);
			/* duration */
		    qt->duration = readl32(f);;
			/* preferred rate (1.0 - normal) */
   		    qt->rate = readf32(f)/(float)4096.0;
		    /* preferred volume (1.0 - full) */
		    qt->volume = readf16(f)/(double)512.0; 
		   	/* skip the rest of the atom */	
			fseek(f,74,SEEK_CUR);
		    continue;
		  }     
		 
		  if (!strncmp(magic,"trak",4)) {
			/* increment track count */ 
		         number_tracks++;
			
			qt->number_tracks++;
			
			/* realloc if not the first track */
			if (number_tracks == (qt->number_tracks - 1) && number_tracks != 0){
			 
			  qt->tracks_array = realloc(qt->tracks_array,qt->number_tracks*sizeof(track_t));
                         }
			/* read contained atoms */ 
     	                 for (i = 0;i<9;i++) { 
			  temp = readl32(f);
			  temp7 = temp7 - temp;
			  size = temp;
	      
			  if (fread(magic, 1, 4, f) != 4)  {
			    return NULL;
			  }
			  if (!strncmp(magic,"mdia",4)) {
                            /* read contained atoms */
			    for (j = 0;j<4;j++) { 
				  temp = readl32(f);
				  temp1 = temp;
				  size = size - temp;
				  if (fread(magic, 1, 4, f) != 4)  {
				    return NULL;
				  }
				  if(!strncmp(magic,"mdhd",4)){

                                    fseek(f,12,SEEK_CUR);
				    /* track's time scale */
				    qt->tracks_array[number_tracks].Scale = readl32(f);
				    /* track's duration */
				    qt->tracks_array[number_tracks].Duration = readl32(f);
				    fseek(f,4,SEEK_CUR);
				  
				  } else
				  if(!strncmp(magic,"minf",4)) {
				    for (k = 0;k<4;k++) {
		
			             temp = readl32(f);
				     temp2 = temp;
				     temp1 = temp1 - temp;
				     if (fread(magic, 1, 4, f) != 4)  {
				       return NULL;
				     }
				     if(!strncmp(magic,"vmhd",4)){
					   strcpy(qt->tracks_array[number_tracks].type,"video");
					   fseek(f,temp-8,SEEK_CUR);
				     }
				     else if (!strncmp(magic,"smhd",4)) {
				       
				       strcpy(qt->tracks_array[number_tracks].type,"audio");
					   fseek(f,temp-8,SEEK_CUR);
				     }else
				     if (!strncmp(magic,"stbl",4)) {
					  for(n=0;n<7;n++) {
					     temp = readl32(f);
					     temp2 = temp2 - temp;
					     if (fread(magic, 1, 4, f) != 4)  {
					       return NULL;
					     }
					     if(!strncmp(magic,"stsd",4)){
					       fseek(f,12,SEEK_CUR); 
					       if(fread(qt->tracks_array[number_tracks].format,1,4,f)!=4){ 
  						  return NULL; 
					       } 
					       if(temp == 102){
						 fseek(f,16,SEEK_CUR);
					       qt->tracks_array[number_tracks].temporal_qualityV = readl32(f);
						qt->tracks_array[number_tracks].spatial_qualityV = readl32(f);
						 fseek(f,16,SEEK_CUR);
						 qt->tracks_array[number_tracks].frame_countV = readl16(f);
						 fseek(f,temp - 66,SEEK_CUR);
						
					       }
					       else if (temp == 52) {
						 fseek(f,16,SEEK_CUR);
						 qt->tracks_array[number_tracks].channelsA = readl16(f);
						 qt->tracks_array[number_tracks].sample_sizeA = readl16(f);
						 fseek(f,temp - 44,SEEK_CUR);
						
					       }
					       else
					       fseek(f,temp - 24,SEEK_CUR); 
					     }
					     else  
					     if(!strncmp(magic,"stts",4)){
						   /* populate Samples_per_Chunk array */
					       fseek(f,4,SEEK_CUR);
						   /* Number of entries */
					       temp4 = readl32(f);
					       qt->tracks_array[number_tracks].d = temp4;
						   z = (int) temp4;
						  
					       qt->tracks_array[number_tracks].Time_to_Sample = calloc(
						     1, sizeof(time_sample_t) * z); 

					       for(x = 0;x<z;x++){
						  /* first chunk */
					          temp4 = readl32(f);
					          qt->tracks_array[number_tracks].Time_to_Sample[x].count = temp4;
						  /* sampels per chunk */
					          qt->tracks_array[number_tracks].Time_to_Sample[x].duration =readl32(f);
						
					       }
					       if (!strncmp(qt->tracks_array[number_tracks].type,"video",5)) {
						 qt->frames_per_sec = (double) qt->tracks_array[number_tracks].Scale / qt->tracks_array[number_tracks].Time_to_Sample[0].duration;}
					     }
					      else  
					     if(!strncmp(magic,"stsc",4)){
						   /* populate Samples_per_Chunk array */
					       fseek(f,4,SEEK_CUR);
						   /* Number of entries */
					       temp4 = readl32(f);
					       qt->tracks_array[number_tracks].a = temp4;
						   z = (int) temp4;
					       qt->tracks_array[number_tracks].Samples_per_Chunk = calloc(
						    1, sizeof(info_t) * z); 

					       for(x = 0;x<z;x++){
						  /* first chunk */
					          qt->tracks_array[number_tracks].Samples_per_Chunk[x].a = readl32(f);
						  /* sampels per chunk */
					          qt->tracks_array[number_tracks].Samples_per_Chunk[x].b = readl32(f);
						   /* sample id */
						  qt->tracks_array[number_tracks].Samples_per_Chunk[x].c = readl32(f);
					       }
					     }
					     else  
					     if (!strncmp(magic,"stsz",4)) {
					       fseek(f,4,SEEK_CUR);
					       /* sample size */
					       sample_size = readl32(f);
					       /* number of entries */
					       qt->tracks_array[number_tracks].c = readl32(f);
					       z = (int) qt->tracks_array[number_tracks].c;
					       if(sample_size == 0){
					         qt->tracks_array[number_tracks].Sample_Size = 
						   calloc(1,(sizeof(u_int32)*z));
						 for(x = 0;x<z;x++){
						    qt->tracks_array[number_tracks].Sample_Size[x] = readl32(f);
						 }
					       }
					       else{
						 qt->tracks_array[number_tracks].c = 1;
						 qt->tracks_array[number_tracks].Sample_Size =
						   calloc(1,(sizeof(u_int32)*1));  
						  qt->tracks_array[number_tracks].Sample_Size[0] = sample_size;
					       }
					     }
					     else  
					     if(!strncmp(magic,"stco",4)){
					       fseek(f,4,SEEK_CUR);
					       /* number of entries */
					       temp4 = readl32(f);
					       z = (int)temp4;
					       qt->tracks_array[number_tracks].b = temp4;
					       qt->tracks_array[number_tracks].Chunk_Offset = 
						 calloc(1,(sizeof(u_int32)*z));
                                               for (x = 0;x<z;x++) {
					         qt->tracks_array[number_tracks].Chunk_Offset[x] = readl32(f); 
					       }
					     }
					     else
					       fseek(f,(temp - 8),SEEK_CUR);
					     if (temp2 == 8) {
						   break;
					     }
					   }    
				     }
				     else
				       fseek(f,(temp - 8),SEEK_CUR);
				     if (temp1 == 8) {
				       break;
				     }
				    }
				  }		 
				  else
				    fseek(f,(temp - 8),SEEK_CUR);
				  if (!strncmp(magic,"udta",4)) {	 
				    break;
				  }
				  if(size == 8) break;
			    }
			  }
			  else 
			  if (!strncmp(magic,"tkhd",4)) {
			    fseek(f,12,SEEK_CUR);
			    /* track id */
        	            temp4 = readl32(f);
			    if (flag == 1 && temp4 == track_id) {
			      qt->track = number_tracks;
			     
			    }
			    qt->tracks_array[number_tracks].TrackID = temp4;
                            fseek(f,60,SEEK_CUR);
			    /* width */
			    qt->tracks_array[number_tracks].Width = (readf32(f)/(float)4096.0);
			    /* height */
			    qt->tracks_array[number_tracks].Height =(readf32(f)/(float)4096.0);
			  }
			  else
			    fseek(f,(temp - 8),SEEK_CUR);
			    
			  if (!strncmp(magic,"udta",4) || temp7 == 8) {	
			    /* compute chunks and samples here */
			    j = 0 ;
			    if (qt->tracks_array[number_tracks].c == 1 && 
				qt->tracks_array[number_tracks].d ==1 ){
			      qt->tracks_array[number_tracks].samples = (int)qt->tracks_array[number_tracks].Duration / qt->tracks_array[number_tracks].Time_to_Sample[0].duration;
			    } else {
			      qt->tracks_array[number_tracks].samples = qt->tracks_array[number_tracks].c;
			    }
			    qt->tracks_array[number_tracks].chunks  =qt->tracks_array[number_tracks].b;
			    qt->tracks_array[number_tracks].size_seek = 0;
		        /* determine the payload type */
				qt->tracks_array[number_tracks].pt = -1;
				if (!strcmp(qt->tracks_array[number_tracks].type,"video")) {
					qt->tracks_array[number_tracks].pt = 26;
					strcpy(qt->tracks_array[number_tracks].name,"JPEG");
				} else if (!strcmp(qt->tracks_array[number_tracks].type,"audio")) {
					for (i = 0; i < 6;i++) {
						if (!strcmp(qt->tracks_array[number_tracks].format,qt_pt[i].description)) {
							qt->tracks_array[number_tracks].pt = qt_pt[i].pt;
							strcpy(qt->tracks_array[number_tracks].name,qt_pt[i].name);
							break;
						}
					}
				}

				break;
			  }
			 }
	      
		  }
		  else {
		    fseek(f,(temp - 8),SEEK_CUR); 
		  }
	    }
	  }
	  else {
	    fseek(f,(temp - 8),SEEK_CUR); 
	  }
   }/*end while loop*/
 
  return qt;

}/*end QTOpen */

