/*
* RecordService()
* Receive RTP packets from network and record to disk.
*/

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "sysdep.h"
#include <pthread.h>
#include "rtspd.h"
#include "snd.h"
#include "mime.h"
#include <string.h>  /* for bzero() called from FD_ZERO() */

void RecordService(Stream *st, request_t *r)
{
  double npt;                  /* normal play time (relative) */
  struct timespec next;
  struct timeval  tv;
  char buffer[8192];
  char *ptr;
  int len;
  rtp_hdr_t *rtp_hdr;
  fd_set readfds;
  struct timeval now;
  int e = 0;
  unsigned long maxsize = -1;
  unsigned long currentsize = 0;

  /* Loop for packet receiving. */
  npt = 0.0;

  debug("RecordService","Entering\n");

  gettimeofday(&now, NULL);
  r->range.effective = now.tv_sec + now.tv_usec / 1e6;
  
  maxsize = RTSP_MaxSize(&r->p);
  debug("RecordService", "Max size is %lu\n", maxsize);

  do {

    next.tv_sec  = (time_t)(r->range.effective + npt);
    next.tv_nsec = (long)(((r->range.effective + npt) - next.tv_sec) * 1e9);

    //debug("RecordService","timedwait(%g) -- %d\n",npt, st->state);
    e = pthread_cond_timedwait(&st->queue_change, &st->lock, &next);
    //debug("RecordService","timedwait exiting\n");

    if (st->state == S_PAUSE || st->state == S_TEARDOWN) {
      debug("RecordService","PAUSE/TEARDOWN");
      break;
    }    

    /* Read RTCP also XXX using NON blocking mode. */
    tv.tv_sec = 0; tv.tv_usec = 0;
    FD_ZERO(&readfds);
    FD_SET(st->rtp.socket[0],&readfds);
    if (select(st->rtp.socket[0]+1, &readfds, NULL, NULL, &tv) > 0) {
      /* if FD_ISSET(st->rtp.socket[0], &readfds) XXX */
      len = recvfrom(st->rtp.socket[0], buffer, sizeof(buffer), 0, NULL,NULL);


      rtp_hdr = (rtp_hdr_t *) buffer;
      ptr = buffer;
      len -= 12;
      ptr += 12;

      if ( currentsize < maxsize ) {
        currentsize += len;
        (*st->write)( 
                     st->fp, 
                     rtp_hdr->pt, 
                     ntohl(rtp_hdr->ts), 
                     ntohs(rtp_hdr->seq), 
                     ptr, len, &npt);
      }
      else {
        // debug("RecordService", "Max size is reached %lu\n", currentsize);
      }
    }

    npt += 160.0/8000.0; // XXX 

  } while (npt < r->range.end || r->range.end < 0);

  debug("RecordService","Exiting\n");

  //fclose(wfile); 
} /* RecordService */

