#ifndef REQUEST_H
#define REQUEST_H

#include <limits.h>
#include "rfc822.h"
#include "mime.h"
#include "http.h"
#include "rtsp.h"
#include "uri.h"
#include "dstring.h"
#include "request-common.h"

/* List of pending requests.
 *
 * NOTE: The top part of this structure (until "COMMON ENDS HERE") is a
 * shared structure defined in libcine/request-common.h as request_common_t.
 * DO NOT CHANGE THE TOP PART OF THIS STRUCTURE WITHOUT CHANGING THAT ONE AS
 * WELL! 
 */
struct request_t {
  /* Basic request information */
  double time_received;          /* time request was received */

  /* Where is the response going to? */
  int type;                      /* SOCK_DGRAM, SOCK_STREAM */
  int socket;                    /* socket id for response */
  struct sockaddr_in source_sin; /* socket address request came from */
  struct sockaddr_in sin;        /* socket address for response */

  /* Information about the request */
  parser_t p;                    /* header for this request */
  body_t request_body;           /* The body of the request; */

  http_authorization_t authorization;  /* Authorization header */

  /* Information about the response */
  int status;
  char *reason;
  headers_t headers;                  /* Headers of response */
  body_t response_body;               /* Body to send with respnse */

  void *server; /* pointer to server object. Not used */

  /* COMMON ENDS HERE */

  char *version;               /* request version */
  long seq;                    /* request sequence number */
  int reference;               /* reference counter */
  char path[PATH_MAX];         /* file name to retrieve */
  rtsp_transport_t transport;  /* transport parameters */
  rtsp_range_t range;          /* range parameters */
  struct Session *session;     /* session this request belongs to */
  struct request_t *next;        /* next request in queue */ 
};

/* Request */
extern const char *GetMethodsAllowed();
extern void RequestProcess(request_t *r);
extern request_t *RequestCreate(int type, int socket, struct sockaddr_in sin);
extern void  RequestFree(request_t *r);
extern request_t *RequestSearch(request_t *r);

/* Response */
extern const char *GetStandardReason(int status);
extern int ResponseSend(request_t *r, int fd);
extern int ResponseBuildDefault(request_t *r);
extern void ResponseSetStatus(request_t *r,
                              int status,
                              const char *reason);
extern void ResponseSetBody(request_t *r, const char *body,
             const char *content_type, int content_length);

#endif
