#ifndef RTSP_H
#define RTSP_H

#ifndef HAVE_BOOLEAN_T
typedef enum { B_FALSE, B_TRUE } boolean_t;
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>  /* inet_addr() */

#include "sysdep.h"

#include "rfc822.h"
#include "http.h"

enum {T_PLAY = 1, T_RECORD = 2}; /* Changed 7/24/99 by AM */

/*
* RTSP header definitions.
*/
typedef struct rtsp_transport {       /* transport options */
  int port[2];
  int client_port[2];
  int server_port[2];
  int interleaved;
  int ssrc;
  int ttl;
  int layers;
  struct in_addr destination;
  boolean_t multicast;
  boolean_t compressed;
  boolean_t append;
  enum {P_RTP} protocol;
  enum {T_UDP,T_TCP} transport;
  enum {Pr_AVP} profile;
  int mode;                            /* PLAY and/or RECORD */
} rtsp_transport_t;

typedef struct rtsp_range {            /* range */
  enum {R_clock, R_npt, R_smpte} type; /* type of range */
  double begin, end;                   /* beginning and ending time */
  double effective;                    /* wallclock time when to play/record */
} rtsp_range_t;

typedef struct rtsp_session {         /* session id string */
  char *id;
  int timeout;
} rtsp_session_t;

typedef struct rtsp_timestamp {       /* range */
  double timestamp;
  double delay;
} rtsp_timestamp_t;

extern int                 RTSP_CSeq          (parser_t *);
extern rtsp_range_t        RTSP_Range         (parser_t *);
extern unsigned long       RTSP_MaxSize       (parser_t *);
extern float               RTSP_Scale         (parser_t *);
extern rtsp_session_t      RTSP_Session       (parser_t *);
extern const char         *RTSP_Subject       (parser_t *);
extern rtsp_timestamp_t    RTSP_Timestamp     (parser_t *);
extern rtsp_transport_t    RTSP_Transport     (parser_t *);

extern double strtoutc  (char *utc);
extern double strtonpt  (char *npt);
extern double strtosmpte(char *smpte);
extern void utctostr(double t, char *buffer);
extern void npttostr(double t, char *buffer);
extern void smptetostr(double smpte, char *buffer);
#endif
