#include "stream.h"
#include "rtspd.h"
#include <pthread.h>

/*
* Teardown()
* Remove stream; implicit PAUSE. If there are no more streams for
* a session, remove that session.
*/
int Teardown(Stream *st, request_t *r)
{
  /* Pause the stream. */
  Pause(st, r);

  pthread_mutex_lock(&st->lock);
  /* Mark stream as being torn down and signal stream thread. */
  st->state = S_TEARDOWN;
  pthread_cond_signal(&st->queue_change);

  debug("Teardown", "Waiting for stream thread %p\n", st);
  pthread_mutex_unlock(&st->lock);

  pthread_join(st->tid, NULL);   /* wait for stream thread to complete */

  pthread_mutex_lock(&st->lock);
  SessionDetach(r->session, st);
  pthread_mutex_unlock(&st->lock);

  StreamFree(st);
  debug("Teardown", "Stream %p freed.\n", st);

  /* If no more streams, eliminate session. */
  if (r->session->streams == NULL) {
    SessionFree(r->session->session_id);
  }

  ResponseSetStatus(r, 200, NULL);
  return 200;
} /* Teardown */
