#! /usr/bin/env perl -w

# Prioritize messages whose 'From:' matches 'sip:hgs@' by proxying them
# with 'Priority: urgent'.

# Translate the REGISTRATIONS env variable into a list of
# registration addresses, without name-addr forms or parameters.
sub get_regs {
    my($reg_str, @regs);

    if (!defined($ENV{REGISTRATIONS})) { return (); }
    
    $reg_str = $ENV{REGISTRATIONS};

    # Kill quoted strings.
    # A quoted string consists of a pair of quotes, surrounding the following:
    # 1. any character other than a backslash or a quote
    # 2. a backslash preceeding any character
    # This handles all "some number of backslashes before the trailing quote"
    # problems.
    $reg_str =~ s/\"([^\"\\]|\\.)*\"//g;

    # Now, all commas are syntaxtically significant, so it is safe to split
    # on them.
    @regs = split(",", $reg_str);

    grep {
        # Eliminate parameters, then strip <> forms.
        s/;.*//;
        if (/\<(.*)\>/) { $_ = $1; }
    } @regs;

    return @regs;
}


if (defined $ENV{SIP_FROM} && $ENV{SIP_FROM} =~ /sip:hgs@/) {
    foreach $reg (get_regs()) {
        print "CGI-PROXY-REQUEST $reg SIP/2.0\n";
        print "Priority: urgent\n\n";
    }
}
