#! /usr/bin/env perl -w

# Force a 'Record-Route' header to be added to proxied calls.  This in
# itself is just useful for demonstration and testing purposes; it doesn't
# actually do anything for the call.

sub parse_nameaddr {
    my($str) = @_;

    # Kill quoted strings.
    # A quoted string consists of a pair of quotes, surrounding the following:
    # 1. any character other than a backslash or a quote
    # 2. a backslash preceeding any character
    # This handles all "some number of backslashes before the trailing quote"
    # problems.
    $str =~ s/\"([^\"\\]|\\.)*\"//g;

    # Now, all commas are syntaxtically significant, so it is safe to split
    # on them.
    @addrs = split(",", $str);

    grep {
        # Eliminate parameters, then strip <> forms.
        s/;.*//;
        if (/\<(.*)\>/) { $_ = $1; }
    } @addrs;

    return @addrs;
}

# Translate the REGISTRATIONS env variable into a list of
# registration addresses, without name-addr forms or parameters.

sub get_regs {
    my($reg_str, @regs);

    if (!defined($ENV{REGISTRATIONS})) { return (); }
    
    return parse_nameaddr($ENV{REGISTRATIONS});
}

sub continue_transaction {
    print "CGI-AGAIN yes SIP/2.0\n\n";
}

if (defined $ENV{REQUEST_METHOD}) {
    # This is a request
    if ($ENV{REQUEST_METHOD} !~ /CANCEL/i || 
        $ENV{REQUEST_METHOD} !~ /ACK/i) {

        # XXX: port?
        $rr_header .= "<" . $ENV{REQUEST_URI} . 
            "; maddr=" . $ENV{SERVER_NAME} . ">";

        # Append previous record-route headers
        if (defined($ENV{SIP_RECORD_ROUTE})) {
            $rr_header .= "," . $ENV{SIP_RECORD_ROUTE};
        }

        foreach $reg (get_regs()) {
            print "CGI-PROXY-REQUEST $reg SIP/2.0\n";
            print "Record-Route: $rr_header\n\n";
        }
    }
}

continue_transaction();

exit(0);

