#! /usr/bin/env perl -w

# Forward to voicemail if the phone is busy, or if there's no answer,
# or if there are no registered addresses.

$voicemail_addr = "sip:lennox-immediate\@ind.cs.columbia.edu";

$delay = 10;

# Translate the REGISTRATIONS env variable, or a SIP_CONTACTS header,
# into a list of registration addresses, without name-addr forms or
# parameters.
sub get_regs($) {
    my($reg_str) = @_;
    my(@regs);

    if (!defined($reg_str)) { return (); }
    
    # Kill quoted strings.
    # A quoted string consists of a pair of quotes, surrounding the following:
    # 1. any character other than a backslash or a quote
    # 2. a backslash preceeding any character
    # This handles all "some number of backslashes before the trailing quote"
    # problems.
    $reg_str =~ s/\"([^\"\\]|\\.)*\"//g;

    # Now, all commas are syntaxtically significant, so it is safe to split
    # on them.
    @regs = split(",", $reg_str);

    grep {
        # Eliminate parameters, then strip <> forms.
        s/;.*//;
        if (/\<(.*)\>/) { $_ = $1; }
    } @regs;

    return @regs;
}

sub send_to_voicemail {
    print "CGI-PROXY-REQUEST $voicemail_addr SIP/2.0\n\n";
}

sub set_cookie {
    print "CGI-SET-COOKIE " . $_[0] . " SIP/2.0\n\n";
}

sub continue_transaction {
    print "CGI-AGAIN yes SIP/2.0\n\n";
}

if (defined $ENV{RESPONSE_STATUS}) {
    # This is a response
    if ($ENV{RESPONSE_STATUS} >= 300) {
        my($branch_count) = $ENV{SCRIPT_COOKIE};
        $branch_count--;
        if ($ENV{RESPONSE_STATUS} < 400 && defined($ENV{SIP_CONTACT})) {
            # 3xx response
            @regs = get_regs($ENV{SIP_CONTACT});

            foreach $reg (@regs) {
                print "CGI-PROXY-REQUEST $reg SIP/2.0\n";
                print "Expires: $delay\n\n";
            }
            $branch_count += $#regs + 1;
        }
        if ($branch_count <= 0) {
            # Failure.  Forward to voicemail.
            send_to_voicemail();
            exit(0);
        }
        else {
            set_cookie($branch_count);
            continue_transaction();
            exit(0);
        }
    }
    elsif ($ENV{RESPONSE_STATUS} >= 200) {
        # Success.  Perform the default action and get out of the way, 
        exit(0);
    }
    else {
        # Trying.  Perform the default action but stay in the loop.
        continue_transaction();
        exit(0);
    }
}
elsif (defined $ENV{REQUEST_METHOD}) {
    # This is a request
    if ($ENV{REQUEST_METHOD} =~ /CANCEL/i || 
        $ENV{REQUEST_METHOD} =~ /ACK/i) {
        exit(0);
    }

    if (!defined($ENV{REGISTRATIONS})) {
        send_to_voicemail();
    }
    else {
        @regs = get_regs($ENV{REGISTRATIONS});
        foreach $reg (@regs) {
            print "CGI-PROXY-REQUEST $reg SIP/2.0\n";
            print "Expires: $delay\n\n";
        }
        set_cookie($#regs + 1);
        continue_transaction();
        exit(0);
    }
}
else {
    # Shouldn't get here
    exit(1);
}

exit(0);
