# Create a new CINEMA MySQL database.
# Note: if you edit this script, also edit the 'altersip.sql' script to reflect
# your changes.

create database sip;
use sip;

create table put (
  user varchar(255) primary key,
  hash_value varchar(255) not null,
  realm varchar(150) not null,
  sip_groups varchar(100),
  auth enum('required','request','never') default 'never',
  algorithm varchar(20) not null,
  sip_methods varchar(100) default 'any',
  remote_user text,
  busy varchar(255),
  noresponse varchar(255),
  message_template text,
  um_timeout smallint(5) unsigned,
  max_msgsize_kb int(10) unsigned not null default 200,
  last_modified timestamp(14),
  gwclass varchar(100),
  email text
);

create table contacts (
  user text not null,
  contact text not null,
  expires datetime default '9999-12-31 23:59:59',
  q float(10,2),
  action enum('Proxy','Redirect'),
  last_modified timestamp(14),
  display_name varchar(100),
  sip_methods varchar(100) default 'any',
  primary key (user (250),contact (250))
);

create table aliases (
  alias varchar(255) primary key,
  primary_user varchar(255) not null,
  last_modified timestamp(14)
);

create table subscription (
  sub_to varchar(255) not null,
  sub_from varchar(255) not null,
  expires datetime,
  contact varchar(255),
  call_id int(15) unsigned not null,
  record_route text,
  last_modified timestamp(14),
  key(sub_to)
);

create table gwclass (
  gwclass varchar(100) primary key not null
);

create table requestlog (
  id integer(11) auto_increment primary key,
  request_time datetime, 
  method varchar(100),
  sip_status int(3) unsigned default 0,
  request_url varchar(255), 
  sip_callid varchar(255),
  sip_from varchar(255),
  sip_to varchar(255),
  period datetime,
  charge decimal(9,3),
  bye integer(11)
);

create table tariff (
  id integer auto_increment primary key,
  valid_from datetime,
  valid_to   datetime,
  rate decimal(9,3),
  daytime_start time,
  daytime_end time,
  prefix varchar(100),
  gwclass varchar(100), 
  increment int(11) unsigned
);

# Populate the gwclass with some starter values
insert into gwclass values ('guest'),('full');

# address book entry, one per person
create table person (
  id integer auto_increment primary key,
  # owner of this record (e.g., foo@domain)
  owner varchar(100),
  # last modified
  last_modified timestamp,
  # Dr., Mr., etc.
  title varchar(30),
  # first (given) name
  givenname varchar(50),
  # middle name
  middlename varchar(50),
  # last name
  surname varchar(50),
  nickname varchar(50),
  spouse varchar(50),
  organization varchar(50),
  department varchar(50),
  # job title (e.g., assistant manager)
  jobtitle varchar(50),
  # URL pointing to photo
  photo varchar(100),  
  # general URL
  url varchar(100),
  # birthday
  birthday date,
  # preferred address class
  preferred varchar(50),
  # SIP URL
  sip varchar(50),
  # email addresses
  email varchar(50),
  email_alt varchar(50),
  # user categories user belongs to
  categories varchar(255),
  # access rights
  #  private: only owner
  #  local:   shared on server
  #  public:  searchable
  access enum('private','local','public') default 'private'
);

# address, possibly several per person
create table address (
  person integer not null,
  addressclass varchar(50) not null,
  companyname varchar(250),
  department varchar(250),
  room varchar(250),
  street varchar(250),
  city varchar(50),
  state enum('', 'AK', 'AL', 'AR', 'AZ',
    'CA', 'CO', 'CT', 'DC', 'DE', 'FL',
    'GA', 'HI', 'IA', 'ID', 'IL', 'IN',
    'KS', 'KY', 'LA',
    'MA', 'MD', 'ME', 'MI', 'MN', 'MO', 'MS', 'MT',
    'NC', 'ND', 'NE', 'NH', 'NJ', 'NM', 'NV', 'NY',
    'OH', 'OK', 'OR',
    'PA', 'RI', 'SC', 'SD', 'TN', 'TX',
    'UT', 'VA', 'VT', 'WA', 'WI', 'WV', 'WY') default '',
  province varchar(50),
  postalcode varchar(20),
  country varchar(50),
  # tzfile timezone names
  timezone varchar(50),
  # geographic location, as LAT LON, in decimal
  geo varchar(50),
  phone varchar(50),
  mobile varchar(50),
  pager varchar(50),
  fax varchar(50),
  primary key (person,addressclass)
);

# groups, one row for each group
create table persongroup (
  id integer auto_increment primary key,
  # name of group
  name varchar(100) not null,
  # owner of group, reference into person table
  owner varchar(100) not null,
  key (owner,name)
);

# one row per group member
create table groupmember (
  # reference to persongroup table
  persongroup integer not null,
  # reference to person table
  person integer not null,
  key (persongroup)
);

# notes for each person
create table personnote (
  last_modified timestamp,
  # person this refers to
  person integer not null,
  # owner of note
  owner integer not null,
  # actual text
  note text,
  key (person)
);

# conference
create table conferences (
  url varchar(255) not null,
  owner varchar(100),
  description text,
  start_time datetime default '0000-00-00 00:00:00',
  end_time datetime default '9999-12-31 00:00:00',
  authentication enum('digest', 'basic', 'none') default 'none',
  conference_type enum('restricted', 'public') default 'public',
  participant_list_type enum('restricted', 'public') default 'public',
  media_allowed varchar(255) default 'audio',
  max_participant_count int(10) unsigned default NULL,
  password varchar(255),
  current_participant_count int(10) unsigned default 0,
  primary key(url)
);

# conference user
create table confusers (
  confurl varchar(255),
  user varchar(255),
  type enum('dialin', 'dialout') default'dialin',
  status enum('notconnected', 'connected') default 'notconnected',
  audio enum('off', 'send', 'receive', 'send-receive') default 'send-receive',
  video enum('off', 'send', 'receive', 'send-receive') default 'send-receive',
  privilege varchar(255) default 'send receive admin',
  key(confurl)
);
