# Columbia CINEMA servers makefile fragment for sip323
# Copyright 1999-2000 by Columbia University; all rights reserved 

sip323_dir := $(topdir)/sip323

SIP323_MODULES := docs

# Include the description for each sub-module
include $(patsubst %,$(topsrcdir)/$(sip323_dir)/%/module.mk,$(SIP323_MODULES))

SIP323_SRC := \
	$(sip323_dir)/main.cxx

SIP323_HEADERS := 

SIP323_MISC := \
	$(sip323_dir)/module.mk \
	$(sip323_dir)/Makefile \
	$(sip323_dir)/sip323.dsp

SIP323_MISCDIST := \
	$(sip323_dir)/README \
	$(sip323_dir)/README.win32 \
	$(sip323_dir)/sip323.license

SIP323_BIN := \
	$(sip323_dir)/sip323

SIP323_REQS := need-pwlib need-openh323

# Target for sip323

# sip323liblinks is needed because 
# of recursive makefile. 

sip323 :: sip++
sip323 :: libh323
sip323 :: sip323liblinks
sip323 :: $(SIP323_REQS)
sip323 :: $(SIP323_BIN)

# SIP323_LIBS, starts with -L. so that all the libraries
# are first looked into current directry for symlinks. 

$(sip323_dir)/sip323 ::
	@echo "Building sip323"
	mkdir -p sip323; TOPDIR="`pwd`/sip323"; export TOPDIR; \
	SIP323_LIBS="$(LDFLAGS) $(LIBH323_LIB) $(SIPAPI_LIB) $(LIBSIP_LIB) $(LIBCINE_LIB) $(LIBRESPARSE_LIB) $(LIBDICT_LIB) $(PGP_SDKLIBS) $(LDAPLIBS) $(DBLIBS) $(RESOLVLIBS) $(CRYPTLIBS) $(NISLIBS) $(SOCKLIBS)"; \
        export SIP323_LIBS; \
	cd $(topsrcdir); \
	IPTELAPI_INC="-I`pwd`/libsip++/"; export IPTELAPI_INC; \
	SIPAPI_INC="-I`pwd`/libsip++/"; export SIPAPI_INC; \
	H323API_INC="-I`pwd`/libh323/"; export H323API_INC; \
	$(MAKE) -C ./sip323 opt;

cleansip323 ::
	set -e; $(MAKE) -C $(topsrcdir)/sip323 clean;

sip323liblinks ::
	topdirabs=`pwd`; echo ":$$topdirabs:"; \
	cd $(topsrcdir); topsrcdirabs=`pwd`; echo ":$$topsrcdirabs:"; \
	rm -f ./sip323/libsip++.a; \
	rm -f ./sip323/libsip.a; \
	rm -f ./sip323/libcine.a; \
	rm -f ./sip323/libresparse.a; \
	rm -f ./sip323/libdict.a; \
	rm -f ./sip323/libh323.a; \
	ln -ns $$topdirabs/libsip++/libsip++.a ./sip323/libsip++.a; \
	ln -ns $$topdirabs/libsip/libsip.a ./sip323/libsip.a; \
	ln -ns $$topdirabs/libcine/libcine.a ./sip323/libcine.a; \
	ln -ns $$topdirabs/resparse-1.3/libresparse.a ./sip323/libresparse.a; \
	ln -ns $$topdirabs/libh323/libh323.a ./sip323/libh323.a; \
	ln -ns $$topdirabs/libdict/libdict.a ./sip323/libdict.a

.PHONY: sip323

SIP323_DIST := $(SIP323_SRC) $(SIP323_HEADERS) \
	$(SIP323_MISC) $(SIP323_MISCDIST)

SIP323_BINDIST := $(SIP323_BIN) $(SIP323_MISCDIST)

SIP323_BINDIST_WINNT := $(SIP323_MISCDIST) \
		$(sip323_dir)/obj_winnt/PTLibd.dll \
		$(sip323_dir)/obj_winnt/PWLibd.dll \
		$(sip323_dir)/obj_winnt/sip323.exe

# Binary and source distribution.

.PHONY: sip323-bindist sip323-dist

SIP323_BINDIST_NAME := sip323-$(VERSION)-$(DATE)-$(SYSNAME)

sip323-bindist:: $(SIP323)
	$(MKDIST) $(topsrcdir) $(SIP323_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(SIPAPI_BINDIST) \
		$(SIP323_BINDIST) \
		$(SIP323_DOCS_BINDIST)

SIP323_BINDIST_WIN32_NAME := sip323-$(VERSION)-$(DATE)-WinNT

sip323-bindist-win32::
	$(MKDIST) $(topsrcdir) $(SIP323_BINDIST_WIN32_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(SIPAPI_BINDIST) \
		$(SIP323_BINDIST_WINNT) \
		$(SIP323_DOCS_BINDIST)

SIP323_DIST_NAME := sip323-$(VERSION)-$(DATE)

sip323-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(SIP323_DIST_NAME) \
		$(TOP_DIST) \
		$(LIBCINE_DIST) \
		$(LIBSIP_DIST)  \
		$(LIBRESPARSE_DIST) \
		$(LIBDICT_DIST) \
		$(DBAPI_DIST) \
		$(SIPAPI_DIST) \
		$(LIBH323_DIST) \
		$(WIN32_DIST) \
		$(SIP323_DIST) \
		$(SIP323_DOCS_DIST)


