# Columbia SIP makefile fragment for sipconf
# Copyright 2000 by Columbia University; all rights reserved 

sipconf_dir := $(topdir)/sipconf

SIPCONF_SRC := \
  $(sipconf_dir)/main.cpp

SIPCONF_HEADERS := 

SIPCONF_MISC := \
  $(sipconf_dir)/module.mk \
  $(sipconf_dir)/sipconf.dsp

SIPCONF_MISCDIST := \
  $(sipconf_dir)/conflist.txt \
  $(sipconf_dir)/sipconf.license \
  $(sipconf_dir)/README

SIPCONF_OBJ := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(SIPCONF_SRC)))
SIPCONF_DEP := \
        $(SIPCONF_OBJ:.o=.d)

# Target to make.
SIPCONF := $(sipconf_dir)/sipconf

# Include files required locally.
SIPCONF_INCS := $(SIPAPI_INC) $(LIBSIP_INC) $(LIBCINE_INC) $(LIBMIXER_INC) \
		$(DBAPI_INC)
SIPCONF_LIBS := $(SIPAPI_LIB) $(LIBSIP_LIB) $(LIBMIXER_LIB) \
        $(LIBCINE_LIB) $(LIBRESPARSE) \
	$(LIBDICT_LIB) $(LDAPLIBS) $(DBLIBS) $(DBAPI_LIB) \
	$(RESOLVLIBS) $(MYSQLLIBS) $(NISLIBS) $(SOCKLIBS)
SIPCONF_DEPENDENCIES := $(SIPAPI) $(LIBMIXER) $(DBAPI)
SIPCONF_REQS :=

# Order of -c and -o matters.
$(SIPCONF_OBJ): %.o: %.cpp
	@echo "compiling $<..."
	@$(CXX) $(SIPCONF_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -o $@ -c $<

$(SIPCONF_DEP): %.d: %.cpp
	@echo "dependencies for $<..."
	@$(DEPEND) -d $(sipconf_dir) -o $@ $(SIPCONF_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# Do not add this --- all: $(SIPCONF)
.PHONY: sipconf
sipconf: $(SIPCONF_REQS) $(SIPCONF_DEP) $(SIPCONF) 

# SIPCONF uses libsip++ == SIPAPI

$(SIPCONF): $(SIPCONF_OBJ) $(SIPCONF_DEPENDENCIES) $(SIPCONF_DEP)
	@echo "creating application $@..."
	$(CXX) -o $@ -w $(SIPCONF_INCS) $(SIPCONF_OBJ) \
	$(LDFLAGS) $(SIPCONF_LIBS) $(LIBS)

MISC      += $(SIPCONF_MISC)
MISCDIST  += $(SIPCONF_MISCDIST)
MISCCLEAN += $(SIPCONF_OTHER_MISCCLEAN)

.PHONY: sipconf-dist

SIPCONF_DIST := $(SIPCONF_SRC) $(SIPCONF_HEADERS) $(SIPCONF_MISC) $(SIPCONF_MISCDIST)
SIPCONF_DIST_NAME := sipconf-$(VERSION)-$(DATE)

sipconf-dist:
	$(MKDIST) $(topsrcdir) $(SIPCONF_DIST_NAME) \
	$(TOP_DIST) \
	$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
	$(LIBDICT_DIST) \
	$(WIN32_DIST) \
	$(DBAPI_DIST) \
	$(SIPAPI_DIST) \
	$(SIPCONF_DIST) \
	$(LIBMIXER_DIST)

SIPCONF_BINDIST := $(SIPCONF_MISCDIST) $(SIPCONF)
SIPCONF_BINDIST_NAME := sipconf-$(VERSION)-$(DATE)-$(SYSNAME)

.PHONY: sipconf-bindist

sipconf-bindist:: $(SIPCONF)
	$(MKDIST) $(topsrcdir) $(SIPCONF_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(SIPAPI_BINDIST) \
		$(SIPCONF_BINDIST) \
		$(SIPCONF_DOCS_BINDIST)


