/*
 * callbacks.c
 * Libsip calls a number of functions for the functionality wrapping it.
 * This file provides the implementation of these functions for sipd.
 */

/* Copyright 1998-2001 Columbia University.  All rights reserved. */

#include <string.h>
#include <sys/types.h>
#include <regex.h>

#include "strdupn.h"
#include "sip.h"
#include "request-sip.h"
#include "libsip.h"
#include "config.h"
#include "canonicalize.h"

#include "method.h"

/*
 * SIP_OnIncomingRequest()
 * Implementation of SIP_OnIncomingRequest() for sipd.
 * Just a wrapper around Method().
 */
void SIP_OnIncomingRequest(request_t *r)
{
  Method(r);
  return;
} /* SIP_OnIncomingRequest */


/* 
 * SIP_OnIncomingAck()
 * Dummy implementation of incoming ACK processing
 * for sipd server. sipd just ignores the unsolicited 
 * incoming ACK with no matching INVITE.
 */
void SIP_OnIncomingAck(request_t *r)
{
  /* Don't do anything */
  pthread_mutex_lock(&r->mutex);
  /* No need to add in the list */
  SIP_OnIncomingRequest(r);
  
  return;
} /* SIP_OnIncomingAck */

/*
 * SIP_OnEndToEndAck()
 * Dummy implementation of the incoming ACK processing
 * when already 200 is been sent. sipd just
 * ignores this ACK
 */
void SIP_OnEndToEndAck(request_t *r)
{
  return;
} /* SIP_OnEndToEndAck */


/*
 * SIP_HandleRequestLocally()
 * Tell libsip whether an incoming request is handled locally or
 * remotely (i.e. whether to use WWW-Authenticate and Require, or 
 * Proxy-Authenticate and Proxy-Require, respectively).
 * For sipd, REGISTER requests whose host part matches the local host
 * pattern, and which are on the local port, are local; everything else is
 * remote. 
 * XXX: OPTIONS requests sent to the registrar address?
 */
int SIP_HandleRequestLocally(char *method, uri_t uri)
{
  if (strcasecmp(method, "REGISTER") != 0) {
    return 0;
  }
  if (regexec(config.domain, uri.host, 0, NULL, 0) != 0) {
    return 0;
  }

  if (!(uri.port == config.port ||
        (config.port == default_sip_port && uri.port == 0))) {
    return 0;
  }

  return 1;
}
