/*
* Read configuration files.
* # comment
* <empty line>
* parameter value value ... \
*   where 'value' can be a quoted string.
*/

#include <stdio.h>
#include <ctype.h>    /* isspace() */
#include "dstring.h"

#define MAX_LINE 1024

/*
* ConfigRead()
* Fill array argv[] (up to *argc entries) with arguments from file 'f'. 
* Allow quoted strings.
* Return number of arguments, 0 on end-of-file.
*/
int ConfigRead(FILE *f, int *argc, char *argv[])
{
  char line[MAX_LINE], *s;
  DString word;
  enum {C_word, C_quoted, C_between} state = C_between;
  int k = 0;

  do {
    if ((s = fgets(line, sizeof(line), f))) {
      if (*s == '#' || *s == '\n' || *s == '\r') continue;
      DStringInit(&word);
      for (s = line; *s; s++) {
        switch(state) {
        case C_between:
          if (*s == '"') {
            state = C_quoted;
          }
          else if (!isspace((unsigned char)*s)) {
            s--;
            state = C_word;
          }
          break;

        case C_quoted:
          if (*s == '"') {
            state = C_between;
            argv[k++] = DStringValue(&word);
            DStringInit(&word);
          }
          else {
            if (*s == '\\') s++;
            DStringAppend(&word, s, 1);  
          }
          break;

        case C_word:
          if (isspace((unsigned char)*s)) {
            if (k < *argc) argv[k++] = DStringValue(&word);
            DStringInit(&word);
            state = C_between;
          }
          else DStringAppend(&word, s, 1);
          break;
        }
      } /* for */
      if (k < *argc && DStringLength(&word)) argv[k++] = DStringValue(&word);
      if (k > 0) break;
    } /* fgets */
  } while (s);
  *argc = k;
  return *argc;
} /* ConfigRead */
