/*
* enginepgp.h
* 
* Written-by: Triantafyllos Alexiou
*             ta154@columbia.edu t.alexiou@ieee.org alexiou@hyper.gr
* December 1999
*/


#ifndef __ENGINEPGP_H
#define __ENGINEPGP_H

#define _Pgp_ERRSTRING_MAXBUF_ 1000
#include "pgpUtilities.h"
#include "pgpErrors.h"
#include "pgpEncode.h"
#include "pgpPubTypes.h"
#include "pgpKeys.h"
#include "pgpFeatures.h"
#include <stdio.h>

extern void InitializePGPsdk(void);

extern int gPgpDebugFlag;
extern FILE*  gPgpReportFile;
extern PGPContextRef gPgpContext;
extern PGPKeySetRef gPgpKeySet, gPgpSecKeySet, gPgpPubKeySet;

extern PGPError 
EnginePgpInit(char *secRingFullPathName,
	      char *pubRingFullPathName, int debugFlag);


extern PGPError EnginePgpDeInit();


extern PGPHashAlgorithm EnginePgpReturnHashAlgID (char *theAlgName);


extern PGPError
EnginePgpOSignWithPvtKey(PGPContextRef thePgpContext,
			 PGPKeySetRef theKeySet, 
			 char* theText, PGPSize theTextSize,
			 char **theSignatureRef,
			 PGPSize *theSgnSizePtr,
			 char* theSigner, char *thePassPhrase,
			 PGPKeyRef theSignKey,
			 PGPHashAlgorithm theHashAlgID,
			 PGPBoolean theIsArmored
			 );


extern PGPError 
EnginePgpOVerifySignatureWithPubKey(PGPContextRef thePgpContext,
				    PGPKeySetRef theKeySet, 
				    char* theText, PGPSize theTextSize,
				    char *theSignature,
				    PGPSize theSgnSize,
				    char* theSigner,
				    PGPKeyRef theSignKey,
				    PGPHashAlgorithm theHashAlgID,
				    PGPBoolean theIsArmored);


extern char* 
EnginePgpPrepareOneLineArmoredSignature (PGPContextRef thePgpContext,
						char* theInputSgn, 
						PGPSize theInputSgnSize);

extern PGPError
EnginePgpOEncyptToUserIDs( PGPContextRef thePgpContext,
			   PGPKeySetRef theKeySet, 
			   char* theText, PGPSize theTextSize,
			   char** theCipherTextRef,
			   PGPSize *theCipherTextSizePtr,
			   char** theRecipients,
			   PGPKeySetRef *theRcptsKeySetRefPtr,
			   PGPBoolean theIsArmored );

extern PGPError
EnginePgpODecryptWithPvtKey( PGPContextRef thePgpContext,
			     PGPKeySetRef theKeySet, 
			     char** theTextRef, PGPSize *theTextSizePtr,
			     char* theCipherText,
			     PGPSize theCipherTextSize,
			     char* theRecipient,  char *thePassPhrase,
			     PGPKeySetRef *theRcptsKeySetRefPtr,
			     PGPBoolean theIsArmored
			     );

extern PGPError 
EnginePgpPrintEvent(FILE *theRptFile, PGPEvent *theEvent);

extern 
PGPError EnginePgpSearchForUserKey(PGPContextRef thePgpContext,
				   PGPKeySetRef theKeySet,
				   char*  usrID, PGPKeyRef *theFoundKey);

extern void 
EnginePgpExtractNameAndEmail(PGPContextRef thePgpContext,
			     char *theSigner, char** theName, char** theEmail);


extern PGPError
EnginePgpGenerateKey(PGPContextRef thePgpContext, 
		     PGPKeySetRef theKeySet,
		     void  *theKey,
		     char *theName, PGPSize theNameSize,
		     PGPKeyRef theMasterKey,
		     PGPBoolean theDoFastGen,
		     char *thePassphrase,
		     PGPPublicKeyAlgorithm thePubKeyAlg,
		     PGPUInt32 theKeySize,
		     PGPUInt32 theExpirationDays
		     );

#define Pgp_DEBUG(file, msg, err) { \
char *errStr; \
fprintf(file,"enginepgp.c:%s:%d",msg, err); \
errStr=malloc( sizeof(char) * _Pgp_ERRSTRING_MAXBUF_ ); \
if (errStr) { \
PGPGetErrorString( err, _Pgp_ERRSTRING_MAXBUF_ , errStr); \
fprintf(file,":%s",errStr); \
free(errStr); \
} \
fprintf(file,"\n"); \
fflush(file); \
}


extern PGPError 
EnginePgpSearchForUserKey(PGPContextRef thePgpContext, PGPKeySetRef theKeySet,
			  char* const usrID, PGPKeyRef *theFoundKey);


extern PGPError 
EnginePgpGenerateKey(PGPContextRef thePgpContext, 
		     PGPKeySetRef theKeySet,
		     void  *theKey,
		     char *theName,PGPSize theNameSize,
		     PGPKeyRef theMasterKey,
		     PGPBoolean theDoFastGen,
		     char *thePassphrase,
		     PGPPublicKeyAlgorithm thePubKeyAlg,
		     PGPUInt32 theKeySize,
		     PGPUInt32 theExpirationDays
		     );

#endif

