# Columbia SIP server makefile fragment for sipd
# Copyright 1998-2001 by Columbia University; all rights reserved 

sipd_dir := $(topdir)/sipd

SIPD_MODULES := test

# Include the description for each sub-module
include $(patsubst %,$(topsrcdir)/$(sipd_dir)/%/module.mk,$(SIPD_MODULES))


SIPD_BIN := \
  $(sipd_dir)/sipd

SIPD_SRC := \
  $(sipd_dir)/callbacks.c \
  $(sipd_dir)/cgi.c \
  $(sipd_dir)/config.c \
  $(sipd_dir)/expire.c \
  $(sipd_dir)/gateways.c \
  $(sipd_dir)/proxy.c \
  $(sipd_dir)/redirect.c \
  $(sipd_dir)/route.c \
  $(sipd_dir)/method.c \
  $(sipd_dir)/register.c \
  $(sipd_dir)/script.c \
  $(sipd_dir)/sipd.c \
  $(sipd_dir)/sipd-config.c \
  $(sipd_dir)/sipd-support.c \
  $(sipd_dir)/telmap.c  \
  $(sipd_dir)/presence.c \
  $(sipd_dir)/log.c 

ifeq ($(USE_PGP_SDK),yes)
SIPD_SRC += \
  $(sipd_dir)/enginepgp.c
endif

SIPD_MAYBESRC := \
  $(sipd_dir)/enginepgp.c

SIPD_HEADERS := \
  $(sipd_dir)/cgi.h \
  $(sipd_dir)/enginepgp.h \
  $(sipd_dir)/gateways.h \
  $(sipd_dir)/method.h \
  $(sipd_dir)/proxy.h \
  $(sipd_dir)/register.h \
  $(sipd_dir)/script.h \
  $(sipd_dir)/sipd-config.h \
  $(sipd_dir)/sipd-policy.h \
  $(sipd_dir)/sipd-support.h \
  $(sipd_dir)/sipd.h \
  $(sipd_dir)/telmap.h \
  $(sipd_dir)/presence.h \
  $(sipd_dir)/log.h

SIPD_MISC := \
  $(sipd_dir)/README.pgpsdk \
  $(sipd_dir)/module.mk \
  $(sipd_dir)/sipd.dsp \
  $(sipd_dir)/sipd.dsw

SIPD_MISCDIST := \
  $(sipd_dir)/README \
  $(sipd_dir)/README.win32 \
  $(sipd_dir)/TODO \
  $(sipd_dir)/BUGS \
  $(sipd_dir)/sipd.conf \
  $(sipd_dir)/sipd.conf_win32 \
  $(sipd_dir)/sipd.license \
  $(sipd_dir)/start-sipd \
  $(sipd_dir)/sipd.conf.win32_template \
  $(sipd_dir)/gateways.sample \
  $(sipd_dir)/files/passwd \
  $(sipd_dir)/logs/placeholder \
  $(sipd_dir)/scripts/placeholder

SIPD_NEEDED_LIBRARIES_LIST := \
	$(sipd_dir)/sipd-needed-shared-libraries

SIPD_NEEDED_LIBRARIES_SOURCES := \
	$(sipd_dir)/sipd-needed-shared-libraries-sources

SIPD_OBJ := \
	$(patsubst %.c,%.o,$(filter %.c,$(SIPD_SRC)))

SIPD_DEP := \
	$(SIPD_OBJ:.o=.d)

SIPD_MISCCLEAN := \
	$(SIPD_NEEDED_LIBRARIES_LIST)

SIPD_INCS := $(LIBSIP_INC) $(LIBCINE_INC) $(LIBRESPARSE_INC) $(LIBDICT_INC) \
	$(DBAPI_INC) $(LIBSIPSNMP_INC)
SIPD_LIBS := $(DBAPI_LIB) $(LIBSIP_LIB) $(LIBCINE_LIB) \
	$(LIBRESPARSE_LIB) $(LIBDICT_LIB) \
	$(PGP_SDKLIBS) $(MYSQLLIBS) $(ZLIBLIBS) $(LIBSIPSNMP_LIB) \
	$(RESOLVLIBS) $(CRYPTLIBS) $(NISLIBS) $(SOCKLIBS) $(MATHLIBS) \
	$(ENUMLIBS) $(CANONLIBS)

SIPD_DEPENDENCIES := $(LIBSIP) $(LIBCINE) $(LIBRESPARSE) $(LIBDICT) $(DBAPI) $(LIBSIPSNMP) $(canonicalize_dir)/libcanon.so

SIPD_REQS := need-mysql

$(SIPD_OBJ): %.o: %.c
	@echo "compiling $<..."
	$(CC) $(SIPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(SIPD_DEP): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(sipd_dir) -o $@ $(SIPD_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

$(sipd_dir)/sipd: $(SIPD_OBJ) $(SIPD_DEPENDENCIES)
	@echo "linking $@..."
	$(CXX) -o $@ $(SIPD_OBJ) $(LDFLAGS) $(SIPD_LIBS) $(LIBS)

$(SIPD_NEEDED_LIBRARIES_SOURCES): $(sipd_dir)/sipd
	@echo "Building list of non-system shared libraries..."
	rm -f $(SIPD_NEEDED_LIBRARIES_SOURCES)
	$(LDD) $(sipd_dir)/sipd | \
		sed -n -e '/not found/ s/^/XXX SET LD_LIBRARY_PATH: /p;' \
			-e '/=>/ s|^.* \([^ ]*/[^ ]*\).*$$|\1|p' | \
			egrep -v '^(/usr)?/lib/' \
			> $(SIPD_NEEDED_LIBRARIES_SOURCES); \
			true
	if grep XXX $(SIPD_NEEDED_LIBRARIES_SOURCES) > /dev/null; \
	then \
		echo "Missing shared library; can't continue"; \
		cat $(SIPD_NEEDED_LIBRARIES_SOURCES); \
		rm -f $(SIPD_NEEDED_LIBRARIES_SOURCES); \
		exit 1; \
	fi

$(SIPD_NEEDED_LIBRARIES_LIST): $(SIPD_NEEDED_LIBRARIES_SOURCES)
	@echo "Building local names of non-system shared libraries..."
	rm -f $(SIPD_NEEDED_LIBRARIES_LIST)
	@sipd_needed_libraries=`cat $(SIPD_NEEDED_LIBRARIES_SOURCES)`; \
	for library in $$sipd_needed_libraries; \
	do\
		echo $(sipd_dir)/`basename $$library`; \
	done > $(SIPD_NEEDED_LIBRARIES_LIST)

.PHONY: sipd
all: sipd
sipd: $(SIPD_REQS) $(SIPD_BIN) $(SIPD_TEST_BIN)

BIN       += $(SIPD_BIN)
SRC       += $(SIPD_SRC)
HEADERS   += $(SIPD_HEADERS)
MISCCLEAN += $(SIPD_OTHER_MISCCLEAN)


SIPD_DIST := $(SIPD_SRC) $(SIPD_MAYBESRC) $(SIPD_HEADERS) \
	$(SIPD_MISC) $(SIPD_MISCDIST)

SIPD_BINDIST := $(SIPD_BIN) $(SIPD_MISCDIST)

SIPD_MISC_DLLS := \
  $(sipd_dir)/libmySQL.dll \
  $(sipd_dir)/cygwinb19.dll

SIPD_MSVC_DLLS  := 

SIPD_BIN_WIN32 := \
  $(sipd_dir)/Debug/sipd.exe \
  $(sipd_dir)/Release/sipd.exe

SIPD_BINDIST_WIN32 := $(SIPD_BIN_WIN32) $(SIPD_MISCDIST) \
		      $(SIPD_MISC_DLLS) $(SIPD_MSVC_DLLS)

.PHONY: sipd-dist sipd-bindist sipd-bindist-win32 copy-sipd-shared-libraries

copy-sipd-shared-libraries: $(SIPD_NEEDED_LIBRARIES_SOURCES)
	@echo "Copying non-system shared libraries..."
	@sipd_needed_libraries=`cat $(SIPD_NEEDED_LIBRARIES_SOURCES)`; \
	if test "x$$sipd_needed_libraries" != "x"; \
	then\
		echo cp $$sipd_needed_libraries $(sipd_dir); \
		cp $$sipd_needed_libraries $(sipd_dir); \
	fi


SIPD_DIST_NAME := sipd-$(VERSION)

sipd-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(SIPD_DIST_NAME) \
		$(TOP_DIST) \
		$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
		$(LIBDICT_DIST) $(DBAPI_DIST) \
                $(LIBSIPSNMP_DIST) \
		$(SIPD_DIST) \
		$(TOOLS_DIST) \
		$(SCRIPTS_DIST) \
		$(WIN32_DIST) \
		$(SIPD_TEST_DIST) \
		$(CINEMA_WEB_SIPD_DIST) \
		$(DOC_DIST) \
		$(EXTRA_DIST_FILES)

SIPD_BINDIST_NAME := sipd-$(VERSION)-$(SYSNAME)

sipd-bindist: need-licensing sipd tools copy-sipd-shared-libraries \
			$(SIPD_NEEDED_LIBRARIES_LIST)
	$(MKDIST) $(topsrcdir) $(SIPD_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) $(LIBSIP_BINDIST) $(LIBRESPARSE_BINDIST) \
		$(LIBDICT_BINDIST) \
                $(LIBSIPSNMP_BINDIST) \
		$(SIPD_BINDIST) \
		$(TOOLS_BINDIST) \
		$(SCRIPTS_BINDIST) \
		$(SIPD_TEST_BINDIST) $(SIPD_DOCS_BINDIST) \
		$(CINEMA_WEB_SIPD_BINDIST) \
		$(DOC_BINDIST) \
		`cat $(SIPD_NEEDED_LIBRARIES_LIST)` \
		$(EXTRA_BINDIST_FILES)

SIPD_BINDIST_WIN32_NAME := sipd-$(VERSION)-WinNT

sipd-bindist-win32: need-licensing
	$(MKDIST_WIN32) $(topsrcdir)  $(SIPD_BINDIST_WIN32_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) $(LIBSIP_BINDIST) $(LIBRESPARSE_BINDIST) \
		$(LIBDICT_BINDIST) \
                $(LIBSIPSNMP_BINDIST_WIN32) \
		$(SIPD_BINDIST_WIN32) \
		$(TOOLS_BINDIST_WIN32) \
		$(SCRIPTS_BINDIST) \
		$(WIN32_MISCDIST) \
		$(SIPD_TEST_BINDIST_WIN32) \
		$(CINEMA_WEB_SIPD_BINDIST) \
		$(DOC_BINDIST) \
		$(EXTRA_BINDIST_FILES)
