
#include <stddef.h>
#include <stdlib.h>

#include "sysdep.h" 
#include "config.h"
#include "strdupn.h"

#include "sipd-policy.h"
#include "register.h"


/* Redirect Policy: handle redirections */

/* redirect_init()
 * Called when a request which should be redirected first arrives.
 * This does all the work of the redirection, since redirection is
 * simple request/response.
 */
static int redirect_init(request_t *r, registration_t *reg, int reg_status) {
  int i;
  DString uri_ip, uri_ipnameaddr;
  headers_t delta = {NULL, NULL};

  /* Create Contact headers for response. */
  DStringInit(&uri_ip);
  DStringInit(&uri_ipnameaddr);
    
  for (i = 0; i < reg->contacts && reg->contact[i].uri.scheme; i++) {
    if (reg->contact[i].q >= 0) {
      char *contact_hdr;

      contact_hdr = SIP_BuildContact(&reg->contact[i]);
      
      HeaderAppend(&delta, "Contact", contact_hdr);

      free_valid(contact_hdr);
    }
  }
  policy_send_modified_new_response(r, reg_status, NULL, &delta,
                                    NULL, NULL);
  /* Wait for the timeout period */
  policy_set_timeout(r, 30);

  HeaderFree(&delta);
  DStringFree(&uri_ip);
  DStringFree(&uri_ipnameaddr);
  return 0;
} /* redirect_init */


/* redirect_handle_request()
 * Handle a subsequent request after a redirection.  This shouldn't happen
 * -- we haven't sent a 2xx, so we won't see ACK, and we've sent a final
 * response, so we won't see CANCEL.
 */
static int redirect_handle_request(request_t *r, message_t *m)
{
  error("redirect_handle_request", "Shouldn't be called");
  return 1;
} /* redirect_handle_request */


/* redirect_handle_response()
 * handle a subsequent response after a redirection.  This definitely
 * shouldn't happen, since we don't have any branches.
 */
static int redirect_handle_response(request_t *r, message_t *m, branch_t *b)
{
  error("redirect_handle_request", "Shouldn't be called");
  return 1;
} /* redirect_handle_response */


/* redirect_timeout_expired()
 * Called when the final timeout expires.  When this happens, clean up.
 */
static int redirect_timeout_expired(request_t *r)
{
  policy_done(r);
  return 0;
} /* redirect_timeout_expired */


/* redirect_cleanup()
 * redirect doesn't keep any local state, so do nothing.
 */
static int redirect_cleanup(request_t *r)
{
  return 0;
}


policy_t redirect_policy =
{
  "redirect",
  &redirect_init,
  &redirect_handle_request,
  &redirect_handle_response,
  &redirect_timeout_expired,
  &redirect_cleanup
};
