
//package SipSImplementation;

import org.ietf.sip.*;
import java.util.Iterator;
import java.util.HashMap;

/**
 * The ServletConfig class represents servlet configuration information.
 * The mechanism for specifying configuration parameters and associating
 * them with servlet instances is outside the scope of the SIP Servlet API
 * itself, but this information is made available to servlets using this
 * class.
 */
public class ServletConfigImpl implements ServletConfig {

   private HashMap initParams;
   private SipServletContextImpl servletContext;


   public ServletConfigImpl() {
      initParams = new HashMap();
      servletContext = new SipServletContextImpl();
   }

   public ServletConfigImpl(HashMap inits, String logFile, HashMap attrs) {
      initParams = new HashMap(inits);
      servletContext = null;
      if(attrs!=null)
         servletContext = new SipServletContextImpl(logFile, new HashMap(attrs));
      else
         servletContext = new SipServletContextImpl(logFile, new HashMap());
   }

   /**
    * Returns the value of the named parameter, or null if is undefined.
    */
   public String getInitParameter(String name) {
      return (String) initParams.get(name);
   }

   /**
    * Returns an iterator over all parameters defined for the servlet for
    * which this ServletConfig applies. The iterator returns Strings.
    */
   public Iterator getInitParameterNames(){
      return initParams.keySet().iterator();
   }

   /**
    * Returns the ServletContext associated with the servlet corresponding
    * to this ServletConfig.
    */
   public SipServletContext getServletContext() {
      return servletContext;
   }
}
