
//package SipSImp;

import org.ietf.sip.*;
import java.util.*;
import java.security.AccessControlException;


/**
 * The SipServletContext represents the servlet engine, that is, the SIP stack,
 * to SipServlets. It provides access to a variety of functionality:
 * logging, server attributes, server software information,
 * factory methods for SIP messages, URLs, etc, as well as access to the
 * servers contact database.
 */
public class SipServletContextImpl extends SipFactoryImpl implements SipServletContext {

   protected static int majorVersion = 0;
   protected static int minorVersion = 8;
   protected static String serverInfo = "SIPServlet API 0.8";
   protected HashMap attributes;
   private String logFile;
   private static ContactDatabaseImpl contactDB = new ContactDatabaseImpl();

   public SipServletContextImpl() {
      super();
      attributes = new HashMap();
      logFile = null;
   }

   public SipServletContextImpl(String log, HashMap attr) {
      super();
      if(attr!=null)
         attributes = new HashMap(attr);
      else
         attributes = new HashMap();
      logFile = log;
   }

   /**
    * Returns  the major version number of the SIP Servlet API specification
    * implemented by this  object. For this specification 1 must be returned.
    */
   public int getMajorVersion(){
      return majorVersion;
   }

   /**
    * Returns  the minor version number of the SIP Servlet API specification
    * implemented by this  object. For this specification 0 must be returned.
    */
   public int getMinorVersion(){
      return minorVersion;
   }

   /**
    * Returns  a string identifying the server software, typically
    * including name and version information.
    */
   public String getServerInfo(){
      return serverInfo;
   }

   /**
    * Returns  the value of the specified attribute, or null if it is
    * undefined.
    */
   public Object getAttribute(String name){
      return attributes.get(name);
   }

   /**
    * Returns  an iterator over all defined attributes.
    */
   public Iterator getAttributeNames(){
      return attributes.keySet().iterator();
   }

   /**
    * Associates  the named attribute with the specified value. A set of
    * servlets associated  with the same servlet engine sees
    */
   public void setAttribute(String  name, Object value){
      attributes.put(name, value);
   }

   /**
    * Removes  the specified attribute.
    */
   public void removeAttribute(String name){
      attributes.remove(name);
   }

   /**
    * Causes the  servlet engine to log the specified string. It is up to
    * the server  implementation to decide on an underlying logging
    * mechanism
    */
   public void log(String  msg){
      if(logFile==null)
         return;
      // log written to logFile
   }

   /**
    * Logs  the specified string and exception.
    */
   public void log(String msg, Throwable t){
      if(logFile==null)
         return;
      // log written to logFile
   }


   /**
    * user ContactDatabaseImpl for access to contact database
    * (calls corresponding methods)
    */
   public List getContacts(SipAddress sipAddr) {
      return contactDB.getContacts(sipAddr);
   }

   public void setContacts(SipAddress sipAddr, List contacts) {
      contactDB.setContacts(sipAddr, contacts);
   }

   public void addContact(SipAddress sipAddr, Contact contact) {
      contactDB.addContact(sipAddr, contact);
   }

   public void removeContact(SipAddress sipAddr, Contact contact) {
      contactDB.removeContact(sipAddr, contact);
   }
}
