
/* Note:  */
#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <jni.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <time.h>
#include <string.h>

#include "modified-script.h"
#include "uri.h"
#include "http.h"      // which also contains rfc822.h, uri.h
#include "rfc822.h"
#include "request-sip.h"
#include "message.h"
#include "sip.h"
#include "policy.h"
#include "config.h"
#include "register.h"
#include "sysdep.h"
#include "headers.h"

// sip JVM handling functions
#include "sipJVM.h"

#define THREADS 1

// resembles sipd
extern int debug_flag;
struct config_t config;
/* In RTSP, missing content means zero.  In SIP, it means infinite. */
const int missing_content_length_means_zero = 0;
/* The name of this server */
const char *server_name = "Columbia-Sip-Server";



/* The virtual machine instance
 * Gloabal var. initialized when
 * main creates JVM, used by functions
 * interacting with servlet container
 */
JavaVM *jvm;

typedef struct passObj {
   registration_t *reg;
   request_t *r;
   message_t *m;
   branch_t *b;
} passObj_t;

struct config_t config;

void assignUri(uri_t *u, char *user);
void assignBody(body_t *b);
void assignParser(parser_t *p, uri_t ur);
void assignReq(request_t *r, uri_t ur);
void assignMsg(message_t *m);
void assignReg(registration_t *reg, uri_t u);
void assignBranch(branch_t *br, uri_t ur);


int main() {

    int tid[5];
    int res, i;
    char *tmp;

    JNIEnv *env;

    /**
     * this section assigns some sample values
     * (which are quite often illegal) to structures
     * to perform the test run
     */
    parser_t p;
    body_t b;
    uri_t uri, uri2;
    branch_t br;

    /* temporary */
    passObj_t pass;
    request_t req;
    message_t msg;
    registration_t reg;

    assignUri(&uri, "watson");
    assignUri(&uri2, "bill");
    // msg fields
    assignParser(&(msg.p), uri);
    assignBody(&(msg.body));
    assignMsg(&msg);

    assignBranch(&br, uri);
    assignBody(&(req.request_body));
    assignBody(&(req.response_body));
    assignParser(&(req.p), uri);
    assignReq(&req, uri);
    assignReg(&reg, uri);

    // assgn branch fields
    br.parent = &req;
    br.orig = &msg;
    br.mlist = NULL;
    // back to msg field
    msg.branch = &br;
    // req fields
    req.branches = &br;
    req.branch_count = 1;
    // pass obj
    pass.reg = &reg;
    pass.r = &req;
    pass.m = &msg;
    pass.b = &br;
    /*** end value assignment here */


    /* this is a crucial section which must be put
     into sipd's main routine */

    // main must start JVM where servlet container will run
    if((res = startJVM(&env)) < 0) {
       fprintf(stderr, "Failure when creating Java VM\n"
                       "exiting...\n");
       exit(-1);
    }

    printf("main: JVM started\n");

    if(startServletEngine(env) <= 0) {
       fprintf(stderr, "Could not start servlet engine!\n");
       exit(-1);
    }
    printf("main: Servlet Engine started\n");

    for (i = 0; i < THREADS; i++) {
        /* We pass the thread number to every thread */
        pthread_create(&(tid[i]), NULL, sipsRequestThread, (void *) &pass);
    }

    for (i = 0; i < THREADS; i++) {
       pthread_join(tid[i], NULL);
    }

    destroyJVM(env);
    return 0;
}



void assignBody(body_t *b)
{
   b->content_type = "application/sdp";
   b->content ="\n"
         "v=0\n"
         "o=bell 53655765 2353687637 IN IP4 128.3.4.5\n"
         "s=Mr. Watson, come here.\n"
         "c=IN IP4 kton.bell-tel.com\n"
         "m=audio 3456 RTP/AVP 0 3 4 5\n";
   b->content_length = strlen(b->content);
}

void assignUri(uri_t *ur, char *user)
{
   uri_nv_t *nvp = (uri_nv_t *) malloc(3*sizeof(uri_nv_t));
   //uri_nv_t *nvp;

   ur->scheme = "sip";
   ur->user = user;
   ur->password = NULL;
   ur->host = "boston.bell-tel.com";
   ur->port = 5060;
   ur->path = "";
   ur->param = ";a=1;b=2";
   ur->search = "uri search";

   //nvp = nv[0];
   nvp[0].name = "a";  nvp[0].value = "1";
   nvp[1].name = "b";  nvp[1].value = "2";
   nvp[2].name = NULL;
   ur->parameters = nvp;
   ur->headers = nvp;
}



void assignParser(parser_t *p, uri_t ur)
{
   parser_header_t *ht, *ht2;
   parser_element_t *e1, *e2;
   parser_parameter_t *pr1, *pr2, *pr3;

   ht = (parser_header_t *) malloc(1*sizeof(parser_header_t));

   e1 = (parser_element_t *) malloc(1*sizeof(parser_element_t));
   e2 = (parser_element_t *) malloc(1*sizeof(parser_element_t));
   pr1 = (parser_parameter_t *) malloc(1*sizeof(parser_parameter_t));
   pr2 = (parser_parameter_t *) malloc(1*sizeof(parser_parameter_t));
   pr3 = (parser_parameter_t *) malloc(1*sizeof(parser_parameter_t));

   ht->name = "Call-ID";
   e1->element = "elem1";
   e1->value = "value1";
   e1->comment = "comment1";
   e2->element = "elem2";
   e2->value = "val2";
   e2->comment = "comment2";
   pr1->attribute = "1attr1";
   pr1->value = "1aval1";
   pr2->attribute = "1attr2";
   pr2->value = "1aval2";
   pr1->value = "1aval3";
   pr3->attribute = "1attr3";
   pr3->value = "1aval3";

   pr1->next = pr2;
   pr2->next = NULL;
   pr3->next = NULL;
   e1->parameters = pr1;
   e2->parameters = pr3;
   e1->next = e2;
   e2->next = NULL;

   ht->elements = e1;
   ht->last_element = e2;

   //another
   ht2 = (parser_header_t *) malloc(1*sizeof(parser_header_t));

   e1 = (parser_element_t *) malloc(1*sizeof(parser_element_t));
   e2 = (parser_element_t *) malloc(1*sizeof(parser_element_t));
   pr1 = (parser_parameter_t *) malloc(1*sizeof(parser_parameter_t));
   pr2 = (parser_parameter_t *) malloc(1*sizeof(parser_parameter_t));

   ht2->name = "header2";
   e1->element = "elem21";
   e1->value = "value21";
   e1->comment = "comment21";
   e2->element = "elem22";
   e2->value = "val22";
   e2->comment = "comment22";
   pr1->attribute = "2attr1";
   pr1->value = "2aval1";
   pr2->attribute = "2attr2";
   pr2->value = "2aval2";
   pr1->value = "2aval3";


   pr1->next = pr2;
   pr2->next = NULL;
   e1->parameters = NULL;
   e2->parameters = pr1;
   e1->next = e2;
   e2->next = NULL;

   ht2->elements = e1;
   ht2->last_element = e2;

   ht->next = ht2;
   ht2->next = NULL;

   p->headers = ht;

   // regular setup
   p->type = http_request;
   p->method = "INVITE";
   p->URI_string = "sip:watson@boston.bell-tel.com SIP/2.0";
   p->URI = URI_Copy(ur);
   p->status = 1;
   p->reason = "REASON";
   p->protocol = "SIP";
   p->version = "2.0";
   p->header_body = "some header body goes here";
   p->header_length = -1;
}



void assignReq(request_t *r, uri_t ur)
{
   sip_contact_t *c = (sip_contact_t *) malloc(sizeof(sip_contact_t));
   headers_t *h1;
   struct header_t *h2, *h3, *h4;
   //h1 = (headers_t *) malloc(sizeof(headers_t));
   h2 = (struct header_t *) malloc(sizeof(struct header_t));
   h3 = (struct header_t *) malloc(sizeof(struct header_t));
   h4 = (struct header_t *) malloc(sizeof(struct header_t));

   r->time_received = (double) time(NULL);
   r->type = SOCK_STREAM;
   r->socket = 1;

   //r->authorization
   r->status = 200;
   r->reason = "OK";

   r->server = NULL;
   r->version = "2.0";

   r->source_sin.sin_addr.s_addr = inet_addr("128.59.16.15");
   r->source_sin.sin_port = htons(1234);
   r->sin.sin_addr.s_addr = inet_addr("128.59.16.16");
   r->sin.sin_port = htons(1234);

   strcpy(r->cseq.method, "CSEQ method");
   r->cseq.seq = 1111;
   r->callID = (char *) malloc(strlen(ur.user)+strlen("@localhost")+1);
   strcpy(r->callID, ur.user);
   strcat(r->callID, "@localhost");

   r->to.uri = URI_Copy(ur);
   r->from.uri = URI_Copy(ur);
   r->to.display_name = "\"To Name\"";
   r->from.display_name = "\"From Name\"";

   r->via.type = SOCK_STREAM;
   r->via.sin.sin_addr.s_addr = inet_addr("128.59.16.16");
   r->via.sin.sin_port = htons(1234);
   r->via.received.s_addr = inet_addr("128.59.16.23");
   r->via.hostname = "cnn.com";
   r->via.islocal = 0;
   r->via.branch.str = "tmp_str";
   r->via.branch.id = 33;
   r->via.branch.hash = 100;

   r->expires = 999999;

   r->authorization.http_method = "auth http_method";
   r->authorization.method = "auth method";
   r->authorization.userid = "auth uid";
   r->authorization.userid_mapped = "auth uid mapped";
   r->authorization.password = "auth passwd";
   r->authorization.realm = "auth realm";
   r->authorization.nonce = "auth digest";
   r->authorization.digest_uri = "auth digest";
   r->authorization.response_digest = "authoriz resp";
   r->authorization.entity_digest = "auth digest";
   r->authorization.opaque = "auth opaq";
   r->authorization.algorithm = "authoriza algorith";
   r->authorization.valid = 1; /* flag: header syntax is correct */
   r->authorization.version = "1.3";  /* The pgp version */
   r->authorization.signed_by = "myself"; /* The key id that created the signature */
   r->authorization.signature = "M.R.";  /* The pgp signature */

   r->authenticate.http_method = "authenticate httpmethod";
   r->authenticate.method = "authenticate method";
   r->authenticate.realm = "authenticate realm";
   r->authenticate.domain = "authenticate domain";
   r->authenticate.nonce = "authenticate nonce";
   r->authenticate.opaque = "authenticate opaque";
   r->authenticate.stale = "authenticate stale";
   r->authenticate.algorithm = "authenticate algorithm";

   h2->name="h name1";
   h2->value="h val1";
   h3->name="h name2";
   h3->value="h val2";
   h4->name="h name3";
   h4->value="h val3";
   h2->next = h3;
   h3->next = h4;
   h4->next = NULL;

   r->headers.headers = h2;
   r->headers.last = h4;

   // branches & count set from top
   r->branch_space = 10;
   r->branches_outstanding = 2;

   c->expires = 111;
   c->action = L_unspecified;
   c->q = 6.0;
   c->uri = URI_Copy(ur);
   c->nameaddr = "name address";
   c->comment = "contact-comment";
   r->contact = c;

   r->user_agent = "req-user-agent";
   r->hash = 456;
   r->acked = 0;
   r->sent_final = 0;
   r->sent_200 = 1;
   r->done = -1;

   //policy omitted
   r->timeout.tv_sec = 4000;
   r->timeout.tv_nsec = 7890;

   r->call = NULL;
   r->remote_sin.sin_addr.s_addr = inet_addr("128.59.16.28");
   r->remote_sin.sin_port = htons(1234);
   r->preferred_type = SOCK_DGRAM;

   r->outbound_proxy = URI_Copy(ur);
}


void assignMsg(message_t *m)
{
   m->type = message_real_message;
   m->handled = policy_handled;

   m->source.sin_addr.s_addr = inet_addr("128.59.23.17");
   m->source.sin_port = htons(5060);
   m->policy_info = NULL;
}


void assignBranch(branch_t *br, uri_t ur)
{
   br->branch = 12345;
   br->tid = 11002;

   br->uri = URI_Copy(ur);

   br->mlist = NULL;

   br->sent_200 = 1;
   br->sent_final = 2;
   br->shutdown = 3;
   br->marked_done = 4;
   br->sent_something = 1;
}


void assignReg(registration_t *reg, uri_t ur)
{
   sip_contact_t *c = (sip_contact_t *) malloc(sizeof(sip_contact_t));
   script_t *s1 = (script_t *) malloc(sizeof(script_t));
   //script_t *s1 = (script_t *) malloc(sizeof(script_t));

   c->expires = 333;
   c->action = L_unspecified;
   c->q = 6.0;
   c->uri = URI_Copy(ur);
   c->nameaddr = "reg-name-address";
   c->comment = "reg-contact-comment";
   reg->contact = c;
   reg->contacts = 1;
   reg->update = time(NULL);

   /* sample.java in 'watson' dir used to test */
   s1->disposition = disposition_sip_servlet;
   s1->disp_str = "sip_servlet";
   s1->content_type = "JAVA CLASS";
   s1->filename = "myservlet.class";
   s1->lastmod = time(NULL);

   reg->scriptset.root_dir = "/home/msr52/cinema/sips/test/script";
   reg->scriptset.scripts = s1;
   reg->scriptset.count = 1;
}

