/*
 * script.h: handle registered scripts: add, retrieve, remove
 *
 * Copyright 2000 Columbia University.  All rights reserved.
 */

#ifndef SCRIPT_H_
#define SCRIPT_H_

#include "dstring.h"

typedef enum {
  disposition_invalid = 0,
  disposition_unknown,
  disposition_sip_cgi,
  disposition_cpl,
  disposition_sip_servlet
} script_disposition_t;

typedef struct {
  script_disposition_t disposition;
  char *disp_str;
  char *content_type;
  char *filename;
  time_t lastmod;
} script_t;

typedef struct {
  char *root_dir;
  script_t *scripts;
  int count;
} scriptset_t;

int GetScriptset(const char* base_dir, const char* username, scriptset_t *set);

script_t* GetScriptByDisposition(const scriptset_t *set,
                                 script_disposition_t disp);

void FreeScriptset(scriptset_t *set);

int HasExecutableScript(const scriptset_t *set);

int AddScriptToSet(scriptset_t *set, const char *disp_str,
                   const char *content_type, const char *content, int length);

int DeleteScriptFromSet(scriptset_t *set, const char *disp_str);

int ReadScriptToDString(script_t *script, DString *str);

#endif /* SCRIPT_H_ */
