
/* Note:  */
#include <stdio.h>
#include <unistd.h>
#include <jni.h>

#include "sipJVM.h"

/* Java VM created in main -> global sipd variable*/
extern JavaVM *jvm;


/**
 * Starts JVM
 * instance of JVM is required throughout lifetime
 * of server to maintain servlet engine object
 */
int startJVM(JNIEnv **env)
{
    int i;
    jint res;

    int ret = 0;

    /* starts JVM and set class and library paths */
    #ifdef JNI_VERSION_1_2
        JavaVMInitArgs vm_args;
        JavaVMOption options[2];
        options[0].optionString = "-Djava.library.path=" SIPS_LDPATH;
        options[1].optionString = "-Djava.class.path=" SIPS_CLASSPATH;
        vm_args.version = 0x00010002;
        vm_args.options = options;
        vm_args.nOptions = 2;
        vm_args.ignoreUnrecognized = JNI_TRUE;
    /* Create the Java VM */
        res = JNI_CreateJavaVM(&jvm, (void**)env, &vm_args);
    #else
        JDK1_1InitArgs vm_args;
        char classpath[1024*3];
        vm_args.version = 0x00010001;
        JNI_GetDefaultJavaVMInitArgs(&vm_args);
        /* Append USER_CLASSPATH to the default system class path */
        sprintf(classpath, "%s%s%s", vm_args.classpath, PATH_SEPARATOR, SIPS_CLASSPATH);
        vm_args.classpath = classpath;
    /* Create the Java VM */
        res = JNI_CreateJavaVM(&jvm, env, &vm_args);
    #endif /* JNI_VERSION_1_2 */
    if (res < 0) {
       fprintf(stderr, "Can't create Java VM\n");
       ret = -1;
    }
    else
       ret = 1;

    return ret;
}



/**
 * Functions initiates sip servlet container after the JVM has been
 * started from main routine in sipd.c
 * Assumes JVM is running and is identifiable by global
 * variable: *jvm;
 *
 */
int startServletEngine(JNIEnv *env)
{
    jclass cls;
    jmethodID mid;
    jstring jstr;
    int ret;

    ret = 1;
    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == NULL) {
        ret = -1;
    }
    mid = (*env)->GetStaticMethodID(env, cls, "startUpContainer", "()V");
    if (mid == NULL) {
        ret = -2;
    }
    (*env)->CallStaticVoidMethod(env, cls, mid);

    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        ret = 0;
    }
    return ret;
}



/**
 * destroys instance of JVM and shuts down servlet container
 */
void destroyJVM(JNIEnv *env) {

    jclass cls;
    jmethodID mid;
    jstring jstr;

    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == NULL) {
       goto destroy;
    }
    mid = (*env)->GetStaticMethodID(env, cls, "shutDownContainer", "()V");
    if (mid == NULL) {
        goto destroy;
    }
    (*env)->CallStaticVoidMethod(env, cls, mid);

    if ((*env)->ExceptionOccurred(env)) {
        (*env)->ExceptionDescribe(env);
        goto destroy;
    }

   destroy:
   /* instance of JVM is closed */
   (*jvm)->DestroyJavaVM(jvm);

   return;
}

