
/**************************************************************
* Program:  <name>.c
* Header:   <hfile>.c
*
* Matthias Rusinski
* msr52@columbia.edu
*
*/

#include <stdio.h>
#include <stdlib.h>     /* calloc() */
#include <assert.h>
#include <pthread.h>
#include <unistd.h>     /* close() */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <jni.h>

#include "SipResponseWrapper.h"
#include "JNI_util.h"
#include "policy.h"
#include "message.h"
#include "request-sip.h"
#include "config.h"
#include "sip.h"
#include "sips_wrapper.h"

/* sipd structures included for compatibility */
struct config_t config;
/* The name of this server */
const char *server_name = "Columbia-Sip-Server";


/**
 * This function is called from SipResponse send() method
 * It performs actual send operation by calling policy
 * functions on sipd structures converted from sips objects
 */
JNIEXPORT jint JNICALL
Java_SipResponseWrapper_processSend(JNIEnv *env, jobject obj, jobject req, jobject msg,
                                    jint st, jstring rs)
{
   jint ret = 1;
   request_t *r;
   message_t *m;
   int i;

   /* temp structures */
   struct sockaddr_in sin;

   /* settings for sipd compatibility */
   config.defaultreg = L_redirect;

   if(req==NULL || msg==NULL)
      return -2;

   /* create empty request */
   if((r = RequestCreate(0,0,sin))==NULL)
      return -1;

   if(build_request_t(env, req, r)<0)
      return (jint) -1;

   /*
   printf("PARSER!: %s, %s, %s\n", r->p.method, r->p.URI_string, r->p.protocol);
   printf("AUTHRZ!: %s, %s, %s\n", r->authorization.http_method, r->authorization.method, r->authorization.userid);
   if(r->callID!=NULL)
      printf("CALL-ID: %s, %d\n", r->callID, 0);
   */

   /*
   ph = r->p.headers;
   while(ph!=NULL) {
      printf("IN PARSER \n");
      if(ph->name != NULL) {
         parser_element_t *el;
         printf(" PARSER HEADER!: %s, %d\n", ph->name, 0);
         el = ph->elements;
         while(el!=NULL) {
            parser_parameter_t *pr;
            printf("   PARSER ELEM!: %s, %d\n", el->element, 0);

            pr = el->parameters;
            while(pr!=NULL) {
               printf("      PARSER PARAM!: %s, %d\n", pr->attribute, 0);
               pr = pr->next;
            }
            el = el->next;
         }
      }
      ph = ph->next;
   }
   */


   /* build message type */
   if((m = MessageCreateTrivial(0,0))==NULL)
      return -1;
   if(build_message_t(env, msg, m)<0)
      return (jint) -1;

   printf("   --> sips: Response: native code: processSend: request_t, message_t built\n");
   /* send request using ResponseSendSocket */

   /* new socket must be created since original socket from request_t is not longer available in JVM */
   /* comonents of message to be sent are in request_t (message_t is not needed here, but is also
    * created and assigned values if a need arises in the future

    * r->headers has headers for outgoinh message
    * r->p (parser_t) has URI_string (for Request) and status, reason (for Response)
    * r->reponse_body has body Request (empty for Response)
    * r->sin should be the socket structure of the destination
    */
   /* ResponseSend should be called */

   return ret;
}



