
/* Note:  */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>
#include <pthread.h>

#include <jni.h>

/* JNI utility functions */
#include "JNI_util.h"
#include "sipJVM.h"
#include "sips_interface.h"

#include "request-sip.h"
#include "message.h"
#include "config.h"
#include "register.h"
#include "http.h"
#include "sip.h"
#include "policy.h"
#include "config.h"
#include "sysdep.h"
#include "uri.h"


/* Java VM created in main */
extern JavaVM *jvm;

int assign_parser_t(JNIEnv *env, jobject *jp, parser_t *p);
int assign_body_t(JNIEnv *env, jobject *jb, body_t *b);
int assign_uri_t(JNIEnv *env, jobject *juri, uri_t *uri);
int assign_branch_t(JNIEnv *env, jobject *jbranch, branch_t *branch);
int assign_sip_contact_t(JNIEnv *env, jobject *jcontact, sip_contact_t *contact);
int assign_sip_via_t(JNIEnv *env, jobject *jvia, sip_via_t *via);
int assign_request_t(JNIEnv *env, jobject *jreq, request_t *req);
int assign_message_t(JNIEnv *env, jobject *jmsg, message_t *msg);
int assign_registration_t(JNIEnv *env, jobject *jreg, registration_t *reg);

int assign_pthread_mutex_t(JNIEnv *env, jobject *mutex, pthread_mutex_t *pmutex);
int assign_pthread_cond_t(JNIEnv *env, jobject *cond, pthread_cond_t *pcond);



/* SipS policy structure */
policy_t sip_servlet_policy =
{
  "sip_servlet",
  &sips_init,
  &sips_handle_request,
  &sips_handle_response,
  &sips_timeout_expired,
  &sips_cleanup
};



/**
 * java object request_t corresponds to sipd request_t - transaction object
 * message_t -> sipd's message_t
 * ... etc ...
 */


/* Policy functions: corresponding to policy function signatures
 * to be called from function: execute_policy
 */
/**
 * Invokes sip servlet container with start of transaction,
 * servlet instantiated for transaction
 */
int sips_init(request_t *r, registration_t *reg, int reg_status)
{
   int ret;
   jobject request, registration;
   jint res, sipsResult;
   jclass cls;
   jmethodID mid;

   /* env will be assigned when thread is attached to jvm */
   JNIEnv *env;

   //printf("sips_init: IN SIPS INIT\n");

   /* attach this thread to JVM */
#ifdef JNI_VERSION_1_2
   res = (*jvm)->AttachCurrentThread(jvm, (void**)&env, NULL);
#else
   res = (*jvm)->AttachCurrentThread(jvm, &env, NULL);
#endif
   if (res < 0) {
      fprintf(stderr, "sips_init: Attach failed\n");
      return -1;
   }

    if((ret = assign_request_t(env, &request, r))<0) ;
    //printf("sips_init: ret %d\n", ret);
    if((ret = assign_registration_t(env, &registration, reg))<0) ;
    //printf("sips_init: ret %d\n", ret);
    if(ret<0)  goto detach;

    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == 0)  { printf("no cls\n"); goto detach;  }
    mid = (*env)->GetStaticMethodID(env, cls, "transactionInit", "(Lrequest_t;Lregistration_t;)I");
    if (mid == 0)  { printf("no mid\n"); goto detach;  }

    sipsResult = (*env)->CallStaticIntMethod(env, cls, mid, request, registration);
    ret = (int) sipsResult;

detach:
   if ((*env)->ExceptionOccurred(env)) {
      (*env)->ExceptionDescribe(env);
   }
   (*jvm)->DetachCurrentThread(jvm);

   return ret;
}


/**
 * handles request from sipd
 * Each message is passed to servlet container
 * to instance of servlet
 *
 */
int sips_handle_request(request_t *r, message_t *m)
{
   int ret;
   jint res, sipsResult;
   jobject request, message;
   jclass cls;
   jmethodID mid;


   /* env will be assigned when thread is attached to jvm */
   JNIEnv *env;

   /* attach this thread to JVM */
#ifdef JNI_VERSION_1_2
   res = (*jvm)->AttachCurrentThread(jvm, (void**)&env, NULL);
#else
   res = (*jvm)->AttachCurrentThread(jvm, &env, NULL);
#endif
   if (res < 0) {
      fprintf(stderr, "sips_init: Attach failed\n");
      return -1;
   }

    ret = assign_message_t(env, &message, m);
    if(ret<0)  goto detach;
    ret = assign_request_t(env, &request, r);
    if(ret<0)  goto detach;

    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == 0)  { printf("no cls\n"); goto detach;  }
    mid = (*env)->GetStaticMethodID(env, cls, "transactionRequest", "(Lrequest_t;Lmessage_t;)I");
    if (mid == 0)  { printf("no mid\n"); goto detach;  }

    sipsResult = (*env)->CallStaticIntMethod(env, cls, mid, request, message);
    ret = (int) sipsResult;

detach:
   //printf(" detaching  \n");
   if ((*env)->ExceptionOccurred(env)) {
      (*env)->ExceptionDescribe(env);
   }
   (*jvm)->DetachCurrentThread(jvm);

   return ret;
}


/**
 * handles response from sipd
 * passes to instance of servlet
 */
int sips_handle_response(request_t *r, message_t *m, branch_t *b)
{
   int ret;
   jint res, sipsResult;
   jobject request, message, branch;
   jclass cls;
   jmethodID mid;

   /* env will be assigned when thread is attached to jvm */
   JNIEnv *env;

   /* attach this thread to JVM */
#ifdef JNI_VERSION_1_2
   res = (*jvm)->AttachCurrentThread(jvm, (void**)&env, NULL);
#else
   res = (*jvm)->AttachCurrentThread(jvm, &env, NULL);
#endif
   if (res < 0) {
      fprintf(stderr, "sips_init: Attach failed\n");
      return -1;
   }

    if((ret = assign_request_t(env, &request, r))<0) ;
    //printf("sips_handle_response: return value %d\n", ret);
    if((ret = assign_message_t(env, &message, m))<0) ;
    //printf("sips_handle_response: return value %d\n", ret);
    if((ret = assign_branch_t(env, &branch, b))<0) ;
    //printf("sips_handle_response: return value %d\n", ret);
    if(ret<0)  goto detach;

    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == 0)  { printf("no cls\n"); goto detach;  }
    mid = (*env)->GetStaticMethodID(env, cls, "transactionResponse", "(Lrequest_t;Lmessage_t;Lbranch_t;)I");
    if (mid == 0)  { printf("no mid\n"); goto detach;  }

    sipsResult = (*env)->CallStaticIntMethod(env, cls, mid, request, message, branch);
    ret = (int) sipsResult;

detach:
   if ((*env)->ExceptionOccurred(env)) {
      (*env)->ExceptionDescribe(env);
   }
   (*jvm)->DetachCurrentThread(jvm);

   return ret;
}


/**
 * timeout
 */
int sips_timeout_expired(request_t *r)
{
   int ret;
   jint res, sipsResult;
   jobject request;

   /* env will be assigned when thread is attached to jvm */
   JNIEnv *env;

   /* attach this thread to JVM */
#ifdef JNI_VERSION_1_2
   res = (*jvm)->AttachCurrentThread(jvm, (void**)&env, NULL);
#else
   res = (*jvm)->AttachCurrentThread(jvm, &env, NULL);
#endif
   if (res < 0) {
      fprintf(stderr, "sips_init: Attach failed\n");
      return -1;
   }

   // remove transaction
   ret = sips_cleanup(r);

detach:
   if ((*env)->ExceptionOccurred(env)) {
      (*env)->ExceptionDescribe(env);
   }
   (*jvm)->DetachCurrentThread(jvm);

   return ret;
}


/**
 * Closes transaction
 * Removes instance of servlet from servlet container
 */
int sips_cleanup(request_t *r)
{
   int ret;
   jint res, sipsResult;
   jobject request;
   jclass cls;
   jmethodID mid;

   /* env will be assigned when thread is attached to jvm */
   JNIEnv *env;

   /* attach this thread to JVM */
#ifdef JNI_VERSION_1_2
   res = (*jvm)->AttachCurrentThread(jvm, (void**)&env, NULL);
#else
   res = (*jvm)->AttachCurrentThread(jvm, &env, NULL);
#endif
   if (res < 0) {
      fprintf(stderr, "sips_init: Attach failed\n");
      return -1;
   }

    if((ret = assign_request_t(env, &request, r))<0) ;
    //printf("sips_cleanup: ret %d\n", ret);
    if(ret<0)  goto detach;

    cls = (*env)->FindClass(env, SERV_ENGINE);
    if (cls == 0)  { printf("no cls\n"); goto detach;  }
    mid = (*env)->GetStaticMethodID(env, cls, "transactionCleanUp", "(Lrequest_t;)I");
    if (mid == 0)  { printf("no mid\n"); goto detach;  }

    sipsResult = (*env)->CallStaticIntMethod(env, cls, mid, request);
    ret = (int) sipsResult;

detach:
   if ((*env)->ExceptionOccurred(env)) {
      (*env)->ExceptionDescribe(env);
   }
   (*jvm)->DetachCurrentThread(jvm);

   return ret;
}






/* ================= internal functions ==================
 * these functions assign java objects corresponding to sipd structures
 * to values found in those structures
 *
 * Each of the functions located below instantiates an object in JVM
 * that corresponds to the sipd structure the function is working on,
 * and initializes its values to those found in the sipd structure.
 *
 * Every function returns a positive number upon success, and a negative
 * numbers if errors were encountered, and the object could not be
 * initialized correctly.
 */


/**
 * assigns values to java object from corresponding structure
 * from sipd
 */
int assign_parser_t(JNIEnv *env, jobject *jp, parser_t *p)
{
   int ret = 1;
   jclass pCls, headCls, elemCls, paramCls, listCls;
   jobject header;
   jobject uri, list;
   parser_header_t *headerPtr;
   parser_element_t *elemPtr;
   parser_parameter_t *paramPtr;

   pCls = (*env)->FindClass(env, "parser_t");
   if (pCls == NULL) { return -1; }
   headCls = (*env)->FindClass(env, "parser_header_t");
   if (headCls == NULL) { return -1; }
   elemCls = (*env)->FindClass(env, "parser_element_t");
   if (elemCls == NULL) { return -1; }
   paramCls = (*env)->FindClass(env, "parser_parameter_t");
   if (paramCls == NULL) { return -1; }

   //printf("about to call new request_t object \n");
   if( ((*jp) = allocateEmptyJObject(env, "parser_t")) == NULL)
      return -2;

   if(setJObjectIntField(env, jp, pCls, "type", p->type) < 0) ret = -3;
   if(setJObjectIntField(env, jp, pCls, "status", p->status) < 0) ret = -3;
   if(setJObjectIntField(env, jp, pCls, "header_length", p->header_length) < 0) ret = -3;

   if(setJObjectStringField(env, jp, pCls, "method", p->method) < 0) ret = -3;
   if(setJObjectStringField(env, jp, pCls, "URI_string", p->URI_string) < 0) ret = -3;
   if(setJObjectStringField(env, jp, pCls, "reason", p->reason) < 0) ret = -3;
   if(setJObjectStringField(env, jp, pCls, "protocol", p->protocol) < 0) ret = -3;
   if(setJObjectStringField(env, jp, pCls, "version", p->version) < 0) ret = -3;
   if(setJObjectStringField(env, jp, pCls, "header_body", p->header_body) < 0) ret = -3;

   if(assign_uri_t(env, &uri, &(p->URI)) < 0) return -4;
   if(setJObjectObjectField(env, jp, pCls, "URI", "Luri_t;", uri) < 0) ret = -5;

   if(p->headers) {
      // create list of header objects
      list = allocateNewJObject(env, "java/util/LinkedList");
      if(list == NULL)  return -2;
      listCls = (*env)->FindClass(env, "java/util/LinkedList");
      if (listCls == NULL) { return -1; }

      headerPtr = p->headers;
      // process each header
      while(headerPtr != NULL) {
         jobject header = allocateEmptyJObject(env, "parser_header_t");
         if(header == NULL)   break;
         if(setJObjectStringField(env, &header, headCls, "name", headerPtr->name) < 0) ret = -7;

         if(headerPtr->elements) {
            // create list of elements objects for each header
            jobject eList = allocateNewJObject(env, "java/util/LinkedList");
            if(eList == NULL) return -2;
            elemPtr = headerPtr->elements;

            // process each element
            while(elemPtr != NULL) {
               jobject element = allocateEmptyJObject(env, "parser_element_t");
               if(element==NULL) break;
               if(setJObjectStringField(env, &element, elemCls, "element", elemPtr->element) < 0) ret = -7;
               if(setJObjectStringField(env, &element, elemCls, "value", elemPtr->value) < 0) ret = -7;
               if(setJObjectStringField(env, &element, elemCls, "comment", elemPtr->comment) < 0) ret = -7;

               if(elemPtr->parameters) {
                  // create list of parameters objects for each element
                  jobject paramList = allocateNewJObject(env, "java/util/LinkedList");
                  if(paramList == NULL)   return -2;

                  paramPtr = elemPtr->parameters;
                  // process each parameter
                  while(paramPtr != NULL) {
                     jobject param = allocateEmptyJObject(env, "parser_parameter_t");
                     if(param==NULL)   break;
                     if(setJObjectStringField(env, &param, paramCls, "attribute", paramPtr->attribute) < 0) ret = -7;
                     if(setJObjectStringField(env, &param, paramCls, "value", paramPtr->value) < 0) ret = -7;
                     if(setJObjectIntField(env, &param, paramCls, "value_type", paramPtr->value_type) < 0) ret = -7;

                     // add param to list of params
                     if(addJListObjectObject(env, &paramList, listCls, param)<0) ret = -8;

                     paramPtr = paramPtr->next;
                  }//end paramPtr processing
                  if(setJObjectObjectField(env, &element, elemCls, "parameters", "Ljava/util/LinkedList;", paramList) < 0) ret = -10;
               }

               // add element to list of elements
               if(addJListObjectObject(env, &eList, listCls, element)<0) ret = -8;

               elemPtr = elemPtr->next;
            }//end elemPtr processing
            if(setJObjectObjectField(env, &header, headCls, "elements", "Ljava/util/LinkedList;", eList) < 0) ret = -10;
         }

         // add header to list of headers
         if(addJListObjectObject(env, &list, listCls, header)<0) ret = -8;
         headerPtr = headerPtr->next;
      }// end headerPtr processing

      // add list of headers to parser_t
      if(setJObjectObjectField(env, jp, pCls, "headers", "Ljava/util/LinkedList;", list) < 0) ret = -10;
   }
   return ret;
}



/**
 * uri_t
 */
int assign_uri_t(JNIEnv *env, jobject *juri, uri_t *uri)
{
   int ret = 1, i, subRet;
   jclass uCls, mapCls;
   jobject params, headers;

   uCls = (*env)->FindClass(env, "uri_t");
   if (uCls == 0) { return -1; }
   mapCls = (*env)->FindClass(env, "java/util/HashMap");
   if (mapCls == 0) { return -1; }

   if( ((*juri) = allocateEmptyJObject(env, "uri_t")) == NULL)
      return -2;
   params = allocateNewJObject(env, "java/util/HashMap");
   headers = allocateNewJObject(env, "java/util/HashMap");
   if(params==NULL || headers==NULL) {
      return -4;
   }

   if(setJObjectStringField(env, juri, uCls, "scheme", uri->scheme) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "user", uri->user) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "password", uri->password) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "host", uri->host) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "path", uri->path) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "param", uri->param) < 0) ret = -3;
   if(setJObjectStringField(env, juri, uCls, "search", uri->search) < 0) ret = -3;
   if(setJObjectIntField(env, juri, uCls, "port", uri->port) < 0) ret = -3;

   if (uri->parameters) {
      for (i = 0; uri->parameters[i].name != NULL; i++) {
        subRet = addJHashMapObjectPair(env, &params, mapCls, (uri->parameters[i]).name, (uri->parameters[i]).value);
        if(subRet < 0) ret = 33;
       }
   }
   if (uri->headers) {
     for (i = 0; uri->headers[i].name != NULL; i++) {
        subRet = addJHashMapObjectPair(env, &headers, mapCls, (uri->headers[i]).name, (uri->headers[i]).value);
        if(subRet < 0) ret = 33;
     }
   }
   if(setJObjectObjectField(env, juri, uCls, "parameters", "Ljava/util/HashMap;", params) < 0) ret = -5;
   if(setJObjectObjectField(env, juri, uCls, "headers", "Ljava/util/HashMap;", headers) < 0) ret = -5;
   return ret;
}



/**
 * body_t
 */
int assign_body_t(JNIEnv *env, jobject *jb, body_t *b)
{
   int ret = 1;
   jclass cls;

   cls = (*env)->FindClass(env, "body_t");
   if (cls == 0) { return -1; }
   if( ((*jb) = allocateEmptyJObject(env, "body_t")) == NULL)
      return -2;
   if(setJObjectStringField(env, jb, cls, "content_type", b->content_type) < 0) ret = -3;
   if(setJObjectStringField(env, jb, cls, "content", b->content) < 0) ret = -3;
   if(setJObjectIntField(env, jb, cls, "content_length", b->content_length) < 0) ret = -3;

   return ret;
}



/**
 * branch_t
 */
int assign_branch_t(JNIEnv *env, jobject *jbranch, branch_t *branch)
{
   int ret = 1;
   jclass cls;
   jobject uri, msg, req, pthreadCond;

   cls = (*env)->FindClass(env, "branch_t");
   if (cls == 0) { return -1; }
   if( ((*jbranch) = allocateEmptyJObject(env, "branch_t")) == NULL)
      return -2;

   if(setJObjectIntField(env, jbranch, cls, "branch", branch->branch) < 0) ret = -3;
   if(setJObjectIntField(env, jbranch, cls, "sent_200", branch->sent_200) < 0) ret = -3;
   if(setJObjectIntField(env, jbranch, cls, "sent_final", branch->sent_final) < 0) ret = -3;
   if(setJObjectIntField(env, jbranch, cls, "marked_done", branch->marked_done) < 0) ret = -3;
   if(setJObjectIntField(env, jbranch, cls, "shutdown", branch->shutdown) < 0) ret = -3;
   if(setJObjectIntField(env, jbranch, cls, "sent_something", branch->sent_something) < 0) ret = -3;

   if(setJObjectLongField(env, jbranch, cls, "tid", branch->tid) < 0) ret = -3;

   if(assign_uri_t(env, &uri, &(branch->uri)) < 0) return -4;
   if(setJObjectObjectField(env, jbranch, cls, "uri", "Luri_t;", uri) < 0) ret = -5;

   if(assign_pthread_cond_t(env, &pthreadCond, &(branch->response_received)) < 0) return -4;
   if(setJObjectObjectField(env, jbranch, cls, "response_received", "Lpthread_cond_t;", pthreadCond) < 0) ret = -5;

   if(branch->orig !=NULL) {
      if(assign_message_t(env, &msg, branch->orig) < 0) return -4;
      if(setJObjectObjectField(env, jbranch, cls, "orig", "Lmessage_t;", msg) < 0) ret = -5;
   }
   if(branch->parent != NULL) {
      if(assign_request_t(env, &req, branch->parent) < 0) return -4;
      if(setJObjectObjectField(env, jbranch, cls, "parent", "Lrequest_t;", req) < 0) ret = -5;
   }

   return ret;
}


/**
 * message_t
 */
int assign_message_t(JNIEnv *env, jobject *jmsg, message_t *msg)
{
   int ret = 1;
   jclass cls;
   jobject p, b, branch;

   cls = (*env)->FindClass(env, "message_t");
   if (cls == 0) { return -1; }
   if( ((*jmsg) = allocateEmptyJObject(env, "message_t")) == NULL)
      return -2;

   if(setJObjectIntField(env, jmsg, cls, "type", msg->type) < 0) ret = -3;
   if(setJObjectIntField(env, jmsg, cls, "handled", msg->handled) < 0) ret = -3;
   if(setJObjectStringField(env, jmsg, cls, "source", inet_ntoa(msg->source.sin_addr)) < 0) ret = -3;
   if(setJObjectIntField(env, jmsg, cls, "source_port", (int) ntohs(msg->source.sin_port)) < 0) ret = -3;

   if(assign_parser_t(env, &p, &(msg->p)) < 0) return -4;
   if(setJObjectObjectField(env, jmsg, cls, "p", "Lparser_t;", p) < 0) ret = -5;

   if(assign_body_t(env, &b, &(msg->body)) < 0) return -4;
   if(setJObjectObjectField(env, jmsg, cls, "body", "Lbody_t;", b) < 0) ret = -5;

   if(msg->branch != NULL && msg->branch->orig != msg) {
      if(assign_branch_t(env, &branch, msg->branch) < 0) return -6;
      if(setJObjectObjectField(env, jmsg, cls, "branch", "Lbranch_t;", branch) < 0) ret = -5;
   }

   return ret;
}

/**
 * sip_contact_t
 */
int assign_sip_contact_t(JNIEnv *env, jobject *jcontact, sip_contact_t *contact)
{
   int ret = 1;
   jclass cls;
   jobject uri;

   cls = (*env)->FindClass(env, "sip_contact_t");
   if (cls == 0) { return -1; }
   if( ((*jcontact) = allocateEmptyJObject(env, "sip_contact_t")) == NULL)
      return -2;

   if(setJObjectLongField(env, jcontact, cls, "expires", contact->expires) < 0) ret = -3;
   if(setJObjectIntField(env, jcontact, cls, "action", contact->action) < 0) ret = -3;
   if(setJObjectDoubleField(env, jcontact, cls, "q", (double) contact->q) < 0) ret = -3;
   if(setJObjectStringField(env, jcontact, cls, "nameaddr", contact->nameaddr) < 0) ret = -3;
   if(setJObjectStringField(env, jcontact, cls, "comment", contact->comment) < 0) ret = -3;

   if(assign_uri_t(env, &uri, &(contact->uri)) < 0) return -4;
   if(setJObjectObjectField(env, jcontact, cls, "uri", "Luri_t;", uri) < 0) ret = -5;

   return ret;
}


/**
 * sip_via_t
 */
int assign_sip_via_t(JNIEnv *env, jobject *jvia, sip_via_t *via)
{
   int ret = 1;
   jclass cls;
   int sock_type = 0;

   cls = (*env)->FindClass(env, "sip_via_t");
   if (cls == 0) { return -1; }
   if( ((*jvia) = allocateEmptyJObject(env, "sip_via_t")) == NULL)
      return -2;

   if(via->type == SOCK_DGRAM) sock_type = 1;
   else if(via->type == SOCK_STREAM) sock_type = 2;
   if(setJObjectIntField(env, jvia, cls, "type", sock_type) < 0) ret = -3;

   if(setJObjectStringField(env, jvia, cls, "sin", inet_ntoa(via->sin.sin_addr)) < 0) ret = -3;
   if(setJObjectIntField(env, jvia, cls, "sin_port", (int) ntohs(via->sin.sin_port)) < 0) ret = -3;
   if(setJObjectStringField(env, jvia, cls, "received", inet_ntoa(via->received)) < 0) ret = -3;

   if(setJObjectStringField(env, jvia, cls, "hostname", via->hostname) < 0) ret = -3;
   if(setJObjectIntField(env, jvia, cls, "islocal", via->islocal) < 0) ret = -3;
   if(setJObjectStringField(env, jvia, cls, "b_str", via->branch.str) < 0) ret = -3;
   if(setJObjectLongField(env, jvia, cls, "b_hash", via->branch.hash) < 0) ret = -3;
   if(setJObjectIntField(env, jvia, cls, "b_id", via->branch.id) < 0) ret = -3;

   return ret;
}



/**
 * registration_t
 */
int assign_registration_t(JNIEnv *env, jobject *jreg, registration_t *reg)
{
   int ret = 1, i;
   jclass cls, vCls;
   jobject vect;
   char *temp;
   script_t *scripts;

   if(reg->scriptset.scripts==NULL || reg->scriptset.count<1)
      return -1;

   cls = (*env)->FindClass(env, "registration_t");
   if (cls == 0) { return -1; }
   if( ((*jreg) = allocateEmptyJObject(env, "registration_t")) == NULL)
      return -2;
   vCls = (*env)->FindClass(env, "java/util/Vector");
   if (vCls == 0) { return -1; }
   vect = allocateNewJObject(env, "java/util/Vector");
   if(vect == NULL) return -2;

   if(setJObjectIntField(env, jreg, cls, "contacts", reg->contacts) < 0) ret = -3;
   if(setJObjectLongField(env, jreg, cls, "update", reg->update) < 0) ret = -3;

   for(i=0; i<reg->contacts; i++) {
      jobject contact;
      if(assign_sip_contact_t(env, &contact, &(reg->contact[i])) < 0) return -4;
      // contact is added to j Vector (list method is called (same signature as vector method))
      if(addJListObjectObject(env, &vect, vCls, contact)<0) ret = -5;
   }
   if(setJObjectObjectField(env, jreg, cls, "contact", "Ljava/util/Vector;", vect) < 0) ret = -6;

   scripts = reg->scriptset.scripts;

   if(setJObjectStringField(env, jreg, cls, "sipsClass", scripts[0].filename) < 0) ret = -3;
   if(setJObjectStringField(env, jreg, cls, "dispString", scripts[0].disp_str) < 0) ret = -3;
   if(setJObjectStringField(env, jreg, cls, "rootDir", reg->scriptset.root_dir) < 0) ret = -3;

   return ret;
}




/**
 * reqeust_t
 * top-level function called from sips policy functions
 */
int assign_request_t(JNIEnv *env, jobject *jreq, request_t *req)
{
   int ret = 1, i;
   jclass cls;
   int sock_type=0;
   jobject p, b, via, contact, vect;
   jobject jtoAddr, jfromAddr, jtoUri, jfromUri, cSeq;
   jclass addrCls, cSeqCls, tSpecCls, vCls, authCls, authenCls;
   jobject threadCond, threadMutex, tSpec, branch, pUri, auth, authen;

   cls = (*env)->FindClass(env, "request_t");
   if (cls == 0) { return -1; }

   /* allocate request_t object in JVM */
   if( ((*jreq) = allocateEmptyJObject(env, "request_t")) == NULL)
      return -2;

   if(setJObjectDoubleField(env, jreq, cls, "time_received", req->time_received) < 0) ret = -3;

   /* sockets and addresses */
   if(req->type == SOCK_DGRAM) sock_type = 1;
   else if(req->type == SOCK_STREAM) sock_type = 2;
   if(setJObjectIntField(env, jreq, cls, "type", sock_type) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "socket", req->socket) < 0) ret = -3;
   if(setJObjectStringField(env, jreq, cls, "source_sin", inet_ntoa(req->source_sin.sin_addr)) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "source_sin_port", (int) ntohs(req->source_sin.sin_port)) < 0) ret = -3;
   if(setJObjectStringField(env, jreq, cls, "sin", inet_ntoa(req->sin.sin_addr)) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "sin_port", (int) ntohs(req->sin.sin_port)) < 0) ret = -3;

   /* parser and body */
   if(assign_parser_t(env, &p, &(req->p)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "p", "Lparser_t;", p) < 0) ret = -5;
   if(assign_body_t(env, &b, &(req->request_body)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "request_body", "Lbody_t;", b) < 0) ret = -5;

   /* http authorization */
   auth = allocateEmptyJObject(env, "http_authorization_t");
   if(auth == NULL) return -2;
   authCls = (*env)->FindClass(env, "http_authorization_t");
   if (authCls == NULL) { return -1; }
   if(setJObjectStringField(env, &auth, authCls, "http_method", req->authorization.http_method) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "method", req->authorization.method) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "userid", req->authorization.userid) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "userid_mapped", req->authorization.userid_mapped) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "password", req->authorization.password) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "realm", req->authorization.realm) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "nonce", req->authorization.nonce) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "digest_uri", req->authorization.digest_uri) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "response_digest", req->authorization.response_digest) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "entity_digest", req->authorization.entity_digest) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "opaque", req->authorization.opaque) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "algorithm", req->authorization.algorithm) < 0) ret = -3;
   if(setJObjectIntField(env, &auth, authCls, "valid", req->authorization.valid) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "version", req->authorization.version) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "signed_by", req->authorization.signed_by) < 0) ret = -3;
   if(setJObjectStringField(env, &auth, authCls, "signature", req->authorization.signature) < 0) ret = -3;
   if(setJObjectObjectField(env, jreq, cls, "authorization", "Lhttp_authorization_t;", auth) < 0) ret = -5;

   if(setJObjectIntField(env, jreq, cls, "status", req->status) < 0) ret = -3;
   if(setJObjectStringField(env, jreq, cls, "reason", req->reason) < 0) ret = -3;

   /* headers are omitted-> they are in parser_t object */

   b = NULL;
   if(assign_body_t(env, &b, &(req->response_body)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "response_body", "Lbody_t;", b) < 0) ret = -5;

   if(setJObjectStringField(env, jreq, cls, "version", req->version) < 0) ret = -3;

   /* sip cseq */
   cSeq = allocateEmptyJObject(env, "sip_cseq_t");
   if(cSeq==NULL)
      return -4;
   cSeqCls = (*env)->FindClass(env, "sip_cseq_t");
   if (cSeqCls == 0) { return -1; }
   if(setJObjectIntField(env, &cSeq, cSeqCls, "seq", req->cseq.seq) < 0) ret = -3;
   if(setJObjectIntField(env, &cSeq, cSeqCls, "seq", req->cseq.seq) < 0) ret = -3;
   if(setJObjectObjectField(env, jreq, cls, "cseq", "Lsip_cseq_t;", cSeq) < 0) ret = -5;

   /* callID */
   if(setJObjectStringField(env, jreq, cls, "callID", req->callID) < 0) ret = -3;

   /* sip address: to and from */
   addrCls = (*env)->FindClass(env, "sip_addr_t");
   if (addrCls == 0) { return -1; }
   jtoAddr = allocateEmptyJObject(env, "sip_addr_t");
   jfromAddr = allocateEmptyJObject(env, "sip_addr_t");
   if(jtoAddr==NULL || jfromAddr==NULL) {
      return -4;
   }
   if(setJObjectStringField(env, &jtoAddr, addrCls, "display_name", req->to.display_name) < 0) ret = -6;
   if(assign_uri_t(env, &jtoUri, &(req->to.uri)) < 0) return -4;
   if(setJObjectObjectField(env, &jtoAddr, addrCls, "uri", "Luri_t;", jtoUri) < 0) ret = -6;
   if(setJObjectObjectField(env, jreq, cls, "to", "Lsip_addr_t;", jtoAddr) < 0) ret = -7;

   if(setJObjectStringField(env, &jfromAddr, addrCls, "display_name", req->from.display_name) < 0) ret = -6;
   if(assign_uri_t(env, &jfromUri, &(req->from.uri)) < 0) return -4;
   if(setJObjectObjectField(env, &jfromAddr, addrCls, "uri", "Luri_t;", jfromUri) < 0) ret = -6;
   if(setJObjectObjectField(env, jreq, cls, "from", "Lsip_addr_t;", jfromAddr) < 0) ret = -7;

   if(assign_sip_via_t(env, &via, &(req->via)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "via", "Lsip_via_t;", via) < 0) ret = -5;

   if(setJObjectLongField(env, jreq, cls, "expires", req->expires) < 0) ret = -3;

   if(assign_sip_contact_t(env, &contact, req->contact) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "contact", "Lsip_contact_t;", contact) < 0) ret = -5;

   if(setJObjectStringField(env, jreq, cls, "user_agent", req->user_agent) < 0) ret = -3;
   if(setJObjectLongField(env, jreq, cls, "hash", req->hash) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "acked", req->acked) < 0) ret = -3;

   /* http_authenticate */
   authen = allocateEmptyJObject(env, "http_authenticate_t");
   if(authen == NULL) return -2;
   authenCls = (*env)->FindClass(env, "http_authenticate_t");
   if (authenCls == NULL) { return -1; }
   if(setJObjectStringField(env, &authen, authenCls, "http_method", req->authenticate.http_method) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "method", req->authenticate.method) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "realm", req->authenticate.realm) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "domain", req->authenticate.domain) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "nonce", req->authenticate.nonce) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "opaque", req->authenticate.opaque) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "stale", req->authenticate.stale) < 0) ret = -3;
   if(setJObjectStringField(env, &authen, authenCls, "algorithm", req->authenticate.algorithm) < 0) ret = -3;
   if(setJObjectObjectField(env, jreq, cls, "authenticate", "Lhttp_authenticate_t;", authen) < 0) ret = -5;

   /** not used **
   if(assign_pthread_cond_t(env, &threadCond, &(req->cond)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "cond", "Lpthread_cond_t;", threadCond) < 0) ret = -5;
   if(assign_pthread_mutex_t(env, &threadMutex, &(req->mutex)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "mutex", "Lpthread_mutex_t;", threadMutex) < 0) ret = -5;
   */

   if(setJObjectIntField(env, jreq, cls, "sent_final", req->sent_final) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "sent_200", req->sent_200) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "done", req->done) < 0) ret = -3;

   tSpecCls = (*env)->FindClass(env, "timespec");
   if (tSpecCls == NULL) { return -1; }
   tSpec = allocateEmptyJObject(env, "timespec");
   if(tSpec == NULL) return -2;
   if(setJObjectLongField(env, &tSpec, tSpecCls, "tv_sec", req->timeout.tv_sec) < 0) ret = -3;
   if(setJObjectLongField(env, &tSpec, tSpecCls, "tv_nsec", req->timeout.tv_nsec) < 0) ret = -3;
   if(setJObjectObjectField(env, jreq, cls, "timeout", "Ltimespec;", tSpec) < 0) ret = -5;

   vCls = (*env)->FindClass(env, "java/util/Vector");
   if (vCls == 0) { return -1; }
   vect = allocateNewJObject(env, "java/util/Vector");
   if(vect == NULL) return -2;
   for(i=0; i<req->branch_count; i++) {
      if(req != req->branches[i].parent) {
         branch = NULL;
         if(assign_branch_t(env, &branch, &(req->branches[i])) < 0) return -6;
         // contact is added to j Vector (list method is called (same signature as vector method))
         if(addJListObjectObject(env, &vect, vCls, branch)<0) ret = -7;
      }
   }
   if(setJObjectObjectField(env, jreq, cls, "branches", "Ljava/util/Vector;", vect) < 0) ret = -6;

   if(setJObjectIntField(env, jreq, cls, "branch_count", req->branch_count) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "branch_space", req->branch_space) < 0) ret = -3;
   if(setJObjectIntField(env, jreq, cls, "branches_outstanding", req->branches_outstanding) < 0) ret = -3;

   if(assign_uri_t(env, &pUri, &(req->outbound_proxy)) < 0) return -4;
   if(setJObjectObjectField(env, jreq, cls, "outbound_proxy", "Luri_t;", pUri) < 0) ret = -6;

   if(req->preferred_type == SOCK_DGRAM) sock_type = 1;
   else if(req->preferred_type == SOCK_STREAM) sock_type = 2;
   if(setJObjectIntField(env, jreq, cls, "preferred_type", sock_type) < 0) ret = -3;

   return ret;
}



/**
 * methods below were created in case thread mutex and cond structures needed to
 * be preserved, but they are not needed in current implementation of servlet engine
 */
int assign_pthread_mutex_t(JNIEnv *env, jobject *mutex, pthread_mutex_t *pmutex)
{
   int ret = 1;
   jclass cls;
   jintArray mutexFlag;
   jintArray mutexPad;
   int i;
   jint mutex_flag[4], mutex_pad[8];

   cls = (*env)->FindClass(env, "pthread_mutex_t");
   if (cls == 0) { return -1; }
   if( ((*mutex) = allocateEmptyJObject(env, "pthread_mutex_t")) == NULL)
      return -2;

   mutexFlag = (*env)->NewIntArray(env, 4);
   if(mutexFlag == NULL)
      return -3;
   mutexPad = (*env)->NewIntArray(env, 8);
   if(mutexPad == NULL)
      return -3;

   for(i=0; i<4; i++) {
      mutex_flag[i] = pmutex->__pthread_mutex_flags.__pthread_mutex_flag[i];
   }
   for(i=0; i<8; i++) {
      mutex_pad[i] = pmutex->__pthread_mutex_lock.__pthread_mutex_lock64.__pthread_mutex_pad[i];
   }
   (*env)->SetIntArrayRegion(env, (jintArray) mutexFlag, (jsize) 0, (jsize) 4, mutex_flag);
   (*env)->SetIntArrayRegion(env, mutexPad, 0, 8, mutex_pad);

   if(setJObjectObjectField(env, mutex, cls, "__pthread_mutex_flag", "[I", mutexFlag) < 0) ret = -5;
   if(setJObjectObjectField(env, mutex, cls, "__pthread_mutex_pad", "[I", mutexPad) < 0) ret = -5;

   if(setJObjectLongField(env, mutex, cls, "__pthread_mutex_type",
         pmutex->__pthread_mutex_flags.__pthread_mutex_type) < 0) ret = -3;
   if(setJObjectLongField(env, mutex, cls, "__pthread_mutex_data",
         pmutex->__pthread_mutex_data) < 0) ret = -3;

   return ret;
}

int assign_pthread_cond_t(JNIEnv *env, jobject *cond, pthread_cond_t *pcond)
{
   int ret = 1;
   jclass cls;
   jintArray condFlag;
   int i;
   jint cond_flag[4];

   cls = (*env)->FindClass(env, "pthread_cond_t");
   if (cls == 0) { return -1; }
   if( ((*cond) = allocateEmptyJObject(env, "pthread_cond_t")) == NULL)
      return -2;

   condFlag = (*env)->NewIntArray(env, 4);
   if(condFlag == NULL)
      return -3;

   for(i=0; i<4; i++) {
      cond_flag[i] = pcond->__pthread_cond_flags.__pthread_cond_flag[i];
   }
   (*env)->SetIntArrayRegion(env, condFlag, 0, 4, cond_flag);

   if(setJObjectObjectField(env, cond, cls, "__pthread_cond_flag", "[I", condFlag) < 0) ret = -5;

   if(setJObjectLongField(env, cond, cls, "__pthread_cond_type",
         pcond->__pthread_cond_flags.__pthread_cond_type) < 0) ret = -3;
   if(setJObjectLongField(env, cond, cls, "__pthread_cond_data",
         pcond->__pthread_cond_data) < 0) ret = -3;

   return ret;
}
