
/**
 * This class defines Java objects which correspond to
 * sipd structures received from sipd.
 *
 * Every object is named the as the sipd structure it replicates.
 * These objects are used by SIP servlet engine to instantiate SIP API
 * objects, by SIP API objects that retain and update the underlying
 * sipd structures they are derived from, and by dynamic libraries
 */

import java.util.*;

public class sipdObjects {
   public final String ver = "1.0";
}

// request_t decribes transaction and
// and is also used to specify request or response to be sent
class request_t {

   /* Basic request information */
   public double time_received;          /* time request was received */

   /* Where is the response going to? */
   public int type;                      /* SOCK_DGRAM, SOCK_STREAM */
   public int socket;                          /* socket id for response */
   public String source_sin;             /* socket address request came from */
   public int source_sin_port;
   public String sin;                    /* socket address for response */
   public int sin_port;

   /* Information about the request */
   public parser_t p;                    /* header for this request */
   public body_t request_body;           /* The body of the request; */

   public http_authorization_t authorization;  /* Authorization header */

   /* Information about the response */
   public int status;
   public String reason;
   public LinkedList headers;                 /* Headers of response */
   public body_t response_body;               /* Body to send with respnse */

   //public Object server;    /* server object associated with this request */
   /* COMMON ENDS HERE */

   /* SIP-specific information about the request */
   public String version;                 /* request version */

   /* SIP request headers */
   public sip_cseq_t cseq;                /* sequence number of request */
   public String callID;                  /* the call ID of the request */
   public sip_addr_t to, from;            /* addresses */
   public sip_via_t via;                  /* top-most Via header */
   public long expires;                   /* Expires header */
   public sip_contact_t contact;          // sip_contact_t
   public String user_agent;              /* SIP User-Agent */

   public long hash;                      /* Hash of identifying headers */

   public int acked;                      /* request has been ACKed */

   /* SIP-specific Information about the response .*/
   public http_authenticate_t authenticate;   /* SIP WWW-Authenticate (response) */

   /* Transaction state: */
   /* Locking. */
   public pthread_cond_t cond;               /* condition variable */
   public pthread_mutex_t mutex;             /* mutex for access */

   /* Communication with subsequent requests and responses */
   //message_t *mqueue;                  /* Message queue for this transaction */
   public int sent_final;                /* Whether we've sent a final
                                         response to the original request */
   public int sent_200;                  /* Whether we've sent a 200 response
                                         to to the original request. */
   public int done;                      /* Whether the policy handler is done
                                         with this transaction */

   /* Transaction policies */
   //public policy_t policy;             /* Policy to use for this transaction */
   public timespec timeout;              /* Policy timeout, or 0 */
   //void *policy_info;                  /* Info specific to a policy */

   /* clients (proxy branches) */
   public Vector branches;               // vector of branches
   public int branch_count;              /* number of parallel searches (branches) */
   public int branch_space;              /* number of branch_t's allocated */
   public int branches_outstanding;      /* current branches awaiting responses */

   //void *call;                         /* The call object associated with the request*/

   public uri_t   outbound_proxy;        /* Address of the outbound proxy if any */
   public int     preferred_type;        /* SOCK_DGRAM or SOCK_STREAM or
                                         0 as a preferred socket type
                                         for outgoing request,
                                         typically used by a user
                                         agent */
}


class sip_cseq_t {
   public int seq;
   public String method;
}

class sip_addr_t {
   public String display_name;
   public uri_t uri;
}

class sip_via_t {
   public int type;            /* SOCK_DGRAM or SOCK_STREAM */
   public String sin;          // host address
   public int sin_port;
   public String received;     // address received from
   public String hostname;     /* hostname */
   public int islocal;         /* this host is us! */

   public String b_str;       // brach ID
   public long b_hash;        // hash part of branch ID
   public int b_id;           // mumeric branch id
}

class sip_contact_t {
   public long expires;
   public int action;
   public double q;           // converted later to float
   public uri_t uri;
   public String nameaddr;
   public String comment;
}


// uri_t
class uri_t {
   public String scheme;      /* URI scheme: http, rtsp, ... */
   public String user;        /* user */
   public String password;    /* password */
   public String host;        /* host name */
   public int port;           /* port number */
   public String path;        /* path name */
   public String param;
   public String search;      /* search path (after ?) */
   public HashMap parameters;
   public HashMap headers;
}


class body_t {
   public String content_type;   /* Content-Type header */
   public int content_length;    /* Content-Length header */
   public String content;
}

// element in parser_t
class parser_header_t {
   public String name;
   public LinkedList elements;     //list of parser_element_t
}

// element in parser_header_t
class parser_element_t {
   public String element;            /* The whole element */
   public String value;              /* top-level value */
   public String comment;            /* comment (in parentheses) */
   public LinkedList parameters;     // map: (attribute -> {value, value_type}=parameter_t )
}

// element in parser_element_t
class parser_parameter_t {
   public String attribute;
   public String value;
   public int value_type;
}

class parser_t {
   public int type;                 // message type
   public String method;            /* Request: method */
   public String URI_string;        /* Request: request-URI, whole string */
   public uri_t URI;                /* Request: URI components */
   public int status;               /* Response: response code */
   public String reason;            /* Response: reason phrase */
   public String protocol;          /* R/R: protocol name */
   public String version;           /* R/R: protocol version */
   public String header_body ;      /* pointer to headers and body */

   public LinkedList headers;       //list of parser_header_t
   public int header_length;
}


// message_t
// also referenced in SipMessageImpl, SipResponseImpl, SipRequestImpl
class message_t {
   public int type;
   public int handled;
   public parser_t p;
   public body_t body;
   // address of sender (stored as str)
   public String source;
   public int source_port;
   branch_t branch;
}


class branch_t {
   public int branch;            /* branch index; -1 if none */
   public request_t parent;      /* parent request */
   public uri_t uri;             /* URI to try */
   long tid;                     /* thread id for branch */

   public  message_t orig;       /* Original request for this branch */
   //public Vector mlist;        /* Message queue for this branch */
   public int sent_200;          /* Have we sent a 200 response to the parent? */
   public int sent_final;        /* Have we sent a final response to the parent? */

   pthread_cond_t response_received;

   public int marked_done;           /* has the parent noticed that we finished? */
   public int shutdown;              /* Have we shut down our state machine? */
   public int sent_something;

   // Object policy_info;
}


// registration_t - simplified
class registration_t {
   public int contacts;          /* number of locations */
   public long update;           /* last updated */
   public Vector contact;        /* contains references to contacts: sip_contact_t */
   //public scriptset_t scriptset;  /* user's registered content */
   public String sipsClass;
   public String dispString;
   public String rootDir;
}


class http_authorization_t {
   public String http_method;
   public String method;
   public String userid;
   public String userid_mapped;
   public String password;
   public String realm;
   public String nonce;
   public String digest_uri;
   public String response_digest;
   public String entity_digest;
   public String opaque;
   public String algorithm;
   public int valid;
   public String version;   /* The pgp version */
   public String signed_by; /* The key id that created the signature */
   public String signature; /* The pgp signature */
}


class http_authenticate_t {
   public String http_method;
   public String method;
   public String realm;
   public String domain;
   public String nonce;
   public String opaque;
   public String stale;
   public String algorithm;
}


class timespec {              /* definition per POSIX.4 */
   public long tv_sec;        /* seconds */
   public long tv_nsec;       /* and nanoseconds */
}



/**
 * classes simulating pthread structures
 * NOT USED at this time
 * (made to simulate structures from sys/types.h)
 *
 * pthread structures must be boken up and translated into
 * objects in JVM to be translated back into C structures when
 * a servlet calls a policy function from sipd
 *
 */
class pthread_mutex_t {
   public int __pthread_mutex_flag[];   // array of 4 elememnts
   public long __pthread_mutex_type;

   // members of original union
   public int __pthread_mutex_pad[];    // array of 8 elements

   public long __pthread_mutex_data;
}

class pthread_cond_t {
   public int __pthread_cond_flag[];
   public long __pthread_cond_type;

   public long __pthread_cond_data;
}


/**
 *** pthread C structs assumed in this implementation ***
 *
typedef  struct   _pthread_mutex {
   struct {
      uint8_t     __pthread_mutex_flag[4];
      uint32_t    __pthread_mutex_type;
   } __pthread_mutex_flags;
   union {
      struct {
         uint8_t  __pthread_mutex_pad[8];
      } __pthread_mutex_lock64;
      upad64_t __pthread_mutex_owner64;
   } __pthread_mutex_lock;
   upad64_t __pthread_mutex_data;
} pthread_mutex_t;

typedef  struct   _pthread_cond {
   struct {
      uint8_t     __pthread_cond_flag[4];
      uint32_t    __pthread_cond_type;
   } __pthread_cond_flags;
   upad64_t __pthread_cond_data;
} pthread_cond_t;

**/