package org.ietf.sip;

/**
 * This interface defines a number of constants used throughout the SIP API.
 * It defines constants corresponding to SIP methods and status codes.
 * For convenience this interface is extended by several other interfaces.
 */
public interface SipConstants {

    //
    // SIP request methods
    //
    public static final String INVITE   = "INVITE";
    public static final String ACK      = "ACK";
    public static final String OPTIONS  = "OPTIONS";
    public static final String BYE      = "BYE";
    public static final String CANCEL   = "CANCEL";
    public static final String REGISTER = "REGISTER";

    //
    // Common SIP headers
    //
    public final static String VIA      = "Via";
    public final static String FROM     = "From";
    public final static String TO       = "To";
    public final static String CALLID   = "Call-ID";
    public final static String CSEQ     = "CSeq";
    public final static String SUBJECT  = "Subject";
    public final static String SERVER   = "Server";
    public final static String CONTACT  = "Contact";
    public final static String TIMESTAMP= "Timestamp";

    public final static String REQUIRE  = "Require";
    public final static String PROXY_REQUIRE    = "Proxy-Require";
    public final static String RECORD_ROUTE     = "Record-Route";

    //
    // SIP status codes
    //

    // Informational 1xx
    public final static int SC_TRYING                   = 100;
    public final static int SC_RINGING                  = 180;
    public final static int SC_CALL_FORWARDING          = 181;
    public final static int SC_QUEUED                   = 182;
    // Successful 2xx
    public final static int SC_OK                       = 200;
    // Redirection 3xx
    public final static int SC_MULTIPLE_CHOICES         = 300;
    public final static int SC_MOVED_PERMANENTLY        = 301;
    public final static int SC_MOVED_TEMPORARILY        = 302;
    public final static int SC_SEE_OTHER                = 303;
    public static final int SC_USE_PROXY                = 305;
    public final static int SC_ALTERNATIVE_SERVICE      = 380;
    // Request Filaure 4xx
    public final static int SC_BAD_REQUEST              = 400;
    public final static int SC_UNAUTHORIZED             = 401;
    public static final int SC_PAYMENT_REQUIRED         = 402;
    public final static int SC_FORBIDDEN                = 403;
    public final static int SC_NOT_FOUND                = 404;
    public static final int SC_METHOD_NOT_ALLOWED       = 405;
    public static final int SC_PROXY_AUTHENTICATION_REQUIRED = 407;
    public static final int SC_REQUEST_TIMEOUT          = 408;
    public static final int SC_CONFLICT                 = 409;
    public static final int SC_GONE                     = 410;
    public static final int SC_LENGTH_REQUIRED          = 411;
    public static final int SC_PRECONDITION_FAILED      = 412;
    public static final int SC_REQUEST_BODY_TOO_LARGE   = 413;
    public static final int SC_REQUEST_URI_TOO_LARGE    = 414;
    public static final int SC_UNSUPPORTED_MEDIA_TYPE   = 415;
    public final static int SC_BAD_EXTENSION            = 420;
    public final static int SC_TEMPORARILY_NOT_AVAILABLE = 480;
    public final static int SC_INVALID_CALL_ID          = 481;
    public final static int SC_LOOP_DETECTED            = 482;
    public final static int SC_TOO_MANY_HOPS            = 483;
    // Server Failure 5xx
    public final static int SC_INTERNAL_SERVER_ERROR    = 500;
    public final static int SC_NOT_IMPLEMENTED          = 501;
    public final static int SC_BAD_GATEWAY              = 502;
    public final static int SC_SERVICE_UNAVAILABLE      = 503;
    public static final int SC_GATEWAY_TIMEOUT          = 504;
    public static final int SC_VERSION_NOT_SUPPORTED    = 505;
    // Global Failures 6xx
    public static final int SC_BUSY                     = 600;
    public static final int SC_DECLINE                  = 603;
    public static final int SC_DOES_NOT_EXIST           = 604;
    public static final int SC_NOT_ACCEPTABLE           = 606;

    /*
    //
    // Constants classes of status codes
    //
    public static final int CL_INFORMATIONAL            = 1;
    public static final int CL_SUCCESS                  = 2;
    public static final int CL_REDIRECTION              = 3;
    public static final int CL_CLIENT_ERROR             = 4;
    public static final int CL_SERVER_ERROR             = 5;
    public static final int CL_GLOBAL_FAILURE           = 6;
    */
}
