package org.ietf.sip;

/**
 * Represents SIP responses.
 */
public interface SipResponse extends SipMessage {
    
    /**
     * Returns the SIP status code of this response. This is a three
     * digit (decimally speaking) integer, usually one of the RFC 2543
     * defined SipConstants.SC_*.
     * @see SipConstants
     */
    public int getStatus();
    
    /**
     * Returns the reason phrase of this response.
     */
    public String getReasonPhrase();
    
    /**
     * Returns the "request token" associated with this response.
     * The request token identifies a particular request and is useful only
     * for matching up responses to requests. This is particularly useful
     * in parallel, forking proxies.
     */
    public Object getRequestToken();
    
    /**
     * Sets the status code and message for this response.
     * @param statusCode    the SIP status code, one of SipConstants.SC_*
     * @param reason        the human readable reason phrase
     * @see SipConstants
     */
    public void setStatus(int code, String reason);
    
    /**
     * Sets the status code for this response.
     * @param statusCode    the SIP status code, one of SipConstants.SC_*
     * @param reason        the human readable reason phrase
     * @see SipConstants
     */
    public void setStatus(int code);
    
    /**
     * Causes this response object to get transmitted. The protocol and
     * destination is chosen according to the value of the Via headers of
     * this object.
     * 
     * <p>If this server is acting as a proxy for this transaction,
     * the Via headers were in the response when it was received and this 
     * method will cause it to strip off the top-most Via address in
     * accordance with RFC 2543 and transmit the request to the next hop.
     * Otherwise, if the response was created by this servlet from a request
     * object, then the Via headers were derived from that request.
     */
    public void send();
}
