package org.ietf.sip;

/**
 * The interface which must be implemented by SIP servlets in order
 * to extend SIP servlet engines.
 */
public interface SipServlet extends SipConstants {
    /**
     * Invoked by the servlet engine to initialize this servlet. This may
     * be a good place to acquire resources. Implementations will typically
     * store the config parameter so as to be able to return it in subsequent
     * calls to getServletConfig.
     * @param config    provides access to parameters of this servlet
     *                  as well as to the servlet context.
     */
    public void init(ServletConfig config);
    
    /**
     * Invoked by the servlet engine to indicate to this servlet that it has
     * received a request. The servlet can proxy the request or respond to it
     * itself.
     * @param req   provides acces to information about the request.
     */
    public boolean gotRequest(SipRequest req) throws ServletException;
    
    /**
     * Invoked by the servlet engine to indicate to this servlet that it has
     * received a response. The servlet can forward the response upstream or
     * not as appropriate.
     * @param res   provides access to information about the response.
     */
    public boolean gotResponse(SipResponse res) throws ServletException;
    
    /**
     * Invoked by the servlet engine to indicate to the servlet that it is
     * being unloaded. This provides the servlet with an opportunity to clean
     * up and release any resources it might have acquired.
     * 
     * <p>After invoking this method the engine shouldn't make any further
     * calls to this object.
     */
    public void destroy();
    
    /**
     * Returns the configuration object of this servlet.
     */
    public ServletConfig getServletConfig();
    
    /**
     * Returns a short plain text string describing this servlet.
     */
    public String getServletInfo();
}
