package org.ietf.sip;

import java.security.AccessControlException;
import java.util.Iterator;
import java.util.List;

/**
 * The SipServletContext represents the servlet engine, that is, the SIP stack,
 * to SipServlets. It provides access to a variety of functionality:
 * logging, server attributes, server software information,
 * factory methods for SIP messages, URLs, etc, as well as access to the
 * servers contact database.
 */
public interface SipServletContext extends SipFactory, ContactDatabase {
    /**
     * Returns the major version number of the SIP Servlet API specification
     * implemented by this object. For this specification 1 must be returned.
     */
    public int getMajorVersion();
    
    /**
     * Returns the minor version number of the SIP Servlet API specification
     * implemented by this object. For this specification 0 must be returned.
     */
    public int getMinorVersion();
    
    /**
     * Returns a string identifying the server software, typically
     * including name and version information.
     */
    public String getServerInfo();
    
    /**
     * Returns the value of the specified attribute, or null if it is
     * undefined.
     */
    public Object getAttribute(String name);
    
    /**
     * Returns an iterator over all defined attributes.
     */
    public Iterator getAttributeNames();
    
    /**
     * Associates the named attribute with the specified value. A set of
     * servlets associated with the same servlet engine sees
     */
    public void setAttribute(String name, Object value);
    
    /**
     * Removes the specified attribute.
     */
    public void removeAttribute(String name);
    
    /**
     * Causes the servlet engine to log the specified string. It is up to
     * the server implementation to decide on an underlying logging
     * mechanism
     */
    public void log(String msg);
    
    /**
     * Logs the specified string and exception.
     */
    public void log(String msg, Throwable t);
}
