package org.ietf.sip;

import java.util.Date;

/**
 * Miscellaneous static utility methods.
 */
public abstract class SipUtils { 
    /**
     * Returns Java time (absolute time in milliseconds since Jan 1 1970 -
     * same unit as returned by System.currentTimeMillis()) corresponding to
     * the specified SIP time.
     * @param time  the time specification. This should be 'delta-seconds'
     *      (an integer) or an RFC 1123 date specification.
     * @return  the Java time corresponding to te argument, or -1 if the
     *      argument is null or fails to parse correctly.
     */
    public static long getJavaTime(String time) {
        if (time != null) {
            try {
                if (time.indexOf(' ') > -1) {
                    return System.currentTimeMillis() + (Long.parseLong(time)*1000L);
                } else {
                    return new Date(time).getTime();
                }
            } catch (Exception _) {}
        }
        return -1;
    }
    
    /**
     * Returns the SIP delta time corresponding to a string which matches
     * <pre>
     *    t = (SIP-date | delta-seconds)
     * </pre>
     * See RFC 2543, sections 6.20 and 6.13.
     * @param time  the time specification
     * @return  the difference between t and the current time in seconds,
     *          or -1 if an error occurred
     */
    public static long getSipDeltaTime(String time) {
        if (time != null && time.length() > 0) {
            try {
                if (time.indexOf(' ') == -1) {
                    return Long.parseLong(time);
                } else {
                    long jt = new Date(time).getTime();
                    return (jt - System.currentTimeMillis())/1000;
                }
            } catch (Exception _) {}
        }
        return -1;
    }
}
