# Columbia SIP makefile fragment for sipua
# Copyright 1999-2000 by Columbia University; all rights reserved 

sipua_dir := $(topdir)/sipua

SIPUA_BIN := \
  $(sipua_dir)/sipua

SIPUA_SRC := \
  $(sipua_dir)/main.cpp \
  $(sipua_dir)/command.cpp \
  $(sipua_dir)/radio.cpp

SIPUA_HEADERS := \
  $(sipua_dir)/radio.h \
  $(sipua_dir)/rtp.h \
  $(sipua_dir)/pcmu_l.h \
  $(sipua_dir)/types.h \
  $(sipua_dir)/command.h


SIPUA_MISC := \
  $(sipua_dir)/module.mk \
  $(sipua_dir)/sipua.dsp \
  $(sipua_dir)/resolv.conf

SIPUA_MISCDIST := \
  $(sipua_dir)/resolv.conf

SIPUA_OBJ := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(SIPUA_SRC)))

SIPUA_DEP := \
        $(SIPUA_OBJ:.o=.d)

# Include files required locally.
SIPUA_INCS := $(SIPAPI_INC) $(LIBSIP_INC) $(LIBCINE_INC)
SIPUA_LIBS := $(SIPAPI_LIB) $(LIBSIP_LIB) $(DBAPI_LIB) $(LIBCINE_LIB) \
	$(LIBRESPARSE) $(LIBDICT_LIB)  \
	$(LDAPLIBS) $(DBAPI_LIB) \
        $(MYSQLLIBS) $(ZLIBLIBS) $(RESOLVLIBS) $(NISLIBS) $(SOCKLIBS)
SIPUA_DEPENDENCIES := $(SIPAPI)
SIPUA_REQS :=
# need-ldap need-db

# Order of -c and -o matters.
$(SIPUA_OBJ): %.o: %.cpp
	@echo "compiling $<..."
	$(CXX) $(SIPUA_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -o $@ -c $<

$(SIPUA_DEP): %.d: %.cpp
	@echo "dependencies for $<..."
	$(DEPEND) -d $(sipua_dir) -o $@ $(SIPUA_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<

# Do not add this --- all: $(SIPUA)
.PHONY: sipua
sipua: $(SIPUA_REQS) $(SIPUA_DEP) $(SIPUA_BIN)

# SIPUA uses libsip++ == SIPAPI

$(sipua_dir)/sipua: $(SIPUA_OBJ) $(SIPUA_DEPENDENCIES)
	@echo "creating application $@..."
	$(CXX) -o $@ -w $(SIPUA_INCS) $(SIPUA_OBJ) \
	$(LDFLAGS) $(SIPUA_LIBS) $(LIBS)

MISC      += $(SIPUA_MISC)
MISCDIST  += $(SIPUA_MISCDIST)
MISCCLEAN += $(SIPUA_OTHER_MISCCLEAN)

.PHONY: sipua-dist

SIPUA_DIST := $(SIPUA_SRC) $(SIPUA_HEADERS) $(SIPUA_MISC) $(SIPUA_MISCDIST)
SIPUA_DIST_NAME := sipua-$(VERSION)-$(DATE)

sipua-dist:
	$(MKDIST) $(topsrcdir) $(SIPUA_DIST_NAME) \
	$(TOP_DIST) \
	$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
	$(LIBDICT_DIST) \
	$(DBAPI_DIST) \
	$(WIN32_DIST) \
	$(SIPAPI_DIST) \
	$(SIPUA_DIST)

SIPUA_BINDIST := $(SIPUA_MISCDIST) $(SIPUA_BIN)
SIPUA_BINDIST_NAME := sipua-$(VERSION)-$(DATE)-$(SYSNAME)

.PHONY: sipua-bindist

sipua-bindist:: $(SIPUA)
	$(MKDIST) $(topsrcdir) $(SIPUA_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIPUA_BINDIST) \
		$(SIPUA_DOCS_BINDIST)


SIPUA_BINDIST_WIN32 := $(SIPUA_MISCDIST) \
		$(sipua_dir)/debug/sipua.exe \
                $(sipua_dir)/release/sipua.exe


SIPUA_BINDIST_WIN32_NAME := sipua-$(VERSION)-$(DATE)-WinNT

sipua-bindist-win32::
	$(MKDIST_WIN32) $(topsrcdir) $(SIPUA_BINDIST_WIN32_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIPUA_BINDIST_WIN32) \
		$(SIPUA_DOCS_BINDIST)
