/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : radio.cpp
#
#  Module       : AIS HW5 exercise 5
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 13 April 2000
#
#  Description  : (Question 5:)
#
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
# 13 Apr 2000  | Kundan      | Modified from HW3.
# 28 Apr 2000  | Kundan      | Modified from HW4.
# 9  Jul 2000  | Kundan      | Modified to have C++ interface for sipua.
#-----------------------------------------------------------------------
*/

#ifdef __cplusplus
extern "C" {
#endif

#include <stdlib.h>     /* getopt */
#include <stdio.h>      /* printf */
#include <sys/time.h>   /* select, struct timeval */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>      /* open */
#include <unistd.h>     /* lseek */
#include <string.h>     /* strdup */
#if !(defined WIN32 || defined __linux__)
  #include <sys/audioio.h>
#else
  #include "sysdep.h"
#endif
#include <sys/socket.h>  /* socket */
#include <netinet/in.h> 
#include <arpa/inet.h>   /* inet_ntoa */
#include <pthread.h>    /* pthread */
#include <signal.h>      /* kill */
#include "pthread-util.h"

#ifdef __cplusplus
}
#endif

#include <math.h>        /* fabs */
#include "radio.h"
#include "sipapi.h"

#if !(defined RTP_LITTLE_ENDIAN || defined RTP_BIG_ENDIAN)
#define RTP_BIG_ENDIAN 1 /* default is solaris on sparc */
#endif

#include "pcmu_l.h"
#include "rtp.h"

#define DEFAULT_THRESHOLD 0 // Change this to 500000
#define DEFAULT_PACKET_SIZE 160 

void Initialize();
int start_audio_tool(int usedevice);
void stop_audio_tool();
unsigned short start_listen_port(unsigned short port);
int set_remote_mediaaddr(struct sockaddr_in *sin);

#ifndef WIN32
#define closesocket close
#endif

static unsigned short portBase = 10000;

extern bool measure_latency;
struct timeval times[256];

AudioTool::AudioTool()
{
  tool = NULL;
  selfport = 0;
  remoteaddr.sin_family = AF_INET;
  remoteaddr.sin_addr.s_addr = INADDR_ANY;
  remoteaddr.sin_port = 0;
  active = false;
  externaltool = 0;
}

AudioTool::AudioTool(const char *path)
{
  tool = (path != NULL) ? strdup(path) : NULL;
  selfport = 0;
  remoteaddr.sin_family = AF_INET;
  remoteaddr.sin_addr.s_addr = INADDR_ANY;
  remoteaddr.sin_port = 0;
  active = false;
  externaltool = 0;
}

AudioTool::~AudioTool()
{
  StopTool();

  if ( tool != NULL ) {
    free(tool);
  }
}

void AudioTool::StartTool(bool usedevice)
{
  if ( !active ) {
    if ( tool == NULL ) {
      /* Internal tool */
      Initialize();
      start_audio_tool(((usedevice == true) ? 1 : 0));
      set_remote_mediaaddr(&remoteaddr);
    }
    else {
      /* external tool */
      char command[500];
      if ( selfport == 0 || selfport == ntohs(remoteaddr.sin_port)) {
        sprintf(command, "%s %s/%d", tool,
                inet_ntoa(remoteaddr.sin_addr),
                ntohs(remoteaddr.sin_port));
      }
      else {
        sprintf(command, "%s %s/%d/%d", tool,
                inet_ntoa(remoteaddr.sin_addr),
                ntohs(remoteaddr.sin_port),
                selfport);
      }
      cout << "audiotool: " << command << endl;
      if ( (externaltool = fork()) == 0 ) {
        /* child runs the tool */
        system(command);
        exit(0);
      }
      else {
        /* parent continues */
        if ( externaltool < 0 ) {
          /* Error */
          perror("fork");
          externaltool = 0;
        }
      }
    }
    active = true;
  }
  else {
    cerr << "StartTool: AudioTool is already active" << endl;
  }
}

void AudioTool::StopTool()
{
  if ( active ) {
    selfport = 0;
    if ( tool == NULL ) {
      /* internal tool */
      stop_audio_tool();
    }
    else {
      /* external tool */
      if ( externaltool > 0 ) {
        kill(externaltool, SIGKILL);
      }
      else {
        cerr << "No subprocess for audio tool" << endl;
      }
    }
    active = false;
  }
  else {
    cerr << "StopTool: AudioTool is not active" << endl;
  }
  externaltool = 0;
}

void AudioTool::SetSelfPort(unsigned short port)
{
  selfport = port;
}

unsigned short AudioTool::GetSelfPort()
{
  if ( selfport == 0 ) {
    /* Allocate and bind socket, preferably to remote port */
    if ( tool == NULL ) {
      return (selfport = start_listen_port(selfport));
    }
    else {
      selfport = ::portBase;
      ::portBase += 2;
    }
  }
  return selfport;
}

void AudioTool::SetRemoteAddress(const struct sockaddr_in sin)
{
  remoteaddr = sin;
}

void AudioTool::GetRemoteAddress(struct sockaddr_in & sin)
{
  sin = remoteaddr;
}

//----------------------------------------------------------------

/* C functions for implementing the 
 * playout delay buffer */

typedef struct ThreadParam {
  int in_sock;  /* Recv socket */
  int out_sock; /* send socket */
  int au_dev;   /* audio device */
  int o_send;   /* Flag to send the packets */
  int o_silence;    /* silence suppression flag */
  double threshold; /* SS thread hold */
} ThreadParam;

#define SQR(x) (x)*(x)

extern struct in_addr host2ip(char *host);

/*
 * Caculate the SNR value
 */
double EnergyLevel (unsigned char *dataBuf, int dataLen) 
{
  int i, n;
  double sum_x, avg_x;

  n= dataLen;

  /* Following portion of the code is copied from HW2 */
  for (sum_x=0, i=0; i < n; i++) { sum_x += pcmu_l16(dataBuf[i]); }
  avg_x = sum_x/n;
  for (sum_x=0, i=0; i < n; i++) { sum_x += 
                SQR(pcmu_l16(dataBuf[i]) - avg_x); }
  avg_x = sum_x/n;

  return (avg_x);
}

/*
 * Play out circular buffer parameters. 
 */
#define MAX_PLAYOUT_BUFFER_SIZE 8000  /* for 1 seconds */
char p_buf[MAX_PLAYOUT_BUFFER_SIZE];
volatile int  r_ptr = 0; /* read pointer */
volatile int  w_ptr = 0; /* write pointer */
double alpha = 0.998;
double mu    = 8;
volatile double di = 0; /* initial values are 0 */
volatile double vi = 0;
long INITIAL = 0;
long INITIALOFFSET = 0; /* 200ms */
long INITIALPJ = 0;
volatile long  pi = 0, pj = 0;
volatile long  ti = 0, tj = 0;
volatile double  ni = 0;
pthread_mutex_t mutex;
int oConnected = 0;
int oActive    = 0;

/* Initialise the globals */

void Initialize()
{
  r_ptr = 0; /* read pointer */
  w_ptr = 0; /* write pointer */
  alpha = 0.998;
  mu    = 8;
  di = 0; /* initial values are 0 */
  vi = 0;
  INITIAL = 0;
  INITIALOFFSET = 0; /* 200ms */
  INITIALPJ = 0;
  pi = 0; pj = 0;
  ti = 0; tj = 0;
  ni = 0;
  oConnected = 0;
  oActive = 0;
}

/* Increments the index by the offset, takes care of wrapping */
int AddIndex(int oldindex, int increment) 
{
  return (oldindex + increment) % MAX_PLAYOUT_BUFFER_SIZE;
}

/* 
Compares the two indices, read and write.
retuns >0 is read is more than write, 0 if
both are same and <0 if read is less than write.
Ideal buffer will always have <= 0 return value 
*/
int CompareIndex(int r_idx, int w_idx)
{
  if ( r_idx < w_idx ) return -1;
  if ( r_idx > w_idx ) return -1;
  if ( r_idx == w_idx) return 0;
  return 1; /* Not correct */
}

/* CopyTo copies from p_buf to packet from r_ptr */
int CopyTo(char *buf, int len)
{
  int i;

  for (i=0; i<len; i++) {
    if ( r_ptr == w_ptr ) { /* empty */
      buf[i] = 0; /* silence */
    }
    else {
      buf[i] = p_buf[r_ptr];
      p_buf[r_ptr] = 0;

    }
    if ( r_ptr == w_ptr ) {
      //fprintf(stderr,"r>w ");
      w_ptr ++;
      if ( w_ptr == MAX_PLAYOUT_BUFFER_SIZE ) {
        w_ptr = 0;
      }
    }
    r_ptr++;
    if ( r_ptr == MAX_PLAYOUT_BUFFER_SIZE ) {
      r_ptr = 0;
    }
  }
  return 0;
}

/* CopyFrom copies to the p_buf from the packet */
int CopyFrom(unsigned index, char *buf, int len)
{
  int i;

  index = w_ptr;

  for (i=0; i<len; i++) {
    p_buf[index] = buf[i];
    if ( index == w_ptr ) {
      w_ptr++;

      if ( w_ptr == MAX_PLAYOUT_BUFFER_SIZE ) {
        w_ptr = 0;
      }
      if ( w_ptr == r_ptr ) {
        p_buf[r_ptr] = 0; /* make it empty and increment */
        r_ptr = ( r_ptr + 1 ) % MAX_PLAYOUT_BUFFER_SIZE;
      }
    }
    index++;
    if ( index == MAX_PLAYOUT_BUFFER_SIZE ) {
      index = 0;
    }
  }
  return 0;
}

/* PutPacket: Put into the playout buffer */
int PutPacket(char *buffer, int len, long ai) /* Includes RTP header */
{
  rtp_hdr_t *rtp = (rtp_hdr_t *) buffer;
  static int first_time = 1;
  static unsigned last_seq = 0;
  int index;
  static int late_count = 0;

  pthread_mutex_lock(&mutex);
  
  tj = rtp->ts;

  last_seq ++;
  if ( first_time ) {
    di = (double) (ai - tj);
    INITIALPJ = (long) di;
    last_seq = rtp->seq;
  }

  if ( rtp->seq < last_seq ) {
    late_count++;
    //fprintf(stderr,"Received late, total count = %d\n", late_count);
    last_seq = rtp->seq;
  }

  ni = (double) (ai - tj); /* network delay */
  di = alpha * di + ( 1-alpha) * ni;
  vi = alpha * vi * ( 1-alpha) * fabs( (double) di - (double) ni);

  /*
  fprintf(stderr, "%u-%u=ni%g\n",
          ai,tj, ni - INITIALPJ);
  */
  if ( rtp->m == 1 ) {
    ti = tj;
    /* First packet in talk spurt */
    pi = ti + di + mu * vi;
    pj = pi;

    //fprintf(stderr, "relative delay is %d\n", ((long)di) - INITIALPJ); 
  } 
  else {
    /* Subsequent packets in talk spurt */
    pj = pi + tj - ti; 
  }

  len    -= 12; /* For RTP */
  buffer += 12;
 
  /* Map 'pj' to index in buffer */
  if ( first_time ) {
    first_time = 0;
    INITIAL = -pj + w_ptr; /* Initial corresponds to offset 0 in buffer */

    index = ( (unsigned) (  pj - INITIAL + INITIALOFFSET))
      % MAX_PLAYOUT_BUFFER_SIZE;
  }
  else {
    index = ( (unsigned) (  pj - INITIAL))
      % MAX_PLAYOUT_BUFFER_SIZE;
  }

  CopyFrom(index,buffer,len);
  //fprintf(stderr,"Storing at %d,ts%u,rptr%d,%d\n", index, pj, w_ptr, len);

  pthread_mutex_unlock(&mutex);

  return 0;
}

/* GetPacket: Get from the playout buffer */
int GetPacket(char *buffer, int len) /*does not include RTP */
{
  pthread_mutex_lock(&mutex);

  //fprintf(stderr, "Getting from %d,%d\n", r_ptr, len);
  CopyTo(buffer, len);

  pthread_mutex_unlock(&mutex);

  return len;
}

/* Global sequence number and time stamp */
unsigned short seq = 0; /* Some random numner */
unsigned long ts = 0; /* some random number */
static int oUsingDevice = 1;

/* 
 * Thread used to record from audio device and sent to
 * remote.
 * This thread also reads from the playout buffer and
 * plays to audio device.
 */
void *RecordThread(void *ptr) 
{
  ThreadParam *th = (ThreadParam *) ptr;
  unsigned char buffer[1500];
  int len;
  int size;
  int recv_count = 0;
  int send_count = 0;
  rtp_hdr_t *rtp;
  int in_talk_spurt = 0;
  unsigned long ssrc;

  ssrc = abs(rand() % 100000);

  fprintf(stderr, "Record thread started\n");

  size = DEFAULT_PACKET_SIZE; /* Read 160 samples 20 ms worth data */

  while ( oActive ) {

    if ( recv_count >= 100 ) {
      /*
      printf("Transmitted %d out of %d recorded\n",
              send_count, recv_count);
      */
      recv_count = 0;
      send_count = 0;
    }

    rtp = (rtp_hdr_t *) buffer;
    rtp->version = 2;
    rtp->p       = 0;
    rtp->x       = 0;
    rtp->cc      = 0;
    rtp->pt      = 0;
    rtp->ssrc    = ssrc; /* some random number */
    rtp->m = (in_talk_spurt ? 0 : 1 );
    in_talk_spurt = 1;
    rtp->seq     = seq;
    rtp->ts      = ts;

    if ( oUsingDevice ) {    
      len = read(th->au_dev, &buffer[12], size);
    }
    else {
      unsigned char a;
      struct timeval tv;
      tv.tv_sec = 0;
      tv.tv_usec = (size) * 125; /* 8bytes == 1 millisec */
#ifdef WIN32
      Sleep(tv.tv_usec / 1000);
#else
      select(0, NULL, NULL, NULL, &tv);
#endif

      gettimeofday(&tv, NULL);
      /* 2 bits from seconds and 6 bits from micro seconds */
      a = ((tv.tv_sec % 10) * 1000 + (tv.tv_usec /1000)) & 0x0ff;
      memset(&buffer[12], a, size);

      times[seq & 0x0ff] = tv;

      len = size;
      //printf("Time for %d: %u:%u\n", buffer[12], tv.tv_sec, tv.tv_usec);
    }

    if ( len != size ) {
      memset (&buffer[12+len],0,size-len);
      //len = size;
    }

    if ( (len < size) || (len <= 0) ) {
      if ( len < 0 ) {
        len = 0;
      }

      /* Should sleep for some time, proportional to size */
      //perror("audio read error");
      struct timeval tv;
      tv.tv_sec = 0;
      tv.tv_usec = (size - len) * 125; /* 8bytes == 1 millisec */
      //fprintf(stdout, "RecordThread: waiting for %lu ms\n", tv.tv_usec );
#ifdef WIN32
      Sleep(tv.tv_usec / 1000);
#else
      select(0, NULL, NULL, NULL, &tv);
#endif
    }

    ts += len; /* Increment ts by 160 */

    recv_count ++;

    if ( th->o_send ) {  /* transmit if needed */
      int o_silence = 0;
      if ( th->o_silence ) {      
        if ( EnergyLevel(&buffer[12], len) < 
             th->threshold ) {
          o_silence = 1; /* Do not transmit */
          in_talk_spurt = 0; /* not in talk spurt */
        } 
      }
      if ( !o_silence ) {
        if ( len > 0 ) {
          if ( oConnected ) {
            write(th->out_sock, buffer, len+12);
            send_count ++;
            seq ++;
            in_talk_spurt = 1; /* In talk spurt */
          }
        }
      }
    }

    /* Play out */
    GetPacket((char *) buffer, len); /* Get len number of samples from playout
                               buf */
    if ( oUsingDevice ) {
      len = write(th->au_dev, buffer, len);
    }
  }

  free(th);

  fprintf(stderr, "Record thread is terminating\n");

  return NULL;
}

/* 
 * Thread to recv the packet and put it in the
 * playout buffer
 */
void *PlayThread(void *ptr) 
{
  ThreadParam *th = (ThreadParam *) ptr;
  unsigned char buffer[1500];
  int len;
  int size;
  int recv_count = 0;

  fprintf(stderr, "Play thread started\n");

  size = sizeof(buffer);

  while ( oActive ) {

    if ( recv_count >= 100 ) {
      /*
      printf("%d received\n",
              recv_count);
       */
      recv_count = 0;
    }

    len = read(th->in_sock, buffer, size);

    /*
     * Display the bitrate of incoming stream.
     */
    {
      static time_t lasttime = 0;
      static long bytes = 0;
      time_t thistime = 0;

      if ( lasttime == 0 ) {
        lasttime = time(0);
      }
      bytes += (len - 12);

      thistime = time(0);
      if ( thistime > lasttime ) {
        printf("Bytes per sec=%ld, (%g kb/s)\n",
               bytes, ((bytes / (thistime - lasttime)) *
                       (64.0/8000.0)));
        lasttime = thistime;
        bytes = 0;
      }
      
    }

    /* Typecast to RTP header and print the timestamp */
    //printf("TS=%lu\n", *((unsigned long *) &buffer[4])); 
    /*
    printf("First byte=%x: Padding byte(%d) is %u\n", 
           *((unsigned char *) buffer),
           len-1,
           *((unsigned char *) &buffer[len-1]));
           */

    if ( len <= 0 ) {
      //perror("recv error");
      struct timeval tv;
      tv.tv_sec = 0;
      tv.tv_usec = 160 * 125; /* 8bytes == 1 millisec */
      //fprintf(stdout, "RecordThread: waiting for %lu ms\n", tv.tv_usec );
#ifdef WIN32
      Sleep(tv.tv_usec / 1000);
#else
      select(0, NULL, NULL, NULL, &tv);
#endif
    }
    recv_count ++;

    //fprintf(stdout, "Received %d bytes\n", len);

    /* Print the debug info for latency */
    {
      struct timeval tv, tv2;
      double dt;
      unsigned char a;

      gettimeofday(&tv2, NULL);

      //printf("Current Time for %u: %u:%u\n", buffer[12], tv2.tv_sec, tv2.tv_usec);

      tv = times[(unsigned) buffer[12]];

      dt = (tv2.tv_sec - tv.tv_sec ) * 1000.0 +
        (tv2.tv_usec - tv.tv_usec) / 1000.0;

      a = buffer[12];
      dt = (unsigned long) (((tv2.tv_sec % 10) * 1000 + (tv.tv_usec/1000)) & 0x0ff)
        - (unsigned long) a;
      /* printf("Latency is %g ms\n", dt); */
    }

    /* Put the packet in the play out buffer */
    PutPacket((char *)buffer, len, ts);
  }

  fprintf(stderr, "Play thread is terminating\n");

  free(th);

  return NULL;
}

/* Main routine */
static int in_sock      = -1;  
static int out_sock     = -1;
static int in_port      = 0;
static int  au = -1;

int start_audio_tool(int usedevice)
{
  int ac;
  struct sockaddr_in sin;
  int o_send       = 1; /* send multicast packets */
  int o_recv       = 1; /* Receive packets */
  int o_multicast  = 1;
  int o_loop       = 0; /* Do not loop back multicast packets */
  int o_silence    = 1; /* enable silence suppression */
  double threshold = DEFAULT_THRESHOLD;
#if !(defined WIN32 || defined __linux__)
  audio_info_t ai;
  mode_t mode      = O_RDWR;
#endif

  oUsingDevice = usedevice;

  srand(time(NULL));
  ts = abs(rand() % 100000);
  seq = rand();

#if !(defined WIN32 || defined __linux__)
  if ( usedevice ) {
    fprintf(stderr, "opening audioctl...\n");
    if ( (ac = open("/dev/audioctl", O_RDWR | O_NDELAY)) < 0 ) {
      perror("open, audioctl");
      //return 1;
    }
    fprintf(stderr, "...done\n");

    /* Set speaker and microphone ports */
    AUDIO_INITINFO(&ai);
    ai.record.port = AUDIO_MICROPHONE;
    ai.play.port = AUDIO_SPEAKER;
    ai.record.sample_rate = 8000;
    ai.record.channels = 1;
    ai.record.encoding = 1; /* Mu law */
    ai.play.sample_rate = 8000;
    ai.play.channels = 1;
    ai.play.encoding = 1;

    /* 
     * XXX somehow the audio info does not seem to be set,
     * for encoding. Use "cp bg.au /dev/audio" before
     * running this program to set the encoding.
     */ 
    ioctl(ac, AUDIO_SETINFO, &ai);
    close(ac);
  }

  /* Open audio device */
  if ( au >= 0 ) {
    close (au);
  }

  if ( usedevice ) {
    fprintf(stderr, "opening audio device...\n");
    //mode |= (O_NDELAY);
    if ( (au = open("/dev/audio", mode | O_NDELAY | O_NONBLOCK)) < 0 ) {
      perror("open audio");
      //return 1;
    }
    fprintf(stderr, "...done\n");

    //fcntl(au, F_SETFL, 0 /*O_NDELAY*/);

  }
#endif
  
  /* Open the socket for listening and sending */
  if ( in_sock == -1 ) {
    in_port = start_listen_port(in_port);
  }
  else {
    /* Socket is already bound */
  }

  /* Open the socket for listening and sending */
  if ((out_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) {
    perror("socket(out)");
    return 1;
  }


  /* Initialize the mutex */
  pthread_mutex_init(&mutex, NULL);

  oActive = 1;

  /* 
   * Setup threads to listen to incoming packets and send outgoing
   * packets 
   */
  {
    int  e;
    pthread_t tid[2];

    /* Recv thread */
    ThreadParam *th;
    th = (ThreadParam *)malloc(sizeof(ThreadParam));
    th->in_sock = in_sock;
    th->out_sock = out_sock;
    th->au_dev = au;
    th->o_send = o_send; /* Don't care */
    th->o_silence = 0; /* No silence detection for playout */
    
    //fprintf(stdout, "pthread_create(PlayThread)\n");
    if ((e = pthread_create_detached(&tid[0], NULL, 
                            PlayThread, (void *)th)) != 0) {
      perror("pthread_create_detached");
    }

    /* Record/send and play  thread */
    th = (ThreadParam *)malloc(sizeof(ThreadParam));
    th->in_sock = in_sock;
    th->out_sock = out_sock;
    th->au_dev = au;
    th->o_send = o_send;

    th->o_silence = o_silence;
    th->threshold = threshold;
    
    //fprintf(stdout, "pthread_create(RecordThread)\n");
    if ((e = pthread_create_detached(&tid[1], NULL, 
                            RecordThread, (void *)th)) != 0) {
      perror("pthread_create_detached");
    }

  }

  return 0;
}

void stop_audio_tool()
{
  oActive = 0; /* This terminates the threads */
#if !(defined WIN32 || defined __linux__)
  if ( au >= 0 ) {
    close(au);
  }
#endif
  au = -1;

  oUsingDevice = 1;

  closesocket(in_sock);
  closesocket(out_sock);
  in_sock = -1;
  out_sock = -1;
  in_port = 0;
  pthread_mutex_destroy(&mutex);
}

unsigned short start_listen_port(unsigned short port)
{
  if ( in_sock == -1 ) {
    struct sockaddr_in sin;
    int sinlen;

    while ( 1 ) {    
      if ((in_sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) {
        perror("socket(in)");
        return 1;
      }

      sin.sin_family = AF_INET;
      sin.sin_addr.s_addr = INADDR_ANY;
      sin.sin_port = htons(port);
      if ( (bind(in_sock, (struct sockaddr *)&sin, sizeof(sin))) < 0) {
        perror("bind");
        return 1;
      }

      sinlen = sizeof(sin);
      if ( getsockname(in_sock, (struct sockaddr *)&sin, &sinlen) < 0 ) {
        perror("getsockname");
      }
      in_port = ntohs(sin.sin_port);

      if ( port == 0 ) {
        if ( in_port % 2 == 0 ) {
          break;
        }
        else {
          closesocket(in_sock);
          in_sock = -1;
          /* Continue */
        }
      }
      else {
        break;
      }
    }
  }

  printf("Audio tool listening at %u\n", in_port);
  return in_port;
}

int set_remote_mediaaddr(struct sockaddr_in *sin)
{
  printf("Audio tool connecting to %s:%u\n", 
         inet_ntoa(sin->sin_addr), ntohs(sin->sin_port));

  if ((connect(out_sock, (struct sockaddr *)sin, 
               sizeof(struct sockaddr_in))) < 0 ) {
    perror("connect");
    return 0;
  }
  oConnected = 1;
  return 0;
}

//---------------------------------------------------------------------

/*
** This table converts from ulaw to 8 and 16 bit linear, faster.
**
** Jef Poskanzer
** 23 October 1989
**
** Input: 8 bit ulaw sample
** Output: signed 16 bit linear sample
*/

int16 pcmu_l16_table[256] = {
/*   0 */  -32124, -31100, -30076, -29052, -28028, -27004, -25980, -24956,
/*   8 */  -23932, -22908, -21884, -20860, -19836, -18812, -17788, -16764,
/*  16 */  -15996, -15484, -14972, -14460, -13948, -13436, -12924, -12412,
/*  24 */  -11900, -11388, -10876, -10364,  -9852,  -9340,  -8828,  -8316,
/*  32 */   -7932,  -7676,  -7420,  -7164,  -6908,  -6652,  -6396,  -6140,
/*  40 */   -5884,  -5628,  -5372,  -5116,  -4860,  -4604,  -4348,  -4092,
/*  48 */   -3900,  -3772,  -3644,  -3516,  -3388,  -3260,  -3132,  -3004,
/*  56 */   -2876,  -2748,  -2620,  -2492,  -2364,  -2236,  -2108,  -1980,
/*  64 */   -1884,  -1820,  -1756,  -1692,  -1628,  -1564,  -1500,  -1436,
/*  72 */   -1372,  -1308,  -1244,  -1180,  -1116,  -1052,   -988,   -924,
/*  80 */    -876,   -844,   -812,   -780,   -748,   -716,   -684,   -652,
/*  88 */    -620,   -588,   -556,   -524,   -492,   -460,   -428,   -396,
/*  96 */    -372,   -356,   -340,   -324,   -308,   -292,   -276,   -260,
/* 104 */    -244,   -228,   -212,   -196,   -180,   -164,   -148,   -132,
/* 112 */    -120,   -112,   -104,    -96,    -88,    -80,    -72,    -64,
/* 120 */     -56,    -48,    -40,    -32,    -24,    -16,     -8,      0,
/* 128 */   32124,  31100,  30076,  29052,  28028,  27004,  25980,  24956,
/* 136 */   23932,  22908,  21884,  20860,  19836,  18812,  17788,  16764,
/* 144 */   15996,  15484,  14972,  14460,  13948,  13436,  12924,  12412,
/* 152 */   11900,  11388,  10876,  10364,   9852,   9340,   8828,   8316,
/* 160 */    7932,   7676,   7420,   7164,   6908,   6652,   6396,   6140,
/* 168 */    5884,   5628,   5372,   5116,   4860,   4604,   4348,   4092,
/* 176 */    3900,   3772,   3644,   3516,   3388,   3260,   3132,   3004,
/* 184 */    2876,   2748,   2620,   2492,   2364,   2236,   2108,   1980,
/* 192 */    1884,   1820,   1756,   1692,   1628,   1564,   1500,   1436,
/* 200 */    1372,   1308,   1244,   1180,   1116,   1052,    988,    924,
/* 208 */     876,    844,    812,    780,    748,    716,    684,    652,
/* 216 */     620,    588,    556,    524,    492,    460,    428,    396,
/* 224 */     372,    356,    340,    324,    308,    292,    276,    260,
/* 232 */     244,    228,    212,    196,    180,    164,    148,    132,
/* 240 */     120,    112,    104,     96,     88,     80,     72,     64,
/* 248 */      56,     48,     40,     32,     24,     16,      8,      0
};

char pcmu_l8_table[256] = {
/*   0 */ -125, -121, -117, -113, -109, -105, -101, -97,
/*   8 */  -93, -89, -85, -81, -77, -73, -69, -65,
/*  16 */  -62, -60, -58, -56, -54, -52, -50, -48,
/*  24 */  -46, -44, -42, -40, -38, -36, -34, -32,
/*  32 */  -31, -30, -29, -28, -27, -26, -25, -24,
/*  40 */  -23, -22, -21, -20, -19, -18, -17, -16,
/*  48 */  -15, -15, -14, -14, -13, -13, -12, -12,
/*  56 */  -11, -11, -10, -10,  -9,  -9,  -8,  -8,
/*  64 */   -7,  -7,  -7,  -7,  -6,  -6,  -6,  -6,
/*  72 */   -5,  -5,  -5,  -5,  -4,  -4,  -4,  -4,
/*  80 */   -3,  -3,  -3,  -3,  -3,  -3,  -3,  -3,
/*  88 */   -2,  -2,  -2,  -2,  -2,  -2,  -2,  -2,
/*  96 */   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
/* 104 */   -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
/* 112 */    0,   0,   0,   0,   0,   0,   0,   0,
/* 120 */    0,   0,   0,   0,   0,   0,   0,   0,
/* 128 */  125, 121, 117, 113, 109, 105, 101,  97,
/* 136 */   93,  89,  85,  81,  77,  73,  69,  65,
/* 144 */   62,  60,  58,  56,  54,  52,  50,  48,
/* 152 */   46,  44,  42,  40,  38,  36,  34,  32,
/* 160 */   31,  30,  29,  28,  27,  26,  25,  24,
/* 168 */   23,  22,  21,  20,  19,  18,  17,  16,
/* 176 */   15,  15,  14,  14,  13,  13,  12,  12,
/* 184 */   11,  11,  10,  10,   9,   9,   8,   8,
/* 192 */    7,   7,   7,   7,   6,   6,   6,   6,
/* 200 */    5,   5,   5,   5,   4,   4,   4,   4,
/* 208 */    3,   3,   3,   3,   3,   3,   3,   3,
/* 216 */    2,   2,   2,   2,   2,   2,   2,   2,
/* 224 */    1,   1,   1,   1,   1,   1,   1,   1,
/* 232 */    1,   1,   1,   1,   1,   1,   1,   1,
/* 240 */    0,   0,   0,   0,   0,   0,   0,   0,
/* 248 */    0,   0,   0,   0,   0,   0,   0,   0
};

/************************** END OF FILE ********************************
*/

