#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# Above lines are more portable than hardcoding the tclsh path.

# Installation script for sipum system from
# binary distribution.

proc waitforuser {} {
    puts -nonewline "\nPress <return> to continue ..."
    gets stdin dummy
}

proc getpath {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    while { [gets stdin dummy] > 0} {
        if { [file exists "$dummy"] == 1} {
            return $dummy
        } else {
            puts "This path does not exists. Enter another."
            puts -nonewline "> "
            flush stdout
        }
    }
    puts "Assuming default value as\n $def"
    return $def
}

proc getstring {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    if { [gets stdin dummy] > 0} {
        return $dummy
    } else {
        puts "Assuming default value as $def"
        return $def    
    }
}

puts "sipum, (c) 2000, Columbia University, installation script."
puts "  If the installation gets terminated abmormally, please
  uncompress the binary distribution again and then try."

if { $argv0 != "./install.tcl" } {
    puts "Change your current directory to the directory where
  install.tcl is present and invoke the script as ./install.tcl"
    exit 1;
}

set installdir [file dirname [pwd]]

puts "Installation directory is $installdir"

set hostname "localhost"

# Inform the columbia server than installation is being done.
if { [catch {set sock [socket "www.cs.columbia.edu" 80] }] == 0 } {
    catch {set hostname [lindex [fconfigure $sock -sockname] 1]}
    catch {close $sock}
}

set vmailserver "$hostname:10022"
set vmailserver [getstring "
Enter the host name and port number (for configuration) of the 
voice mail server. default is $vmailserver" $vmailserver]

set rtspserver "$hostname:554"
set rtspserver [getstring "
Enter the host name and port number of the 
RTSP media server. default is $rtspserver" $rtspserver]

set httpserver "$hostname"
set dummylist [split $httpserver .]
set dummylist [lrange $dummylist 1 [llength $dummylist]]
set domain    [join $dummylist .]
set dummylist [linsert $dummylist 0 www]
set httpserver [join $dummylist .]

set httpserver [getstring "
Enter the HTTP server address you will use.
default is $httpserver" $httpserver]

set emaildomain $domain
set emaildomain [getstring "
Enter the email domain you will be using.
default is $emaildomain" $emaildomain]
puts ""

set LOGNAME [exec whoami]
puts "Administrator's login id is $LOGNAME"

set adminemail "$LOGNAME@$emaildomain"
puts "Administrator's email is $adminemail"

set vmailscriptdir "$installdir/sipum/script"
puts "Script directory is $vmailscriptdir"

set vmaildocsdir "$installdir/sipum/docs"
puts "Script directory is $vmaildocsdir"

set vmailtoolsdir "$installdir/sipum/tools"
set sipumtoolsdir   "$vmailtoolsdir"
puts "Script directory is $vmailtoolsdir"

set sipuminstalldir "$installdir/sipum"
puts "Directory with vmail.conf is $sipuminstalldir"

set defaultdomain "voicemail.com"
puts "Default voicemail domain is $defaultdomain. This is not important."

set vmailhomeurl "http://$httpserver/~$LOGNAME/software/sipum/docs"
set vmailhomeurl [getstring "
Enter the SIPUM home page address you will be using.
Make sure to create a softlink to sipum/docs for this path.
default is $vmailhomeurl" $vmailhomeurl]

# Construct script url from home url.
set dummylist [split $vmailhomeurl / ]
if { [lindex $dummylist [expr [llength $dummylist] - 1]] == {} } {
    puts "$dummylist"
    set dummylist [lrange $dummylist 0 [expr [llength $dummylist] - 2]]
}
set $vmailhomeurl [join $dummylist /]

set vmailscripturl [join [lrange $dummylist 0 [expr [llength $dummylist] - 2]] /]
set vmailscripturl "$vmailscripturl/script"
set vmailscripturl [getstring "
Enter the SIPUM script directory address (URL) you will be using.
This directory should contain the script vmail.cgi and other relavent files.
Make sure to map $vmailscriptdir to this link.
default is $vmailscripturl" $vmailscripturl]

set vmailuserdir  "/home/$LOGNAME/.vmusers"
set vmailuserdir [getpath "
Enter the absolute directory path where you would like to create 
per-user directory. Default is $vmailuserdir" $vmailuserdir]
puts ""

set cgitcldir   "/usr/local/lib/cgi1.2"
set cgitcldir [getpath "
Enter the absolute directory path where you have installed 
cgi-tcl (http://expect.nist.gov/cgi.tcl/
If you have not installed cgi-tcl then please install it
and re-run this script.
On CS machines it is /u/kon/hgs/lib/cgi1.2
Default location is $cgitcldir" $cgitcldir]

set tclpath  "/usr/bin/tclsh"
if { [catch {set tclpath [exec which tclsh]}] == 0 } {
    puts "Tcl is detected at $tclpath"
} else {
    puts "Tcl is not installed. Please install it (http://dev.scriptics.com),
add it in your PATH environment variable
and re-run this installation script."
    exit 1
}

set rtspdserverroot "$installdir/rtspd"
puts "Directory with rtspd.conf is $rtspdserverroot"

set rtspddocumentroot "$vmailuserdir"
puts "rtspd document root is $vmailuserdir"

set onbusy "rtsp://SERVER/audio/welcome.au"
catch {exec mkdir -p "$rtspddocumentroot"}
catch {exec ln -s "$installdir/sipum/audio" "$rtspddocumentroot/audio"}

puts "RTSP onbusy message URL is $onbusy"

set noresponse "$onbusy"
puts "RTSP noresponse message URL is $noresponse"

set topdir "$vmailuserdir/LOGNAME/"
puts "topdir is $topdir"

set msgfile "$topdir/inbox/MSGID.au"
puts "msgfile is $msgfile"

set rtspmsgurl "rtsp://SERVER/LOGNAME/inbox/MSGID.au"
puts "rtspmsgurl is $rtspmsgurl"

set httpmsgurl "$vmailscripturl/vmail.cgi"
puts "httpmsgurl is $httpmsgurl"

set vmailhome  "$vmailscripturl/vmail.cgi"
puts "vmailhome is $vmailhome"

set register "LOGNAME@DOMAIN default"
puts "register is $register"

set waittimeout "10"
puts "waittimeout is $waittimeout"

set emailto "LOGNAME@DOMAIN"
puts "emailto is $LOGNAME@DOMAIN"

set randomdigits "6"
puts "randomdigits is $randomdigits"

set totalsize_KB "2000"
puts "totalsize_KB is $totalsize_KB"

set max_msgsize_kb "1000"
puts "max_msgsize_kb is $max_msgsize_kb"

puts "\nI will create the relevant configuration files now...."
waitforuser

puts "Creating $rtspdserverroot/rtspd.conf..."
set f [open "$rtspdserverroot/rtspd.conf" "w" 0644 ]
puts $f "DocumentRoot $rtspddocumentroot
CNAME        $adminemail
ServerRoot   $rtspdserverroot"
catch {close $f}

puts "Creating $vmailscriptdir/config.tcl..."
set f [open "$vmailscriptdir/config.tcl" "w" 0755 ]
puts $f "#! /dev/null
# This file is auto-generated by install.tcl
# Add the library path for cgi-tcl. (cgi.tcl package) and include
# the package.

lappend auto_path $cgitcldir

# URL of the home page and terms of service page of the software.
# Do not edit this XXX.
set g_vmailhomeurl  \"$vmailhomeurl\"

# URL of the voice mail viewer script.
set g_vmailscripturl \"$vmailscripturl\"

# Directory for voicemail script package.
set g_vmailscriptdir \"$vmailscriptdir\"

# Email of the administrator
set g_adminemail      \"$adminemail\"

# Default domain for the voicemail account.
# Voice mail accounts are registered as
# userlogin@defaultdomain.
set g_defaultdomain \"$defaultdomain\"

# Top level directory, under which all user accounts are maintained.
set g_vmailuserdir \"$vmailuserdir\"

# directory where sipum is installed. (list of all users).
# The config file (vmail.conf) is assumed to be in this directory.
set g_sipuminstalldir \"$sipuminstalldir\"

# Directory where the sipum tools are available.
# Tools used are sipum-conf, etc.
set g_sipumtoolsdir  \"$sipumtoolsdir\"


# Following parameters are copied from the default config file for
# users available in vmailscriptdir/config.
set g_vmailserver \"$vmailserver\"
set g_rtspserver  \"$rtspserver\"
set g_httpserver  \"$httpserver\"
set g_emaildomain \"$emaildomain\"

set g_onbusy      \"$onbusy\"
set g_noresponse  \"$noresponse\"
set g_topdir      \"$topdir\"
set g_msgfile     \"$msgfile\"
set g_rtspmsgurl  \"$rtspmsgurl\"
set g_httpmsgurl  \"$httpmsgurl\"
set g_vmailhome   \"$vmailhome\"
set g_register    \"$register\"
set g_waittimeout $waittimeout
set g_randomdigits $randomdigits
set g_totalsize_KB $totalsize_KB
set g_max_msgsize_kb $max_msgsize_kb

return 1
"
catch {close $f}

puts "Creating $vmailscriptdir/config..."
set f [open "$vmailscriptdir/config" "w" 0755 ]
puts $f "vmailserver $vmailserver
rtspserver $rtspserver
httpserver  $httpserver
emaildomain $emaildomain

onbusy      $onbusy
noresponse  $noresponse
topdir      $topdir
msgfile     $msgfile
rtspmsgurl  $rtspmsgurl
httpmsgurl  $httpmsgurl
vmailhome   $vmailhome
register    $register
waittimeout $waittimeout
emailto     $emailto
randomdigits $randomdigits
totalsize_KB $totalsize_KB
max_msgsize_kb $max_msgsize_kb
"
catch {close $f}

proc copywithtclpath {filename tclpath} {
    return
    exec echo "#!$tclpath" > tmpfile
    exec cat $filename >> tmpfile
    exec mv -f tmpfile $filename
    exec rm -f tmpfile
}

puts "Creating $vmailscriptdir/notify.tcl..."
copywithtclpath "$vmailscriptdir/notify.tcl" "$tclpath
set g_vmailscriptdir \"$vmailscriptdir\"
"

puts "Creating $vmailscriptdir/vmail.cgi..."
copywithtclpath "$vmailscriptdir/vmail.cgi" "$tclpath"
catch {close $sock}

# Create default in vmail.conf using sipum-conf

puts "Changing permissions..."
puts "755 for subdirectory $installdir"
catch {exec chmod -fR 755 $installdir}
set tempdir [file dirname $installdir]
while { $tempdir != "/" } {
    puts "755 for directory $tempdir"
    catch {exec chmod -f 755 $tempdir}
    set tempdir [file dirname $tempdir]
}

puts "Updating the default config..."
catch { exec $vmailtoolsdir/sipum-conf -c "add default $vmailscriptdir/config"}

puts "Installation done...
Please report any problem to kns10@cs.columbia.edu or hgs@cs.columbia.edu
or visit http://www.cs.columbia.edu/~kns10/software/sipum"
