# Columbia CINEMA servers makefile fragment for sipum
# Copyright 1999-2000 by Columbia University; all rights reserved 

sipum_dir := $(topdir)/sipum

SIPUM_MODULES := docs script audio

# include the description for each sub-modules if exist.
include $(patsubst %,$(topsrcdir)/$(sipum_dir)/%/module.mk,$(SIPUM_MODULES))

SIPUM_SRC := \
	$(sipum_dir)/vmail.cxx \
	$(sipum_dir)/vmailinfo.cxx \
	$(sipum_dir)/rtspapi.cxx \
	$(sipum_dir)/rtspparser.cxx

SIPUM_HEADERS := \
	$(sipum_dir)/vmail.h \
	$(sipum_dir)/rtspapi.h \
	$(sipum_dir)/rtspparser.h

SIPUM_MISC := \
	$(sipum_dir)/module.mk \
	$(sipum_dir)/Makefile \
	$(sipum_dir)/sipum.dsp \
	$(sipum_dir)/todolist.txt

SIPUM_MISCDIST := \
	$(sipum_dir)/README \
        $(sipum_dir)/sipum.license

SIPUM_BIN := \
	$(sipum_dir)/sipum

SIPUM_REQS := need-pwlib

# Target for sipum (formarly v-mail)

.PHONY: sipum v-mail

v-mail: sipum
sipum : $(SIPAPI)
sipum : $(DBAPI)
sipum : sipumliblinks
sipum : $(SIPUM_REQS)
sipum : $(SIPUM_TOOLS_BIN)
sipum : tools

sipum :
	@echo "Building sipum"
	@mkdir -p sipum; TOPDIR="`pwd`"; export TOPDIR; \
	SIPUM_LIBS="$(LDFLAGS) $(SIPAPI_LIB) $(LIBSIP_LIB) $(LIBCINE_LIB) $(LIBRESPARSE_LIB) $(LIBDICT_LIB) $(PGP_SDKLIBS) $(DBLIBS) $(DBAPI_LIB) $(MYSQLLIBS) -lgdbm $(RESOLVLIBS) $(CRYPTLIBS) $(NISLIBS) $(SOCKLIBS) $(MATHLIBS)"; \
	export SIPUM_LIBS; \
	cd $(topsrcdir); \
	SIPUM_INCS="-I`pwd`/$(sipapi_dir) -I`pwd`/$(libsip_dir) -I`pwd`/$(libcine_dir) -I`pwd`/$(dbapi_dir) -I$(MYSQL_DIR)/include -I$(MYSQL_DIR)/include/mysql"; export SIPUM_INCS; \
	$(MAKE) -C ./sipum opt; # change it to opt later XXX


cleansipum ::
	set -e; $(MAKE) -C $(topsrcdir)/sipum clean;


sipumliblinks ::
	@echo "creating symbolic links for libraries..."
	@topdirabs=`pwd`; \
	cd $(topsrcdir); topsrcdirabs=`pwd`; \
	rm -f ./sipum/libsip++.a; \
	rm -f ./sipum/libsip.a; \
	rm -f ./sipum/libcine.a; \
	rm -f ./sipum/libresparse.a; \
	rm -f ./sipum/libdict.a; \
	rm -f ./sipum/libdb++.a; \
	ln -ns $$topdirabs/libsip++/libsip++.a ./sipum/libsip++.a; \
	ln -ns $$topdirabs/libsip/libsip.a ./sipum/libsip.a; \
	ln -ns $$topdirabs/libcine/libcine.a ./sipum/libcine.a; \
	ln -ns $$topdirabs/resparse-1.3/libresparse.a ./sipum/libresparse.a; \
	ln -ns $$topdirabs/libdict/libdict.a ./sipum/libdict.a; \
	ln -ns $$topdirabs/libdb++/libdb++.a ./sipum/libdb++.a


# Binary and source distribution.

SIPUM_DIST := $(SIPUM_SRC) $(SIPUM_HEADERS) \
	      $(SIPUM_MISC) $(SIPUM_MISCDIST)
SIPUM_BINDIST := $(SIPUM_BIN) $(SIPUM_MISCDIST)


.PHONY: sipum-bindist sipum-dist

SIPUM_BINDIST_NAME := sipum-$(VERSION)-$(DATE)-$(SYSNAME)

sipum-bindist:: need-licensing $(SIPUM) $(SIPUA_BIN) $(RTSPD_BIN) 
	$(MKDIST) $(topsrcdir) $(SIPUM_BINDIST_NAME) \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) $(LIBSIP_BINDIST) $(LIBRESPARSE_BINDIST) \
		$(LIBDICT_BINDIST) \
		$(TOOLS_BINDIST) \
                $(CINEMA_WEB_SIPUM_BINDIST) \
                $(DOC_BINDIST) \
		$(RTSPD_BINDIST) \
		$(RTSPD_DOCS_BINDIST) \
		$(SCRIPTS_BINDIST) \
		$(SIPUA_BINDIST) \
		$(SIPUA_DOCS_BINDIST) \
		$(SIPUM_SCRIPT_BINDIST) \
		$(SIPUM_AUDIO_BINDIST) \
		$(SIPUM_BINDIST) \
		$(SIPUM_DOCS_BINDIST) \
                $(EXTRA_BINDIST_FILES)

SIPUM_DIST_NAME := sipum-$(VERSION)-$(DATE)

sipum-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(SIPUM_DIST_NAME) \
		$(TOP_DIST) \
		$(LIBCINE_DIST) \
		$(LIBSIP_DIST)  \
		$(LIBRESPARSE_DIST) \
		$(LIBDICT_DIST) \
		$(DBAPI_DIST) \
                $(TOOLS_DIST) \
		$(SCRIPTS_DIST) \
		$(SIPAPI_DIST) \
		$(WIN32_DIST) \
		$(SIPUM_DIST) \
		$(SIPUM_DOCS_DIST) \
                $(CINEMA_WEB_SIPUM_DIST) \
                $(DOC_DIST) \
                $(EXTRA_DIST_FILES)

