/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : rtspparser.cpp
#
#  Module       : RtspParser Source
#
#  Author(s)    : Tarun Kapoor
#
#  Created      : 8 Feb 2000
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |				| 
#-----------------------------------------------------------------------
*/

/******************* INCLUDES ******************************************
#***********************************************************************
*/
#include <ptlib.h>
#include <ptclib/http.h>
#include "rtspparser.h"  /* rtsparser calls are declared here */

#define new PNEW

static PString MULTICAST("multicast");
static PString UNICAST("unicast");
/*************** Rtsp Parser ***************************/
RtspParser::RtspParser()
: PHTTP()
{
  responseStatusLine = false;
}

RtspParser::~RtspParser()
{
  /* make sure the dictionary is cleaned out */
  headers.RemoveAll();
}

PINDEX RtspParser::ParseResponse(const PString& line)
{
  /* call the parent's ParseResponse only if first line of response message */
  if (!responseStatusLine)
  {
    PINDEX index = PHTTP::ParseResponse(line);
    responseStatusLine = true;
    return index;
  }
  else
  {
    PINDEX nameIndex = line.Find(':');
    PString name = line(0,nameIndex-1);
    PString value = line(nameIndex+1,line.GetLength()-1);
    headers.SetAt(name, new PString(value));
    return nameIndex;
  }
}

const PString RtspParser::GetHeaderValue(const PString& headerName)
{
  PString *header_v = (headers.GetAt(headerName));
  if ( header_v == NULL ) {
	return PString("");
  }
  PString header_value = *header_v;
  PINDEX endOfValue = header_value.Find(';');
  if (endOfValue != P_MAX_INDEX)
  {
    return header_value(0,endOfValue-1);
  }
  else
  {
    return header_value;
  }
}

const PString RtspParser::GetParameter(const PString& headerName, 
                                        const PString& paramName)
{
  PString header_value = *(headers.GetAt(headerName));
  PINDEX parmIndex = header_value.Find(';');
  if (parmIndex == P_MAX_INDEX) /* if no parameters in header */
  {
    return PString("");
  }
  else
  {
    PString parm; /* entire parameter, including name and value */
    PString name; /* name of desired parameter, e.g., client_port */
    PString parmValue; /* value of desired paramter */
    
    while (parmIndex != P_MAX_INDEX)
    {
      parm = header_value(0,parmIndex-1);
      PINDEX valueIndex = parm.Find('='); 
      if (valueIndex == P_MAX_INDEX) /* if only one value in parameter */
      {
        name = parm;
        parmValue = PString(""); 
      }
      else
      {
        name = parm(0,valueIndex-1);
        parmValue = parm(valueIndex+1,parm.GetLength()-1);
      }
      
      if (name == paramName)
      {
        return parmValue;  
      }
      else
      {
        header_value = header_value(parmIndex+1,header_value.GetLength()-1);
        parmIndex = header_value.Find(';');
      }
    }

    /* if the whole header was parsed, but the parameter was not found (i.e., */
    /* the header does not contain that parameter, then return the empty      */
    /* string                                                                 */
    return PString("");
  }
}

bool RtspParser::IsParameterPresent(const PString& headerName, 
                                    const PString& paramName)
{
  PString header_value = *(headers.GetAt(headerName));
  PINDEX parmIndex = header_value.Find(';');

  if (parmIndex == P_MAX_INDEX) /* if no parameters in header */
  {
    return false;
  }
  else
  {
    PString parm; /* entire parameter, including name and value */
    PString name; /* parameter name */
    
    while (parmIndex != P_MAX_INDEX)
    {
      parm = header_value(0,parmIndex-1);
      PINDEX valueIndex = parm.Find('='); 
      if (valueIndex == P_MAX_INDEX) /* if no value in parameter */
      {
        if (parm == paramName)
          return true;
      }
      else
      {
        name = parm(0,valueIndex-1);
        if (name == paramName)
        {
          return true;
        }
        else
        {
          header_value = header_value(parmIndex+1,header_value.GetLength()-1);
          parmIndex = header_value.Find(';');
        }
      }
    }

    return false;
  }
}


