/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : rtspparser.h
#
#  Module       : RtspParser
#
#  Author(s)    : Tarun Kapoor
#
#  Created      : 8 Feb 2000
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |				| 
#-----------------------------------------------------------------------
*/

#ifndef _RTSP_PARSER_H_
#define _RTSP_PARSER_H_
/******************* INCLUDES ******************************************
#***********************************************************************
*/
#include <ptlib.h>  /* PString */
#include <ptclib/http.h>

PDICTIONARY(HeadersList, PString, PString);

/* this is a class used for parsing rtsp messages. */
class RtspParser : public PHTTP
{
  PCLASSINFO(RtspParser, PHTTP)

public:
  RtspParser();
  ~RtspParser();
  
  virtual PINDEX ParseResponse(
      const PString & line // used to determine if server response was okay 
                           // or not.  (i.e., < 300 response status code)
    );

  /* This method should be used by any client just before receiving a new */ 
  /* response to their request.  Then, the parser will know to parse the  */
  /* status code first, as the first line, and not to extract any RTSP    */
  /* header information                                                   */
  void SetNewResponse() { responseStatusLine = false; }

  /* Get functions for header values */
  const PString GetHeaderValue(const PString& headerName);

  /* return the paramater value of a header.  If it is not present, return */
  /* an empty PString.                                                     */
  const PString GetParameter(const PString& headerName, const PString& paramName);

  /* determine if parameter is present, before calling GetParameter */
  bool IsParameterPresent(const PString& headerName, const PString& paramName);

  int GetLastResponseCode() { return lastResponseCode; }

private:

  HeadersList headers; /* dynamic list of rtsp headers */
  bool responseStatusLine; /* First line of any RTSP response will contain */
                           /* the return status code.                      */
};



#endif
