#! /bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" ${1+"$@"}

# Modify the following lines to reflect the
# email template file location for each user and
# the default email template file location.
# Specify the complete path. Do not forget to use
# \ for $ when using the variable $user for
# user login name.

set default_notification_template "script/email.tmp"
set g_adminemail "CINEMA administrator"

# Modification needed upto this only.

# Procedures
# Report problem
proc reportproblem {} {
    global g_adminemail
    return "Report problem to $g_adminemail.\n"
}

# Convert URI to email address
proc email {uri} {
    if { $uri != "" } {
        set value ""
        regexp "^(\[^:]+):(.*)" $uri dummy scheme value
        return "$value" 
    } else {
        return "unknown"
    }
}

# Output a http link <a href=...>..</a>
proc url {name link} {
    if { $link != "" } {
        if { $name == "" } {
            set name $link
        }
        return "<a href=\"$link\">$name</a>"
    } else {
        return "<b>broken link</b>"
    }
}

# output the data url
proc dataurl {name link} {
    if { $link != "" } {
        set value ""
        set scheme ""
        regexp "^(\[^:]+):(.*)" $link dummy scheme value
        return [url "$name" "data:application/$scheme;$link"] 
    } else {
        return "<b>broken dataurl</b>"
    }
}

# Put mandatory variable intializer header
set user     ""
set to       ""
set from     ""
set priority "normal"
set name     ""
set subject  ""
set rtspurl  ""
set httpurl  ""
set sipurl   ""
set msgid    ""
set administrator "CINEMA Administrator ([lindex [array get env USER] 1])"

for {set i 0} {$i < [llength $argv]} {incr i} {
    regexp "^(\[^=]+)=(.*)" [lindex $argv $i] dummy header value
    set $header $value
}

# Set the variables to default values.
# while { [gets stdin line]>0} {
#   regexp "^(\[^:]+):(.*)" $line dummy header value
#   set $header $value
# }

# Set the derived variables.
if { $user == "" } {
    set scheme ""
    set domain ""
    set value ""
    regexp "^(\[^:]+):(.*)" $from dummy scheme value
    regexp "^(\[^:]+)@(.+)" $value dummy user domain
}
if { $user == "" } {
    puts "No userid found.\n[reportproblem]"
    exit 0
}
if { $name == "" } {
    if { [ catch {set pwent [exec ypcat passwd | grep $user]} msg] == 0} {
        regexp "^(\[^:]+):(\[^:]+):(\[^:]+):(\[^:]+):(\[^:]+):" \
                $pwent dummy dummy1 dummy2 dummy3 dummy4 name
    } else {
        set name $user
    }

}

# Default template
set template $default_notification_template

if {[file exists $template] == 0 } {
    puts "No template found.\n[reportproblem]"
    exit 0
}

set outmail [source $template]

puts $outmail

