#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

#***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : vmail.cgi
#
#  Module       : VoiceMail viewer.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#                 Tarun Kapoor (tk358@cs.columbia.edu)
#
#  Created      : 18 Mar 2000
#
#  Description  : The TCL/CGI script to view the voicemails of a user.
#                A single script takes care of every thing,
#                password validation, folder management,
#                playback of voicemails, etc.
#                 Individual users are encouraged to modify the script
#                as per their taste. Potential modification points
#                are marked as TODO in this script.
#                
#  Notes        : The script uses cgi-tcl, [refer
#                 http://expect.nist.gov/cgi.tcl/ for details].
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------

# Include the header containing all the 
# configuration items.

set g_newmailimgurl ""
set g_vmailhomeurl ""

source config.tcl

package require cgi

# Enable or disable debuging, --TODO.
# Comment out this line to disable the debugging.
# cgi_debug -on

set scriptname $argv0

# Derive other config variables.
set g_vmailsoftwarepage "http://www.cs.columbia.edu/~kns10/software/sipum"
set g_vmailtermsurl "$g_vmailsoftwarepage/terms.html"
set g_vmailhelpurl  "$g_vmailsoftwarepage/help.html"
# URL of the image to be used to show new mails.
set g_newmailimgurl "$g_vmailscripturl/newmail.gif"
# URL of the email icon
set g_emailiconurl  "$g_vmailscripturl/emailicon.gif"

# URL of the voice mail logo.
set g_vmaillogofile   "$g_vmailscripturl/sipumlogo.gif"
# Background image
set g_backgroundimage  "$g_vmailscripturl/bg.gif"

# Name of the user script
set g_scriptname  "vmail.cgi"

# Update the name of the script URL
set g_vmailscripturl  "$g_vmailscripturl/$g_scriptname"

# File path for different files.
set g_vmailscriptfile "$g_vmailscriptdir/g_scriptname"

# Sample config file.
set g_vmailuserconfigfile "$g_vmailscriptdir/config.tmp"

# sipum tools
set g_sipumtoolconf "$g_sipumtoolsdir/sipum-conf"

# sipum config file
set g_sipumconffile "$g_sipuminstalldir/vmail.conf"

# Password file for the users. Password is stored in this file as
# userlogin:encryptedpassword.
# e.g., 
# abc:fXdXL8NrlPoas
# kundan:/0tNbVK4XXlno
# gautam:)-1h00/cCwe1U
# jon:7cdnm.Xs2SoGE
#
# If this file can not be opened then /misc/yp/passwd is checked.
#
set g_passwdfile "$g_vmailscriptdir/.vmaccess"

# Set it to 0 to disable checking of password.
# Set it to 1 to enable it.
set g_checkpasswd 1

# Tools  for checking and creating passwords
set g_passwdchecker "$g_sipumtoolsdir/ckpasswd"
set g_passwdmaker   "$g_sipumtoolsdir/mkpasswd"

# Page attributes, colors. --TODO.
# Background = white, link color is blue.
cgi_body_args bgcolor=#ffffff text=#000000 link=#0000ee vlink=#551a8b alink=#ff0000
#  background=$g_backgroundimage

# newmailimg() : URL for indicating a new message as an image.
proc newmailimg {} {
    global g_newmailimgurl
    return "[img $g_newmailimgurl alt='NewMail' hspace=1 border=0 height=11 width=11]"
} 

# emailicon() : Icon for email. This is displayed besides the
# from address of message send. Clicking on this icon 
# invokes the appropriate email-tool to reply back.
proc emailicon {} {
    global g_emailiconurl
    return "[img $g_emailiconurl alt='Re:email' hspace=1 border=0 height=11 width=15]"
} 

# Home page link  for vmail 
proc vmailhomelogo {} {
    global g_vmailhomeurl
    global g_vmaillogofile
    global g_vmailsoftwarepage
    puts "[font size=+4 [bold [link vmailhomelogo "<img src='$g_vmaillogofile' alt='SIPUM' border=0>" "$g_vmailsoftwarepage"]]]"
}

# Print the organization spcific URLs
proc print_organization_urls {} {
  table width="100%" cols=2 border=0 cellpadding=0 cellspacing=0 {
    table_row {
      table_data nowrap align=left width="70%" {
        puts [url "IRT Lab." "http://www.cs.columbia.edu/~hgs/research/IRT"]
        br;puts [url "Computer Science Dept." "http://www.cs.columbia.edu"]
        br;puts [url "Columbia University" "http://www.columbia.edu"]
          
      }
      table_data nowrap align=right {
        puts [img "http://www.cs.columbia.edu/~kns10/software/vmail/cls_crown.gif" alt="Columbia University" height=56 width=58 align=texttop]
      }
    }
  }
}

# Print copyright notice.
proc print_copyright {} {
    global g_vmailtermsurl
    # Copyright notice
    table border=0 cellspacing=0 cellpadding=3 cols=1 width="100%" bgcolor="#000099" {
        table_row {
            table_data {
                cgi_division align=right {
                    puts "[bold [font color=#ffffff "[cgi_copyright] 2000, Columbia University. All rights reserved."]] [url "<font color=#ffffff>Terms of service</font>" "$g_vmailtermsurl"]"
                }
            }
        }
    }
}
proc print_topline {} {
    global g_vmailscripturl
    table width="100%" border=0 cellpadding=0 cellspacing=0 bgcolor="#66ddff" {
        table_row {
            table_data nowrap align=left {
                vmailhomelogo
            }
            table_data nowrap align=right {
                h3 "New User? [url "Sign up now!" "$g_vmailscripturl?_VMaction=signup"]"
            }
        }
    }
}
proc print_problemline {} {
    global g_adminemail
    global g_vmailhelpurl
    table width="100%" border=0 cellpadding=0 cellspacing=0 bgcolor="#ddddff" {
        table_row {
            table_data nowrap align=left {
                puts "[url "Problems signing in?" "mailto:$g_adminemail"]"
            }
            table_data nowrap align=right {
                puts "[url "Forgot your password?" "$g_vmailhelpurl#forgotpassword"]"
            }
        }
    }
}

# Check upgrade checks if the script running is the latest or not.
# If it is not, then it prompts the user to update the script.
# It compares the timestamps on the script file with
# the master copy at /home/kns10/html/vmail/software/package/vmail.cgi
# This is applicable only for CS account installations
# which have a local script in users home area.
 
proc check_upgrade {} {
    global scriptname
    global g_vmailscriptfile
    global g_vmailhomeurl
    set mastercopy "$g_vmailscriptfile"
    if [file exists $mastercopy] {
        scan [file mtime $mastercopy] "%d" t1
        scan [file mtime $scriptname] "%d" t2
        if { $t2 < $t1 } {
            # Old local copy.
            cgi_division bgcolor=#cccccc {
                puts "[bold "You are running older version of the software.Please reinstall using the new user [url "signup" "$g_vmailhomeurl"] instructions"]"
            }
        } else {
            # New copy. Do nothing.
        }
    }
    # Don;t do anything if no master copy found.
}

# Password verification program.
proc verify {user password} {

    global g_checkpasswd
    global g_passwdfile
    global g_passwdchecker

    # Check local .vmaccess file.
    # If not present use /misc/yp/passwd
    # Else crib.
    # XXX uses a local ckpasswd program (solaris executable of ckpasswd.c )
    # Might be a problem if PATH is changed and some other util is
    # executed instead.

    set u ""
    set p ""

    if { $g_checkpasswd == 0 } {
        return 1
    }

    # Local .vmaccess file.
    if {[catch {set td [open "$g_passwdfile" r]}] == 0} {
        while { -1 != [gets $td str]} {
            regexp "(\[^:]+):(\[^:]+)" $str dummy u p
            if { $u == $user } {
                set r 1
                if {[catch {set r [exec $g_passwdchecker $password $p]} msg] == 0 } {
                    if { $r == 0 } {
                        return 1
                    }
                }
            }
            # else continue to next. A userid may have multiple password XXX
        }
        catch { close $td}
    }
    
    if { [catch {set td [open "/misc/yp/passwd" r]}] == 0} {
        while { -1 != [gets $td str]} {
            regexp "^(\[^\:]+)\:(\[^\:]+)" $str dummy u p
            if { $u == $user } {
                set r 1
                if {[catch {set r [exec $g_passwdchecker $password $p]} msg] == 0 } {
                    if { $r == 0 } {
                        return 1
                    }
                }
            }
            # else continue to next. A userid may have multiple password XXX
        }
        catch { close $td}
    }
    return 0
}

# Default domain.
proc def_domain {} {
    global g_defaultdomain
    return "$g_defaultdomain"
}

# Print footer for login screens
proc print_footer {} {
    global g_adminemail
    global g_vmailhomeurl
    puts "If you do not have a account, contact [url "mailto:$g_adminemail"]"
    puts "or visit [link vmailhome "VoiceMail home" "g_vmailhomeurl"]"
}

# TCL routine to print the date/time. Date is printed in a shorter form,
# e.g., only day and time for this week's mail, only time for todays mail,
# only date for this months mail and year/month/date for older mails.
# Please update this routine if you would like a different format or
# policy for displaying the date/time of the message.
proc printdate {input} {
    set date $input

    # Get useful date/time information as integers.
    set s1 [clock scan $input]
    set s2 [clock seconds]
    set y1 [clock format $s1 -format "%Y"]
    set y2 [clock format $s2 -format "%Y"]
    set y [clock format $s1 -format "%m"]
    scan $y "%d" m1
    set y [clock format $s2 -format "%m"]
    scan $y "%d" m2
    set y [clock format $s1 -format "%U"]
    scan $y "%d" w1
    set y [clock format $s2 -format "%U"]
    scan $y "%d" w2
    set y [clock format $s1 -format "%j"]
    scan $y "%d" j1
    set y [clock format $s2 -format "%j"]
    scan $y "%d" j2
    set y [clock format $s1 -format "%d"]
    scan $y "%d" d1
    set y [clock format $s2 -format "%d"]
    scan $y "%d" d2
    
    if { $y1 == $y2 } {
        #  Same year
        # puts "$j1,$j2"
        set x [ expr $j1 + 4 ]
        if { $x >= $j2 } {  
            # Same past week 
            if { $j1 == $j2 } {
                # Today: print hr:min:sec
                set date [clock format $s1 -format "%I:%M:%S %p"]
            } else  { 
                if { $j1 < $j2 } {
                    # Other day of past week: print day,hr:min
                    set date [clock format $s1 -format "%a %I:%M %p"]
                } else {
                    # Date is in future.
                    set date [clock format $s1 -format "%b %d, %Y; %I:%M:%S %p (future)"]
                }   
            }
        } else {
            if { $m1 == $m2 } {
                # Same month: print date,hr:min
                set date [clock format $s1 -format "%a %d, %I %p"]
                
            } else {
                # Print: month, date, 
                set date [clock format $s1 -format "%b %d"]
            }
        }
    } else {
        # Print year:month,date
        set date [clock format $s1 -format "%b %d, %Y"]
    } 
}

# Check if the server is running or not.
# returns 0 if success, returns 1 if server is not running.
proc restart_vmail_server {} {
    global g_vmailserver 
    set hostport [split $g_vmailserver :]
    set vmailserver [lindex $hostport 0]

    if { [llength $hostport] > 1 } {
        set vmailserverport [lindex $hostport 1]
    } else {
        set vmailserverport 10022
    }

    if {[catch {set fd [socket "$vmailserver" $vmailserverport]} ] == 0 } {
        if { $fd == -1 } {
            return 1
        } else {
            catch {close $fd}
            return 0
        }
    } else {
        return 1
    }
}

# cgi_eval() mail routine to display the page.

cgi_eval {

    cgi_input

    # Get all the user/password information.
    # Check if .htaccess file is present. If yes no need to
    # do extra password checking/login mechanism.
    # If "user" param is present, use that,
    # else try to import "user" cookie.
    # If password param is present, use that, else, 
    # prompt for password.
    # If user param is not known then prompt for user login also.

    # user, e.g., kns10
    # userdomain, e.g., kns10@cs.columbia.edu, tk358@voicemail
    # domain, e.g., cs.columbia.edu, voicemail.
    set user ""
    set password ""
    set userdomain "" 
    set domain ""
    set userknown 0
    set command "displayfolder"
    set pwcorrect 1
    
    if { [catch {cgi_import _VMaction}] == 0 } {
        set command [cgi_import _VMaction]
    } 

    if { [catch {cgi_import user}] } {
        # User "name" param is *not* present.
        if { [ catch {cgi_import_cookie userdomain}] } {
            #  User "name" cookie is *not* present.
            # Set user to empty string.
        } else {
            # Set user to "user" cookie.
            set userdomain [cgi_import_cookie userdomain]
            # Extract user from userdomain.
            regexp "(\[^\@]+)@(.*)" $userdomain dummy user domain
            if { $domain == "" } {
                # Default domain is cs.columbia.edu.
                set domain [def_domain]
            }
        }
        # Do not use old password param.

    } else {
        # "user" param is present. Use that.
        set userknown 1

        set user [cgi_import user]
        if { [catch {cgi_import domain}] == 0} {
            set domain [cgi_import domain]
        }
        if { $domain == "" } {
            set domain [def_domain]
        }
        set userdomain "$user@$domain"
        
        # Get password param.
        if { [ catch {cgi_import password}] } {
            # No passwd.
        } else {
            # Verify passwd
            set password [cgi_import password]
            set pwcorrect [verify "$user" "$password"] 
            if { $pwcorrect == 0 } {
                # verification failed.
                set password ""
                # puts "Could not authenticate, probably password is incorrect"
            }
        }
    }

    if { $command == "signup" } {
        # Signup request or confirmation.

        cgi_head {
            title "Voicemail sign up"
        }
        cgi_body {
            print_topline
            set newuserflag 1

            check_upgrade

            if { [catch {set newuser [cgi_import newuser]}] == 0 } {
                global g_vmailuserdir
                global g_defaultdomain
                global g_passwdfile
                global g_vmailscripturl
                global g_passwdmaker
                global g_sipumconffile
                global g_sipumtoolconf

                # user entered his user id.
                set maindir "$g_vmailuserdir/$newuser"
                set newpassword ""
                set reenterpassword ""
                set domain "$g_defaultdomain"
                
                catch {set newpassword [cgi_import newpassword]}
                catch {set reenterpassword [cgi_import reenterpassword]}
                catch {set domain [cgi_import domain]}

                if { $newpassword == "" } {
                  h3 "Enter a valid password"
                  set newuserflag 1
                } elseif { $newpassword != $reenterpassword } {
                  h3 "Please type the same password while re-entering"
                  set newuserflag 1
                } elseif {[file exists $maindir] } {
                  h3 "User name \"$newuser\" is already registered, try another."
                  set newuserflag 1
                } elseif {[catch {set encr [exec $g_passwdmaker $newpassword]} msg] != 0 } {
                    h3 "Could not encrypt the password: $msg, please try again."
                    set newuserflag 1
                } elseif {[catch {set ppf [open "$g_passwdfile" a]} msg] != 0} {
                    h3 "Could not create $g_passwdfile: $msg, please try again."
                    set newuserflag 1
                } elseif {[catch {set cfgcr [exec $g_sipumtoolconf -f $g_sipumconffile -c "add $newuser $g_vmailuserdir/$newuser/config"]} msg] != 0} {
                    h3 "Could not update $g_sipumconffile: $msg, please try again. Contact $g_adminemail if problem persists"
                    set newuserflag 1
                } else {
                    global g_vmailuserconfigfile

                    puts $ppf "$newuser:$encr"
                    catch {close $ppf}

                    file mkdir $maindir
                    file attributes $maindir -permissions 0711
                    file mkdir "$maindir/inbox"
                    file attributes "$maindir/inbox" -permissions 0711
                    file copy -force "$g_vmailuserconfigfile" "$maindir/config"

                    p
                    puts "Thankyou for signing up"
                    p
                    puts "Your user account $newuser@$domain has been created."
                    puts "Your voicemail home is at [url "$g_vmailscripturl" "$g_vmailscripturl?user=$newuser&domain=$domain"]."
                    p
                    puts "Please visit [url "your voicemail home" "$g_vmailscripturl?user=$newuser&domain=$domain"] and create a bookmark to this link in your browser"
                    p

                    set e [restart_vmail_server]
                    if { $e != 0 } {
                        puts "The voice-mail server is not yet up. Contact administrtor to start the voice-mail server."
                    }
                    set newuserflag 0
                }
            } else {
                h3 "Enter your preferred login name and password"  
                p
            }
            
            if { $newuserflag == 1 } {
                cgi_form vmail {
                    global g_defaultdomain

                    put "Username: "; cgi_text newuser=$user size=16
                    put "[cgi_nbspace][bold "\@$g_defaultdomain"]"
                    cgi_export domain=$g_defaultdomain
                    cgi_export _VMaction=signup
                    br
                    put "Password: "; cgi_text newpassword= type=password size=16
                    put "[cgi_nbspace][cgi_nbspace]"
                    br
                    put "Reenter password: "; cgi_text reenterpassword= type=password size=16
                    put "[cgi_nbspace][cgi_nbspace]"
                    submit_button "=Sign in"
                    p
                }
            }
            print_problemline
            print_copyright
        }
    } elseif { [file exists .htaccess]==0 &&  ($userknown == 0 || $user == "") } {
        # Export the "user" cookie. and "user" and "password" params.
        cgi_http_head {
            cgi_content_type text/html
            cgi_export_cookie userdomain expires=60s
        }

        # User loging in.
        # Prompt for user login and password.
        # This spits out the User-Login page. Put any advertisement, reminder,
        # upgrade information here. --TODO.
        cgi_head {
            title "Voicemail login"
        }
        cgi_body {
            print_topline
            cgi_form vmail {
              global g_defaultdomain

              check_upgrade
              p

              if { $pwcorrect == 0 } {
                 puts "Either username or password or both are incorrect"
              }

              h3 "Login please"  
              p
              
              put "Username: ";
              cgi_text user=$user size=16
              put "[cgi_nbspace][bold "\@ $g_defaultdomain"]"
              cgi_export domain=$g_defaultdomain
              br
              put "Password: "; 
              cgi_text password= type=password size=16
              put "[cgi_nbspace][cgi_nbspace]"
              submit_button "=Sign in"
              p
            }
            print_problemline
            print_copyright
       }
   } elseif {[file exists .htaccess]==0 && $password == "" } {
        # Export the "user" cookie. and "user" and "password" params.
        cgi_http_head {
            cgi_content_type text/html
            cgi_export_cookie userdomain expires=60s
        }

        # Prompt for password verification.
        cgi_head {
            title "Voicemail login for $userdomain"
        }
        cgi_body {
            print_topline
            cgi_form vmail {
                global g_scriptname

                check_upgrade
                p
                
                if { $pwcorrect == 0 } {
                    puts "Either username or password or both are incorrect"
                }

                h3 "Password required for user \"$user@$domain\""
                p
                puts "Password: "; cgi_text password= type=password size=16
                p
                cgi_export user=$user
                cgi_export domain=$domain
                # Prompt for change in user id. 
                puts [cgi_link changeuser "change user id" "$g_scriptname"]
                br
                print_footer
            }
            print_problemline
            print_copyright
        }
    } elseif { $command == "retrieve" } {
        global g_vmailuserdir

        # Set user and domain here. XXX

        # Format other important links and variables
        set folder  "inbox"
        set fname   ""
        if { [catch {cgi_import folder}] == 0 } {
            set folder [cgi_import folder]
        }
        if { [catch {cgi_import fname}] == 0 } {
            set fname [cgi_import fname]
        }
        set topdir "$g_vmailuserdir/$user"

        set folderdir "$topdir/$folder"
        if { $folder == "" || $fname == "" } {
            cgi_error "Invalid mail folder or mail file" 
            cgi_exit
        } else {
            # Export the "user" cookie. and "user" and "password" params.
            cgi_http_head {
                cgi_content_type audio/basic
            }
            
            # Set the status as read...
            if { [catch {set td [open "$folderdir/$fname" r+]} msg] == 0} {
              # Could open the file.
              set readsize 0
              while { -1 != [set linesize [gets $td str]]} {
                set linesize [expr $linesize + 1 ]
                set statusitem "R"
                if {[regexp "Status:(\[^\|\n]+)" $str dummy statusitem]==1} {
                  if { $statusitem != "R" } {
                    seek $td $readsize
                    # Set the status to R
                    puts $td "Status:R"
                  }
                  break
                }
                set readsize [expr $readsize + $linesize]
              }
              catch { close $td}
            }

            set td [open "$folderdir/$fname" r]
            while { -1 != [gets $td str]} {
                puts $str
            }
            catch { close $td}
        }
        # Retrive the voicemail and play back.

    } else {
        # Export the "user" cookie. and "user" and "password" params.
        cgi_http_head {
            cgi_content_type text/html
            cgi_export_cookie userdomain expires=60s
        }

      # Proceed to execute the command, or display the inbox.

      cgi_head {
        title "Voicemail for $user@$domain"

        # Javascripts to check all and open new help window.
        javascript {
          puts {
            function CheckAll()
            {
              for (var i=0;i<document.vmail.elements.length;i++)
              {
                var e = document.vmail.elements[i];
                if (e.name != 'allbox')
                  e.checked = document.vmail.allbox.checked;
              }
            }
          
            function NewWindow(url)
            {
              var hWnd = window.open(url,
              "HelpWindow",
              "width=600,height=400,toolbar=yes,resizable=yes,scrollbars=yes");
            }
          }
        }
        noscript {
            puts "Sorry - your browser doesn't understand Javascript."
        }
      }
# ------------- Body of Mail voicemail page. --------------
    cgi_body {

        # Get all CGI parameters, _VMaction, folder, etc.
        set folder  "inbox"
        set sort    "date"
        
        check_upgrade

        if { [catch {cgi_import folder}] == 0 } {
            set folder [cgi_import folder]
        }
        if { [catch {cgi_import sort}] == 0 } {
            set sort [cgi_import sort]
        }

        cgi_form vmail name=vmail {
            global g_vmailuserdir
            global g_vmailhelpurl
            global g_newmailimgurl

          # Display the page

          # Export those variables.
          cgi_export user=$user
          cgi_export domain=$domain
          cgi_export password=$password
          cgi_export folder=$folder
          cgi_export sort=$sort
          cgi_export _VMaction=$command

        # Format other important links and variables
        set topdir "$g_vmailuserdir/$user"
        set folderdir "$topdir/$folder"

        cgi_link inboxlink    "[cgi_font color=#ffffff Inbox]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.folder.value='inbox';document.vmail.submit();"
        cgi_link folderslink  "[cgi_font color=#ffffff Folders]" "javascript:document.vmail._VMaction.value='displayfolderlist';document.vmail.submit();" 
        cgi_link optionslink "[cgi_font color=#ffffff Options]" "javascript:document.vmail._VMaction.value='displayoptions';document.vmail.submit();"
        cgi_link helplink     "[cgi_font color=#ffffff Help]" "javascript:NewWindow('$g_vmailhelpurl')"

        cgi_imglink newmaillink "$g_newmailimgurl" alt="Sort by New Mail" hspace=1 border=0 height=11 width=11

        cgi_link  datelink "[cgi_font color=#000000 Date]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='date';document.vmail.submit();"
        cgi_link  fromlink "[cgi_font color=#000000 From]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='from';document.vmail.submit();"
        cgi_link  subjectlink "[cgi_font color=#000000 Subject]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='subject';document.vmail.submit();"
        cgi_link  sizelink "[cgi_font color=#000000 Size]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='size';document.vmail.submit();"
        cgi_link  deletelink "Delete<br>Selected" "javascript:document.vmail._VMaction.value='delete';document.vmail.submit();"
        cgi_link  forwardlink "Forward" "javascript:document.vmail._VMaction.value='forward';document.vmail.submit();"
        cgi_link  refreshlink "Refresh" "javascript:document.vmail._VMaction.value='refresh';document.vmail.submit();"
        cgi_link  logoutlink "Logout" "javascript:document.vmail._VMaction.value='logout';document.vmail.submit();"
        cgi_link  signoutlink "Permanent<br>Signout" "javascript:document.vmail._VMaction.value='signout';document.vmail.submit();"
        cgi_link  restartlink "Restart server" "javascript:document.vmail._VMaction.value='restart';document.vmail.submit();"

        if { $command == "delete" } {
            # Delete the voicemails present in DeleteVMailList.
            if {0==[catch {cgi_import DeleteVMailList}]} {
                # Uncomment following line for notification of deletion.--TODO
                # h4 "The following voice mails were deleted:"
                foreach mail $DeleteVMailList {
                    # Uncomment following line for notification of deletion --TODO.
                    # h5 $mail 
                    if [catch {exec rm -f $folderdir/$mail} msg] {
                        # Uncomment following line for notification --TODO.
                        # h4 "$msg"
                    }
                }
            }
            # Display the remaining mails in folder.
            set command "displayfolder"

        } elseif { $command == "forward" } {
            if {[catch {set mails [cgi_import DeleteVMailList]}] == 0 } {
                global g_adminemail
                global g_vmailhomeurl

                # Forward the selected voicemails as 
                # an attachement to the emaillist.
                set emaillist ""
                catch {set emaillist [cgi_import emaillist]}
                # XXX use random boundary....
                set boundary "------------FDAAFA6929D67C4D187555EF"
                # XXX reply to should have the actual email id.
                set emailmsg "MIME-Version: 1.0
From: $user@$domain
To: $emaillist
Reply-To: $user@$domain
Subject: \[Forwarded voice mail(s)\]
Content-Type: multipart/mixed; boundary=\"$boundary\"

This is a multi-part message in MIME format.
--$boundary
Content-Type: text/plain; charset=us-ascii


$user@$domain has forwarded the attached voice mail(s) to you.

---
administrator ($g_adminemail)
$g_vmailhomeurl

--$boundary"

# puts "Email header is $emailmsg"

                foreach mail $DeleteVMailList {
                    # Read the Subject from the voicemail.
                    set subject "voicemail"
                    set fp [open "$folderdir/$mail" r]
                    if { $fp != 0 } {
                        # puts "File $folderdir/$mail is opened"
                        fconfigure $fp -translation binary
                        set magic [read $fp 4]
                        set x [read $fp 4]
                        binary scan $x I hdrsize
                        # Skip rest of the 16 bytes.
                        read $fp 16  
                        set hdrsize [ expr $hdrsize - 16 ]
                        set y [read $fp $hdrsize]
                        binary scan $y A* info
                        
                        regexp "Subject:(\[^\|\n]+)" $info dummy subject
                        # puts "Subject is $subject"

                        catch {close $fp}
                    } else {
                        # puts "File $folderdir/$mail could not be opened"
                    }
                    
                    set emailmsg "$emailmsg
Content-Type: audio/basic; name=\"$mail\"
Content-Transfer-Encoding: 8bit
Content-Description: $subject
Content-Disposition: attachment; filename=\"$mail\"

[exec cat $folderdir/$mail]
--$boundary"
                }

                set emailmsg "$emailmsg--"

                # puts "$emailmsg"
                # puts "Sending mail to $emaillist"

                if { [catch {set mf [open "|mail $emaillist" "w"] } ] == 0 } {
                    # puts $emailmsg
                    puts $mf $emailmsg
                    # puts $mf "Hi"
                    # puts $mf "."
                    catch { close $mf }                    
                }
                
            } else {
                puts "Select some mails and then forward."
            }

            # Display the mails in folder.
            set command "displayfolder"


        } elseif { $command == "MoveTo" } {
            # Move the voicemails present in DeleteVMailList to a different folder
            # given by tolist.
            set tobox ""
            if { [catch {cgi_import tobox}] == 0 } {
                set tobox [cgi_import tobox]
            }
            if { $tobox != "" } {
                set newdir "$topdir/$tobox"

                if {0==[catch {cgi_import DeleteVMailList}]} {
                    foreach mail $DeleteVMailList {
                        if [catch {exec \mv $folderdir/$mail $newdir} msg] {
                            puts "Can not move $folderdir/$mail to $newdir: $msg"
                            br
                        }
                    }
                }
            }
            # Display the remaining mails in folder.
            set command "displayfolder"
        } elseif { $command == "refresh" } {
            # Since no caching, no need to refresh.
            set command "displayfolder"
        } elseif { $command == "createfolder" } {
            # Create a new folder.
            set newfolder ""
            if { [catch {cgi_import newfolder}] == 0 } {
                set newfolder [cgi_import newfolder]
            }
            if { $newfolder != "" } {
                if [catch {exec mkdir "$topdir/$newfolder"} msg] {
                    puts "Can not create $topdir/$newfolder: $msg"
                    br
                }
            }
            # Display the current list folder.
            set command "displayfolderlist"
        } elseif { $command == "deletefolder" } {
            # Delete empty selected folders.
            if {0==[catch {cgi_import DeleteVMailList}]} {
                foreach fldr $DeleteVMailList {
                    if [catch {exec rmdir $topdir/$fldr} msg] {
                        puts "Can not delete folder $topdir/$fldr: $msg"
                        br
                    }
                }
            }

            # Display the current list folder.
            set command "displayfolderlist"

        } elseif { $command == "changeoption" } {
            # Change user option.
            if {0==[catch {set option [cgi_import option]}]} {
              if { 0==[catch {set $option [cgi_import $option]}]} {
                if { $option == "newpassword" } {
                    if [file exists ".htaccess"] {
                        puts "[bold "You have a .htaccess file in your html/vmail directory. You should change password referred by that file using htpasswd command. Contact administrator if you face some problem."]"
                    } else {
                      global g_passwdmaker
                      if { [catch {
                        set encry [exec $g_passwdmaker [cgi_import $option]]}]==0} {
                        global g_passwdfile
                        if {[catch {set pf [open "$g_passwdfile" w]}]==0} {
                            puts $pf $user:$encry
                            catch {close $pf}
                        } else {
                            puts "Can not write into $g_passwdfile file\n"
                        }
                      } else {
                          puts "Can not generate a password\n"
                      }
                    } 
                } else {
                    set configfile "$topdir/config"
                    set optionslist [list]
            
                    if {[catch {set cf [open $configfile [list RDWR CREAT]]} msg] == 0 } {
                      # Opened the file successfully.
                      while { -1 != [gets $cf str]} {
                        regexp "^(\[^\ ]+)(\ +)(.+)" $str dummy h dummy2 v
                        if { $h != "" } {
                            if { $h == $option } {
                                # puts "Old $option: $v"
                            } else {
                                lappend optionslist [list "$h" "$v"]
                            }
                        }
                      }
                      catch {close $cf}
                      if {[catch {set cf [open $configfile [list w]]} msg] == 0 } {
                          

                          lappend optionslist [list "$option" "[cgi_import $option]"]
                          # puts "New options: $optionslist"
                          for { set i 0} {$i < [llength $optionslist]} {incr i} {
                              set hv [lindex $optionslist $i]
                              puts $cf "[lindex $hv 0] [lindex $hv 1]"
                          }
                          catch { close $cf}
                      } else {
                          p
                          h3 "Could not open the config file, $configfile: $msg"
                          p
                      }
                    } else {
                      p
                      h3 "Could not open the config file, $configfile: $msg"
                      p
                    }
                }
              } else {
                puts "Error: No option value specified for $option"
              }
            } else {
                puts "Error: No option parameter"
            }

            # Display the current list folder.
            set command "displayoptions"

        } elseif { $command == "restart" } {
            # Restart the server
            set f [restart_vmail_server]
            if { $f == 0 } {
                cgi_h3 "Server is re-started"
            } else {
                cgi_h3 "Server is not running. Report problem to administrator"
            }

            # Display the current list folder.
            set command "displayfolder"

        } elseif { $command == "signout" || $command == "signoutconfirm" } {
            # Permanently signout the user.
            # delete all inboxes and config info and the directory 
            # structure.

            # set domain "voicemail"
            global g_vmailscripturl
            global g_vmailuserdir

            set htmldir ""
            set maindir "$g_vmailuserdir/$user"

            if { $command == "signout" } {
                h3 "Warning !"
                p
                puts "Signing out permanently means your account will be closed permanently and your will not be able use your voicemail system later. You are always welcome to signup in future. However, you will lose all your current settings, configuration, inbox messages and other folders."
                p

                puts "If you really want to signout click [url "here" "javascript:document.vmail._VMaction.value='signoutconfirm';document.vmail.submit();"] else click on back button of your browser."
                p

            } else {
                # signout confirm.
                # user entered his user id.
                p
                if {[catch {exec rm -rf $maindir} msg] == 0 } {
                    puts "Removed $maindir."
                } else {
                    puts "Can not remove $maindir: $msg"
                }
                if { $htmldir != "" } {
                    # CS account.
                    if {[catch {exec rm -rf $htmldir} msg] == 0 } {
                        puts "Removed $htmldir."
                    } else {
                        puts "Can not remove $htmldir: $msg"
                    }
                } else {
                    # web hosting.
                    global g_passwdfile
                    global g_sipumtoolconf
                    global g_sipumconffile
                    global g_adminemail

                    # XXX remove from vmail.conf also.

                    if {[catch {exec grep -v $user < $g_passwdfile > tmp} msg] != 0} {
                        puts "Can not remove $user from password file: $msg"
                    } 

                    if {[catch {exec mv -f ./tmp $g_passwdfile} msg] != 0} {
                        puts "Can not rename new password file: $msg"
                    } else {
                        puts "Removed $user from password file"
                    }

                    if { [catch {exec $g_sipumtoolconf -f $g_sipumconffile -c "remove $user"} msg] != 0 } {
                        puts "Can not remove $user from sipum configuration file $g_sipumconffile: $msg."
                        puts "Please send a mail to $g_adminemail describing this problem."
                    } else {
                        puts "Removed $user from config file"
                    }
                }
                p
            }
        }

        # Set the current folder list.
        set folderslist [list] 
        set td [open "|/bin/ls -t1 $topdir" r]
        while { -1 != [gets $td filename]} {
            if { [file isdirectory "$topdir/$filename"] } {
                lappend folderslist "$filename"
            }
        }
        catch { close $td}

          table border=0 cellspacing=0 cellpadding=0 cols=1 width="500" {
            table_row {
              table_data {
                # Top line "VoiceMail, UserId, ColumbiaUniversity
                table cellspacing=0 cellpadding=0 cols=3 width="100%" {
                  table_row valign=top {
                    table_data width="1" {
                      vmailhomelogo
                    }
                    table_data width="100" {
                    }
                    table_data {
                      print_organization_urls
                    }
                  }
                }
                cgi_division align=center {
                    puts [bold $user]
                    puts [bold "@$domain"]
                }
                p

                # Inbox, Folders, Options, Help.
                table border=0 cellspacing=0 cellpadding=0 cols=4 width="100%" bgcolor="#006600" {
                  table_row {
                    table_data {
                      cgi_center {
                        puts "[bold [cgi_link inboxlink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                          puts "[bold [cgi_link folderslink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                          puts "[bold [cgi_link optionslink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                       puts "[bold [cgi_link helplink]]"
                      }
                    }
                  }
                }

                if { $command == "displayfolder" } {                
                  # Current Folder and number of new messages.XXX
                  table border=0 cellspacing=0 cellpadding=0 cols=1 width="100%" bgcolor="#33cc00" {
                    table_row {
                      table_data {
                        cgi_center {
                          puts "[bold [font color="#000000" "$folder"]]"
                        }
                      }
                    }
                  }
                
                  # List all the folder content
                  #--------------------------------------------
                  # Folder content
                  table cellspacing=0 width="100%" bgcolor="#eeeedd" {
                    # Select, NewMail, Date, From, Subject, Size.
                    table_row bgcolor="#66ffcc" {
                      table_data width="1" {
                        cgi_checkbox allbox="CheckAll" onclick='CheckAll()'
                      }
                      table_data width="1" {
                        puts "[cgi_imglink newmaillink]"
                      }
                      table_data width="1" {
                        puts "[cgi_link datelink]"
                      }
                      table_data width="150" {
                        puts "[cgi_link fromlink]"
                      }
                      table_data align=center width="200" {
                        puts "[cgi_link subjectlink]"
                      }
                      table_data width="20" {
                        puts "[cgi_link sizelink]"
                      }
                    }

                    set f [open "|/bin/ls -t1 $folderdir" r]
                    set msgcount 0
                    set maillist [list]
                    cgi_export fname=

                    while { -1 != [gets $f fname]} {
                      set fp [open "$folderdir/$fname" r]
                      if { $fp != 0 } {
                          set msgcount [expr $msgcount + 1]
                          fconfigure $fp -translation binary
                          set magic [read $fp 4]
                          set x [read $fp 4]
                          binary scan $x I hdrsize
                          # Skip rest of the 16 bytes.
                          read $fp 16  
                          set hdrsize [ expr $hdrsize - 16 ]
                          set y [read $fp $hdrsize]
                          binary scan $y A* info
                          
                          set organization ""
                          set subject ""
                          set priority ""
                          set date ""

                          # R for read, N for unread (new)
                          # default is R.
                          set status "R"

                          set from ""
                          set to   ""
                          set date ""
                          set subject ""
                          set priority ""
                          set organization ""
                          set status ""

                          regexp "From:(\[^\|\n]+)" $info dummy from
                          regexp "To:(\[^\|\n]+)"   $info dummy to
                          regexp "Date:(\[^\|\n]+)" $info dummy date
                          regexp "Subject:(\[^\|\n]+)" $info dummy subject
                          regexp "Priority:(\[^\|\n]+)" $info dummy priority
                          regexp "Organization:(\[^\|\n]+)" $info dummy organization
                          regexp "Status:(\[^\|\n]+)" $info dummy status

                          set filesize [file size "$folderdir/$fname" ]
                          set filesizeink [ expr $filesize / 1000 ]
                          set filesize [ expr $filesize / 8000 ] 
                          set filesizestr "$filesize s ($filesizeink KB)"
                          if { $filesize >= 60 } {
                              set min [expr $filesize / 60]
                              set sec [expr $filesize % 60]
                              set filesizestr "$min:$sec min ($filesizeink KB)"
                          } 
                          
                          set datebin [binary format A25 $date ]
                          binary scan $datebin A* date
                          set dateitem "[font size=-1 "[printdate $date]"]"
                          
                          set from_id ""
                          set to_id   ""
                          regexp "^sip:(.*)" $from dummy from_id
                          regexp "^sip:(.*)" $to   dummy to_id
                          
                          # From should display short address.
                          # use ... if address is long.
                          set fromdisplay $from
                          set fromdisplaysplits [split $fromdisplay {}]
                          if { [llength $fromdisplaysplits] > 20} {
                              set fromdisplaysplits [lrange $fromdisplaysplits 0 17]
                              lappend fromdisplaysplits "." "." "."
                              set fromdisplay [join $fromdisplaysplits {}]
                          } 

                          # Store the mails in a list.
                          set from [cgi_quote_url "$from"]
                          set fromdisplay [cgi_quote_html "$fromdisplay"]
                          set fromitem "[url "$fromdisplay" "data:application/sip,$from"]"
                          if { $organization != "" } {
                              set fromitem "$fromitem <br> {$organization}";
                          }
                          set fromitem "$fromitem [url "[emailicon]" "mailto:$from_id"]"
                          set toitem "$to"
                          if { $subject == "" } {
                              set subjectitem "Message: $fname"
                          } else {
                              set subjectitem $subject
                          }
                          if { $priority != "" } {
                              set subjectitem "\[$priority\] $subjectitem"
                          }
                          set subjectitem "[url "$subjectitem" "javascript:document.vmail._VMaction.value='retrieve';document.vmail.fname.value='$fname';document.vmail.submit();"]" 
                          
                          set sizeitem "[font size=-1 "$filesizestr"]"

                          if { $status == "N" } {
                              set statusitem "[newmailimg]"
                          } else {
                              set statusitem "[cgi_nbspace]"
                          }

                          # Set the date as integer.
                          set dateint 0
                          scan "[clock scan $date]" "%d" dateint


                          set mailentry [list "$fname" [list "$date" "$dateitem"] \
                            [list "$from" "$fromitem"] [list "$subject" "$subjectitem"]\
                            [list $filesize "$sizeitem"] "$statusitem"  \
                            $dateint \
                            $filesize \
                          ]

                          lappend maillist $mailentry

                          catch {close $fp}

                        }
                        # If { $fp != 0 }
                     }
                     # while { all files }

                     if { $sort == "date" } {
                         # Sort by date.
                         set maillist [lsort -integer -decreasing -index 6 $maillist]
                     } elseif { $sort == "from" } {
                         # Sort using sender URI.
                         set maillist [lsort -dictionary -index 2 $maillist]
                     } elseif { $sort == "subject" } {
                         set maillist [lsort -dictionary -index 3 $maillist]
                     } elseif { $sort == "size" } {
                         set maillist [lsort -integer -index 7 $maillist]
                     }

                     for { set c 0 } { $c < [llength $maillist]} {incr c} {
                       set mailentry [lindex $maillist $c]

                       set fname       "[lindex $mailentry 0]"
                       set dateitem    "[lindex [lindex $mailentry 1] 1]"
                       set fromitem    "[lindex [lindex $mailentry 2] 1]"
                       set subjectitem "[lindex [lindex $mailentry 3] 1]"
                       set sizeitem    "[lindex [lindex $mailentry 4] 1]"
                       set statusitem  "[lindex $mailentry 5]"

                       table_row {
                         table_data { cgi_checkbox DeleteVMailList=$fname }
                         # XXX How do we decide if this is a new mail.
                         table_data { puts "$statusitem" }
                         table_data nowrap  { puts "$dateitem" }
                         table_data nowrap { puts "$fromitem" }
                         # table_data { puts "$toitem"}
                         table_data align=center { puts "$subjectitem" } 
                         table_data nowrap align=right { puts "$sizeitem" }
                       }
                     }
                     # Table row for loop

                     # Display the table rows for each element in the list

                     # Comment out following for loop lines if you do not
                     # want additional blank table rows in case number of messages
                     # is less than, say 7. --TODO
                     for { set c $msgcount } { $c < 7 } { incr c } {
                       table_row {
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                       }
                     }
                     # End of Additional blank table rows.

                   }
                   # End of table for folder display
                   #------- end of folder display --------------

                   # Delete and Move to Folder 
                   table border=0 cellspacing=2 cellpadding=0 width="100%" {
                       table_row {
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link deletelink]]</dvi>"
                           }
                           table_data nowrap bgcolor="#cccccc" {
                             cgi_select tobox "onChange=document.vmail._VMaction.value='MoveTo';document.vmail.submit();" {
                               cgi_option "-Move to Folder-" value= selected
                               for { set i 0} {$i < [llength $folderslist]} {incr i} {
                                   set fld "[lindex $folderslist $i]"
                                   if { $folder != $fld } {
                                       cgi_option "$fld" value=$fld
                                   }
                               }
                               # List other user defined folders.XXX
                             }
                           }
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link refreshlink]]</dvi>"
                           }
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link change_user "Change<br>User" "vmail.cgi"]]</dvi>"
                           }
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link signoutlink]]</dvi>"
                           }
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link restartlink]]</dvi>"
                           }
                       }
                   }

                   table border=0 cellspacing=2 cellpadding=0 width="100%" {
                       table_row nowrap {
                         table_data nowrap bgcolor="#cccccc" {
                           puts "<dvi class='buttn'>[font color=#000066 [cgi_link forwardlink]]</dvi>"
                           set emaillist ""
                           put "selected mails to these email(s):";  cgi_text emaillist size=16
                         }
                       }
                   }
                   # End of Delete and Move to folder.

                 }  elseif { $command == "displayfolderlist" } {
                   # List of available folders.
                   # Number of messages in each and total content size.

                   # Print the directory name.
                   table border=0 cellspacing=0 cellpadding=0 width="100%" bgcolor="33cc00" {
                       table_row {
                         table_data {
                           cgi_center {
                             puts "[bold [font color="#000000" "Folders in $topdir"]]"
                           }
                         }
                       }
                   }

                   # Print the table of folders.
                   table cellspacing=0 width="100%" bgcolor="#eeeedd" {
                     # Foldername, Number of files, Total Size.
                     table_row bgcolor="#66ffcc" {
                       table_data width="1" {
                           puts "[cgi_nbspace]"
                           # cgi_checkbox allbox="CheckAll" onClick='CheckAll()'
                       }
                       table_data width="50%" { puts "[bold "Folder(s)"]" }
                       table_data width="100" align=right { puts "[bold "Messages"]" }
                       table_data width="100" align=right { puts "[bold "Size"]" }
                     }
                     
                     # Print each folder info.
                     for { set i 0 } { $i < [llength $folderslist] } {incr i} {
                       set fldr "[lindex $folderslist $i]"
                       table_row {
                         table_data { cgi_checkbox DeleteVMailList=$fldr }
                         table_data { 
                           cgi_link fldrlink "$fldr" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.folder.value='$fldr';document.vmail.submit();"
                           puts "[cgi_link fldrlink]"
                         }
                         table_data align=right {
                           set td [open "|/bin/ls -t1 $topdir/$fldr" r]
                           set c 0
                           set totalsize 0
                           while { -1 != [gets $td filename] } {
                             if { [file isfile "$topdir/$fldr/$filename"] } {
                               incr c
                               set totalsize [expr $totalsize + [file size "$topdir/$fldr/$filename"]]
                             }
                           }
                           catch {close $td}
                           puts "$c"
                         }
                         table_data align=right {
                             set kbsize [expr $totalsize / 1000]
                             puts "$kbsize KB"
                         }
                      }
                    }

                     # Comment out following for loop lines if you do not
                     # want additional blank table rows in case number of folders
                     # is less than, say 4. --TODO
                    for { set i [llength $folderslist] } { $i < 4 } { incr i } {
                       table_row {
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                       }
                     }
                     # End of Additional blank table rows.
                  }

                  p
                  puts "<dvi class='buttn'>[font color=#000066 "[cgi_link deletefolder  "Delete" "javascript:document.vmail._VMaction.value='deletefolder';document.vmail.submit();"] selected folders"]</dvi>"
                  br

                  set newfolder ""
                  put "Create new folder:";  cgi_text newfolder size=16
                  puts "<dvi class='buttn'>[font color=#000066 [cgi_link createfolder  "Create" "javascript:document.vmail._VMaction.value='createfolder';document.vmail.submit();"]]</dvi>"

                 } elseif { $command == "displayoptions" } {
                     # Display and prompt for modification of options.
                     set configfile "$topdir/config"
                     set optionslist [list]

                     if {[catch {set cf [open $configfile r]}] == 0 } {
                         # Opened the file successfully.
                         lappend optionslist [list "newpassword" ""]
                         while { -1 != [gets $cf str]} {
                             regexp "^(\[^\ ]+)(\ +)(.+)" $str dummy h dummy2 v
                             if { $h != "" } {
                                 lappend optionslist [list "$h" "$v"]
                             }
                         }
                         catch { close $cf}

                     } else {
                         p
                         h3 "No configuration file found. Creating default."
                         p
                         set optionslist [list [list "newpassword" ""] \
                                 [list "emailto" "default"]     \
                                 [list "rtspserver" "default"] \
                                 [list "httpserver" "default"] \
                                 [list "emaildomain" "default"] \
                                 [list "onbusy" "default" ]     \
                                 [list "noresponse" "default" ] \
                                 [list "topdir" "default" ] \
                                 [list "msgfile" "default" ] \
                                 [list "rtspmsgurl" "default" ] \
                                 [list "vmailhome" "default" ] \
                                 [list "register" "default" ] \
                                 [list "waittimeout" "default" ] \
                                 [list "randomdigits" "default" ] \
                                 [list "httpmsgurl" "default" ] \
                                 ]
                     }
                     set option ""
                     cgi_export option

                     table border=0 {
                       set emailto_present 0

                       for { set c 0} {$c < [llength $optionslist]} {incr c} {
                         set hv [lindex $optionslist $c]
                         set h [lindex $hv 0]
                         set v [lindex $hv 1]

                         table_row valign=top {
                         switch $h {
                           newpassword  {
                             table_data {
                               global g_passwdfile
                               puts "[bold "Your password"] (newpassword)"
                               br; puts "You can change your password."
                             }
                             table_data {
                               cgi_text $h=$v type=password size=20
                             }
                           }
                           emailto   {
                             global g_emaildomain
                             table_data {
                               puts "[bold "Email notification to"] (emailto)[blink "This is very important"]"
                               br; puts "Specify the email address at which you would like to get notified of a new mail, e.g., $user@$g_emaildomain"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                             set emailto_present 1
                           }
                           rtspserver {
                             global g_rtspserver
                             table_data {
                               puts "[bold "RTSP server address"] (rtspserver)"
                               br; puts "Specify the RTSP server address including the appropriate port number, e.g., $g_rtspserver"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           httpserver {
                             global g_httpserver
                             table_data {
                               puts "[bold "HTTP server address"] (httpserver)"
                               br; puts "Specify the HTTP server address, e.g., $g_httpserver"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           emaildomain {
                             global g_defaultdomain
                             table_data {
                               puts "[bold "Email domain"] (emaildomain)"
                               br; puts "Specify the default emaildomain, e.g., $g_emaildomain"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           topdir {
                             global g_vmailuserdir
                             table_data {
                               puts "[bold "Users top directory"] (topdir)"
                               br; puts "Specify the top level directory for the users home, e.g., $g_vmailuserdir/LOGNAME"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           msgfile {
                             global g_msgfile
                             table_data {
                               puts "[bold "User's message file"] (msgfile)"
                               br; puts "Specify the message file name, e.g., $g_msgfile"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           rtspmsgurl {
                             global g_rtspmsgurl
                             table_data {
                               puts "[bold "RTSP URL template"] (rtspmsgurl)"
                               br; puts "Specify the RTSP URL for message, e.g., $g_rtspmsgurl"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           httpmsgurl {
                             global g_httpmsgurl
                             table_data {
                               puts "[bold "HTTP URL template"] (httpmsgurl)"
                               br; puts "Specify the HTTP URL for message, e.g., $g_httpmsgurl"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           vmailhome {
                             global g_vmailhome
                             table_data {
                               puts "[bold "User's home"] (vmailhome)"
                               br; puts "Specify HTTP URL for user's home, e.g., $g_vmailhome"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           register {
                             global g_register
                             table_data {
                               puts "[bold "SIP registration"] (register)"
                               br; puts "Specify the SIP URI to register with SIP registrar and the address of the SIP registrar, e.g., $g_register"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           waittimeout {
                             global g_waittimeout
                             table_data {
                               puts "[bold "Waiting time"] (waittimeout)"
                               br; puts "Specify the numberof seconds to wait before transferring a call to voice mail system, e.g., $g_waittimeout"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           randomdigits {
                             global g_randomdigits
                             table_data {
                               puts "[bold "File name for messages"] (randomdigits)"
                               br; puts "Specify the number of digits to be used in generating the random filename for your messages, e.g., $g_randomdigits"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           onbusy {
                             global g_onbusy
                             table_data {
                               puts "[bold "Audio (.au) file for busy"] (onbusy)"
                               br; puts "Specify the RTSP URL of the .au file you would like to be played back to the caller when your phone is busy, e.g., $g_onbusy"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           noresponse {
                             global g_noresponse
                             table_data {
                               puts "[bold "Audio (.au) file for no response"] (noresponse)"
                               br; puts "Specify the RTSP URL of the .au file you would like to be played back to the caller when there is no response from your phone,e.g., $g_noresponse"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           default { 
                             table_data {
                               global g_vmailhomeurl
                               puts "[bold "$h"]"
                               br; puts "This is a user defined option."

                               puts "Visit [url "$g_vmailsoftwarepage" "$g_vmailsoftwarepage"] for more information"
                             }
                             table_data {
                               cgi_text $h=$v size=20
                             }
                           }
                           }
                           table_data {
                            cgi_link mylink \
                            "Change" "javascript:document.vmail._VMaction.value='changeoption';document.vmail.option.value='$h';document.vmail.submit();"
                            puts "<dvi class='buttn'>[cgi_link mylink]</dvi>"
                           }
                         }
                       }
                       if { $emailto_present == 0 } {
                           # Add email to field.
                           table_data {
                               puts "[bold "Email notification to"] (emailto)"
                               br; puts "Specify the email address at which you would like to get notified of a new mail. [bold "This is very important"]"
                           }
                           table_data {
                               cgi_text emailto= size=20
                           }
                           table_data {
                            cgi_link mylink \
                            "Change" "javascript:document.vmail._VMaction.value='changeoption';document.vmail.option.value='emailto';document.vmail.submit();"
                            puts "<dvi class='buttn'>[cgi_link mylink]</dvi>"
                           }
                       }
                     }
                 }
                 print_copyright

                 # End of the mail table.
               }
               # End of toplevel table data 
           }
           # End of top level table row.
         }
         # End of top level table.
         }
         # End of cgi_form
     }	
     # End of cgi_body.
   } 
   # End of diplay of VoiceMail page (Non login page)
}
# End of cgi_eval


#************************** END OF FILE ********************************
