/***************************** FILE HEADER *****************************
#               Copyright (c) 2000 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>  /* getopt */
#include <gdbm.h>

void usage(char *prog) 
{
  fprintf(stderr,"Usage: %s [-f filename] [-c command]\n", prog);
  exit (1);
} /* usage */

void errorfxn(char *str)
{
  printf("GDBMERR: %s\n", str);
}

int main(int argc, char *argv[])
{
  int c;
  char line[200];
  char *command = NULL;
  int  inlinecommand = 0;
  GDBM_FILE dbf;

  static char *help = 
    "list                : to list all users\n"
    "add  login conffile : to add a new user\n"
    "remove login        : to remove an existing user\n"
    "quit or exit        : to exit the application\n"
    "help                : to get list of commands\n"
    ;

  char *filename = strdup("vmail.conf");

  /* Parse command line options */
  while((c = getopt(argc, argv, "hf:c:")) != EOF) {
    switch(c) {
    case 'h' :
      usage(argv[0]);
      return 1;
    case 'f' :
      filename = strdup(optarg);
      break;
    case 'c' :
      strcpy(line,optarg);
      inlinecommand = 1;
      break;
    default:
      usage(argv[0]);
      exit(1); /* Not required */
    }
  }

  if ( !inlinecommand ) {
    printf("Enter \"help\" to get list of commands\n");
  }

  do {
    if ( !inlinecommand ) {
      printf("sipum-conf> ");
      fflush(stdout);
      if ( fgets(line, sizeof(line)-1, stdin) == NULL ) {
        break;
      }
      line[strlen(line)-1] = 0;
    }

    command = strtok(line, " ");
    if ( command == NULL || !(strcmp(command, "\n"))) {
      if ( inlinecommand ) {
        break;
      }
      else {
        continue;
      }
    }

    if ( !strcmp(command, "list") ) {
      datum key, nextkey, record;
      if ((dbf = gdbm_open(filename, 512, GDBM_WRCREAT, 0755, errorfxn))
          == NULL) {
        perror("gdbm_open");
      }
      else {
        key = gdbm_firstkey(dbf);
        while ( key.dptr ) {
          record = gdbm_fetch(dbf, key);
          if ( inlinecommand ) {
            printf("%s:%s\n", key.dptr, *record.dptr ?
                   record.dptr : "<default>");
          }
          else {
            printf("\t%s\t: %s\n", key.dptr, *record.dptr ?
                   record.dptr : "<default>");
          }
          nextkey = gdbm_nextkey(dbf, key);
          free(record.dptr);
          free(key.dptr);
          key = nextkey;
        }
        gdbm_close(dbf);
      }
    }
    else if ( !strcmp(command, "add") ) {
      char *user = strtok(NULL, " ");
      char *path = strtok(NULL, " ");
      datum key, record;

      if ( user == NULL ) {
        printf("usage: add userlogin config_file_path\n");
      }
      else {
        if ( path == NULL ) {
          path = "";
        }
        
        if ((dbf = gdbm_open(filename, 512, GDBM_WRCREAT, 0755, NULL))
            == NULL) {
          perror("gdbm_open");
        }
        else {
          key.dsize = strlen(user) + 1;
          key.dptr = user;
          record.dsize = strlen(path) + 1;
          record.dptr = path;
          gdbm_store(dbf, key, record, GDBM_REPLACE);
          gdbm_close(dbf);
        }
      }
    }
    else if ( !strcmp(command, "remove") ) {
      char *user = strtok(NULL, " ");
      datum key;

      if ( user == NULL ) {
        printf("usage: remove userlogin\n");
      }
      else {
        if ((dbf = gdbm_open(filename, 512, GDBM_WRCREAT, 0755, NULL))
            == NULL) {
          perror("gdbm_open");
        }
        else {
          key.dsize = strlen(user) + 1;
          key.dptr = user;
          gdbm_delete(dbf, key);
          gdbm_close(dbf);
        }
      }
    }
    else if ( !strcmp(command, "help") ) {
      printf("%s", help);
    }
    else if (  !strcmp(command, "exit") ||
               !strcmp(command, "quit") ) {
      break;
    }
    else {
      fprintf(stderr, "Unknown command %s\n", command);
    }

  } while ( !inlinecommand );

  if ( !inlinecommand ) {
    printf("Thank you\n");
  }

  return(0);
}
