/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : vmail.h
#
#  Module       : Main header for Voice Mail Server.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 26 Feb 2000
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

/* Common includes */
#include <iostream.h>
#include <stdio.h>

/* Portable library includes */
#include <ptlib.h>
#include <ptclib/inetmail.h>

PDICTIONARY(UserParameterList, PString, PString);

class RegistrationInfoObject : public PObject
{
  PCLASSINFO(RegistrationInfoObject,PObject);
 public:
  PString localName;
  PString registrar;
};

PLIST(RegistrationInfoList, RegistrationInfoObject);

class VoiceMailInfo : public PObject
{
  PCLASSINFO(VoiceMailInfo,PObject);

  public :
  VoiceMailInfo(const PString user);
  ~VoiceMailInfo();

  /**
   * Read the config file and register all the users 
   */
  static void RegisterAllUsers(SIPEndpoint & sip);

  /*
   * Set the default user login, from the user id of person
   * executing the software
   */
  static void SetAdministratorLogin(PString user) { administrator = user; }

  /* Returns default userlogin e.g., kns10 */
  static PString GetAdministratorLogin() { return administrator; }

  /* Returns the unix userid of the user */
  static int GetUserId(PString user);

  /*
   * Set the default domain 
   */
  static void SetDefaultDomain(PString domain) { def_domain = domain; }

  /* Retrive the default domain */
  static PString GetDefaultDomain() { return def_domain; }

  /* Set the config file */
  static void SetConfigFile(PString file) { conf_file = file; }

  /* Get the config file name */
  static PString GetConfigFile() { return conf_file; }

  /* Set the default server */
  static void SetDefaultServer(PString server) { def_server = server; }

  /* get the default server */
  static PString GetDefaultServer() { return def_server; }

  /** Get the users list */
  static PStringArray GetUsers();

  /** Set the authentication file */
  static void SetAuthenticationFile(PString ptr);

  /** Is Authentication needed */
  static bool IsAuthenticationPresent();

  /** Get the Authentication file name */
  static PString GetAuthenticationFile();

 private:
  /* Class members */
  static PString Subst(PString string, PString value, PString
                       newvalue);
  /** Get the config file name for the user */
  static PString GetUserConfigFile(PString userlogin);

  static PString administrator; /* Default login */
  static PString def_domain; /* Default domain */
  static PString def_server; /*SIP server*/
  static PString conf_file;  /* config file */
  static VoiceMailInfo *defaultvmailinfo;
  static PString auth_file;  /* Name of authentication file */
 public:
  static bool    use_local_db;

  /* Object members */
 private:
  /** retrieve the parameter value from config file */
  PString GetConfig(PString attr);
  /** retrieve the parameter value from config file.
      return def if not found */
  PString GetConfig(PString attr,PString def);

  PString userlogin;
  UserParameterList param;
  PString user_conf;

 public:
  /* Retrieve the appropriate URLs or URIs for different types of
   * user agents to retrieve a message */
  PString GetRTSPMessageURL(int msgid);
  PString GetHTTPMessageURL(int msgid);
  PString GetSIPMessageURI(int msgid);
  PString GetFileName(int msgid);
  PString GetHTTPHomeURL(); /* Voice mail home page */

  /* Gettters for different user settable parameters */
  PString GetWelcomeURL(PString type = PString("default"));
  PString GetOnBusyURL();
  PString GetNoResponseURL();
  PString GetEmailAddress();
  int     GetWaitTimeout(); /* in seconds */
  /* Get the IP address of RTSP server */
  PString GetRTSPServerIP(PString host);
  int     GetNewMessageID(); /* new message */

  /* Get the size limit */
  long GetPerMsgSizeLimit();

  RegistrationInfoList GetRegistrationInfo();
};


/************************** END OF FILE ********************************
*/
