/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : vmailinfo.cxx
#
#  Module       : Voice Mail Server.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 26 Feb 2000
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

/* Common includes */
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>  /* srand */
#include <time.h>    /* time */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>  /* strtok_r */
#include <pwd.h>     /* pwgetnam */

#include <gdbm.h> /* Use of gdbm */

#include <dbapi.h>

/* Portable library includes */
#include <ptlib.h>
#include <ptclib/inetmail.h>

/* SIP API */
#include "iptelapi.h"
#include "sipapi.h"
#include "rtspapi.h"
#include "vmail.h"

#include <sys/types.h>
#include <sys/stat.h>  /* stat */
#include <dirent.h>    /* opendir, readdir */

/*
 * Global trace option.
 */
extern bool sipum_debug_flag;
#define TRACE(x) if (sipum_debug_flag) { cout << x << endl; }

#define MAX_RANDOM_DIGITS 6

PString VoiceMailInfo::administrator 
= "kns10"; /* Default login */
PString VoiceMailInfo::def_domain 
= "cs.columbia.edu"; /* Default domain */
PString VoiceMailInfo::def_server 
= "cs.columbia.edu"; /*SIP server*/
PString VoiceMailInfo::conf_file 
= "vmail.conf";  /* config file */
VoiceMailInfo *VoiceMailInfo::defaultvmailinfo = NULL;
PString VoiceMailInfo::auth_file 
= "";  /* Invalid file */
bool VoiceMailInfo::use_local_db
= false; /* Use MySQL instead */

#ifdef USE_MYSQL
void printtable(ResultTable table, int rows, int cols)
{
  if ( rows <= 0 ) {
    printf("Table is empty (size = %d)\n", rows);
  }
  else {
    printf("Table with %d rows\n", rows);

    for (int i=0; i<rows; i++) {
      for ( int j=0; j<cols; j++) {
        printf("%s:", table[i][j] ? table[i][j] : "NULL");
      }
      printf("\n");
    }
  }
}
#endif

PString VoiceMailInfo::GetUserConfigFile(PString userlogin)
{
#ifdef USE_MYSQL
  if ( userlogin == "default" ) {
    return "script/config";
  }
#else
  GDBM_FILE dbf;

  if ((dbf = gdbm_open ((char *) (const char *) conf_file, 
                        512, GDBM_WRCREAT, 0755, NULL))
      == NULL) {
    TRACE( "VMAIL: GetUserConfigFile, " <<
      "gdbm_open failed" );
    return "";
  }

  datum key;
  datum record;
  key.dsize = strlen((const char *)userlogin) + 1;
  key.dptr = (char *)(const char *)userlogin;
  record = gdbm_fetch(dbf, key);
  if ( record.dptr == NULL || *record.dptr == '\0') {
    TRACE( "User config for " << userlogin << " not found" );
  }
  else {
    
    PString name = (const char *) record.dptr;
    free(record.dptr);
    
    TRACE( "User " << userlogin << ":" << name );
    gdbm_close(dbf);
    return name;
  }

  key.dsize = strlen("default") + 1;
  key.dptr = (char *)"default";
  record = gdbm_fetch(dbf, key);
  if ( record.dptr == NULL || *record.dptr == '\0' ) {
    TRACE( "User config for default not found" );
    gdbm_close(dbf);
    return "";
  }
  else {
    
    PString name = (const char *) record.dptr;
    free(record.dptr);
    
    TRACE( "User " << userlogin << ":" << name );
    gdbm_close(dbf);
    return name;
  }
#endif
}

PStringArray VoiceMailInfo::GetUsers()
{
  PStringArray users;

#ifdef USE_MYSQL
  if ( !VoiceMailInfo::use_local_db ) {
    if ( isConnectionEstablished() ) {
      
      int ret = 0;
      ResultTable table = query("put", "", "", "user", &ret); 
      printtable(table, ret, 1);
      for ( int i=0; i<ret; i++) {
        if ( table[i][0] ) {
          if ( strncmp(table[i][0], "default", strlen("default")) != 0) { 
            users.Append(new PString(table[i][0]));
          }
        }
      }
      freeResultTable(table, ret, 1);
    }
    
  }
  else {
#endif

    GDBM_FILE dbf;
    
    if ((dbf = gdbm_open ((char *) (const char *)conf_file, 512, GDBM_WRCREAT, 0755, NULL))
        == NULL) {
      TRACE( "VMAIL: GetUsers" <<
             "gdbm_open failed" );
      return users;
    }
    
    datum key, nextkey;
    key = gdbm_firstkey(dbf);
    
    while ( key.dptr ) {
      nextkey = gdbm_nextkey(dbf, key);
      PString name = (const char *) key.dptr;
      if ( name != "default" ) {
        users.Append(new PString(name));
      }
      free(key.dptr);
      key = nextkey;
    }
    
    gdbm_close(dbf);

#ifdef USE_MYSQL
  }
#endif

  return users;
}

PString VoiceMailInfo::GetConfig(PString attr,
                                 PString def)
{
  if ( param.Contains(attr) ) {
    PString value;
    value = param[attr];
    if ( value == "default" ) {
      TRACE( "VMAIL: Config(" << attr << ") " << def );
      return def; 
    }
    else {
      TRACE( "VMAIL: Config(" << attr << ") " << value );
      return value;
    }
  }
  else {
    TRACE( "VMAIL: Config(" << attr << ") " << def );
    return def; /* return default here */
  }
}

/* Get the name of the config variable from user's config file */
PString VoiceMailInfo::GetConfig(PString attr)
{
  if ( param.Contains(attr) ) {
    PString value;
    value = param[attr];
    if ( value != "default" ) {
      TRACE( "VMAIL: Config(" << attr << ") " << value );
      return value;
    }
  }
  /* comes here for default case */
  return VoiceMailInfo::defaultvmailinfo->GetConfig(attr, "default");
}

PString VoiceMailInfo::GetRTSPMessageURL(int msgid)
{
  /* Get the audio server */
  PString server = GetConfig("rtspserver");
  if ( server == "default") {
    /* self host by default */
    server = PString(SIPLibrary::HostName);
  }
  PString format = GetConfig("rtspmsgurl");

  PString msgidstr(PString::Unsigned, msgid);
  format.Replace("SERVER", server, TRUE);
  format.Replace("LOGNAME", userlogin, TRUE);
  format.Replace("MSGID", msgidstr, TRUE);

  TRACE( "VMAIL: url = " << format );
  return format;
}

/* 
 * Get the URL for the HTTP browser.
 * e.g., http://www.cs.columbia.edu/~kns10/vmail/vmail.cgi
 */
PString VoiceMailInfo::GetHTTPMessageURL(
                             int msgid)
{

  /* Get the audio server */
  PString server = GetConfig("httpserver");
  if ( server == "default") {
    /* self host by default */
    server = "www.cs.columbia.edu";
  }
  PString format = GetConfig("httpmsgurl");

  if ( msgid > 0 ) {
    /* return the message specific URL */
    PString msgidstr(PString::Unsigned, msgid);
    format.Replace("SERVER", server, TRUE);
    format.Replace("LOGNAME", userlogin, TRUE);
    format.Replace("MSGID", msgidstr, TRUE);
  }
  else {
    PString vmailhome = GetConfig("vmailhome");
    if ( vmailhome != "default")  {
      format = vmailhome;
    }
    format.Replace("SERVER", server, TRUE);
    format.Replace("LOGNAME", userlogin, TRUE);
  }
  
  TRACE( "VMAIL: httpurl = " << format );
  return format;
}

/* 
 * Get the URL for the HTTP browser for inbox.
 * e.g., http://www.cs.columbia.edu/~kns10/vmail/vmail.cgi
 */
PString VoiceMailInfo::GetHTTPHomeURL()
{
  return GetHTTPMessageURL(0);
}

/* 
 * Get the URI for the SIP user agent.
 * e.g., sip:kns10-218921-retrieve@server.
 */
PString VoiceMailInfo::GetSIPMessageURI(
                             int msgid)
{

  PString format = GetConfig("sipmsgurl");

  if ( format == "default" ) {
    format = PString("sip:LOGNAME-MSGID-retrieve@") +
      SIPLibrary::HostName;
  }

  /* return the message specific URL */
  PString msgidstr(PString::Unsigned, msgid);
  
  format.Replace("LOGNAME", userlogin, TRUE);
  format.Replace("MSGID", msgidstr, TRUE);
  
  TRACE( "VMAIL: sipurl = " << format );
  return format;
}

/*
 * Get the filename of the user message file from inbox.
 * e.g., /home/kns10/vmail/inbox/8219.au
 */
PString VoiceMailInfo::GetFileName(int msgid)
{
  PString format = GetConfig("msgfile");
  if ( format == "default") {
    format = GetConfig("topdir");
    format += "/inbox/MSGID.au";
  }

  /* return the message specific URL */
  PString msgidstr(PString::Unsigned, msgid);
  format.Replace("LOGNAME", userlogin, TRUE);
  format.Replace("MSGID", msgidstr, TRUE);
  return format;
}

PString VoiceMailInfo::GetWelcomeURL(PString type)
{
  PString server = GetConfig("rtspserver");
  if ( server == "default") {
    /* self host by default */
    server = PString(SIPLibrary::HostName);
  }

  PString topdir = GetConfig("topdir");
  topdir.Replace("LOGNAME", userlogin, TRUE);

  PString filename = "default";

  if ( type == "default" ) {
    type = "noresponse";
  }

  PString path = GetConfig(type);

  /* Check if the URL is not specified then
   * build a default URL */

  if ( path != "" && path.Find("://") == P_MAX_INDEX ) {
    PString newpath = PString("rtsp://SERVER/audio/") +
      path;
    path = newpath;
  }
  path.Replace("LOGNAME", userlogin);
  path.Replace("SERVER", server);

  TRACE( "VMAIL: url = " << path );
  return path;
}

PString VoiceMailInfo::GetOnBusyURL()
{
  return GetWelcomeURL("onbusy");
}

PString VoiceMailInfo::GetNoResponseURL()
{
  return GetWelcomeURL("noresponse");
}

/* Get the email address of the user */
PString VoiceMailInfo::GetEmailAddress()
{
  PString domain = GetConfig("emaildomain");
  PString addr = GetConfig("emailto");
  if ( addr == "default") {
    addr = PString("LOGNAME@DOMAIN");
  }
  addr.Replace("LOGNAME", userlogin, TRUE);
  addr.Replace("DOMAIN", domain, TRUE);

  TRACE( "VMAIL: Email address " << addr );
  return addr;
}

int VoiceMailInfo::GetNewMessageID()
{
  static int firsttime = 1;
  if ( firsttime ) {
    srand(time(NULL));
  }
  firsttime = 0;

  int randomdigits = MAX_RANDOM_DIGITS;
  PString rd = GetConfig("randomdigits");
  if ( rd != "defaults" && rd != "" ) {
    randomdigits = rd.AsInteger();
    if ( randomdigits < 3 ) { /* Min is 3 */
      randomdigits = 3;
    }
    else if ( randomdigits > MAX_RANDOM_DIGITS ) {
      randomdigits = MAX_RANDOM_DIGITS;
    }
  }

  unsigned long max_rand_num = 1;
  for ( int i=0; i<randomdigits; i++) {
    max_rand_num = max_rand_num * 10;
  }

  int maxiteration = 10;
  while ( maxiteration > 0 ) {
    maxiteration --;
    unsigned long r = ((unsigned long) SIPLibrary::Random32());
    r = r % max_rand_num;
    PString filename = GetConfig("msgfile");
    PString msgidstr(PString::Unsigned, r);
    filename.Replace("LOGNAME", userlogin, TRUE);
    filename.Replace("MSGID", msgidstr, TRUE);

    if ( PFile::Exists(filename) ) {
      /* file exists */
      continue;
    }
    /* File does not exist. return the unique number */
    return r;
  }
  /* Could not find till 10 iterations. return 0 as invalid */
  TRACE( "VMAIL: GetNewMessageID() failed" );
  return 0;
}

PString VoiceMailInfo::GetRTSPServerIP(PString messageurl)
{
  PURL url(messageurl);
  PString host = url.GetHostName();
  struct in_addr in;
  in = SIPLibrary::Host2IP((const char *)host);
  PString ip(inet_ntoa(in));
  TRACE( "VMAIL: IP of server is " << ip );
  return ip;
}

/* 
 * Returns the unix userid of the user.
 * returns 0 on error. 
 */
int VoiceMailInfo::GetUserId(PString user)
{
  struct passwd *p;
  p = getpwnam((const char *) user);
  if ( p == NULL ) {
    return 0;
  }
  else {
    return p->pw_uid;    
  }
}

/* Get the waiting period in seconds */
int VoiceMailInfo::GetWaitTimeout()
{
  PString waitstr = GetConfig("waittimeout");
  if (waitstr == "default") {
    waitstr = "10";
  }

  int tm = waitstr.AsInteger();
  if ( tm < 1 ) {
    tm = 1; /* min one seconds delay */
  }

  return tm; 
}

PString VoiceMailInfo::Subst(
         PString str, PString value, PString
         newvalue)
{
  PString substituted = str;
  substituted.Replace(value, newvalue, TRUE /* all occurances */);
  return substituted;
}

RegistrationInfoList VoiceMailInfo::GetRegistrationInfo()
{
  RegistrationInfoList reginfo;

  if ( !VoiceMailInfo::use_local_db ) {
#ifdef USE_MYSQL
    PString registrar;
    if ( isConnectionEstablished() ) {
      
      registrar = VoiceMailInfo::GetDefaultServer();
      
      RegistrationInfoObject *reg = 
        new RegistrationInfoObject();
      reg->localName = userlogin;
      
      if ( registrar != "" && registrar != "default" ) {
        reg->registrar = registrar;
      }
      reginfo.Append(reg);
    }
#endif
  }
  else {

    /* Build the parameter list */
    PTextFile file(user_conf, PTextFile::ReadOnly);
    
    if ( !file.IsOpen() ) {
      TRACE( "VMAIL: config file " << user_conf << " can not be opened"
             );
      return reginfo; /* param list is empty */
    }
    
    PString line;
    while ( file.ReadLine(line) ) {
      PStringArray splits = line.Tokenise(" ", FALSE);
      if ( splits.GetSize() > 0 ) {
        if ( splits[0] == "register" ) {
          PString localName = "LOGNAME@DOMAIN";
          PString registrar = "default";
          
          if (splits.GetSize() > 1 ) {
            localName = splits[1];
            if ( splits.GetSize() > 2 ) {
              registrar = splits[2];
            }
          } 
          if ( registrar == "default" ) {
            registrar = VoiceMailInfo::GetDefaultServer();
          }
          PString domain = VoiceMailInfo::GetDefaultDomain();
          localName.Replace("DOMAIN", domain, TRUE);
          localName.Replace("LOGNAME", userlogin, TRUE);
          registrar.Replace("DOMAIN", domain, TRUE);
          
          
          RegistrationInfoObject *reg = 
            new RegistrationInfoObject();
          reg->localName = localName;
          reg->registrar = registrar;
          reginfo.Append(reg);
        }
      }
    }
  }

  return reginfo;
}

/** Set the authentication file */
void VoiceMailInfo::SetAuthenticationFile(PString ptr)
{
  auth_file = ptr;
}

/** Is Authentication needed */
bool VoiceMailInfo::IsAuthenticationPresent() 
{
  return auth_file != "";
}

/** Get the Authentication file name */
PString VoiceMailInfo::GetAuthenticationFile()
{
  return auth_file;
}

/** Get the size limitation */
long GetTotalDirSize(const char *dirname)
{
  struct stat statbuf;
  long size = 0;
  DIR *dir;
  struct dirent * dirent;

  if ( stat(dirname, &statbuf) != 0 ) {
    return size;
  }

  size += statbuf.st_size;
  
#ifdef S_ISDIR
  if ( !S_ISDIR(statbuf.st_mode) ) {
    return size; /* Return if not a directory */
  }
#endif

  if (( dir = opendir(dirname)) == NULL ) {
    return size;
  }

  while ((dirent = readdir(dir)) != NULL ) {
    if ( strcmp(dirent->d_name, ".") != 0  &&
         strcmp(dirent->d_name, "..") != 0) {
      /* Another file or directory */
      PString buf(PString::Printf, "%s/%s", dirname, dirent->d_name);
      size += GetTotalDirSize(buf);
    }
  }

  closedir(dir);

  return size;
}

long VoiceMailInfo::GetPerMsgSizeLimit()
{
  PString totalstr = GetConfig("totalsize_KB");
  if (totalstr == "default") {
    totalstr = "20000"; /* 20 MB */
  }

  PString permsgstr = GetConfig("max_msgsize_kb");
  if ( permsgstr == "default" ) {
    permsgstr = "10000"; /* 10 MB */
  }

  long totalsize = totalstr.AsInteger() * 1000;
  long permsgsize = permsgstr.AsInteger() * 1000;

  PString format = GetConfig("topdir");
  format.Replace("LOGNAME", userlogin, TRUE);

  long currentsize = GetTotalDirSize((const char *) format);

  if ( currentsize > totalsize ) {
    return 0;
  }
  if ( permsgsize > (totalsize - currentsize) ) {
    return totalsize - currentsize;
  }

  TRACE( "VMAIL: Size limitation of " << permsgsize );
  return permsgsize;
}

VoiceMailInfo::VoiceMailInfo(PString user)
{
  /* Set userlogin */
  userlogin = user;

#ifdef USE_MYSQL
  /* Set the config file name */
  if ( userlogin == "default" ) {
    TRACE( "VMAIL: finding for default");

    user_conf = VoiceMailInfo::GetUserConfigFile(userlogin);

    TRACE( "VMAIL: config file is " << user_conf );
    /* Build the parameter list */
    PTextFile file(user_conf, PTextFile::ReadOnly);
    
    if ( !file.IsOpen() ) {
      TRACE( "VMAIL: config file " << user_conf << " can not be opened"
             );
      return; /* param list is empty */
    }
    
    PString line;
    PString value;
    while ( file.ReadLine(line) ) {
      PStringArray splits = line.Tokenise(" ", FALSE);
      if ( splits.GetSize() > 0 ) {
        if (splits.GetSize() > 1 ) {
          value = splits[1];
          for ( int i=2; i < splits.GetSize(); i++) {
            value += PString(" ") + splits[i];
          }
        } 
        else {
          value = "default";
        }
        param.SetAt(splits[0], new PString(value));
      }
    }
  }
  else {
    if ( isConnectionEstablished() ) {

      int ret = 0;
      PString userhost = userlogin;
      userhost += "@";
      userhost += VoiceMailInfo::GetDefaultDomain();

      ResultTable table = query("put", "user",(char *)((const char
                                                        *)userhost),
                                "*", &ret);
      printtable(table, ret, 13);

      if ( ret > 0 ) {
        PString emailid = userhost;
        param.SetAt("emailto", new PString(emailid));
        /* ALternatively use userlogin@GetDefaultDomain() 
          PString(userlogin) + "@" +
          VoiceMailInfo::GetDefaultDomain()));
        */
        if ( table[0][8] ) {
          param.SetAt("onbusy", new PString(table[0][8]));  //busy
        }
        if ( table[0][9] ) {
          param.SetAt("noresponse", new PString(table[0][9]));
          //noresponse
        }
        if ( table[0][11] ) {
          param.SetAt("waittimeout", new PString(table[0][11]));
          //um_timeout
        }
        
        if ( table[0][10] ) {
          param.SetAt("emailtemplate", new PString(table[0][10]));
          //message_template
        }
      }
 
      freeResultTable(table, ret, 14);
    }
  }

#else  

  user_conf = VoiceMailInfo::GetUserConfigFile(userlogin);
  TRACE( "VMAIL: config file is " << user_conf );
  /* Build the parameter list */
  PTextFile file(user_conf, PTextFile::ReadOnly);

  if ( !file.IsOpen() ) {
    TRACE( "VMAIL: config file " << user_conf << " can not be opened" );
    return; /* param list is empty */
  }

  PString line;
  PString value;
  while ( file.ReadLine(line) ) {
    PStringArray splits = line.Tokenise(" ", FALSE);
    if ( splits.GetSize() > 0 ) {
      if (splits.GetSize() > 1 ) {
        value = splits[1];
        for ( int i=2; i<splits.GetSize(); i++) {
          value += PString(" ") + splits[i];
        }
      } 
      else {
        value = "default";
      }
      param.SetAt(splits[0], new PString(value));
    }
  }

#endif

  if ( VoiceMailInfo::defaultvmailinfo == NULL 
       && userlogin != "default") {
    VoiceMailInfo::defaultvmailinfo = new
      VoiceMailInfo("default");
  }

}

VoiceMailInfo::~VoiceMailInfo()
{
  /* Close the database XXX */
}


/************************** END OF FILE ********************************
*/

