#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "sysdep.h"
#include "dstring.h"
#include "strdupn.h"
#include "md5.h"

#ifndef USE_MYSQL

#warning "MySQL not configured: addsipuser will be non-functional."

int main(int argc, char** argv)
{
  fprintf(stderr, "CINEMA not configured with mysql!\n");
  fprintf(stderr, "%s is non-functional.\n", argv[0]);
  exit(2);
}

#else

#include <mysql.h>

static void printUsage()
{
  printf("Usage: -d sql://user:password@host:port/database ");
  printf("where port is optional ");
  printf("-u sip_user -p sip_password -r realm [-a] [-f to add default@domain]\n");
}

int main(int argc, char**argv)
{
  MYSQL mysql;
  int c; /* used for getopt function */
  extern char* optarg; /* used for getopt function */
  char * mysql_url = NULL;
  char * sip_user = NULL;
  char * sip_password = NULL;
  char * realm = NULL;
  char * md5passwd;
  char * configType;
  char * user;
  char * sqlServer;
  char * portString;
  char * passwd;
  char * database;
  int admin_requested = 0;
  char * domain;
  DString insert;
  DString hash_key;
  int add_default = 0;
  
  while ((c = getopt(argc, argv, "d:u:p:r:af")) != EOF) {
    switch (c) {
    case 'd':
      mysql_url = optarg;
      break;

    case 'u':
      sip_user = optarg;
      break;

    case 'p':
      sip_password = optarg;
      break;

    case 'r':
      realm = optarg;
      break;

    case 'a':
      admin_requested = 1;
      break;

    case 'f':
      add_default = 1;
      break;

    case '?':
      printUsage();
      exit(1);
    }
  }

  if (mysql_url == NULL || sip_user == NULL || sip_password == NULL ||
      realm == NULL) {
    printUsage();
    exit(1);
  }

  mysql_init(&mysql);

  /* get MYSQL connection parameters */
  configType = strtok(mysql_url, ":");
  if (!strcmp(configType, "sql")) { 
    user = strtok(NULL,":");
    user++; /* move past the '//' */
    user++;
    passwd = strtok(NULL, "@");
    printf("mysql password is %s\n", passwd ? passwd : "NULL (ignored)");
    if ( passwd != NULL && !strcmp(passwd,"NULL")) {
      passwd = NULL;
    }
    
    portString = strchr(mysql_url, ':');
    if (!portString) {
      sqlServer = strtok(NULL, "/");
      database = strtok(NULL, "/");

      if (!(mysql_real_connect(&mysql,sqlServer,user,passwd,database,
			       0,NULL,0))) {
	    fprintf(stderr, "Failed to connect to database: Error:\n%s\n", 
		    mysql_error(&mysql));
	    exit(1);
      }
    }
    else {
      sqlServer = strtok(NULL, ":");
      portString = strtok(NULL, "/");
      database = strtok(NULL, "/");
      
      if (!(mysql_real_connect(&mysql,sqlServer,user,passwd,database,
			       atoi(portString),NULL,0))) {
	fprintf(stderr, "Failed to connect to database: Error:\n%s\n", 
		mysql_error(&mysql));
	exit(1);
      }
    }
  }
  else { /* no SQL scheme used */
    fprintf(stderr, "sql uri scheme not used.\n");
    exit(1);
  }

  /* Compute the hash_value */
  DStringInit(&hash_key);
  DStringPrintf(&hash_key, "%s:%s:%s", (const char *) sip_user,
                (const char *) realm,
                (const char *) sip_password);
  md5passwd = MD5L(DStringValue(&hash_key));
  DStringValue(&hash_key);

  /* ---- check if the default@domain is to be added --- */
  domain = strchr(sip_user, '@');
  if ( add_default && domain && *domain && *(domain+1)) {
    domain ++;
    DStringInit(&insert);
    DStringPrintf(&insert, "insert into put (user,hash_value,realm,sip_groups,auth,algorithm,sip_methods,gwclass) values('default@%s','%s','%s','cgi','never','MD5','REGISTER INVITE any','NULL')",
                  domain, md5passwd, realm);
    if ((mysql_query(&mysql, (const char*) DStringValue(&insert))) == 0) {
      fprintf(stderr, "Added user default@%s with password %s\n", 
              domain, sip_password);
    }
    else {
      fprintf(stderr, "Could not add default@%s: Error: %s\n", domain,
          mysql_error(&mysql));
    }
    DStringFree(&insert);
  }

  /* --- Insert the user record --- */
  DStringInit(&insert);
  DStringPrintf(&insert, "insert into put (user,hash_value,realm,sip_groups,auth,algorithm,sip_methods,gwclass) values('%s','%s','%s','cgi%s','never','MD5','REGISTER INVITE any','NULL')",
                sip_user, md5passwd, realm, admin_requested ? " admin" : "");

  if ((mysql_query(&mysql, (const char*) DStringValue(&insert))) != 0) {
    fprintf(stderr, "Database error when trying to add user:\n%s\n",
	    mysql_error(&mysql));
    exit(1);
  }

  DStringFree(&insert);

  mysql_close(&mysql);

  fprintf(stderr, "Successfully added user:%s with password %s\n", 
          sip_user, sip_password);
  return 0;
}

#endif /* USE_MYSQL */
