/*
 * canon_core.h
 * Convert URIs into their canonical forms, as understood by an
 * Internet telephony server (in particular sipd.)
 *
 * Copyright 2001 Columbia University.  All rights reserved.
 */

/* Common header files for canonicalization. */

#ifndef CANONCORE_H_
#define CANONCORE_H_

#include "uri.h"
#include "dbaccess.h"
#include "canonicalizer_interface.h" /* the interface we implement */

typedef enum {
  CANONICALIZE_NO_TRANSFORMATION,
  CANONICALIZE_UNIQUE_TRANSFORMATION,
  CANONICALIZE_MULTIPLE_TRANSFORMATION,
  CANONICALIZE_ERROR
} canonicalize_status;

typedef struct {
  int count;
  char **nameaddrs;
} canonicalize_result;

typedef canonicalize_status (canonicalize_fn)(uri_t*, canonicalize_result*);

extern canonicalize_fn database;
extern canonicalize_fn aliases;
extern canonicalize_fn namemapper;
extern canonicalize_fn dialplan;

extern int init_database  (char **error, cinema_table*);
extern int init_namemapper(char **error);
extern int init_dialplan  (char **error);

struct canonicalize_options {
  int do_aliases;
  int do_namemapper;
  int do_dialplan;
  int require_userparam;  /* Whether user=phone in SIP URL is required for
                           * translating using dialplan */
  char* table_uri;
  char* aliases_file;
  char* dialplan_file;
};

extern struct canonicalize_options options;

#ifdef __GNUC__
/* This function uses printf-style formats, so tell GNUC to do printf
 * format checking. */
#define ATTRIBUTE_PRINTF(fmt, args) \
   __attribute__ ((format (printf, fmt, args)))
#else
#define ATTRIBUTE_PRINTF(fmt, args)
#endif

/* A useful function for setting error strings */
void seterror(char **error, const char *format, ...)
     ATTRIBUTE_PRINTF(2, 3);

#undef ATTRIBUTE_PRINTF


#define DEFAULT_ALIASES_FILE  "/etc/aliases"
#define DEFAULT_DIALPLAN_FILE "dialplan"



/*
 * defined in libsip. those who link to us have to supply implementation
 */
extern int SIP_NameAddr(char *v, DString *name, DString *addr);

#endif /* CANONCORE_H_ */
