# Makefile fragment for canonicalize
# Copyright 1998-1999 by Columbia University; all rights reserved 

# from v1.18 (sipd) canonicalize is split into a main program and
# a shared library that loads as .so / or .dll

canonicalize_dir := $(tools_dir)/canonicalize

DIRS += $(canonicalize_dir)

CANONICALIZE_BIN := \
	$(canonicalize_dir)/libcanon.so \
	$(canonicalize_dir)/canonicalize

CANONICALIZE_SRC := \
	$(canonicalize_dir)/canon_core.c \
        $(canonicalize_dir)/canonicalize.c \
	$(canonicalize_dir)/aliases.c \
	$(canonicalize_dir)/database.c \
	$(canonicalize_dir)/dialplan.c \
	$(canonicalize_dir)/namemapper.c

CANONICALIZE_LIBS := $(DBAPI_LIB) $(MYSQLLIBS) $(ZLIBLIBS) $(LIBCINE_LIB) \
	 $(LIBDICT_LIB) $(NIS_LIBS) $(SOCKLIBS) $(MATHLIBS)

CANONICALIZE_HEADERS := \
	$(canonicalize_dir)/canon_core.h

CANONICALIZE_MISC := \
	$(canonicalize_dir)/module.mk \
	$(canonicalize_dir)/aliastest \
        $(canonicalize_dir)/dialplan.sample \
        $(canonicalize_dir)/canonicalize.dsp \
	$(canonicalize_dir)/canonicalize_exe/canonicalize_exe.dsp \
	$(canonicalize_dir)/canonicalize_interface.txt

CANONICALIZE_MISCDIST := \
	$(canonicalize_dir)/canonicalize.html \
        $(canonicalize_dir)/dialplan.sample


CANONICALIZE_OBJ := \
	$(patsubst %.c,%.o,$(filter %.c,$(CANONICALIZE_SRC)))

CANONICALIZE_DEP := \
	$(CANONICALIZE_OBJ:.o=.d)

# Include files required locally.
CANONICALIZE_INCS := $(DBAPI_INC) $(LIBCINE_INC) $(LIBDICT_INC) $(LIBSIP_INC)

$(CANONICALIZE_OBJ): %.o: %.c
	@echo "compiling $<..."
	$(CC) -fPIC $(CANONICALIZE_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) -c -o $@ $<

$(CANONICALIZE_DEP): %.d: %.c
	@echo "dependencies for $<..."
	$(DEPEND) -d $(canonicalize_dir) -o $@ $(CANONICALIZE_INCS) $(CFLAGS) $(CPPFLAGS) $(DEFS) $<


tools: $(canonicalize_dir)/libcanon.so $(canonicalize_dir)/canonicalize

#we are hardcoding libcanon.so in the ld switch because using $@ wont
#work as it typically contains the path name also.

$(canonicalize_dir)/libcanon.so: $(CANONICALIZE_OBJ) $(DBAPI) $(LIBCINE) $(LIBDICT)
	@echo "building $@..."
	$(LD) -shared -soname libcanon.so -o $@ $^ $(LDFLAGS) $(CANONICALIZE_LIBS) -lc  
	@echo "Before starting sipd, make sure LD_LIBRARY_PATH properly points to $@..."

$(canonicalize_dir)/canonicalize: $(canonicalize_dir)/canonicalize.o $(DBAPI) $(LIBCINE) $(LIBDICT)
	@echo "linking $@..."
	$(CXX) -o $@  $^ $(LDFLAGS) -L$(canonicalize_dir) -lcanon $(CANONICALIZE_LIBS) $(LIBS) 

SRC += $(CANONICALIZE_SRC)
HEADERS += $(CANONICALIZE_HEADERS)

CANONICALIZE_BIN_WIN32 := \
  $(canonicalize_dir)/Release/libcanon.dll \
  $(canonicalize_dir)/Debug/libcanon.dll \
  $(canonicalize_dir)/canonicalize_exe/Debug/canonicalize.exe \
  $(canonicalize_dir)/canonicalize_exe/Release/canonicalize.exe

TOOLS_DIST    += $(CANONICALIZE_SRC) $(CANONICALIZE_HEADERS) \
	$(CANONICALIZE_MISC) $(CANONICALIZE_MISCDIST)
TOOLS_BINDIST += $(CANONICALIZE_BIN) $(CANONICALIZE_MISCDIST)
TOOLS_BINDIST_WIN32 += $(CANONICALIZE_BIN_WIN32) $(CANONICALIZE_MISCDIST)
