/*
 * Returns exit code 0 if user is here, 1 if not or error.
 *
 * Usage: ishere [-cl] name
 *
 * If -c flag is given, the user is only recognized if sitting at the
 * console, with -l flag the user has to be logged in locally rather than
 * through rlogin or telnet.
 *
 * Copyright (c) 1997 by Henning Schulzrinne, Columbia University
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>      /* getopt() */
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "copyright.h"

#ifdef HAVE_UTMPX_H
#  include <utmpx.h>
#else
#  include <utmp.h>
#endif

#if defined(HAVE_UTMP_UT_USER) && !defined(HAVE_UTMP_UT_NAME)
#define ut_name ut_user
#endif

#ifndef HAVE_GETUTENT
#  include "getutent.h"
#endif


static void usage(char *progname)
{
  fprintf(stderr, "Usage: %s [-c] user\n", progname);
  exit(2);
} /* usage */


/*
* Main program.
*/
int main(int argc, char *argv[])
{
  extern char *optarg;
  extern int optind; 
  int islocal = 0;      /* only return 0 if local */
  char *user = NULL;    /* user */
  int c;

#ifdef HAVE_UTMPX_H
  struct utmpx *ut;
#else
  struct utmp  *ut;
#endif

  /* Get commandline arguments. */
  while ((c = getopt(argc, argv, "cl")) != EOF) {
    switch(c) {
    case 'c':
      islocal = 2;
      break;
    case 'l':
      islocal = 1;
      break;
    case '?':
      usage(argv[0]);
      break;
    }
  }
  if (optind < argc) user = argv[optind];
  else usage(argv[0]);

  /* Read all user utmp entries. */
  while (1) {
#ifdef HAVE_UTMPX_H
    if ((ut = getutxent()) == NULL)
#else
    if ((ut = getutent()) == NULL)
#endif
      break;

#if HAVE_UTMP_UT_TYPE
    if (ut->ut_type != USER_PROCESS)
      continue;
#endif

    if (!strcasecmp(ut->ut_name, user)) {
      if (!strcmp(ut->ut_line, "console") ||
          (
#if HAVE_UTMP_UT_SYSLEN
           ut->ut_syslen == 0
#else
           ut->ut_host[0] == '\0'
#endif
           && islocal == 1) ||
          !islocal) return 0;
    }
  }
#ifdef HAVE_UTMPX_H
  endutxent();
#else
  endutent();
#endif
  return 1;
} /* main */
