/*
 * sippasswd.c: simple program for manipulating password file.
 */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "md5.h"

#define LF 10
#define CR 13

#define MAX_STRING_LEN 1024

char *tn;

char *strd(char *s) {
    char *d;

    d=(char *)malloc(strlen(s) + 1);
    strcpy(d,s);
    return(d);
}

void getword(char *word, char *line, char stop) {
    int x = 0,y;

    for(x=0;((line[x]) && (line[x] != stop));x++)
        word[x] = line[x];

    word[x] = '\0';
    if(line[x]) ++x;
    y=0;

    while(line[x] != '\0') {
      line[y++] = line[x++];
    }
}

int getline(char *s, int n, FILE *f) {
    register int i=0;

    while(1) {
        s[i] = (char)fgetc(f);

        if(s[i] == CR)
            s[i] = fgetc(f);

        if((s[i] == 0x4) || (s[i] == LF) || (i == (n-1))) {
            s[i] = '\0';
            return (feof(f) ? 1 : 0);
        }
        ++i;
    }
}

void putline(FILE *f,char *l) {
    int x;

    for(x=0;l[x];x++) fputc(l[x],f);
    fputc('\n',f);
}

int check_for_colon(char *str) {
  if ( str == NULL ) {
    return 0;
  }

  for ( ;*str; str++) {
    if ( *str == ':' )
      return 1;
  }
  return 0;
}

void add_password(char *user, FILE *f, char *realm) {
    char *pw;
    char *newrealm;
    char hash_in[1024];
    char *hash_out;
    
    if ( realm == NULL || *realm == '\0' ) {
      printf("Realm:");
      fflush(stdout);
    }
    else {
      printf("Realm [%s]:", realm);
      fflush(stdout);
    }
    
    newrealm = (char *)calloc(1024, 1);
    if ( getline(newrealm, 1024, stdin) != 0 ||
         *newrealm == CR || *newrealm == LF || *newrealm == '\0') {
      free(newrealm);      
      newrealm = (realm != NULL) ? strd(realm) : NULL;
    }

    if ( newrealm == NULL || 
         check_for_colon(newrealm) ) {
      if ( newrealm == NULL ) {
        fprintf(stderr, "No realm specified, terminating.\n");
      }
      else {
        fprintf(stderr, "Realm can not have \":\", try again.\n");
      }
      if (tn)
        unlink(tn);
      exit(1);
    }

    pw = strd((char *) getpass("New password:"));
    if(strcmp(pw,(char *) getpass("Re-type new password:"))) {
        fprintf(stderr,"They don't match, sorry.\n");
        if(tn)
            unlink(tn);
        exit(1);
    }


    sprintf(hash_in, "%s:%s:%s", user, newrealm, pw);
    hash_out = (char *) MD5L(hash_in);
    fprintf(f, "%s:%s:%s\n", user, newrealm, hash_out);

    free(newrealm);
    free(pw);
}

void usage() {
    fprintf(stderr,"Usage: sippasswd [-c] passwordfile username\n");
    fprintf(stderr,"The -c flag creates a new file.\n");
    exit(1);
}

void interrupted() {
    fprintf(stderr,"Interrupted.\n");
    if(tn) unlink(tn);
    exit(1);
}

int main(int argc, char *argv[]) {
    FILE *tfp,*f;
    char user[MAX_STRING_LEN];
    char line[MAX_STRING_LEN];
    char l[MAX_STRING_LEN];
    char w[MAX_STRING_LEN];
    char r[MAX_STRING_LEN];
    char command[MAX_STRING_LEN];
    int found;

    tn = NULL;
    signal(SIGINT,(void (*)())interrupted);
    if(argc == 4) {
        if(strcmp(argv[1],"-c"))
            usage();
        if(!(tfp = fopen(argv[2],"w"))) {
            fprintf(stderr,"Could not open passwd file %s for writing.\n",
                    argv[2]);
            perror("fopen");
            exit(1);
        }
        printf("Adding password for %s.\n",argv[3]);
        add_password(argv[3],tfp,NULL);
        fclose(tfp);
        exit(0);
    } else if(argc != 3) usage();

    tn = tmpnam(NULL);
    if(!(tfp = fopen(tn,"w"))) {
        fprintf(stderr,"Could not open temp file.\n");
        exit(1);
    }

    if(!(f = fopen(argv[1],"r"))) {
        fprintf(stderr,
                "Could not open passwd file %s for reading.\n",argv[1]);
        fprintf(stderr,"Use -c option to create new one.\n");
        exit(1);
    }
    strcpy(user,argv[2]);

    found = 0;
    while(!(getline(line,MAX_STRING_LEN,f))) {
        if(found || (line[0] == '#') || (!line[0])) {
            putline(tfp,line);
            continue;
        }
        strcpy(l,line);
        getword(w,l,':');
        if(strcmp(user,w)) {
            putline(tfp,line);
            continue;
        }
        else {
          
            printf("Changing password for user %s\n",user);
            getword(r,l,':');
            add_password(user,tfp,r);
            found = 1;
        }
    }
    if(!found) {
        printf("Adding user %s\n",user);
        add_password(user,tfp,NULL);
    }
    fclose(f);
    fclose(tfp);
    sprintf(command,"cp %s %s",tn,argv[1]);
    system(command);
    unlink(tn);
    exit(0);
}
