/*
* Finger client.
* Copyright 1997 by Henning Schulzrinne
*/

#include <sys/types.h>
#include <sys/socket.h>      /* struct sockaddr */
#include <netinet/in.h>      /* sockaddr_in */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>          /* strlen(), strcpy() */
#include <unistd.h>          /* read(), write() */
#include <signal.h>
#include <setjmp.h>
#include <unistd.h>

#include "host2ip.h"

static jmp_buf env;

#define FINGER_PORT    79
#define FINGER_TIMEOUT 10
#define BUFFERED

/*
* Finger timeout handler.
*/
static void finger_timeout(int sig)
{
  if (sig == SIGALRM) longjmp(env, sig);
} /* finger_timeout */


/*
* Retrieve finger information from host 'host' for user 'user' and
* return. Storage is allocated by routine and needs to be freed by caller.
* Since some systems such as Windows/NT don't refuse the connection and
* just hang, the routine times out and returns 0 after 3 seconds.
*/
char *finger(int verbose, char *user, char *host)
{
  int s;
  int len;            /* characters in buffer */
  int char_read = 0;  /* bytes read */
  char line[1024];
  FILE *f;
  char *buffer = 0;
  struct sockaddr_in sin;

  if (setjmp(env)) {
    printf("longjump from alarm.\n");
    return 0;
  }
  signal(SIGALRM, finger_timeout);
  alarm(FINGER_TIMEOUT);
  
  len = 0;

  if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) return 0;
  sin.sin_family = AF_INET;
  sin.sin_port   = htons(FINGER_PORT);
  sin.sin_addr   = host2ip(host);
  if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) return 0;

  buffer = malloc(1);

#ifdef BUFFERED
  f = fdopen(s, "r+");
  fprintf(f, "%s%s\r\n", verbose ? "/W " : "", user);
  fflush(f);
  while (fgets(line, sizeof(line), f)) {
    char_read = strlen(line);
    buffer = realloc(buffer, len + char_read + 1);
    strcpy(&buffer[len], line);
    len += char_read;
  }
  fclose(f);
#else
  if (verbose) write(s, "/W ", 3);
  write(s, user, strlen(user));
  write(s, "\r\n", 2);

  while (read(s, line, sizeof(line))) {
    char_read = strlen(line);
    buffer = realloc(buffer, len + char_read + 1);
    strcpy(&buffer[len], line);
    len += char_read;
  }
  close(s);
#endif
  buffer[len] = '\0';
  alarm(0);

  return buffer;  
} /* finger */
