/*
* Try to check and fix up host name.
* Argument 'host' is assumed to have enough space for fix-up.
*/
#include <sys/types.h>
#include <sys/socket.h>      /* struct sockaddr */
#include <netdb.h>           /* gethostbyname() */
#include <netinet/in.h>      /* sockaddr_in */
#include <arpa/inet.h>       /* inet_addr() */
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include <unistd.h>

static jmp_buf env;

/*
* Timeout handler.
*/
static void signal_handler(int sig)
{
  if (sig == SIGALRM) longjmp(env, sig);
}


/*
* Return 1 is host is a valid DNS name. The routine has a built-in time
* to avoid hanging.
*/
int ishost(char *host)
{
  struct in_addr in;
  register struct hostent *hep;

  if (setjmp(env)) {
    printf("longjump from alarm.\n");
    return 0;
  }
  signal(SIGALRM, signal_handler);

  if (!host) {
    return 0;
  }
  else if ((in.s_addr = inet_addr(host)) != -1) {
    return 1;
  }
  /* Attempt to resolve host name via DNS. */
  else {
    alarm(2);
    if ((hep = gethostbyname(host))) {
      alarm(0);
      return 1;    
    }
    else {
      alarm(0);
      return 0;
    }
  }
} /* ishost */


int hostfix(char *domain, char *host)
{
  char h[256];
  char *s;
  char *d;       /* domain name pointer */
  char h1[256];  /* first part of host name */
  int parts = 1;

  if (!host || !*host) return 0;

  /* count domain parts */
  for (d = host; *d; d++) {
    if (*d == '.') parts++;
  }

  if (ishost(host)) return 1;
  /* Try adding domain, starting with whole and then removing elements
   * from left to right. */
  else if (parts < 2) {
    d = domain;
    while (d) {
      sprintf(h, "%s%s%s", host, *d == '.' ? "" : ".", d); 
      if (ishost(h)) {
        strcpy(host, h);
        return 1;
      }
      d = strchr(d+1, '.');
    }
  }
  /* Try taking first word of host and add pieces of the domain. */
  else {
    /* Extract first part of host name. */
    for (s = host, d = h1; *s && *s != '.'; s++) {
      *d++ = *s; 
    }
    *d = '\0';
 
    d = domain;
    while (d) {
      sprintf(h, "%s%s%s", h1, *d == '.' ? "" : ".", d); 
      if (ishost(h)) {
        strcpy(host, h);
        return 1;
      }
      d = strchr(d+1, '.');
    }
  }
  return 0;
} /* host2ip */
