#! /usr/local/bin/tclsh

# Create or modify an address for the address book.

source where.tcl
source zone.tcl
source countries.tcl

set F(addressclass) {"Address class" 1 text 20}
set F(companyname)  {"Company name"  2 text 50}
set F(department)   {"Department"    3 text 50}
set F(room)         {"Room"          4 text 50}
set F(street)       {"Street"        5 text 50}
set F(city)         {"City"          6 text 50}
set F(state) {"U.S. State" 7 select {"" "AK" "AL" "AR" "AZ"
    "CA" "CO" "CT" "DC" "DE" "FL"
    "GA" "HI" "IA" "ID" "IL" "IN"
    "KS" "KY" "LA"
    "MA" "MD" "ME" "MI" "MN" "MO" "MS" "MT"
    "NC" "ND" "NE" "NH" "NJ" "NM" "NV" "NY"
    "OH" "OK" "OR"
    "PA" "RI" "SC" "SD" "TN" "TX"
    "UT" "VA" "VT" "WA" "WI" "WV" "WY"}}
set F(province)     {"Province"         7 text 50}
set F(postalcode)   {"Postal/zip code"  8 text 20}
set F(country)      [list "Country"         9 select  $countries \
                     tz(this,this.selectedIndex)]
set F(timezone)     [list "Time zone"       10 select $zonelist]
set F(geo)          {"Longitude/Latitude"   11 text 50}
set F(phone)        {"Phone number"         12 text 50}
set F(mobile)       {"Mobile (cell) phone"  13 text 50}
set F(pager)        {"Pager"                14 text 50}
set F(fax)          {"Fax"                  15 text 50}

cgi_eval {
  set level [auth AddressEdit "Create or Modify Address" 2]

  cgi_body {
    import_default action       ""
    import_default person       "#"
    import_default for          ""
    import_default addressclass ""

    cgi_script language=JavaScript src=tz.js {}

    set res [sql "SELECT id FROM person WHERE id=$person AND owner='$for'"]
    if {$res == ""} {
      p "Person $person doesn't exist for owner $for."
      cgi_exit
    }

    # existing address classes
    set ac [sql "SELECT addressclass FROM address WHERE person=$person"]

    if {$action == "Edit"} {
      if {$addressclass == ""} {
        p "Address class must have a value."
        cgi_exit
      }
      set q [tableforminput F]
      if {$person != "" && [lsearch $ac $addressclass] >= 0} {
        sql "UPDATE address SET [join $q ,] WHERE person=$person AND addressclass='$addressclass'"
        h1 "Updated $addressclass address for $person."
      } else {
        sql "INSERT address SET person=$person,[join $q ,]"
        h1 "Inserted $addressclass address for $person."

        # set preferred address class if first address
        if {$ac == ""} {
          sql "UPDATE person SET preferred='$addressclass' WHERE person=$person"
        }
      }
    } elseif {$action == "Delete"} {
      p "Are you sure you want to delete the $addressclass address?"
      form AddressEdit {
        cgi_export person
        cgi_export for
        cgi_export addressclass
        submit_button "action=Yes"
        submit_button "action=No"
      }
    } elseif {$action == "Yes"} {
      p "Address $addressclass deleted."
      sql "DELETE FROM address WHERE person=$person AND addressclass='$addressclass'"
    } elseif {$action == ""} {
      if {$person != "" && $addressclass != ""} {
        h1 "Editing $addressclass address"
        set fieldlist [array names F]
        set res [sql "SELECT [join $fieldlist ","] FROM address WHERE person=$person AND addressclass='$addressclass'"]
        if {$res == ""} {
          p "Address of type $addressclass doesn't exist for $person."
          cgi_exit
        } else {
          maparray [lindex $res 0] $fieldlist V
        }
        set F(addressclass) [lreplace $F(addressclass) 2 2 show_export]
      } else {
        h1 "Adding new addressbook entry"
        set V(country) "US"
      }

      form AddressEdit {
        cgi_export person
        cgi_export for
        tableform {action=Edit action=Delete} F V
      }
    }

    sql disconnect
  }
}
