#! /usr/local/bin/tclsh

# Add or delete aliases for a particular user. Can only be done by admin.

source where.tcl

cgi_eval {
  set level [auth Aliases "Edit User Aliases"]
 
  validate_user_script

  cgi_body {
    import_default add     ""
    import_default delete  ""

    if {$level < 2} {
      cgi_exit
    }

    h1 "SIP Aliases for User $for"

    set nouser 0
    if {$for == ""} {
      set nouser 1
    } else {
      set c [sql "SELECT count(user) from put where user='$for'"]
      if {$c == 0} {
        p "User $for does not exist."
        set nouser 1
      }
    }

    if {$nouser} {
      form Aliases onsubmit="return validate_user(this.user)" {
        cgi_export add
        cgi_export delete

        table {
          table_row {
            td "User identifier ([italic user@domain]):"
            table_data {cgi_text user=$for size=50}
            table_data {submit_button "=Modify aliases"}
          }
        }
      }
    } else {
      # add alias
      if {$add != ""} {
        # alias can't be an existing user name
        set c [sql "SELECT count(user) from put where user='$add'"]
        if {$c > 0} {
          p "Alias $add is already a user identifier and can't be redefined."
        } else {
          # checks whether alias already exists
          if {[catch {sql "INSERT aliases SET alias='$add',primary_user='$for'"} msg]} {
            p "Cannot insert alias $add: $msg."
          } else {
            p "Added alias $add."
          }
        }
      }

      # delete alias
      if {$delete != ""} {
        if {[catch {sql "DELETE FROM aliases WHERE alias='$delete' and primary_user='$for'"} msg]} {
          p "Could not delete alias $delete."
        } else {
          p "Deleted alias $delete."
        } 
      }

      # list current aliases
      set aliascount 0
      set rows [sql "SELECT alias,DATE_FORMAT(last_modified,'%d %b %Y %H:%i') FROM aliases WHERE primary_user='$for'"]
      if {$rows != ""} {
        table {
          table_row {
            th "Alias"
            th "Last modified"
            th "Delete?"
          }
          foreach row $rows {
            maplist $row alias last_modified
            table_row bgcolor=[lindex $bglist [expr $aliascount % 2]] {
              td "$alias"
              td "$last_modified"
              td [url [imglink Delete] [cgi_cgi Aliases [cgi_set for $for] [cgi_set delete $alias]]]
            }
            incr aliascount
          }
        }
      } else {
        p "User [bold $for] has no aliases."
      }

      # field for adding new aliases
      h2 "Adding a new alias"
      form Aliases onsubmit="return validate_user(this.add)" {
        cgi_export for
        table {
          table_row {
            td "Alias user identifier ([italic user@domain]):"
            table_data {cgi_text add= size=50}
            table_data {submit_button "=Add alias"}
          }
        }
      }
    }
  }
}
