#! /usr/local/bin/tclsh

# Write the sipd configuration file.
# If an item has a trailing number (e.g., LogFormat.1), write several.

source where.tcl

cgi_input
cgi_eval {
  cgi_body {
    import_default what  "sipd"
    import_default where "."

    if {$what == "sipd" || $what == "cinema"} {
      set f $what
    } else {
      p "Invalid configuration $what."
      cgi_exit
    }

    # read the current configuration file
    source config-${f}.tcl
    config ${where}/${f}.conf

    h1 "Modifying $what Configuration"

    # check entries
    set errors 0
    foreach item $c {
      set v        [lindex $item 0]
      set d        [lindex $item 1] 
      set t        [lindex $item 2]
      set len      [lindex $item 3]
      set instance [lindex $item 5]

      set vv "${v}${instance}"
      if {![info exists $vv]} {
        set $vv ""
      }
      import_default $vv [set $vv]

      if {$t == "directory" && ![file isdir [set $vv]]} {
        p "$v: [set $vv] is not a directory."
        incr errors
      }
      if {$t == "integer" && ![regexp {^[1-9][0-9]*$} [set $vv]]} {
        p "$v: [set $vv] is not an integer."
        incr errors
      }
      if {$t == "email" && ![regexp {^[A-Za-z0-9+-.]+@[A-Za-z0-9][.A-Za-z0-9-]+$} [set $vv]]} {
        p "$v: [set $vv] is not an email address."
        incr errors
      }
      if {$t == "p_url" && ![validate_path_url [set $vv] scheme user pw host port path]} {
        p "$v: [set $vv] is not a valid URL of the format scheme://user:pw@host/path."
        incr errors
      }
    }

    if {$errors > 0} {
      p "Please go back and fix errors first."
      cgi_exit
    }

    # open the configuration file
    if {[catch {file rename ${where}/${f}.conf ${where}/${f}-[clock seconds].conf} error]} {
      p "Could not rename old configuration: $error"
    }
    if {[catch {set fd [open ${where}/${f}.conf w]} error]} {
      p "Cannot open configuration file $file: $error"
      cgi_exit
    }

    # write the configuration file
    foreach item $c {
      set v        [lindex $item 0]
      set d        [lindex $item 1] 
      set t        [lindex $item 2]
      set len      [lindex $item 3]
      set instance [lindex $item 5]

      set vv "${v}${instance}"

      set description [split [fmt "\# $v: $d"] "\n"]
      set description [join [lrange $description 0 [expr [llength $description] - 2]] "\n\# "]
      puts $fd $description
      set value [set $vv]
      if {$value != ""} {
        # Escape any previous quotes inside the value
        set newvalue $value
        regsub -all {"} $newvalue {\\"} value

        # enclose elements in quotation marks
        if {[regexp {[[:space:]]} $value] && [regexp {^"} $value] == 0} {
          set value "\"$value\""
        }
        puts "$v $value"
        cgi_br
        puts $fd "$v $value"
      }
      puts $fd ""
    }
    close $fd

    cgi_exit
  }
}
