#! /usr/local/bin/tclsh

# List and edit SIP contacts for a user.

source where.tcl

cgi_eval {
  set level [auth Contacts "Contacts" 2]

  validate_user_script

  cgi_body {
    import_default op
    import_default contact
    import_default display_name
    import_default action
    import_default expires
    import_default q

    set nouser 0
    if {$for == ""} {
      set nouser 1
    } else {
      set c [sql "SELECT count(user) from put where user='$for'"]
      if {$c == 0} {
        p "User $for does not exist."
        set nouser 1
      }
    }

    # convert "Display <URI>" into display_name and contact
    if {[regexp {(.*)<([A-z@0-9./:]+)>} $contact x d c]} {
      set display_name [string trim $d]
      set contact $c
    }

    if {$nouser} {
      form Contacts onsubmit="return validate_user(this.user)" {
        cgi_export op
        cgi_export contact
        cgi_export display_name
        cgi_export action
        cgi_export expires
        cgi_export q

        table {
          table_row {
            td "User identifier ([italic user@domain]):"
            table_data {cgi_text user=$for size=50}
            table_data {submit_button "=Modify aliases"}
          }
        }
      }
    } else {
      h1 "Contacts for User $for"

      # convert time in various formats to canonical SQL format
      if {$expires != ""} {
        if { [catch {set e [clock scan $expires]}] == 0 } {
          if { [catch {set expires [clock format $e -format "%Y-%m-%d %T"]}] != 0 } {
            p "Could not scan the date-time format. Try again."
            cgi_exit
          }
        } else {
          p "Could not convert date-time format."
          cgi_exit
        }
      }

      if {$op == "add"} {
        # can't be an existing Contact
        set c [sql "SELECT count(contact) FROM contacts WHERE user='$for' and contact='$contact'"]
        if {$c > 0} {
          p "Contact [bold $contact] is already in contact list."
        } else {
          if {[catch {sql "INSERT contacts SET user='$for',contact='$contact',expires='$expires',action='$action',display_name='$display_name',q='$q'"} msg]} {
            p "Cannot insert contact [bold $contact]: $msg."
          } else {
            p "Added contact [bold $contact] ($action, expires $expires, preference $q)."
          }
        }
      } elseif {$op == "modify"} {
        sql "UPDATE contacts SET \
          expires='$expires',\
          q='$q',display_name='$display_name',\
          action='$action'\
          WHERE user='$for' AND contact='$contact'"
        p "Modified contact [bold $contact] ($action, expires $expires, preference $q)."
      } elseif {$op == "delete"} {
        sql "DELETE FROM contacts WHERE user='$for' AND contact='$contact'"
        p "Deleted contact [bold $contact]."
      }

      # List current contacts
      set contacts [sql "SELECT contact,display_name,expires,
          unix_timestamp(expires),q,action,
          DATE_FORMAT(last_modified,'%d %b %Y %H:%i')
          FROM contacts WHERE user='$for' ORDER BY q"]
      set contactcount 0
      table border=1 {
        table_row {
          th "Contact/display name"
          th [small "Delete?"]
          th [small "Pref."]
          th "Expires"
          th "Action"
          th [small "Last modified"]
          th [nbspace]
        }
        foreach c $contacts {
          incr contactcount
          maplist $c contact display_name exp expires q action last_modified
          form Contacts {
            cgi_export for
            cgi_export contact
            cgi_export op=modify

            table_row bgcolor=[lindex $bglist [expr $contactcount % 2]] {
              table_data {
                puts [url $contact [quote_url $contact]]
                set t [cgi_buffer {cgi_text display_name=$display_name}]
                puts [font size=-1 $t]
                cgi_br
              }
              td [url [imglink Delete] [cgi_cgi Contacts [cgi_set for $for] [cgi_set op delete] [cgi_set contact $contact]]]
              table_data {cgi_text q=$q size=3}
              table_data {
                if {$expires == 0} {
                  set t [cgi_buffer {cgi_text expires= size=18}]
                } else {
                  set t [cgi_buffer {cgi_text expires=$exp size=18}]
                  #set t [cgi_buffer {cgi_text expires=[clock format $expires -format "%d %b %Y %H:%M"] size=18}]
                }
                puts [font size=-1 $t]
              }
              table_data {
                cgi_select action size=1 {
                  foreach a "Proxy Redirect" {
                    cgi_option $a value=$a selected_if_equal=$action
                  }
                }
              }
              td [small $last_modified]
              table_data {submit_button "=Change"}
            }
          }
        }
        form Contacts {
          cgi_export op=add
          cgi_export for

          table_row {
            table_data {
              cgi_text contact= size=20
              cgi_br
              set t [cgi_buffer {cgi_text display_name=}]
              puts [font size=-1 $t]
            }
            td [nbspace]
            table_data {cgi_text q=          size=3}
            table_data {cgi_text expires=    size=18}
            table_data {
              cgi_select action size=1 {
                foreach a "Proxy Redirect" {
                  cgi_option $a value=$a selected_if_equal=$action
                }
              }
            }
            td [nbspace]
            table_data {submit_button "=Add"}
          }
        }
      }

      sql disconnect
    }
  }
}
