#! /usr/local/bin/tclsh

# Create a new user.

source where.tcl

proc SetupVoiceMail {user} {
  global VMAILHOST
  global VMAILUSERDIR 
  global DOMAINS
  if {[info exists VMAILUSERDIR] && $VMAILUSERDIR != "" } {
    if {![info exists VMAILHOST] || $VMAILHOST == ""} {
      set VMAILHOST "vmail.[lindex $DOMAINS 0]"
    }
    # Set the top dir and userid
    set userid [userpart $user]
    # Format other important links and variables
    set topdir "$VMAILUSERDIR/$userid"
    set folderdir "$topdir/inbox"
    if {[file exists $topdir] == 0 } {
      if {[catch {file mkdir "$topdir"} msg] != 0} {
        p "Cannot create $topdir: $msg"
      } elseif {[catch {file mkdir "$topdir/inbox"} msg] != 0} {
        p "Can not create $topdir/inbox: $msg"
      } else {
        catch {file attributes $topdir -permissions o=rwx,og=}
        catch {file attributes "$topdir/inbox" -permissions o=rwx,og=}
        p "Created the voicemail home directory."
      }
    } else {
      if {![file isdirectory $topdir]} {
        p "$topdir is not a directory. Report the problem."
      }
    }
        
    set contact "sip:[userpart $user]@$VMAILHOST"

    set c [sql "SELECT count(contact) FROM contacts WHERE user='$user' and contact='$contact'"]
    if {$c <= 0} {
      if {[catch {sql "INSERT contacts SET user='$user',contact='$contact',expires='',action='proxy',display_name='',q='1.0'"} msg]} {
        p "Cannot insert contact [bold $contact]: $msg."
      } else {
        p "Added contact [bold $contact]."
      }
    }
  }
}


# form for creating new user
proc NewUserForm {{user ""} {email ""}} {
  global MODE tcl_platform

  if {![info exists MODE]} {
    set MODE "remote"
  }

  h2 "Create new user"
  form NewUser {
    table {
      table_row {
        td "Desired user name:"
        table_data {
          cgi_text user=$user type=text size=30
        }
      }
      if {$MODE == "remote"} {
        table_row {
          td "Your email address:"
          table_data {
            cgi_text email=$email type=text size=30
          }
        }
      }
      table_row {
        table_data {
          submit_button "=Create new user"
        }
      }
    }
  }
}


cgi_debug -on
cgi_eval {
  cgi_head {
    cgi_title "CINEMA: Creating New User"
  }
  #validate_user_pw_script

  cgi_body {
    cgi_input
    import_default user
    import_default pw
    import_default email
    config cinema.conf
    banner

    # if empty user, show form
    if {$user == ""} {
      NewUserForm
      cgi_exit
    }

    set result [opendb]

    # add domain if not given
    set uh [split $user "@"]
    if {$user != "" && [llength $DOMAINS] == 1 && [lindex $uh 1] == ""} {
      set user "[lindex $uh 0]@${DOMAINS}"
    }

    # check syntax of user name
    if {[set msg [checkuser $user]] != ""} {
      p $msg
      NewUserForm $user $email
      done
    }

    # check if domain is valid
    set uh [split $user "@"]
    if {[lsearch $DOMAINS [lindex $uh 1]] < 0} {
      if {[llength $DOMAINS] == 1} {
        p "Domain name must [bold $DOMAINS]."
      } else {
        p "Domain name must be one of [bold $DOMAINS]."
      }
      NewUserForm $user $email
      done
    }

    # if not defined, try user name as email
    if {$email == ""} {
      set email $user
    }

    # check email address
    if {[set msg [checkemail $email]] != ""} {
      p $msg
      NewUserForm $user $email
      done
    }

    # check if user exists already
    set olduser [sql "SELECT email FROM put WHERE user='$user'"] 
    if {$olduser != ""} {
      p "The name [bold $user] is already taken. Please choose another
name or [url login [cgi index [cgi_set user $user]]]."
      NewUserForm
      done
    }

    # if on local system, make sure user exists
    if {$MODE == "local" && [localuser $user 1] < 2} {
      NewUserForm $user $email
      done
    }

    # obtain default values from default@domain entry if no existing user
    h2 "Creating new user $user $MODE"

    p "Reading default user [bold default@[domain $user]]."
    set default [sql "SELECT realm,auth,algorithm,sip_methods,sip_groups,gwclass,um_timeout,max_msgsize_kb,busy,noresponse,message_template FROM put WHERE user='default@[domain $user]'"]
    maplist [lindex $default 0] realm auth algorithm sip_methodList sip_groupList gwclass um_timeout max_msgsize_kb busy noresponse message_template
    set hash_value ""

    set pw [randomword 3]
    if {![info exists REALM]} {
      set REALM [lindex $DOMAINS 0]
    }
    set realm $REALM
    set hash [hash $user $realm $pw]
    if {$gwclass == ""} {
      set gwclass [gwclass_default $user]
    }

    sql "INSERT put SET \
      user='$user',\
      realm='$realm',\
      email='$email',\
      auth='$auth',\
      hash_value='$hash',\
      algorithm='$algorithm',\
      sip_groups='$sip_groupList',\
      gwclass='$gwclass',\
      sip_methods='$sip_methodList',\
      message_template='$message_template',\
      um_timeout='$um_timeout',\
      max_msgsize_kb='$max_msgsize_kb',\
      noresponse='$noresponse',\
      busy='$busy'"

    if {[lsearch $sip_groupList voicemail] >= 0} {
      SetupVoiceMail $user
    }

    # Send email to new user
    cgi_mail_start $email
    cgi_mail_add "Subject: Your CINEMA registration"
    cgi_mail_add      
    cgi_mail_add "Your new CINEMA password for $user
in [domain $user] is \"$pw\".
You can login at [cgi_root].
The realm is \"$realm\"."
    cgi_mail_end
    #p "Password is $pw"  
    p "Email with the password for [bold $user] has been sent to [bold $email]."

    p "[url Login [cgi index] target=_top] to your account."

    sql disconnect
  }
}
