#! /usr/local/bin/tclsh

# List address book. Allow sorting by any single field or combination of fields.
# Searching picks from any field.


source where.tcl

set H(title)         {"Title"           0 p}
set H(givenname)     {"Given name"      1 p}
set H(middlename)    {"Middle name"     2 p}
set H(surname)       {"Surname"         3 p}
set H(nickname)      {"Nick name"       4 p}
set H(organization)  {"Organization"    5 p}
set H(department)    {"Dept."           6 p}
set H(jobtitle)      {"Job title"       7 p}
set H(city)          {"City"            8 a}
set H(email)         {"Email"           9 p}
set H(email_alt)     {"Email 2"        10 p}
set H(fax)           {"Fax"            11 a} 
set H(phone)         {"Phone"          12 a} 
set H(url)           {"URL"            13 p}
set H(sip)           {"SIP"            14 p}
set H(birthday)      {"Birthday"       15 p}
set H(last_modified) {"Last modified"  16 p}
set H(owner)         {"Owner"          17 p}
set H(photo)         {"Photo"          18 p}
set H(preferred)     {"Preferred"      19 p}
set H(spouse)        {"Spouse"         20 p}
set H(access)        {"Access"         21 p}
set H(id)            {"ID"             22 p}

proc fieldsort {a b} {
  global H
  return [expr [lindex $H($a) 1] - [lindex $H($b) 1]]
}

cgi_eval {
  set level [auth PersonList "List address book" 2]
  cgi_body {
    import_default orderby "surname"
    import_default letter  "-"
    import_default search  ""
    import_default fieldList "givenname surname email sip"

    if {[llength $fieldList] == 1} {
      set fieldList [lindex $fieldList 0]
    }
    set a_fields ""
    foreach f $fieldList {
      switch [lindex $H($f) 2] {
        p {lappend p_fields $f}
        a {lappend a_fields $f}
      }
    }
    if {[lsearch $p_fields "id"] < 0} {
      set p_fields [concat $p_fields id preferred]
    }
    set alphabet {a b c d e f g h i j k l m n o p q r s t u v w x y z}

    h1 "Addressbook for $for"
    if {$letter != "-"} {
      if {$letter != ""} {
        set q "AND surname LIKE '${letter}%'"
      } else {
        set q ""
      }
      if {$search != ""} {
        set s "AND ((surname LIKE '$search') OR (givenname LIKE '$search') OR (organization LIKE '$search'))"
      } else {
        set s ""
      }
      set res [sql "SELECT [join $p_fields ,] FROM person WHERE owner='$for' $q $s ORDER BY [join $orderby ,]"]

      set fieldsorted [lsort -command fieldsort $fieldList]

      table {
        # backward/forward navigation through alphabet
        if {$letter != ""} {
          table_row {
            set li [lsearch $alphabet $letter]
            if {$letter != "a"} {
              td [url [imglink Prev] [cgi_cgi PersonList \
              [cgi_set letter [lindex $alphabet [expr $li - 1]]] \
                [cgi_set search $search] \
                [cgi_set fieldList $fieldList] \
                [cgi_set orderby $orderby]]]
            }
            if {$letter != "z"} {
              td [url [imglink Next] [cgi_cgi PersonList \
                [cgi_set letter [lindex $alphabet [expr $li + 1]]] \
                [cgi_set search $search] \
                [cgi_set fieldList $fieldList] \
                [cgi_set orderby $orderby]]]
            }
          }
        }

        # table headings
        table_row {
          th "Edit"
          foreach f $fieldsorted {
            if {[lsearch $orderby $f] >= 0} {
              set bgcolor red
            } else {
              set bgcolor white
            }
            if {[lindex $H($f) 2] == "p"} {
              th bgcolor=$bgcolor [url [lindex $H($f) 0] [cgi_cgi PersonList [cgi_set letter $letter] [cgi_set search $search] [cgi_set fieldList $fieldList] [cgi_set orderby $f]]]
            } else {
              th bgcolor=$bgcolor [lindex $H($f) 0]
            }
          }
        }

        # entries
        set count 0
        foreach row $res {
          incr count
          maplist $row [join $p_fields ,]

          if {$a_fields != ""} {
            set address [sql "SELECT [join $a_fields ,] FROM address WHERE person=$id AND addressclass='$preferred'"]
            maplist [lindex $address 0] [join $a_fields ,]
          }

          table_row bgcolor=[lindex $bglist [expr $count % 2]] {
            td [url [imglink Person] [cgi_cgi PersonShow [cgi_set for $for] [cgi_set id $id]] target=PersonShow]
            foreach f $fieldsorted {
              switch $f {
                url {
                  if {$url != ""} {
                    if {![string match "http://*" $url]} {
                      set url "http://$url" 
                    }
                    td [url [imglink WWW] $url]
                  } else {
                    td [blank]
                  }
                }
                photo {
                  td [url [img $photo height=50] $photo]
                }
                email {
                  td [url [blank $email] "mailto:$email"]
                }
                sip {
                  td [url [blank $sip] "sip:$sip"]
                }
                default {
                  td [blank [set $f]]
                }
              }
            }
          }
        }
      }
    }

    # configure search and sorting
    h2 "List entries"

    p "The first checkbox includes the field in the display, the second
determines sorting order."

    set flist [lsort [array names H]]
    set cols 3
    set rows [expr int(ceil([llength $flist] / double($cols)))]
    form PersonList {
      cgi_export for
      table border=1 {
        for {set i 0} {$i < $rows} {incr i} {
          table_row {
            for {set j 0} {$j < $cols} {incr j} {
              table_data {
                set f [lindex $flist [expr $j*$rows + $i]]
                if {$f != ""} {
                  if {[lsearch $fieldList $f] >= 0} {set checked $f} {set checked ""}
                  checkbox fieldList=$f checked_if_equal=$checked
                  if {[lindex $H($f) 2] == "p"} {
                    radio_button orderby=$f checked_if_equal=$orderby 
                  } else {
                    radio_button orderby=$f disabled checked_if_equal=$orderby 
                  }
                  puts " [lindex $H($f) 0]"
                }
              }
            }
          }
        }
        # starting point
        table_row {
          table_data colspan=3 {
            table {
              table_row {
                table_data {
                  radio_button letter= checked_if_equal=$letter
                }
                for {set i 0} {$i < 26} {incr i} {
                  set l [lindex $alphabet $i]
                  table_data {
                    radio_button letter=$l checked_if_equal=$letter
                  }
                }
              }
              table_row {
                td "all"
                for {set i 0} {$i < 26} {incr i} {
                  set l [lindex $alphabet $i]
                  td align=center $l
                }
              }
            }
          }
        }
        table_row {
          td "Search for:"
          table_data colspan=2 {
            text search=$search size=40
          }
        }
        table_row {
          table_data colspan=3 align=center {
            submit_button "action=List"
          }
        }
      }
    }

    sql disconnect
  }
}
