#! /usr/local/bin/tclsh
# show current time at location

# link to map


# Show information about a person

source where.tcl
source iso2c.tcl

# add scheme to URL if missing
# Use scheme like http:// or sip:
proc urlscheme {u scheme} {
  if {![regexp "^${scheme}" $u]} {
    return "${scheme}${u}"
  }
  return $u
}


# Create URL to mapping service
# <a href="http://maps.yahoo.com/py/maps.py?&addr=313+Westview+Ave&csz=Leonia%2C+NJ+07605">
proc maplocation {Aname} {
  upvar $Aname A

  if {$A(country) == "US"} {
    if {$A(city) != "" && $A(state) != ""} {
      set u [cgi_cgi http://maps.yahoo.com/py/maps.py \
        [cgi_set addr $A(street)] [cgi_set csz "$A(city), $A(state) $A(postalcode)"]]
    }
    return [url [imglink Map] $u target=map]
  }
  return ""
}

proc zip9 {Aname} {
  upvar $Aname A
  if {$A(country) == "US"} {
    if {$A(city) != "" && $A(state) != ""} {
      form http://www.usps.gov/cgi-bin/zip4/zip4inq2 target=zip9 {
      #form http://www.cs.columbia.edu/~hgs/test/test-cgi.cgi
        cgi_export Firm=
        cgi_export Urbanization=
        cgi_export "Delivery Address=$A(street)"
        cgi_export "City=$A(city)"
        cgi_export "State=$A(state)"
        cgi_export "Zip Code=$A(postalcode)"
        cgi_submit_button "Submit=Zip+4"
      }
    }
  }
}

cgi_eval {
  set level [auth PersonShow "Show Person" 2]

  cgi_body {
    import_default id     ""
    import_default for    ""
    import_default email  ""

    set rowbg   "#b0cafa"
    set titlebg "#6666cc" 
    set pfieldlist {id title givenname middlename surname nickname jobtitle
      organization department url email email_alt birthday last_modified
      photo preferred sip spouse}

    set afieldlist {addressclass companyname department room street city
      state province postalcode country timezone geo phone mobile pager fax}

    if {$id == "" && $email == ""} {
      p "Must specify either ID number or email address."
      cgi_exit
    }
    if {$email != ""} {
      set res [sql "SELECT [join $pfieldlist ","] 
         FROM person
         WHERE (email='$email' OR email_alt='$email' OR sip='$email')
         AND owner='$for'"]
    } else {
      set res [sql "SELECT [join $pfieldlist ","]
         FROM person WHERE id=$id AND owner='$for'"]
    }
    if {$res == ""} {
      p "The person $id $email doesn't exist for owner $for.  Do you
         want to [url add [cgi_cgi PersonEdit [cgi_set email $email] [cgi_set for $for]]] him/her?"
      cgi_exit
    } else {
      maparray [lindex $res 0] $pfieldlist P
    }

    # group memberships
    set memberships [sql "SELECT persongroup.name,persongroup.id
      FROM persongroup,groupmember
      WHERE groupmember.person=$P(id) and 
      groupmember.persongroup=persongroup.id ORDER BY persongroup.name"]

    table {
      table_row {
        table_data colspan=2 bgcolor=$titlebg {
          puts [font color=white size=+1 "$P(title) $P(givenname) $P(middlename) $P(surname)"]
        }
      }
      table_row {
        table_data valign=top {
          table {
            table_row bgcolor=$rowbg {
              td valign=top "Work:" 
              table_data valign=top {
                foreach v {jobtitle organization department} {
                  if {$P($v) != ""} {
                    puts "$P($v)"
                    cgi_br
                  }
                }
              }
            }
            if {$P(birthday) != ""} {
              table_row bgcolor=$rowbg {
                td "Birthday:"
                td "$P(birthday)"
              }
            }
            if {$P(spouse) != ""} {
              table_row bgcolor=$rowbg {
                td "Spouse:"
                td "$P(spouse)"
              }
            }

            # URL
            if {$P(url) != ""} {
              table_row bgcolor=$rowbg {
                td valign=top "URL:"
                table_data {
                  puts [url $P(url) [urlscheme $P(url) http://] target=url]
                }
              }
            }

            # email
            table_row bgcolor=$rowbg {
              td valign=top "Email:"
              table_data {
                foreach v {email email_alt} {
                  if {$P($v) != ""} {
                    puts [url $P($v) "mailto:$P(givenname) $P(surname) <$P($v)>"]
                    cgi_br
                  }
                }
              }
            }

            # SIP
            table_row bgcolor=$rowbg {
              if {$P(sip) == ""} {
                set P(sip) $P(email)
              }
              if {$P(sip) != ""} {
                td valign=top "SIP:"
                table_data {
                  puts [url $P(sip) [urlscheme $P(sip) sip:] target=url]
                }
              }
            }

            # Group memberships      
            if {$memberships != ""} {
              table_row bgcolor=$rowbg {
                td "Member in:"
                table_data {
                  foreach g $memberships {
                    maplist $g name gid
                    puts $name
                    cgi_br
                  } 
                }
              }
            }

          }
        }
        if {$P(photo) != ""} {
          table_data valign=top {
            puts [img [urlscheme $P(photo) http://] alt="$P(givenname) $P(surname) photo"] 
          }
        }
      }
      table_row {
        table_data colspan=2 {
          table {
            # one address per column
            set addresses [sql "SELECT [join $afieldlist ,] FROM address
              WHERE person='$id'"]
            # header row
            table_row bgcolor=$rowbg {
              td [blank]
              foreach a $addresses {
                maparray $a $afieldlist A
                if {$A(addressclass) == $P(preferred)} {
                  set bgcol "#ff9933"
                } else {
                  set bgcol "#6666cc"
                }
                td bgcolor=$bgcol $A(addressclass)
              }
            }
            # mailing address row
            table_row bgcolor=$rowbg {
              td valign=top "Mailing address:"
              foreach a $addresses { 
                table_data valign=top {
                  maparray $a $afieldlist A
                  table {
                    table_row {
                      table_data {
                        foreach f {companyname department room street} {
                          if {$A($f) != ""} {
                            puts $A($f)
                            cgi_br
                          }
                        }
                        if {$A(country) == "US"} {
                          puts "$A(city), $A(state) $A(postalcode)"
                        } elseif {$A(province) != ""} {
                          puts "$A(postalcode) $A(city), $A(province)"
                        } else {
                          puts "$A(postalcode) $A(city)"
                        }
                        cgi_br
                        if {$A(country) != ""} {
                          puts "$A(country)"
                        }
                      }
                      table_data {
                        puts [blank [maplocation A]]
                        cgi_br
                        zip9 A
                      }
                    }
                  }
                }
              }
            }

            # time zone
            table_row bgcolor=$rowbg {
              td valign=top "Timezone:"
              foreach a $addresses { 
                table_data valign=top {
                  maparray $a $afieldlist A
                  set env(TZ) [pwd]/zoneinfo/$A(timezone)
                  if {$A(timezone) != ""} {
                    table_data {
                      puts [blank $A(timezone)] 
                      cgi_br
                      puts [exec date]
                    }
                  } else {
                    td [blank]
                  }
                }
              }
            }
  
            # geographic location
            table_row bgcolor=$rowbg {
              td valign=top "Geo location:"
              foreach a $addresses { 
                table_data valign=top {
                  maparray $a $afieldlist A
                  td [blank $A(geo)]
                }
              }
            }
     
            # phone
            foreach f {phone mobile pager fax} {
              table_row bgcolor=$rowbg {
                td valign=top "${f}:"
                foreach a $addresses { 
                  table_data valign=top {
                  maparray $a $afieldlist A
                    td [blank $A($f)]
                  }
                }
              }
            }
          }
        } 
      }
    }

    form PersonEdit {
      cgi_export for
      cgi_export id
      cgi_submit_button "=Edit $P(givenname) $P(surname)"
    }

    sql disconnect
  }
}
