#! /usr/local/bin/tclsh

# Rate calls that haven't been rated yet.
# Tariffs are matched most-specific first, so that a tariff for +1201
# matches instead of +1, for example.

# We currently do not handle calls split across two tariffs. We simply
# assume that the tariff in effect at the beginning of a call persists
# throughout the call. Given that time-of-day tariffs are becoming less
# common, it's probably not worth worrying about this.

source where.tcl

cgi_eval {
  set level [auth Rating "Rate calls"]

  cgi_body {
    if {$level < 3} {
      p "Only administrators can rate calls."
      cgi_exit
    }

    set period [clock format [clock seconds] -format "%Y-%m-%d"]

    h1 "Rating Calls ($period)"

    # get tariffs and sort them by prefix length
    set tarifflist [sql "SELECT \
id,unix_timestamp(valid_from),unix_timestamp(valid_to),gwclass,rate,time_to_sec(daytime_start),time_to_sec(daytime_end),prefix,increment,gwclass\
    FROM tariff ORDER BY length(prefix) DESC"]

    # work through request list
    cgi_bullet_list {
      foreach row [sql "SELECT id,unix_timestamp(request_time),method,sip_status,request_url,sip_callid,sip_from,sip_to
FROM requestlog WHERE period is NULL and method = 'INVITE' and sip_status = 200 ORDER BY request_time"] {
        maplist $row id,invite_time,method,sip_status,request_url,sip_callid,sip_from,sip_to

        # find matching BYE
        set bye [sql "SELECT id,unix_timestamp(request_time),request_url FROM requestlog \
WHERE method = 'BYE' and sip_callid='$sip_callid' and sip_from='$sip_from' and sip_to='$sip_to'"]
        if {[llength $bye] > 1} {
          cgi_li "Multiple matching BYEs for $row: $bye."
        } elseif {$bye == ""} {
          cgi_li [font color=orange "No matching BYE from $sip_from to $sip_to, call-id $sip_callid."]
        } else {
          maplist [lindex $bye 0] bye_id bye_time request_url

          # compute duration and seconds-within-day
          set duration [expr $bye_time - $invite_time]
          set inday    [expr [clock format $invite_time -format "%H*3600+%M*60"]]

          # canonicalize request URL: remove sip: and tel:
          regsub {^(sip:|tel:)(.*)} $request_url {\2} request_url
          # remove dashes and periods
          regsub -all {[\.\-]} $request_url {} request_url

          # determine user class
          regsub {^(sip:)(.*)} $sip_from {\2} sip_user
          set u [sql "SELECT gwclass FROM put WHERE user = '$sip_user'"]
          maplist [lindex $u 0] userclass

          # find matching tariffs
          set charge -1
          foreach t $tarifflist {
            maplist $t tariff_id valid_from valid_to gwclass rate daytime_start daytime_end prefix increment gwclass

            #p "invite_time $invite_time $valid_from $valid_to inday=$inday $daytime_start $daytime_end $prefix url=$request_url"

            if {$invite_time >= $valid_from && $invite_time <= $valid_to \
                && ($userclass == $gwclass || $gwclass == "")
                && $inday > $daytime_start && $inday < $daytime_end && \
                [string match $prefix $request_url]} {
              if {$increment > 0} {
                set charge [expr ceil(double($duration) / $increment) * $rate]  
              } else {
                set charge 0
              }
              break
            }
          }
          if {$charge >= 0} {
            cgi_li [font color=green "Call $period on [clock format $invite_time] ($duration s) to $request_url uses tariff $tariff_id ($prefix), \$$rate per
$increment s = \$${charge}."]

            # update request with billing period and price
            sql "UPDATE requestlog SET period='$period',charge=$charge,bye='$bye_id' WHERE id=$id"
          } else {
            cgi_li "Call on [clock format $invite_time] ($duration s) from
$sip_from ($userclass) to $request_url not rated."
          }
        }
      }
    }

    done
  }
}
