#! /usr/local/bin/tclsh

# List all requests.

source where.tcl

cgi_eval {
  set level [auth Requestlog "List requests"]

  cgi_body {
    import_default sortby "request_time"
    import_default select_from "" V(select_from)         
    import_default select_to   "" V(select_to)
    import_default start_time  "9999-01-01" V(start_time)
    import_default end_time    "9999-12-31" V(end_time)
    import_default methodList  "" V(method)

    if {$V(start_time) != ""} {
      h1 "Request Log from $V(start_time) to $V(end_time)"
    }

    set hlist {
      {request_time  "Request"}
      {method        "Method"}
      {sip_status    "Response"}
      {request_url   "Destination"}
      {sip_callid    "Call-ID"}
      {sip_from      "From"}
      {sip_to        "To"}
      {period        "Period"}
      {charge        "Charge"}
    }

    set where ""
    if {$V(start_time) != ""} {
      lappend where "request_time >= '$V(start_time)'"
    }
    if {$V(end_time) != ""} {
      lappend where "request_time <= '$V(end_time)'"
    }
    if {$V(method) != ""} {
      lappend where "FIND_IN_SET(method,'[join $V(method) ,]')"
    }
    if {$level > 2} {
      if {$V(select_from) != ""} {
        lappend where "sip_from REGEXP '$V(select_from)'"
      }
      if {$V(select_to) != ""} {
        lappend where "sip_to REGEXP '$V(select_to)'"
      }
    } else {
      lappend where "sip_from = 'sip:$user' or sip_to = 'sip:$user'"
    }
    set where [join $where " and "]

    p "$where"

    # get request list
    set r 0 
    table {
      table_row {
        foreach h $hlist {
          if {$sortby == [lindex $h 0]} {
            set bgcolor red
          } else {
            set bgcolor white
          }
          th bgcolor=$bgcolor [url [lindex $h 1] [cgi_cgi Requestlog \
            [cgi_set sortby [lindex $h 0]] \
            [cgi_set select_from $V(select_from)] \
            [cgi_set start_time $V(start_time)] \
            [cgi_set end_time $V(end_time)] \
          ]]
        }
      }
      foreach row [sql "SELECT \
request_time,method,sip_status,request_url,sip_callid,sip_from,sip_to,period,charge \
FROM requestlog WHERE $where ORDER BY $sortby"] {
        maplist $row request_time method sip_status request_url sip_callid sip_from sip_to period charge
        incr r
        table_row bgcolor=[lindex $bglist [expr $r % 2]] {
          td "$request_time"
          td "$method"
          td "$sip_status"
          td "$request_url"
          td "$sip_callid"
          td "$sip_from"
          td "$sip_to"
          td [blank $period]
          td [blank $charge]
        }
      }
    }

    set F(start_time) {"Start date and time" 1 text 16}
    set F(end_time)   {"End date and time"   2 text 16}
    if {$level > 2} {
      set F(select_from) {"From" 3 text 16}
      set F(select_to)   {"To"   4 text 16}
    }
    set F(method)     {"Methods"             5 checkbox {INVITE REGISTER BYE}}
 
    form Requestlog {
      cgi_export sortby
      tableform "action=Search" F V
    }

    done
  }
}
