#! /usr/local/bin/tclsh

# Add or edit a tariff.

source where.tcl

cgi_eval {
  set level [auth TariffEdit "Add or edit a tariff" 3]

  cgi_body {
    import_default valid_from    "1000-01-01 00:00"
    import_default valid_to      "9999-12-31 00:00"
    import_default rate
    import_default daytime_start
    import_default daytime_end
    import_default minute
    import_default prefix
    import_default increment     "60"
    import_default sortby        "prefix"
    import_default gwclass
    # if ID is non-empty, edit an existing record
    import_default id

    # check values
    if {$valid_from > $valid_to} {
      p "Time range invalid: start date must precede end date."
      cgi_exit
    }

    if {[llength $valid_from] == 1} {
      set valid_from "$valid_from 00:00:00"
    }
    if {[llength $valid_to] == 1} {
      set valid_to "$valid_to 23:59:59"
    }
    set ds [split $daytime_start :]
    if {[llength $ds] == 2} {
      set daytime_start "$daytime_start:00"
    }
    set de [split $daytime_end :]
    if {[llength $de] == 2} {
      set daytime_end "$daytime_end:59"
    }

    if {$increment == 0 && $minute == 0} {
      set increment 60
      set rate 0
    } elseif {$increment == 0} {
      p "An increment of zero seconds is not valid."
      cgi_exit
    } elseif {$minute != ""} {
      set rate [expr $minute / (60./$increment)]
    }

    if {$gwclass == ""} {
      set gwclass_set "gwclass=NULL"
    } else {
      set gwclass_set "gwclass='$gwclass'"
    }

    # if tariff doesn't exist, insert
    if {$id == ""} {
      sql "INSERT tariff SET \
             valid_from='$valid_from',
             valid_to='$valid_to',
             rate='$rate',
             ${gwclass_set},
             daytime_start='$daytime_start',
             daytime_end='$daytime_end',
             prefix='$prefix',
             increment='$increment'"
      p "Tariff for $prefix does not exist and is being created."
    } else {
      # update tariff
      sql "UPDATE tariff SET \
             valid_from='$valid_from',
             valid_to='$valid_to',
             rate='$rate',
             ${gwclass_set},
             daytime_start='$daytime_start',
             daytime_end='$daytime_end',
             prefix='$prefix',
             increment='$increment'
             WHERE id='$id'"
      p "Tariff for $prefix modified."
    }

    tarifflist $level $sortby

    sql disconnect
  }
}
