#! /usr/local/bin/tclsh

# Allow user to add and delete third parties that can register for that user.

source where.tcl

cgi_eval {
  set level [auth ThirdParty "Edit Third-Party Registrations"]

  cgi_body {
    import_default op
    import_default sip_from

    if {$level < 2} {
      p "Only local users can have third-party registrations."
      cgi_exit
    }

    h1 "Who Can Register for $user"

    if {[lsearch $DOMAINS [domain $user]] < 0} {
      p "Registrand must be a local user."
      cgi_exit
    }

    set row [sql "SELECT remote_user FROM put WHERE user='$for'"]
    maplist [lindex $row 0] remote_user

    # add and delete third parties
    if {$sip_from != ""} {
      if {$op == "add"} {
        if {[lsearch $remote_user $sip_from] >= 0} {
          p "$sip_from is already in the list $remote_user."
        } else {
          lappend remote_user $sip_from
          sql "UPDATE put SET remote_user='$remote_user' WHERE user='$for'"
          p "[bold $sip_from] can now register for [bold $for]."
        }
      } elseif {$op == "delete"} {
        set i [lsearch $remote_user $sip_from]
        if {$i >= 0} {
          set remote_user [lreplace $remote_user $i $i]
          sql "UPDATE put SET remote_user='$remote_user' WHERE user='$for'"
          p "Deleted [bold $sip_from]."
        }
      }
    }

    # list current third parties
    if {$remote_user != ""} {
      table border=1 {
        table_row {
          td "User name"
          td "Registered?"
          td "Delete?"
        }
        foreach u $remote_user {
          table_row {
            td $u
            set reg [sql "SELECT user FROM put WHERE user='$u'"]
            if {$reg != ""} {
              td [imglink Registered]
            } else {
              td [nbspace]
            }
            td [url [imglink Delete] [cgi_cgi ThirdParty [cgi_set op delete] [cgi_set for $for] [cgi_set sip_from $u]]]
          }
        }
      }
    }

    h2 "Add to list of users that can register for $for"

    form ThirdParty {
      cgi_export op=add
      cgi_export for
      table {
        table_row {
          td "User identifier ([italic user@domain]):"
          table_data {cgi_text sip_from= size=30}
          table_data {submit_button "=Add"}
        }
      }
    }

    sql disconnect
  }
}
