#! /usr/local/bin/tclsh

# Add new user.

source where.tcl

cgi_eval {
  set level [auth UserAdd "Edit User Information" 3]

  cgi_body {
    import_default realm

    h1 "Create New User"

    # read default user information
    set u [sql "SELECT email,realm,sip_groups,gwclass,auth,algorithm,hash_value,sip_methods,busy,noresponse,message_template,um_timeout,max_msgsize_kb FROM put WHERE user='default@$DOMAINS'"]
    maplist [lindex $u 0] email realm sip_groupList gwclass auth algorithm hash_value sip_methods busy noresponse message_template um_timeout max_msgsize_kb

    form UserEdit enctype=multipart/form-data onsubmit="return validate_pw_user(this.user,this.pass,this.pass1)" {
      table {
        table_row {
          td "User identifier [italic "(user@$DOMAINS) or tel:"]:"
          table_data {cgi_text for= size=50}
        }
        table_row {
          table_data {
            puts "Realm (prompt for user):"
            cgi_br
            puts "[small "Must supply password to change realm"]"
          }
          table_data {cgi_text realm=$realm size=50}
        }
        table_row {
          td "New password:"
          table_data {cgi_text pass= size=16 type=password}
        }
        table_row {
          td "Reenter password for verification:"
          table_data {cgi_text pass1= size=16 type=password}
        }
        if {$MODE == "remote"} {
          table_row {
            td "Email address:"
            table_data {cgi_text email=$email size=30}
          }
        }
        table_row {
          td "Callers for this user must authenticate:"
          table_data {
            cgi_select auth {
              foreach a {never request required ""} {
                cgi_option $a selected_if_equal=$auth
              }
            }
          }
        }
        table_row {
          td "Algorithm:"
          table_data { 
            cgi_select algorithm {
              foreach a {MD5 MD5-sess} {
                cgi_option $a selected_if_equal=$algorithm
              }  
            }
          }
        }
        table_row {
          td "SIP methods this user can receive:"
          table_data {
            foreach method {REGISTER INVITE any} {
              if {[lsearch $sip_methods $method] >= 0} {
                set allowed $method
              } else {
                set allowed ""
              }
              checkbox sip_methodList=$method checked_if_equal=$allowed
              puts $method
            }
          }
        }
        if {$level > 2 || [lsearch $sip_groupList voicemail] >= 0} {
          table_row {
            table_data { 
              puts "URL of announcement played when busy (typically, a RTSP URL):"
            }
            table_data {
              cgi_text busy=$busy size=30
            }
          }
          table_row {
            table_data {
              puts "URL of announcement played when no answer (typically, a RTSP URL; word SERVER is replaced by voicemail server):"
            }
            table_data {
              cgi_text noresponse=$noresponse size=30
            }
          }
          table_row {
            table_data {
              puts "Message template for unified messages. Templates can
                    contain variables and functions:"
              table {
                table_row {
                  td [typewriter \$user]
                  td "userid"
                }
                table_row {
                  td [typewriter \$to]
                  td "SIP header field of call"
                }
                table_row {
                  td [typewriter \$from]
                  td "SIP header field of call"
                }
                table_row {
                  td [typewriter \$name]
                  td "SIP header field of call"
                }
                table_row {
                  td [typewriter \$priority]
                    td "SIP header field of call"
                }
                table_row {
                  td [typewriter \$subject]
                  td "SIP header field of call"
                }
                table_row {
                  td [typewriter \$sipurl]
                  td "URL for accessing the message via SIP"
                }
                table_row {
                  td [typewriter \$httpurl]
                  td "URL for accessing the message via HTTP"
                }
                table_row {
                  td [typewriter \$rtspurl]
                  td "URL for accessing the message via RTSP"
                }
                table_row {
                  td [typewriter \$msgid]
                  td "the random number of the message"
                }
                table_row {
                  td [typewriter \$administrator]
                  td "userid of the person running sipum"
                }
                table_row {
                  td [typewriter "\[url [italic "name link"]\]"]
                  td "generates link enclosing 'name'"
                }
                table_row {
                  td [typewriter "\[email [italic uri]\]"]
                  td "removes the scheme (if any) from the uri so
                     that it looks like an email address"
                }
                table_row {
                  td [typewriter "\[dataurl [italic "name uri"]\]"]
                  td "creates a link for data URL by extracting the scheme from the URI and using data:application/scheme as the URL type."
                }
              }
            }

            table_data {
              cgi_textarea message_template=$message_template cols=70 rows=20
            }
          }
          table_row {
            td "Timeout (seconds) until voicemail:"
            table_data {cgi_text um_timeout=$um_timeout size=4}
          }
          table_row {
            td "Maximum size (in KB) for each message:"
            table_data {cgi_text max_msgsize_kb=$max_msgsize_kb size=4}
          }
        }
        # parameters that can only be changed by admin
        if {$level > 2} {
          set GWCLASS [sql "SELECT gwclass FROM gwclass ORDER BY gwclass"]
          table_row {
            td "Privileges:"
            table_data {
              foreach policy $POLICY {
                global VMAILUSERDIR
                set value  [lindex $policy 0]
                set prompt [lindex $policy 1]
                if {$value == "voicemail" && (![info exists VMAILUSERDIR] || $VMAILUSERDIR == "")} {continue}
                if {[lsearch $sip_groupList $value] >= 0} {
                  set allowed $value
                } else {
                  set allowed ""
                }
                checkbox sip_groupList=$value checked_if_equal=$allowed
                puts $prompt
                cgi_br
              }
            }
          }
          table_row {
            td "Gateway class:"
            table_data {
              cgi_select gwclass {
                foreach g [concat [list ""] $GWCLASS] {
                  cgi_option $g selected_if_equal=$gwclass
                }
              }
            }
          }
        }
        table_row {
          td ""
          table_data {submit_button "=Change"}
        }
      }
    }

    cgi_hr

    sql disconnect
  } 
}
