#! /usr/local/bin/tclsh

# Create or modify a user with all information fields.

source where.tcl

cgi_eval {
  set level [auth UserEdit "Created or Modified User" 2]

  cgi_body {
    # check syntax of user name
    if {[set msg [checkuser $for]] != ""} {
      p $msg
      cgi_exit
    }

    # fields
    set F(realm)            {"Realm"                     2 text     50}
    set F(email)            {"Email address"             5 text     30}
    set F(auth)             {"Callers authentication"    6 select   {never request required ""}}
    set F(algorithm)        {"Algorithm"                 7 select   {MD5 MD5-sess}}
    set F(sip_methods)      {"SIP methods"               8 checkbox {REGISTER INVITE any}}
    set F(busy)             {"busy"                      9 text     30}
    set F(noresponse)       {"noresponse"               10 text     30}
    set F(message_template) {"template"                 11 textarea {20 70}}
    set F(um_timeout)       {"Timeout until voicemail"  12 text     4}
    set F(max_msgsize_kb)   {"Maximum size"             13 text     4} 
    if {$level > 2} {
      set F(sip_groups)     {"Privileges"               14 checkbox_v {}}
      set F(gwclass)        {"Gateway class"            15 select     {}}
    }
    
    # get current user information
    set fieldlist [array names F]
    set u [sql "SELECT [join $fieldlist ,]
                FROM put WHERE user = '$for'"]
    if {$u != ""} {
      maparray [lindex $u 0] $fieldlist V
    }

    # obtain default values from default@domain entry if no existing user
    if {$u == "" || $V(algorithm) == ""} {
      p "Reading default user [bold default@[domain $for]]."
      set default [sql "SELECT [join $fieldlist ,] 
                        FROM put WHERE user='default@[domain $for]'"]
      maparray [lindex $default 0] $fieldlist V
      set hash_value ""
      if {$V(gwclass) == ""} {
        set V(gwclass) [gwclass_default $for]
      }
    }

    # get information from forms input, using current defaults
    set q [tableforminput F $fieldlist V]

    # if user doesn't exist, insert
    if {$u == ""} {
      sql "INSERT put SET [join $q ,]"
      p "User [bold $for] does not exist and is being created."
    } else {
      # update user
      sql "UPDATE put SET [join $q ,] WHERE user='$for'"
      p "User [bold $for] modified."
    }

    if {[lsearch $V(sip_groups) voicemail] >= 0} {
      global VMAILHOST
      global VMAILUSERDIR 
      global DOMAINS
      if {[info exists VMAILUSERDIR] && $VMAILUSERDIR != "" } {
        if {![info exists VMAILHOST] || $VMAILHOST == ""} {
          set VMAILHOST "vmail.[lindex $DOMAINS 0]"
        }
        # Set the top dir and userid
        set userid [userpart $for]
        # Format other important links and variables
        set topdir "$VMAILUSERDIR/$userid"
        set folderdir "$topdir/inbox"
        if {[file exists $topdir] == 0 } {
          if {[catch {file mkdir "$topdir"} msg] != 0} {
            p "Can not create $topdir: $msg"
          } elseif {[catch {file mkdir "$topdir/inbox"} msg] != 0} {
            p "Can not create $topdir/inbox: $msg"
          } else {
            catch {file attributes $topdir -permissions 0700}
            catch {file attributes "$topdir/inbox" -permissions 0700}
            p "Created the voicemail home directory."
          }
        } else {
          if { [file type $topdir] != "directory" } {
            p "$topdir is not a directory. Report the problem."
          }
        }
        
        set contact "sip:[userpart $for]@$VMAILHOST"

        set c [sql "SELECT count(contact) FROM contacts WHERE user='$for' and contact='$contact'"]
        if {$c <= 0} {
          if {[catch {sql "INSERT contacts SET user='$for',contact='$contact',expires='',action='proxy',display_name='',q='1.0'"} msg]} {
            p "Cannot insert contact [bold $contact]: $msg."
          } else {
            p "Added contact [bold $contact]."
          }
        }
      }
    }
    # Voice mail specific editing ends here
    
    # modify password only if specified
    import_default pass
    if {$pass != ""} {
      set hash_value [hash $for $V(realm) $pass]
      sql "UPDATE put SET \
             hash_value='$hash_value',realm='$realm' WHERE user='$for'"
      p "Password for [bold $for] updated."
    }

    # show public user information
    usershow $for

    sql disconnect
  }
}
