#! /usr/local/bin/tclsh

# Voice mail folders.

source where.tcl

proc folder_delete {topdir folderList} {
  foreach f $folderList {
    checkfilename $f
    if {[glob -nocomplain -- $topdir/$f/*] != ""} {
      p "First, you must delete the messages in folder [bold $f]."
      return
    }
    if [catch {file delete -force -- $topdir/$f} msg] {
      p "Cannot delete $dir/$m: $msg"
    }
  }
}

proc folder_create {topdir folder} {
  if {[catch {file mkdir "$topdir/$folder"} msg]} {
    p "Cannot create $topdir/$folder: $msg"
  }
}

cgi_eval {
  set level [auth VmailFolders "Voicemail Folders" 2]

  import_default folderList ""
  import_default folder  ""
  import_default Action  ""

  cgi_body {
    set topdir    "$VMAILUSERDIR/[userpart $for]/"

    switch $Action {
      Delete {folder_delete $topdir $folderList}
      Create {folder_create $topdir $folder}
    }

    # Current folder list.
    set folderlist [list] 
    foreach f [glob -nocomplain -- ${topdir}/*] {
      if {[file isdirectory $f]} {
        lappend folderlist [file tail $f]
      }
    }
    set folderlist [lsort $folderlist]

    h1 "Voicemail folders for $for"

    set totalsize 0
    set totalcount 0
    set foldercount 0

    form VmailFolders {
      cgi_export for
      table {
        table_row bgcolor=blue {
          td [blank]
          td [bold "Folder Name"]
          td align=right [bold Messages]
          td align=right [bold "Size (kB)"]
        }
        foreach folder $folderlist {
          incr foldercount
          set size 0
          set filelist [glob -nocomplain $topdir/$folder/*]
          foreach f $filelist {
            incr size [file size $f]
          }
          set msgcount [llength $filelist]
          incr totalsize  $size
          incr totalcount $msgcount
          table_row [bgcolor row] {
            if {$folder != "inbox"} {
              table_data {cgi_checkbox folderList=$folder}
            } else {
              td [blank]
            }
            table_data {
              puts [url $folder \
                [cgi_cgi Vmail [cgi_set for $for] [cgi_set folder $folder]]]
            }
            td align=right $msgcount
            td align=right [expr $size / 1000]
          }
        }
        table_row [bgcolor row] {
          td [blank]
          td [bold Total]
          td align=right $totalcount
          td align=right [expr $totalsize / 1000]
        }
        table_row [bgcolor row] {
          if {$foldercount > 1} {
            table_data {
              submit_button "Action=Delete"
            }
          } else {
            td [blank]
          }
          table_data {
            cgi_text folder= 
          }
          td [blank]
          table_data {
            submit_button "Action=Create"
          }
        }
      }
    }
  } 
}
