# Returns authentication level:
#  -7 no cookies
#  -6 cannot hash password
#  -5 no SQL URL
#  -4 cannot open SQL db
#  -3 no user or password, or 'user' is invalid
#  -2 invalid password
#  -1 user and for disagree for non-admin user, or 'for' is invalid
#  0  new user
#  1  valid user
#  2  valid local user
#  3  administrator

proc auth {form {title ""} {mustbe 0}} {
  global user pw for ADMINISTRATOR DOMAINS MODE

  # read configuration file
  config cinema.conf

  # In case of a authentication failure, the failure message must be 
  # deferred until cgi_http_head and cgi_head have finished.
  set auth_errmsg ""

  # configure address for sending error messages
  if {[info exists ADMINISTRATOR]} { 
    cgi_admin_mail_addr $ADMINISTRATOR
  }
  if {[info exists DOMAINS]} {
    set DOMAINS [string tolower $DOMAINS]
  }
  if {![info exists MODE]} {
    set MODE "remote"
  }

  set level -3
  cgi_input
  import_default for

  if {[lsearch [cgi_cookie_list] cinema_pw] >= 0} {
    set pw [cgi_cookie_get cinema_pw]
  } else {
    set level -7
    set pw ""
  }
  if {[lsearch [cgi_cookie_list] cinema_user] >= 0} {
    set user [cgi_cookie_get cinema_user]
  } else {
    set level -7
    set user ""
  }

  # authenticate user
  if {$user != ""} {
    set result [opendb]
    set level    [lindex $result 0]
    set errormsg [lindex $result 1]
    if {$level == 0} {
      set row [sql "SELECT realm,hash_value,sip_groups FROM put WHERE user='$user'"]
      if {[llength $row] == 0} {
        set level 0
        if {$for == ""} {
          set for $user
        }
      } else {
        maplist [lindex $row 0] realm hash_value sip_groups

        set h [hash $user $realm $pw]
        if {$h == ""} {
          set level -6
        } elseif {$h != $hash_value} {
          set level -2
        } else {
          if {[localuser $user]} {
            set level 2
          } else { 
            set level 1
          }
          if {[lsearch $sip_groups "admin"] >= 0} {
            set level 3
          } else {
            # make sure 'for' and 'user' are the same
            if {$for != "" && $for != $user} {
              set level -1
            }
          }
        }
      }
    }
  }

  if {$level < $mustbe} {
    cgi_head {
      cgi_title "CINEMA: Login failure $level"
    }
    cgi_body {
      banner
      if {$level == -7} {
        set auth_errmsg "Please [url "log in" [cgi_cgi Login [cgi_set user $user]] target=_top]
          or [url "create a new user" [cgi_cgi UserNew [cgi_set user $user]] target=_top]."
      } elseif {$level == -6} {
        set auth_errmsg "Could not hash password. Check md5string function."
      } elseif {$level == -5} {
        set auth_errmsg "The [cgi_keyboard cinema.conf] file is missing or does not define SQL; please [url create [cgi_cgi Config [cgi_set what cinema]]] it."
      } elseif {$level == -4} {
        set auth_errmsg "$errormsg"
        cgi_mail_start [cgi_admin_mail_addr]
        if {[catch {cgi_mail_add "Subject: [cgi_name] CINEMA SQL problem on $env(HTTP_HOST)"}] != 0 } {
          # env(HTTP_HOST) might not be present.
          cgi_mail_add "Subject: [cgi_name] CINEMA SQL problem"
        }
        cgi_mail_add
        cgi_mail_add "$errormsg"
        
        if {[catch {cgi_mail_end} error] != 0 } {
          p "Could not send error message to [cgi_admin_mail_addr]"
          p "Email error:\n$error"
          p "Program error:\n$errormsg"
        }
      } elseif {$level == -3 || $level == -2} {
        set auth_errmsg "Invalid user or password. 
          Please [url "try again" [cgi_cgi Login [cgi_set user $user]] target=_top]
          or [url "create a new user" [cgi_cgi UserNew [cgi_set user $user]] target=_top]."
        if {$user != ""} {
          # If not already connected then disconnect will fail.
          catch {sql disconnect}
        }
      } else {
        p [bold "Minimum privilege level of $mustbe needed."]
      }
      # Print the error message if it is set.
      if {$auth_errmsg != ""} {
        p "$auth_errmsg"
      }
      cgi_exit
    }
  } else {
    cgi_head {
      cgi_title "CINEMA: $title ($user)"
    }
  }

  return $level
}

package provide cinema 1.0
