# Like cgi_import, but allow default value.
# If default is '#', complain if no value given.
# If default is '?', complain if value is empty. 
proc import_default {fieldname {default ""} {varname ""}} {
  if {$varname == ""} {
    upvar 1 $fieldname var
  } else {
    upvar 1 $varname var
  }

  if {[lsearch [cgi_import_list] $fieldname] < 0} {
    if {$default != "#"} {
      set var $default
    } else {
      p [bold "Need $fieldname as cgi argument."]
      cgi_exit
    }
  } else {
    cgi_import_as $fieldname var
  }

  if {$var == "" && $default == "?"} {
    p [bold "$varname cannot be an empty string."]
    cgi_exit
  }
  # remove any non-printing characters and trim whitespace
  regsub -all {[^[:graph:][:space:]]} $var "" var

  # remove Windows crap
  regsub -all {\x83} $var {f} var
  regsub -all {\x85} $var {...} var
  regsub -all {\x91} $var {`} var
  regsub -all {\x92} $var {'} var
  regsub -all {\x93} $var {"} var
  regsub -all {\x94} $var {"} var
  regsub -all {\x96} $var {-} var
  regsub -all {\x9a} $var {s} var
  regsub -all {\xa1} $var {} var
  regsub -all {\xa6} $var {} var
  regsub -all {\xad} $var {-} var
  regsub -all {\xb7} $var {.} var
  regsub -all {} $var {c} var

  # remove blank space
  set var [string trim $var]
}

package provide cinema 1.0
