# If user is from local domain, see if this is a local account.
# Uses 'glob ~user' to check if a local account.
# If verbose is 0, don't output <p> tags.  (We're probably somewhere in the
# form where this won't work).
# Returns
#  2  if in local domain and local user 
#  1  if in local domain but not a local user
#  0  if not in local domain or malformed 'user'.

proc localuser {user {verbose 0}} {
  global DOMAINS tcl_platform

  if {[regexp {^[A-Za-z0-9\+\-._]+@[A-Za-z0-9.\-_]+} $user]} {
    if {$verbose > 2} {
      p "Matched user@domain"
    }
    set u [split $user @]
    if {[lsearch $DOMAINS [string tolower [lindex $u 1]]] >= 0} {
      set luser [lindex $u 0]
      if {[info exists tcl_platform(platform)] && \
            $tcl_platform(platform) == "unix" && \
            [catch {set home [glob ~$luser]} msg]} {
        if {$verbose > 1} {
          p "User '[bold $luser]' is from the local domain, but not a local user."
        }
        return 1
      } else {
        if {$verbose > 2} {
          p "[bold $luser] is a local user."
        }
        return 2
      }
    }
    if {$verbose > 1} {
      p "User '[bold $user]' is not from the local domain, $DOMAINS."
    }
  }
  return 0
}

package provide cinema 1.0
