# Present a tabular form.
# Array F: describes form (prompt, order, type, value).  
# Array V: current values.

# http://javascript.internet.com/calendars/date-menu.html

proc fieldsort {a b} {
  upvar F FF
  return [expr [lindex $FF($a) 1] - [lindex $FF($b) 1]]
}


# prompt for date and time
proc datetime {f value} {
  set months {January February March April May June July August
     September October November December}

  regexp {([0-9]+)-([0-9]+)-([0-9]+) ([0-9]+):([0-9]+):([0-9]+)} \
    $value dummy year month day hour min sec
  # month
  cgi_export ${f}=
  cgi_select ${f}_month "onChange=
    datetime(this.form.$f,this.form.${f}_year.selectedIndex,this.form.${f}_month.selectedIndex,this.form.${f}_day.selectedIndex,this.form.${f}_hour.selectedIndex,this.form.${f}_min.selectedIndex)" {
    set m 1
    foreach label $months {
      cgi_option $label value=$m selected_if_equal=$month
      incr m
    }
  }
  # day
  cgi_select ${f}_day "onChange=
    datetime(this.form.$f,this.form.${f}_year.selectedIndex,this.form.${f}_month.selectedIndex,this.form.${f}_day.selectedIndex,this.form.${f}_hour.selectedIndex,this.form.${f}_min.selectedIndex)" {
    for {set d 1} {$d <= 31} {incr d} {
      cgi_option $d selected_if_equal=$day
    }
  }
   
  # year
  cgi_select ${f}_year "onChange=
    datetime(this.form.$f,this.form.${f}_year.selectedIndex,this.form.${f}_month.selectedIndex,this.form.${f}_day.selectedIndex,this.form.${f}_hour.selectedIndex,this.form.${f}_min.selectedIndex)" {
    foreach y {2000 2001 2002} {
      cgi_option $y selected_if_equal=$year
    }
  }

  # hour
  cgi_select ${f}_hour "onChange=
    datetime(this.form.$f,this.form.${f}_year.selectedIndex,this.form.${f}_month.selectedIndex,this.form.${f}_day.selectedIndex,this.form.${f}_hour.selectedIndex,this.form.${f}_min.selectedIndex)" {
    for {set h 1} {$h <= 23} {incr h} {
      cgi_option [format "%2d" $h] selected_if_equal=$hour
    }
  }
  puts ":"
  # minute
  cgi_select ${f}_min "onChange=
    datetime(this.form.$f,this.form.${f}_year.selectedIndex,this.form.${f}_month.selectedIndex,this.form.${f}_day.selectedIndex,this.form.${f}_hour.selectedIndex,this.form.${f}_min.selectedIndex)" {
    for {set m 0} {$m <= 45} {incr m 15} {
      cgi_option [format "%02d" $m] selected_if_equal=$min
    }
  }
}


# one form element
proc element {type f value par Vname {script ""}} {
  upvar $Vname V  

  if {$script != ""} {
    set script "onChange=$script"
  }

  switch $type {
    show_export {
      puts [blank $value]
      cgi_export $f=$value
    }
    show {
      puts [blank $value]
    }
    blank {
      puts [blank]
    }
    datetime {
      datetime $f $value
    }
    textarea {
      cgi_textarea $f=$value rows=[lindex $par 0] cols=[lindex $par 1]
    }
    password {
      cgi_text $f=$value size=$par type=password
    }
    text {
      cgi_text $f=$value size=$par
    }
    checkbox -
    checkbox_v {
      foreach o $par {
        set v [lindex $o 0]
        if {[lsearch $value $v] >= 0} {
          set oo $v
        } else {
          set oo ""
        }
        checkbox ${f}List=$v checked_if_equal=$oo
        if {[lindex $o 1] != ""} {puts [lindex $o 1]} {puts $o}
        if {$type == "checkbox_v"} {cgi_br}
      }
    }
    select {
      cgi_select $f $script {
        foreach v $par {
          set ovalue [lindex $v 0]
          if {[llength $v] <= 1} {
            set olabel [lindex $v 0]
          } else {
            set olabel [lindex $v 1]
          }
          cgi_option $olabel value=$ovalue selected_if_equal=$value
        }
      }
    }
    url_hidden -
    url {
      if {$type == "url"} {cgi_export $f=$value}
      set args ""
      foreach p [lrange $par 1 end] {
        if {[llength $p] == 1} {
          set args "$args[cgi_set [lindex $p 0] $V([lindex $p 0])] "
        } else {
          set args "$args[cgi_set [lindex $p 0] [lindex $p 1]] "
        }
      }
      set c [eval "cgi_cgi [lindex $par 0] $args"]
      puts [url [blank $value] $c]
    }
  }
}


# generate display
proc tableshow {Fname Vname} {
  upvar $Fname F
  upvar $Vname V  

  set fieldlist [array names F]
  set fieldsorted [lsort -command fieldsort $fieldlist]

  table {
    foreach f $fieldsorted {
      set prompt [lindex $F($f) 0]
      set type   [lindex $F($f) 2]
      set par    [lindex $F($f) 3]
      if {$type != "skip"} {
        if {![info exists V($f)]} {
          set V($f) ""
        }
        table_row bgcolor=#b0cafa {
          td valign=top "${prompt}:"
          table_data valign=top {
            switch $type {
              select -
              password -
              text -
              show {
                puts [blank $V($f)]
              }
              textarea {
                preformatted {
                  puts [cgi_quote_html $V($f)]
                }
              }
              checkbox {
                set checked ""
                foreach o $par {
                  if {[lsearch $V($f) $o] >= 0} {
                    set checked [concat $checked $o]
                  }
                  puts [join $checked ,]
                }
              }
            }
          }
        }
      }
    }
  }
}

# 'Vertical' form, with one field per row
proc tableform {submit Fname Vname} {
  upvar $Fname F
  upvar $Vname V  

  cgi_script language=JavaScript src=datetime.js {}

  set fieldlist [array names F]
  set fieldsorted [lsort -command fieldsort $fieldlist]
  table {
    foreach f $fieldsorted {
      set prompt [lindex $F($f) 0]
      set type   [lindex $F($f) 2]
      set par    [lindex $F($f) 3]
      set script [lindex $F($f) 4]
      if {$type != "skip" && $type != "hidden"} {
        if {![info exists V($f)]} {
          set V($f) ""
        }
        table_row bgcolor=#b0cafa {
          td "${prompt}:"
          table_data {
            element $type $f $V($f) $par V $script
          }
        }
      }
      if {$type == "hidden"} {
        cgi_export $f=$V($f)
      }
    }
    table_row {
      foreach s $submit {
        table_data align=center {
           submit_button $s
        }
      }
    }
  }
}


# Create header row
proc tableheader {Fname} {
  upvar $Fname F

  set fieldlist [array names F]
  set fieldsorted [lsort -command fieldsort $fieldlist]

  table_row {
    foreach f $fieldsorted {
      if {[lindex $F($f) 2] != "hidden"} {
        th [lindex $F($f) 0]
      }
    }
  }
}


# 'Horizontal' form, with one field per column
proc tablehform {form bgcolor submit Fname Vname} {
  upvar $Fname F
  upvar $Vname V  

  cgi_script language=JavaScript src=datetime.js {}

  set fieldlist [array names F]
  set fieldsorted [lsort -command fieldsort $fieldlist]
  form $form {
    table_row bgcolor=$bgcolor {
      foreach f $fieldsorted {
        set type   [lindex $F($f) 2]
        set par    [lindex $F($f) 3]
        if {$type != "skip" && $type != "hidden"} {
          if {![info exists V($f)]} {
            set V($f) ""
          }
          table_data {
            element $type $f $V($f) $par V
          }
        }
        if {$type == "hidden"} {
          cgi_export $f=$V($f)
        }
      }
      foreach s $submit {
        table_data align=center {
          submit_button $s
        }
      }
    }
  }
}


# Create SET string for INSERT or UPDATE given field list
proc tableforminput {Fname {fieldlist ""} {Dname ""}} {
  upvar $Fname F
  if {$Dname != ""} {
    upvar $Dname D
  }

  if {$fieldlist == ""} {
    set fieldlist [array names F]
  }
  set q ""
  foreach f $fieldlist {
    if {$Dname != ""} {
      set def $D($f)
    } else {
      set def ""
    }
    if {[lindex $F($f) 2] == "checkbox" || [lindex $F($f) 2] == "checkbox_v"} {
      import_default ${f}List $def $f
    } else {
      import_default $f $def
    }
    # make sure that postalcodes like 07605 are treated as strings
    if {[set $f] == ""} {
      lappend q "$f=NULL"
    } elseif {[regexp {^[1-9][0-9]*$|^[0-9]\.[0-9]+$} [set $f]]} {
      lappend q "$f=[set $f]"
    } else {
      regsub -all {'} [set $f] {''} $f
      set q [concat $q "{$f='[set $f]'}"]
    }
  }
  return $q
}

package provide cinema 1.0
