#! /usr/local/bin/tclsh

# Include the cinema libraries.
source where.tcl
config cinema.conf

proc administrator {} {return [cgi_italic "administrator"] }
proc user {} {return [cgi_italic "regular user"] }
proc userid {} {return [cgi_typewriter "userid"] }
proc password {} {return [cgi_typewriter "password"] }
proc realm {} {return [cgi_typewriter "realm"] }
proc section {title {name ""}} {
  p "[url "top" #top]"; cgi_br
  cgi_hr; cgi_br
  if { $name != "" } {
     puts "[cgi_anchor_name $name]"
  }
  puts "[cgi_h2 "$title"]"
}
proc installmanual {} {
  return "[url "installation manual" "../doc/install.html"]"
}
proc icon {file} {
  return "<img height=30 src=\"$file\">"
}
proc cgi_image {file} {
  return "<img src=\"$file\">"
}

cgi_link sipum "sipum" "http://www.cs.columbia.edu/~hgs/software"
cgi_link sipconf "sipconf" "http://www.cs.columbia.edu/~hgs/software"

cgi_body_args bgcolor=#ffffff

cgi_eval {
cgi_head {
  cgi_title "CINEMA GUI User Manual"
}

cgi_body {

cgi_h1 "CINEMA GUI User Manual"

cgi_h2 "Table of Contents"
cgi_bullet_list {
 li; p [url "Overview" #overview]
 li; p [url "Creating a user" #creating]
 li; p [url "Logging in" #login]
 li; p [url "Edit user profile" #edituser]
 li; p [url "Edit aliases" #editaliases]
 li; p [url "Edit contacts" #editcontacts]
 li; p [url "Third-party registration" #editthird]
 li; p [url "Voice mail" #voicemail]
 li; p [url "Conferencing" #conferencing]
 li; p [url "List of users (admin)" #listofusers]
 li; p [url "Adding new user (admin)" #addinguser]
 li; p [url "Telephony gateway class (admin)" #gwclass]
 li; p [url "Accounting (admin)" #accounting]
 li; p [url "Config files (admin)" #config]
 li; p [url "Logging out" #logout]
 li; p [url "Deleting a user" #deleteuser]
}

section "Overview" overview

p "With the Cinema GUI, various user and system profiles can be
configured using a web browser.  The web interface is a front end for
the database tables used by the system.  There are two types of users: 
[user] or [administrator].  An administrator has additional privileges
compared to a regular user.  The first user created becomes the
[administrator].  He can then add additional users as [administrator]
later, or change the status from [administrator] to [user] or
vice-versa.  A [user] cannot access profiles of other users or change
the status of himself or other users."

p "Users can login from the web page by providing their [userid] and
[password]. The [userid] is a unique identifier for a user
and is of the form [italic "user@domain"]."

p "The system administrator must install the system before the users
can use it. See the [installmanual] available with the distribution."

# new section.
section "Creating a user" creating

p "A new user is created by visiting the [url "main web page" index.cgi].
Enter the userid of the form user@$DOMAINS as shown below. 
In the following screen shot, user [cgi_sample ka191@cs.columbia.edu] is
being created.  You do not need to change the text in the [realm] edit
box."

cgi_division align=center {
  p "[cgi_image gui_login.jpg]"
}

p "The userid [bold "MUST"] be a valid email address.  An
email notification with the password is sent to that userid.
When a new user is created, the default user profile is borrowed from
that of user \"default@$DOMAINS\".  You can change your profile
later.  The default profile can be altered by the [administrator],
typpically during installation."

p "The initial password is randomly choosen.  You are strongly
encouraged to change your password immediately after creation, by
[url "logging" #login] into the system."

# Section.
section "Logging in" login

p "Visit the [url "same page" "index.cgi"] again and enter your userid
as \"user@$DOMAINS\".  Click on [italic "Login or add user"] button.  If
the user is already created, the system will prompt for the
password.  After logging in, you will see the user menu frame similar to
the following picture."

p align=center "[cgi_image gui_menu.jpg]"

p "The exact number of menu items depends on your user type: [administrator]
or [user], and your installation, e.g., whether you have voice
mail or conferencing."

p "Make sure that your browser is configured to enable Java and
JavaScript.  For example, for Netscape, go to
[cgi_keyboard Edit->Preferences->Advanced], and click the check boxes for \"Enable
Java\" and \"Enable JavaScript\".  Also it important to enable \"Accept
all cookies\".  Your browser should support frames.  Both Netscape 4.0
and later and Microsoft Internet Explorer support frames."

p "Placing your mouse on the appropriate icon will display the purpose
of that icon. Icons correspond to \"Edit user profile\", 
\"Edit aliases\",
\"Edit contacts\", \"Log out\", and so on. Administrators also have additional
menu items for \"List users\", \"Add user\", \"Gateway class\", and 
\"Configure\"."

section "Edit User profile" edituser

p "Clicking on the [bold Edit] user icon [icon "passport.gif"], lets you
edit your user profile."

cgi_definition_list {
cgi_term "Realm:"
cgi_term_definition "This is the realm or the prompt for asking for your
password.  Normally you should keep it the same as your domain name,
e.g., [italic "$DOMAINS"].  To change the realm you [bold "MUST"] supply
the password.  The reason is that the password is stored as encrypted
hash of the user id, realm and password.  Changing the realm will make
the encrypted hash invalid.  So you must also fill the \"New password\"
and the \"Reenter password for verification\" edit boxes."

cgi_term "New password:"
cgi_term_definition "You need to supply the new password and reenter it
for verification only if you wish to change your [password] or your
[realm].  You are strongly encouraged to change the password frequently
and not to use common or dictionary words."

cgi_term "Authentication:"
cgi_term_definition "The SIP server supports authentication based on
basic or digest authentication algorithms.  The authentication mechanism
is a global [url "configuration" #config] which can be changed by the
administrator.  However, when authentication is needed can be set on
per-user basis.  The global configuration option overrides per-user
configuration.  Global configuration can set the authentication to
\"digest\", \"basic\" or \"none\".  If the global configuration is set
to use authentication then all SIP REGISTER requests must be
authenticated.  For INVITE and other requests (used for making and
terminating the calls), the per user configuration can be set to (1)
never, which disables proxy-authentication for making a call, (2) request:  try
requesting authentication but complete the call even if authentication
fails, or (3) required:  always requires proxy authentication.  For
initial system users can set this value to \"never\" to allow anobody to
reach them.  <p>Note that the registration authentication is a system
wide global configuration option and can not be changed on per-user
basis, unlike call authentication.  <p>Most User set the algorithm to
MD5 for authentication."

cgi_term "Voice mail:"
cgi_term_definition "Voice mail is an optional component.  Ask your
administrator to license [cgi_link sipum] to avail the voicemail
facility.  <p>If the user has been configured to have voice mail, then he
can set the RTSP URL for the outgoing message prompt which gets played
when the user's phone is busy or there is no response.  Alternatively
you may upload your own recorded message as the prompt.  This prompt is
played when your phone is busy or there is no response.  <p>When a new
voice mail is received, an email notification is sent to the user.  The
format of the email can be altered on a per-user basis.  For example, a
user using web based email system would prefer to have HTTP URLs in the
email.  <p>Users can also set the timeout after which the call is
forwarded to the voicemail system.  And the maximum size of each message
allowed for this user.  The total size of all the messages is a system
level global configuration option."
}

p "This page also allows you to sign-out from the service.
Another confirmation dialog is presented before actually deleting a user 
account."

# section 
section "Edit Aliases" editaliases

p "With an alias, you can be reached under more than one name.  You can
have as many aliases as you like, but aliases must not conflict with
existing user names.  For example, if your userid is [keyboard bob@$DOMAINS] and
you add an alias of [keyboard Bobby@$DOMAINS], then any call for
[italic "sip:Bobby@$DOMAINS"] will be forwarded to you at
[italic "bob@$DOMAINS"]. After you click on the \"Aliases\" icon
[icon "people0a.gif"], you can change your aliases."

# section
section "Edit Contacts" editcontacts

p "The contacts page [icon "rolodex2.gif"] allows you to
view and edit current your SIP registrations. This is also useful to create
a follow-me of service, where you can enter a contact as
\"sip:bob.wilson@hotel-california.com\", and all the calls to
you (\"sip:bob@$DOMAINS\") will be forwarded to this new address."

p "The contact must be a valid URL. Usually the SIP server does not
forward the call to a non-sip URL."

p "The preference field for a particular contact can have values between
0.0 to 1.0, where a higher value is preferred.  Thus, a contact with a
preference value of 1.0 is the most preferred contact.  The SIP server
in proxy mode tries to contact the locations based on the preference
values, highest preference first.  All contacts with same preference
value are contacted at the same time, i.e., all those phones will ring
simultaneously."

p "You can specify the expiration time for a particular contact.  If
nothing is specified, then the contact location never expires."

p "You can choose whether the contact locations should be
contacted in proxy or redirect mode. Proxy mode is transparent to
the calling party and hence may be preferred at times.
Please note that all the contact locations must have the same mode.
You can not have one contact as proxy and another as redirect."

# section
section "Edit Third-party registration" editthird

p "Third-party registration [icon "register3.gif"]
allows others to register contacts for you. You must specify a valid
user id of the form user@${DOMAINS}."

# section
section "Voice mail" voicemail

if {[file executable vmail.cgi]} {
  p "After clicking on the voice mail icon [icon "voicemail1.gif"],
you can view your voice messages. The interface is similar to other
web based email systems like hotmail. An example inbox is shown below."

p align=center "[cgi_image gui_vmail.jpg]"

p "The red arrow indicates that the message is a new message.  The date
column lists the time and date when the message was received.  The
senders address is in the [italic From] column.  [italic Subject] shows
the SIP subject of the original call.  If the SIP call has a priority
higher than [italic normal], then the priority is printed before the
subject.  If there is no subject, the message number is displayed
instead.  The size of the message is given in both seconds and
kilobytes."

p "You can delete messages by selecting them.  Messages are selecting
by clicking on the check box in the left-most column and then clicking
on the \"Delete Selected\" link.  Mails can be moved to other folders by
selecting the mails and then selecting the appropriate folder.  The
selected mails can also be forwarded to a specified email address as
email attachments."

p "Voice messages are stored as [cgi_sample .au] files or G.711 mu-Law 8
kHz 8-bit audio files.  To play the voice message, you can just click on
the subject field of the appropriate message.  Make sure that your
browser is configured to play the MIME-type of [cgi_sample audio/basic]
(or .au extension) properly.  For example, on Solaris using the Netscape
browser, you can edit the preferences as
[cgi_keyboard Edit->Preferences->Navigator->Applications->New].
<pre>
Description: AU 
MIMEType: audio/basic
Suffixes:.au 
Application: audioplay -i -V -v 100 -p speaker %s.
</pre>
"
} else {
  p "Voice mail is an optional component. Ask your administrator to
license [cgi_link sipum] to use the voicemail facility."
}


# section
section "Conferencing" conferencing

if {[file executable ConfList.cgi]} {
p "Click on the conferencing icon [icon "meeting12.gif"]
to display the conferencing page. An example conferencing page is shown
below."

p align=center [cgi_image gui_conference.jpg]

p "The conference is identified using a conference URL.  For example, if
the conference URL is called \"test\", then you can join the conference
by connecting to \"sip:test@$DOMAINS\", where [italic domain] is the
domain your system is in.  Restricted conferences limit who can listen
or speak."
} else {
  p "Conferencing is an optional component.  Ask your administrator to
license [cgi_link sipconf] to make use of the audio, video and text
conferencing facility."
}

# section
section "List of users" listofusers

p "Administrators can view the list of users by clicking on the list
icon [icon "page.gif"].  The list gives a comprehensive listing of all
users.  A pop-up menu, visible when moving the cursor over the user
name, contains more detailed information for each user."

# section
section "Adding a new user" addinguser

p "Administrators can add a new user by clicking on the new user icon
[icon "person_add.gif"]. This is typically needed for 
adding special user ids like \"tel:+12129397130\", for mapping a telephone
number to a user id."

# section
section "Telephony gateway class" gwclass

p "The gateway class configuration menu [icon "permit.gif"] allows an
administrator to update the gateway class needed by a user for making
calls to a telephone number through a SIP-to-PSTN gateway.  Different
gateway classes can have different dialing privileges, e.g., students
may be disallowed from dialing a long distance number whereas faculty
and staff may be allowed to do so.  Every user is assigned a gateway
class, similar to the unix group identifier.  When a user wants to make
a call to a telephone number, the gateway class of the user is compared
against the allowed gateway classes for that type of number.  Please see
the example [cgi_keyboard sipd/gateways.sample] and 
[cgi_keyboard tools/canonicalize/dialplan.sample] files and associated
documentation."

# section
section "Accounting and Logging" accounting

p "Accounting, rating and billing are three stages that are based on the
SQL request log for the SIP server.  This feature is designed to support
simple re-billing, not serve as a multi-user billing interface.  For
example, the mechanism does not support fixed charges, multiple
currencies, sending invoices or collecting credit card information.  It
should be sufficient for many departmental rebilling applications."

p "The [icon requestlog.gif] request log function lists all
requests, if invoked by the administrator, or the requests of the user
logged in.  The request list can be sorted according to time, request
type, source and destination by clicking on the column heading.  At the
bottom of the page, a menu allows to limit the list to a particular time
range.  This feature will only work if the server has been
[url configured config] to log requests to the SQL database."

p "Tariffs can be created and edited [icon paragraph.gif].  A tariff
specifies what a call costs per time unit.  A tariff applies to a
certain date range and during certain times of the day.  Also, it can be
restricted to a [url "user class" gwclass].  The current
implementation does not support tariffs that apply only on certain days,
e.g., weekends and holidays, as most telecom tariffs are now
time-insensitive.  The prefix indicates the user name this tariff
applies to, generally a phone number.  This is a [cgi_italic glob] pattern,
with [cgi_sample ?] representing any single character and [cgi_sample *] any
number of characters.  Thus, [cgi_sample +1212*] represents phone numbers in
Manhattan.  Note:  in almost all cases, a trailing [cgi_sample *] is
required. Tariff durations are automatically rounded, i.e., an end time
of 23:59 is rounded to 23:59:59. There is no need to delete tariffs; if
a particular tariff is no longer valid, mark its end date accordingly."

p "For each billing period, calls can be rated [icon rating.gif],
i.e., assigned a cost.  Each call is only rated once, since
the billing period and cost is recorded in the call detail record.  It
is possible that some calls do not match any tariff and thus cannot be
rated.  Calls that could not be rated are listed in orange."

p "Finally, users can see their bills [icon bill.gif] for each
billing period.  All calls are tallied by time, duration and
destination, with a total duration and expense given."

# section
section "Config files" config

p "The administrator can edit the configuration options from the web by
clicking on the [bold configure] icon [icon "wrench.gif"].  The configure
page allows the administrator to edit various application specific
configuration files, such as the sipd configuration file (\"sipd.conf\")
as well as the global system configuration file \"cinema.conf\".  Please
refer to the [url "sipd documentation" http://www.cs.columbia.edu/IRT/software/cinema/sipd.html] on the
contents of the \"sipd.conf\" file.  Please refer to the installation
manual for editing the \"cinema.conf\" file."

# section
section "Logging out" logout

p "Users are advised to log out [icon "exit_emergency.jpg"] after they
are done.  If a users does not log out, their userid and password are
stored in the browser (via cookies).  If somebody else uses the
same web browser to visit the configuration web page, that user does not
have to enter the password again."

# section
section "Deleting a user" deleteuser

p "A user can remove himself from the system 
(see [url "Edit user" #edituser])
or can be removed by the administrator 
(see [url "User list" #listofusers])."

p "[url "top" #top]"

cgi_hr

}
}
