# Return MD5 hash of concatenation of user:realm:pw.
proc hash {user realm pw} {
  if [catch {set h [exec ./md5string "$user:$realm:$pw"]} msg] {
    return ""
  } else {
    return [lindex $h 0]
  }
}


# return space if element is empty
proc blank {{s ""}} {
  if {$s == ""} {return [nbspace]} {return $s}
}


# Set bgcolor for a table row, using the global color declaration
proc bgcolor {rowname} {
  global bglist
  upvar $rowname row

  if {![info exists row]} {
    set row 0
  }
  incr row
  return "bgcolor=[lindex $bglist [expr $row % 2]]"
}


# return version of variable with ' escaping
proc q {s} {
  regsub -all {'} $s {''} quoted
  return $quoted
} 


# Done with database.
proc done {} {
  catch {sql disconnect}
  cgi_exit
}


# Map list onto list of variable names. Also takes comma-separated list.
proc maplist {list args} {
  set i 0
  set cargs [split $args ,]
  if {[llength $args] == 1 && [llength $cargs] > 1} {
    set args $cargs
  }
  foreach arg $args {
    upvar $arg $arg
    set $arg [lindex $list $i]
    incr i    
  }
}

# Map list onto array.
proc maparray {data fields Vname} {
  upvar $Vname V

  set i 0
  foreach v $fields {
    set V($v) [lindex $data $i]
    incr i
  }
}

# Read an entire file.  Don't accept pipelines.
proc catfile {file} {
  if {[string index $file 0] == "|"} {
    error "Cannot read $file: pipeline not allowed"
  }
  set f [open $file]
  set ret [read $f]
  close $f
  return $ret
}


# Return domain part of user@host identifier.
proc domain {u} {
  return [lindex [split $u @] 1]
}


# Return user part of user@host identifier.
proc userpart {u} {
  return [lindex [split $u @] 0]
}


# Return string in hh:mm:ss format
proc hhmmss {t} {
  set hours [format "%02d" [expr $t / 3600]]
  set mins  [format "%02d" [expr ($t % 3600)/60]]
  set secs  [format "%02d" [expr $t % 60]]
  return "$hours:$mins:$secs"
}


# Validate and parse a path-based URL
proc validate_path_url {url scheme_ user_ pw_ host_ port_ path_} {
  upvar scheme_ scheme
  upvar user_   user
  upvar pw_     pw
  upvar host_   host
  upvar path_   path

  return [regexp -nocase \
  {([a-z]+)://([a-z0-9%_-]+)(:([a-z0-9%_-]*))@([a-z0-9%_\.-]+)(:([1-9][0-9]*))*/([a-z0-9%_-]+)} $url \
  match scheme pwc pw user host portc port path]
}


# Validate whether cookies are enabled.
proc validate_cookies {} {
  cgi_script language=JavaScript {
    cgi_puts {
      hascookies = true;
      function cc() {
        if (document.cookie == "") {
          /* if a cookie is not found, alert user. */
          alert("Cookies need to be enabled in your browser!");
          hascookies = false;
        } else {
          hascookies = true;
        }
      }
      document.cookie = 'killme' + escape('nothing')
    }
  }
}


# validate a user entry - client side
# server-side validation is in checkuser.tcl
proc validate_user_script {} {
  cgi_script language=JavaScript {
    cgi_puts {
      function badURI(field)
      {
        re = /^[A-Za-z0-9+-]+@[A-Za-z0-9][.A-Za-z0-9-]+$/i;
        var f = field.value;
        if (!re.test(f)) {
          window.alert (f + 
             " isn't a valid SIP user name of the form user@host.");
          field.focus();
          return true;
        }
        return false;
      }
      function validate_user(field)
      {
        if (badURI(field)) return false;
        return true;
      }
    }
  }
}

# JavaScript for checking user names
proc validate_user_pw_script {} {
  cgi_script language=JavaScript {
    cgi_puts {
      function badURI(field)
      {
        re = /^[A-Za-z0-9+-]+@[A-Za-z0-9][.A-Za-z0-9-]+$/i;
        tel = /^tel:[0-9]+$/i;
        var f = field.value;
        if (!re.test(f) && !tel.test(f)) {
          window.alert (f + 
             " isn't a valid SIP user name of the form user@host or tel:.");
          field.focus();
          return true;
        }
        return false;
      }
      function validate_user(field,pass,pass1)
      {
        if (pass.value != pass1.value) {
          alert("Please re-enter password.");
          pass.focus();
          return false;
        }
        if (badURI(field)) return false;
        return true;
      }
    }
  }
}

# check for a valid file name without any . or .. entries or
# any /.
proc checkfilename {name} {
  if {$name != [file tail $name]} {
    error "Incorrect path $name."
    cgi_exit
  }
}

package provide cinema 1.0
