#! /usr/local/bin/tclsh

# Voicemail GUI.

# Include the cinema libraries.
source where.tcl

# Derive other config variables.
set g_vmailhelpurl  "usermanual.html"

# URL of the image to be used to show new mails.
set g_newmailimgurl "newmail.gif"

# URL of the email icon
set g_emailiconurl  "emailicon.gif"

# newmailimg() : URL for indicating a new message as an image.
proc newmailimg {} {
  global g_newmailimgurl
  return "[img $g_newmailimgurl alt='NewMail' hspace=1 border=0 height=11 width=11]"
} 

# emailicon() : Icon for email. This is displayed besides the
# from address of message send. Clicking on this icon 
# invokes the appropriate email-tool to reply back.
proc emailicon {} {
  global g_emailiconurl
  return "[img $g_emailiconurl alt='Re:email' hspace=1 border=0 height=11 width=15]"
} 

# Extract the user name from user@host
proc extractuserid {userhost} {
  set uh [split $userhost @ ]
  return [lindex $uh 0]
}

# Default domain.
proc def_domain {} {
  global DOMAINS
  set domains [split $DOMAINS]
  if {[llength $domains] > 0} {
    return [lindex $domains 0]
  } else {
    return "unknowndomain"
  }
}

# check for a valid file name without any . or .. entries or
# any /.
proc checkforfilename {name} {
    if { $name != [file tail $name] } {
        error "Incorrect path"
        cgi_exit
    }
}

# cgi_eval() main routine to display the page.
cgi_eval {
    global VMAILUSERDIR

    cgi_input

    config cinema.conf

    if {![info exists VMAILUSERDIR] || $VMAILUSERDIR == "" } {
      p "Voice Mail is not installed."
      cgi_exit
    }

    import_default _VMaction "displayfolder"
    set command $_VMaction

    # Get all CGI parameters, _VMaction, folder, etc.
    import_default folder "inbox"
    import_default sort   "date"

    checkforfilename $folder

    # Retrieve the voicemail MIME type audio/basic
    if { $command == "retrieve" } {

        # Export the "user" cookie. and "user" and "password" params.
        set level [auth vmail "Voice Mail" 2 audio/basic no]

        # Format other important links and variables
        import_default fname  ""

        checkforfilename $fname

        if { $folder == "" || $fname == "" } {
            error "Invalid mail folder or mail file" 
            cgi_exit
        } else {
            if {$level < 2 || $for == ""} {
              cgi_exit
            }
            
            set userid [extractuserid $for]
            set topdir "$VMAILUSERDIR/$userid"
            set folderdir "$topdir/$folder"        
                  
            # Set the status as read...
            if { [catch {set td [open "$folderdir/$fname" r+]} msg] == 0} {
              # Could open the file.
              set readsize 0
              while { -1 != [set linesize [gets $td str]]} {
                set linesize [expr $linesize + 1 ]
                set statusitem "R"
                if {[regexp "Status:(\[^\|\n]+)" $str dummy statusitem]==1} {
                  if { $statusitem != "R" } {
                    seek $td $readsize
                    # Set the status to R
                    puts $td "Status:R"
                  }
                  break
                }
                set readsize [expr $readsize + $linesize]
              }
              catch { close $td}
            }

            set td [open "$folderdir/$fname" r]
            fconfigure $td -translation binary
            fconfigure stdout -translation binary
            fcopy $td stdout
            # while { -1 != [gets $td str]} {
            #     puts $str
            # }
            catch { close $td}
        }
        # Retrive the voicemail and play back.
    } elseif { $command == "deleteaccount" } {

        # Export the "user" cookie. and "user" and "password" params.
        set level [auth vmail "Voice Mail" 2 text/html no]
        
        if { $level < 1 } {
            p "Unknown account."
            cgi_exit
        }

        # Format other important links and variables
        set userid [extractuserid $for]

        # Format other important links and variables
        set topdir "$VMAILUSERDIR/$userid"

        if { $userid == "" } {
          p "Could not delete voicemail account for empty userid"
        } else {
          if {[catch {file delete --force -- "$VMAILUSERDIR/$userid"} msg] == 0 } {
              p "Deleting the voicemail account (if any)."
          } else {
              p "Could not delete voicemail account $userid: $msg"
          }
        }

    } else {
      # Proceed to execute the command, or display the inbox.

      set level [auth vmail "Voice Mail" 2 text/html no]

      if { $level < 2 } {
        puts "Userid is incorrect"
        cgi_exit
      }

      cgi_head {
        title "Voicemail for $user"

        # Javascripts to check all and open new help window.
        javascript {
          puts {
            function CheckAll()
            {
              for (var i=0;i<document.vmail.elements.length;i++)
              {
                var e = document.vmail.elements[i];
                if (e.name != 'allbox')
                  e.checked = document.vmail.allbox.checked;
              }
            }
          }
        }
        noscript {
            puts "Sorry - your browser doesn't understand Javascript."
        }
      }
# ------------- Body of Mail voicemail page. --------------
    cgi_body {

      cgi_form vmail name=vmail {
        global VMAILUSERDIR
        global g_vmailhelpurl
        global g_newmailimgurl

        # Display the page

        # Export those variables.
        cgi_export user=$user
        cgi_export passwd=$pw
        cgi_export for=$for
        cgi_export folder=$folder
        cgi_export sort=$sort

        # puts "$user:$for:$pw:$folder:$sort"

        set userid [extractuserid $for]

        # Format other important links and variables
        set topdir "$VMAILUSERDIR/$userid"
        set folderdir "$topdir/$folder"

        # create the directory if not already created.
        if { [file exists $topdir] == 0 } {
            # Check if the voice mail is allowed for the
            # user using SQL query. XXX
            if {[catch {file mkdir "$topdir"} msg] != 0} {
                p "Can not create $topdir: $msg"
            } elseif {[catch {file mkdir "$topdir/inbox"} msg] != 0} {
                p "Can not create $topdir/inbox: $msg"
            } else {
                catch {file attributes $topdir -permissions 0700}
                catch {file attributes "$topdir/inbox" -permissions 0700}
                p "First time user, created the voicemail home directory."
            }
        } else {
            if { [file type $topdir] != "directory" } {
                p "$topdir is not a directory. Report the problem."
            }
        }

        cgi_link inboxlink    "[cgi_font color=#ffffff Inbox]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.folder.value='inbox';document.vmail.submit();"
        cgi_link folderslink  "[cgi_font color=#ffffff Folders]" "javascript:document.vmail._VMaction.value='displayfolderlist';document.vmail.submit();" 
        cgi_link optionslink "[cgi_font color=#ffffff Options]" "[cgi_cgi UserEditForm [cgi_set for $for] [cgi_set user $user] [cgi_set pw $pw]]"
        cgi_link helplink     "[cgi_font color=#ffffff Help]" "$g_vmailhelpurl"

        cgi_imglink newmaillink "$g_newmailimgurl" alt="Sort by New Mail" hspace=1 border=0 height=11 width=11

        cgi_link  datelink "[cgi_font color=#000000 Date]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='date';document.vmail.submit();"
        cgi_link  fromlink "[cgi_font color=#000000 From]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='from';document.vmail.submit();"
        cgi_link  subjectlink "[cgi_font color=#000000 Subject]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='subject';document.vmail.submit();"
        cgi_link  sizelink "[cgi_font color=#000000 Size]" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.sort.value='size';document.vmail.submit();"
        cgi_link  deletelink "Delete<br>Selected" "javascript:document.vmail._VMaction.value='delete';document.vmail.submit();"
        cgi_link  forwardlink "Forward" "javascript:document.vmail._VMaction.value='forward';document.vmail.submit();"
        cgi_link  refreshlink "Refresh" "javascript:document.vmail._VMaction.value='refresh';document.vmail.submit();"

        if { $command == "delete" } {
            # Delete the voicemails present in DeleteVMailList.
            if {0==[catch {cgi_import DeleteVMailList}]} {
                # Uncomment following line for notification of deletion.--TODO
                # h4 "The following voice mails were deleted:"
                foreach mail $DeleteVMailList {
                    # Uncomment following line for notification of deletion --TODO.
                    # h5 $mail 
                    if [catch {file delete -force -- $folderdir/$mail} msg] {
                        # Uncomment following line for notification --TODO.
                        # h4 "$msg"
                    }
                }
            }
            # Display the remaining mails in folder.
            set command "displayfolder"

        } elseif { $command == "forward" } {
            if {[catch {set mails [cgi_import DeleteVMailList]}] == 0 } {

                # Forward the selected voicemails as 
                # an attachement to the emaillist.
                set emaillist ""
                catch {set emaillist [cgi_import emaillist]}
                # XXX use random boundary....
                set boundary "------------FDAAFA6929D67C4D187555EF"
                # XXX reply to should have the actual email id.
                cgi_mail_addr $user
                cgi_mail_start $emaillist
                cgi_mail_add "MIME-Version: 1.0
Reply-To: $for
Subject: \[Forwarded voice mail(s)\]
Content-Type: multipart/mixed; boundary=\"$boundary\"

This is a multi-part message in MIME format.
--$boundary
Content-Type: text/plain; charset=us-ascii


$user has forwarded the attached voice mail(s) to you."

# puts "Email header is $emailmsg"

                foreach mail $DeleteVMailList {
                    # Read the Subject from the voicemail.
                    set subject "voicemail"
                    set fp [open "$folderdir/$mail" r]
                    if { $fp != 0 } {
                        # puts "File $folderdir/$mail is opened"
                        fconfigure $fp -translation binary
                        set magic [read $fp 4]
                        set x [read $fp 4]
                        binary scan $x I hdrsize
                        # Skip rest of the 16 bytes.
                        read $fp 16  
                        set hdrsize [ expr $hdrsize - 16 ]
                        set y [read $fp $hdrsize]
                        binary scan $y A* info
                        
                        regexp "Subject:(\[^\|\n]+)" $info dummy subject
                        # puts "Subject is $subject"

                        catch {close $fp}
                    } else {
                        # puts "File $folderdir/$mail could not be opened"
                    }
                    
                    cgi_mail_add "--$boundary
Content-Type: audio/basic; name=\"$mail\"
Content-Transfer-Encoding: 8bit
Content-Description: $subject
Content-Disposition: attachment; filename=\"$mail\"

[catfile $folderdir/$mail]"
                }

                cgi_mail_add "--$boundary--"

                cgi_mail_end
            } else {
                puts "Select some mails and then forward."
            }

            # Display the mails in folder.
            set command "displayfolder"


        } elseif { $command == "MoveTo" } {
            # Move the voicemails present in DeleteVMailList to a different folder
            # given by tolist.
            set tobox ""
            if { [catch {cgi_import tobox}] == 0 } {
                set tobox [cgi_import tobox]
            }
            if { $tobox != "" } {
                set newdir "$topdir/$tobox"

                if {0==[catch {cgi_import DeleteVMailList}]} {
                    foreach mail $DeleteVMailList {
                        if [catch {file rename -- $folderdir/$mail $newdir} msg] {
                            puts "Can not move $folderdir/$mail to $newdir: $msg"
                            br
                        }
                    }
                }
            }
            # Display the remaining mails in folder.
            set command "displayfolder"
        } elseif { $command == "refresh" } {
            # Since no caching, no need to refresh.
            set command "displayfolder"
        } elseif { $command == "createfolder" } {
            # Create a new folder.
            set newfolder ""
            if { [catch {cgi_import newfolder}] == 0 } {
                set newfolder [cgi_import newfolder]
            }
            if { $newfolder != "" } {
                if [catch {file mkdir "$topdir/$newfolder"} msg] {
                    puts "Can not create $topdir/$newfolder: $msg"
                    br
                }
            }
            # Display the current list folder.
            set command "displayfolderlist"
        } elseif { $command == "deletefolder" } {
            # Delete empty selected folders.
            if {0==[catch {cgi_import DeleteVMailList}]} {
                foreach fldr $DeleteVMailList {
                    if [catch {file delete -- $topdir/$fldr} msg] {
                        puts "Can not delete folder $topdir/$fldr: $msg"
                        br
                    }
                }
            }

            # Display the current list folder.
            set command "displayfolderlist"
        }

        # Set the current folder list.
        set folderslist [list] 
        if {[catch {set files [glob -- $topdir/*]}] != 0} {
          set files [list]
        }
        foreach file $files {
          if {[file isdirectory $file]} {
            lappend folderslist [file tail $file]
          }
        }
        lsort $folderslist

          table border=0 cellspacing=0 cellpadding=0 cols=1 width="500" {
            table_row {
              table_data {
                cgi_division align=center {
                    puts [bold $for]
                }
                p

                # Inbox, Folders, Options, Help.
                table border=0 cellspacing=0 cellpadding=0 cols=4 width="100%" bgcolor="#006600" {
                  table_row {
                    table_data {
                      cgi_center {
                        puts "[bold [cgi_link inboxlink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                          puts "[bold [cgi_link folderslink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                          puts "[bold [cgi_link optionslink]]"
                      }
                    }
                    table_data {
                      cgi_center {
                       puts "[bold [cgi_link helplink]]"
                      }
                    }
                  }
                }

                if { $command == "displayfolder" } {                
                  # Current Folder and number of new messages.XXX
                  table border=0 cellspacing=0 cellpadding=0 cols=1 width="100%" bgcolor="#33cc00" {
                    table_row {
                      table_data {
                        cgi_center {
                          puts "[bold [font color="#000000" "$folder"]]"
                        }
                      }
                    }
                  }
                
                  # List all the folder content
                  #--------------------------------------------
                  # Folder content
                  table cellspacing=0 width="100%" bgcolor="#eeeedd" {
                    # Select, NewMail, Date, From, Subject, Size.
                    table_row bgcolor="#66ffcc" {
                      table_data width="1" {
                        cgi_checkbox allbox="CheckAll" onclick='CheckAll()'
                      }
                      table_data width="1" {
                        puts "[cgi_imglink newmaillink]"
                      }
                      table_data width="1" {
                        puts "[cgi_link datelink]"
                      }
                      table_data width="150" {
                        puts "[cgi_link fromlink]"
                      }
                      table_data align=center width="200" {
                        puts "[cgi_link subjectlink]"
                      }
                      table_data width="20" {
                        puts "[cgi_link sizelink]"
                      }
                    }

                    if {[catch {set files [glob -- $folderdir/*]}]} {
                      set files [list]
                    }
                    # Leave 'files' unsorted, since it's sorted manually later.
                    set msgcount 0
                    set maillist [list]
                    cgi_export fname=

                    foreach file $files {
                      set fname [file tail $file]
                      set fp [open "$file" r]
                      if { $fp != 0 } {
                          set msgcount [expr $msgcount + 1]
                          fconfigure $fp -translation binary
                          set magic [read $fp 4]
                          set x [read $fp 4]
                          binary scan $x I hdrsize
                          # Skip rest of the 16 bytes.
                          read $fp 16  
                          set hdrsize [ expr $hdrsize - 16 ]
                          set y [read $fp $hdrsize]
                          binary scan $y A* info
                          
                          set organization ""
                          set subject ""
                          set priority ""
                          set date ""

                          # R for read, N for unread (new)
                          # default is R.
                          set status "R"
                          
                          set from ""
                          set to   ""
                          set date ""
                          set subject ""
                          set priority ""
                          set organization ""
                          set status ""

                          regexp "From:(\[^\|\n]+)" $info dummy from
                          regexp "To:(\[^\|\n]+)"   $info dummy to
                          regexp "Date:(\[^\|\n]+)" $info dummy date
                          regexp "Subject:(\[^\|\n]+)" $info dummy subject
                          regexp "Priority:(\[^\|\n]+)" $info dummy priority
                          regexp "Organization:(\[^\|\n]+)" $info dummy organization
                          regexp "Status:(\[^\|\n]+)" $info dummy status

                          set filesize [file size "$file" ]
                          set filesizeink [ expr $filesize / 1000 ]
                          set filesize [ expr $filesize / 8000 ] 
                          set filesizestr "$filesize s ($filesizeink KB)"
                          if { $filesize >= 60 } {
                              set min [expr $filesize / 60]
                              set sec [expr $filesize % 60]
                              set filesizestr "$min:$sec min ($filesizeink KB)"
                          } 
                          
                          set datebin [binary format A25 $date ]
                          binary scan $datebin A* date
                          set dateitem "[font size=-1 "[printdate $date]"]"
                          
                          set from_id ""
                          set to_id   ""
                          regexp "^sip:(.*)" $from dummy from_id
                          regexp "^sip:(.*)" $to   dummy to_id
                          
                          # From should display short address.
                          # use ... if address is long.
                          set fromdisplay $from
                          set fromdisplaysplits [split $fromdisplay {}]
                          if { [llength $fromdisplaysplits] > 20} {
                              set fromdisplaysplits [lrange $fromdisplaysplits 0 17]
                              lappend fromdisplaysplits "." "." "."
                              set fromdisplay [join $fromdisplaysplits {}]
                          } 

                          # /* Escape " and < and > in fromdisplay and from */
                          # Store the mails in a list.
                          set from [cgi_quote_url "$from"]
                          set fromdisplay [cgi_quote_html "$fromdisplay"]
                          set fromitem "[url "$fromdisplay" "data:application/sip,$from"]"
                          if { $organization != "" } {
                              set fromitem "$fromitem <br> {$organization}";
                          }
                          set fromitem "$fromitem [url "[emailicon]" "mailto:$from_id"]"
                          set toitem "$to"
                          if { $subject == "" } {
                              set subjectitem "Message: $fname"
                          } else {
                              set subjectitem $subject
                          }
                          if { $priority != "" } {
                              set subjectitem "\[$priority\] $subjectitem"
                          }
                          set subjectitem "[url "$subjectitem" "javascript:document.vmail._VMaction.value='retrieve';document.vmail.fname.value='$fname';document.vmail.submit();"]" 
                          
                          set sizeitem "[font size=-1 "$filesizestr"]"

                          if { $status == "N" } {
                              set statusitem "[newmailimg]"
                          } else {
                              set statusitem "[cgi_nbspace]"
                          }

                          # Set the date as integer.
                          set dateint 0
                          scan "[clock scan $date]" "%d" dateint


                          set mailentry [list "$fname" [list "$date" "$dateitem"] \
                            [list "$from" "$fromitem"] [list "$subject" "$subjectitem"]\
                            [list $filesize "$sizeitem"] "$statusitem"  \
                            $dateint \
                            $filesize \
                          ]

                          lappend maillist $mailentry

                          catch {close $fp}

                        }
                        # If { $fp != 0 }
                     }
                     # foreach { files }

                     if { $sort == "date" } {
                         # Sort by date.
                         set maillist [lsort -integer -decreasing -index 6 $maillist]
                     } elseif { $sort == "from" } {
                         # Sort using sender URI.
                         set maillist [lsort -dictionary -index 2 $maillist]
                     } elseif { $sort == "subject" } {
                         set maillist [lsort -dictionary -index 3 $maillist]
                     } elseif { $sort == "size" } {
                         set maillist [lsort -integer -index 7 $maillist]
                     }

                     for { set c 0 } { $c < [llength $maillist]} {incr c} {
                       set mailentry [lindex $maillist $c]

                       set fname       "[lindex $mailentry 0]"
                       set dateitem    "[lindex [lindex $mailentry 1] 1]"
                       set fromitem    "[lindex [lindex $mailentry 2] 1]"
                       set subjectitem "[lindex [lindex $mailentry 3] 1]"
                       set sizeitem    "[lindex [lindex $mailentry 4] 1]"
                       set statusitem  "[lindex $mailentry 5]"

                       table_row {
                         table_data { cgi_checkbox DeleteVMailList=$fname }
                         # XXX How do we decide if this is a new mail.
                         table_data { puts "$statusitem" }
                         table_data nowrap  { puts "$dateitem" }
                         table_data nowrap { puts "$fromitem" }
                         # table_data { puts "$toitem"}
                         table_data align=center { puts "$subjectitem" } 
                         table_data nowrap align=right { puts "$sizeitem" }
                       }
                     }
                     # Table row for loop

                     # Display the table rows for each element in the list

                     # Comment out following for loop lines if you do not
                     # want additional blank table rows in case number of messages
                     # is less than, say 7. --TODO
                     for { set c $msgcount } { $c < 7 } { incr c } {
                       table_row {
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                       }
                     }
                     # End of Additional blank table rows.

                   }
                   # End of table for folder display
                   #------- end of folder display --------------

                   # Delete and Move to Folder 
                   table border=0 cellspacing=2 cellpadding=0 width="100%" {
                       table_row {
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link deletelink]]</dvi>"
                           }
                           table_data nowrap bgcolor="#cccccc" {
                             cgi_select tobox "onChange=document.vmail._VMaction.value='MoveTo';document.vmail.submit();" {
                               cgi_option "-Move to Folder-" value= selected
                               for { set i 0} {$i < [llength $folderslist]} {incr i} {
                                   set fld "[lindex $folderslist $i]"
                                   if { $folder != $fld } {
                                       cgi_option "$fld" value=$fld
                                   }
                               }
                               # List other user defined folders.XXX
                             }
                           }
                           table_data nowrap bgcolor="#cccccc" {
                               puts "<dvi class='buttn'>[font color=#000066 [cgi_link refreshlink]]</dvi>"
                           }
                       }
                   }

                   table border=0 cellspacing=2 cellpadding=0 width="100%" {
                       table_row nowrap {
                         table_data nowrap bgcolor="#cccccc" {
                           puts "<dvi class='buttn'>[font color=#000066 [cgi_link forwardlink]]</dvi>"
                           set emaillist ""
                           put "selected mails to these email(s):";  cgi_text emaillist size=16
                         }
                       }
                   }
                   # End of Delete and Move to folder.

                 }  elseif { $command == "displayfolderlist" } {
                   # List of available folders.
                   # Number of messages in each and total content size.

                   # Print the directory name.
                   table border=0 cellspacing=0 cellpadding=0 width="100%" bgcolor="33cc00" {
                       table_row {
                         table_data {
                           cgi_center {
                             puts "[bold [font color="#000000" "Folders in $topdir"]]"
                           }
                         }
                       }
                   }

                   # Print the table of folders.
                   table cellspacing=0 width="100%" bgcolor="#eeeedd" {
                     # Foldername, Number of files, Total Size.
                     table_row bgcolor="#66ffcc" {
                       table_data width="1" {
                           puts "[cgi_nbspace]"
                           # cgi_checkbox allbox="CheckAll" onClick='CheckAll()'
                       }
                       table_data width="50%" { puts "[bold "Folder(s)"]" }
                       table_data width="100" align=right { puts "[bold "Messages"]" }
                       table_data width="100" align=right { puts "[bold "Size"]" }
                     }
                     
                     # Print each folder info.
                     for { set i 0 } { $i < [llength $folderslist] } {incr i} {
                       set fldr "[lindex $folderslist $i]"
                       table_row {
                         table_data { cgi_checkbox DeleteVMailList=$fldr }
                         table_data { 
                           cgi_link fldrlink "$fldr" "javascript:document.vmail._VMaction.value='displayfolder';document.vmail.folder.value='$fldr';document.vmail.submit();"
                           puts "[cgi_link fldrlink]"
                         }
                         table_data align=right {
                           if {[catch {set files [glob -- $topdir/$fldr/*]}]} {
                             set files [list]
                           }
                           set c 0
                           set totalsize 0
                           foreach file $files {
                               if { [file isfile "$file"] } {
                               incr c
                               set totalsize [expr $totalsize + [file size "$file"]]
                               }
                           }
                           puts "$c"
                         }
                         table_data align=right {
                             set kbsize [expr $totalsize / 1000]
                             puts "$kbsize KB"
                         }
                      }
                    }

                     # Comment out following for loop lines if you do not
                     # want additional blank table rows in case number of folders
                     # is less than, say 4. --TODO
                    for { set i [llength $folderslist] } { $i < 4 } { incr i } {
                       table_row {
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                           table_data { puts "[cgi_nbspace]" }
                       }
                     }
                     # End of Additional blank table rows.
                  }

                  p
                  puts "<dvi class='buttn'>[font color=#000066 "[cgi_link deletefolder  "Delete" "javascript:document.vmail._VMaction.value='deletefolder';document.vmail.submit();"] selected folders"]</dvi>"
                  br

                  set newfolder ""
                  put "Create new folder:";  cgi_text newfolder size=16
                  puts "<dvi class='buttn'>[font color=#000066 [cgi_link createfolder  "Create" "javascript:document.vmail._VMaction.value='createfolder';document.vmail.submit();"]]</dvi>"

                 }
                 # End of the mail table.
               }
               # End of toplevel table data 
           }
           # End of top level table row.
         }
         # End of top level table.

         cgi_export _VMaction=$command
         }
         # End of cgi_form
     }
     # End of cgi_body.
   } 
   # End of diplay of VoiceMail page (Non login page)
}
# End of cgi_eval


#************************** END OF FILE ********************************
