# display voicemail messages

# read a single message and generate a display structure
proc msg {for folder file} {
  set fname [file tail $file]
  set fp [open "$file" r]
  if {$fp == ""} {
   return ""
  }

  fconfigure $fp -translation binary
  set magic [read $fp 4]
  set x [read $fp 4]
  binary scan $x I hdrsize
  # Skip rest of the 16 bytes.
  read $fp 16  
  set hdrsize [expr $hdrsize - 16]
  set y [read $fp $hdrsize]
  binary scan $y A* info
  catch {close $fp}
                          
  set organization ""
  set subject ""
  set priority ""
  set date ""
  set from ""
  set to   ""
  set date ""
  set subject ""
  set priority ""

  # R for read, N for unread (new); default is R
  set status "R"

  regexp "From:(\[^\|\n]+)" $info dummy from
  regexp "To:(\[^\|\n]+)"   $info dummy to
  regexp "Date:(\[^\|\n]+)" $info dummy date
  regexp "Subject:(\[^\|\n]+)" $info dummy subject
  regexp "Priority:(\[^\|\n]+)" $info dummy priority
  regexp "Organization:(\[^\|\n]+)" $info dummy organization
  regexp "Status:(\[^\|\n]+)" $info dummy status

  set filesize   [file size $file]
  set filesizeKB [expr $filesize / 1000]
  set filesize   [expr $filesize / 8000] 
  set filesizestr "$filesize s ($filesizeKB KB)"
  if {$filesize >= 60} {
    set min [expr $filesize / 60]
    set sec [expr $filesize % 60]
    set filesizestr "$min:$sec min ($filesizeKB KB)"
  } 
                          
  set datebin [binary format A25 $date ]
  binary scan $datebin A* date
  set dateitem "[font size=-1 "[printdate $date]"]"
                          
  set from_id ""
  set to_id   ""
  regexp "^sip:(.*)" $from dummy from_id
  regexp "^sip:(.*)" $to   dummy to_id

  # From should display short address.
  # use ... if address is long.
  set fromdisplay $from
  set fromdisplaysplits [split $fromdisplay {}]
  if {[llength $fromdisplaysplits] > 50} {
    set fromdisplaysplits [lrange $fromdisplaysplits 0 17]
    lappend fromdisplaysplits "." "." "."
    set fromdisplay [join $fromdisplaysplits {}]
  } 

  # /* Escape " and < and > in fromdisplay and from */
  # Store the mails in a list.
  set from        [cgi_quote_url $from]
  regsub -all {<} $from "%3C" from
  regsub -all {>} $from "%3E" from
  set fromdisplay [cgi_quote_html $fromdisplay]
  set fromitem    [url $fromdisplay "data:application/sip,$from"]
  if {$organization != ""} {
    set fromitem "$fromitem<br>${organization}"
  }
  if {$from_id != ""} {
    set fromitem "$fromitem [url [imglink Email] "mailto:$from_id"]"
  }
  set toitem "$to"
  if {$subject == ""} {
    set subjectitem $fname
  } else {
    set subjectitem $subject
  }
  if {$priority != ""} {
    set subjectitem "\[$priority\] $subjectitem"
  }
  set subjectitem [url $subjectitem \
     [cgi_cgi VmailListen [cgi_set for $for] \
     [cgi_set folder $folder] \
     [cgi_set m $fname]]]
  set sizeitem [font size=-1 $filesizestr]

  if {$status == "N"} {
    set statusitem [imglink NewMail]
  } else {
    set statusitem [cgi_nbspace]
  }

  # Set the date as integer.
  set dateint 0
  scan "[clock scan $date]" "%d" dateint

  set mailentry [list $fname [list $date $dateitem] \
    [list $from $fromitem] [list $subject $subjectitem]\
    [list $filesize "$sizeitem"] $statusitem  \
    $dateint $filesize]
}


# Display all messages for user 'for', folder 'folder'
proc vmaildisplay {folderdir for folder sort} {
  # Javascripts to check all and open new help window.
  javascript {
    puts {
      function CheckAll() {
        for (var i=0;i<document.vmail.elements.length;i++) {
          var e = document.vmail.elements[i];
          if (e.name != 'allbox') {
            e.checked = document.vmail.allbox.checked;
          }
        }
      }
    }
  }
  noscript {
    puts "Your browser doesn't understand Javascript."
  }

  # Current folder list.
  set folderlist [list] 
  foreach f [glob -nocomplain -- ${folderdir}/../*] {
    if {[file isdirectory $f]} {
      lappend folderlist [file tail $f]
    }
  }
  set folderlist [lsort $folderlist]

  h1 "Voicemail $folder for $for"

  # get list of messages
  if {[catch {set files [glob -nocomplain -- $folderdir/*]}]} {
    set files [list]
  }
  set maillist ""
  foreach f $files {
    lappend maillist [msg $for $folder $f]
  }

  # sort messages
  switch $sort {
    date {
      set maillist [lsort -integer -decreasing -index 6 $maillist]
    }
    from {
      set maillist [lsort -dictionary -index 2 $maillist]
    }
    subject {
      set maillist [lsort -dictionary -index 3 $maillist]
    }
    size {
      set maillist [lsort -integer -index 7 $maillist]
    }
  }

  # main table
  form Vmail name=vmail {
    cgi_export for
    cgi_export from=$folder
    table border=0 cellspacing=2 cellpadding=0 {
      # actions
      table_row bgcolor=#b0cafa {
        table_data colspan=6 {
          table {
            table_data {
              puts [url [imglink Folder] \
                [cgi_cgi VmailFolders [cgi_set for $for]]]
            }
            table_data align=left colspan=3 {
              submit_button "Action=Delete" 
              puts "checked mail"
            }
            if {$folderlist != $folder} {
              table_data align=center colspan=2 {
                submit_button "Action=Change" 
                puts "to folder"
                cgi_select change {
                  foreach f $folderlist {
                    if {$f != $folder} {
                      cgi_option $f
                    }
                  }
                }
              }
            }
            if {$folderlist != $folder} {
              table_data align=right {
                submit_button "Action=Move" 
                puts "checked mail to"
                cgi_select to {
                  foreach f $folderlist {
                    if {$f != $folder} {
                      cgi_option $f
                    }
                  }
                }
              }
            }
          }
        }
      }

      # header row for messages
      table_row [bgcolor row] {
        table_data {
          cgi_checkbox allbox="CheckAll" onclick='CheckAll()'
        }
        th [blank]
        foreach h {{Date date} {Size size} {Sender sender} {Subject subject}} {
          if {$sort == [lindex $h 1]} {
            td [lindex $h 0]
          } else {
            td [url [lindex $h 0] [cgi_cgi Vmail [cgi_set sort [lindex $h 1]] [cgi_set for $for] [cgi_set folder $folder]]]
          }
        }
      }

      # show messages
      foreach m $maillist {
        set fname   [lindex $m 0]
        set date    [lindex [lindex $m 1] 1]
        set from    [lindex [lindex $m 2] 1]
        set subject [lindex [lindex $m 3] 1]
        set size    [lindex [lindex $m 4] 1]
        set status  [lindex $m 5]

        table_row [bgcolor row] {
          table_data {cgi_checkbox msgList=$fname}
          table_data {puts "$status"}
          table_data nowrap { puts "$date" }
          table_data nowrap align=right {puts "$size" }
          table_data nowrap { puts "$from" }
          table_data align=center { puts "$subject" } 
        }
      }
    }
  }
}
