/*
* dstring
* Dynamic string buffers, modeled after the Tcl_DString routines.
*/

#include <malloc.h>  /* malloc() */
#include <string.h>  /* strncpy() */
#include "dstring.h"

/*
* Add a character string to the dynamically growing string buffer 'd'.
*/
char *DStringAppend(DString *d, char *string, int length)
{
  if (d->length + length + 1 >= d->spaceAvl) {
    if (!d->spaceAvl) {
      d->spaceAvl = length + 1;
      d->string = malloc(d->spaceAvl);
    }
    else {
      d->spaceAvl *= 1.5;
      if (d->spaceAvl < d->length + length + 1) {
        d->spaceAvl += length + 1;
      }
      d->string = realloc(d->string, d->spaceAvl);
    }
  }
  strncpy(&d->string[d->length], string, length);
  d->length += length;
  d->string[d->length] = '\0';
  return d->string;
} /* DStringAppend */


/*
* Initialize a new dynamic string object.
*/
void DStringInit(DString *d)
{
  d->length   = 0;
  d->spaceAvl = 0;
  d->string   = 0;
} /* DStringInit */


/*
* Reset and free a dynamic string 'd'.
*/
void DStringFree(DString *d)
{
  d->length   = 0;
  d->spaceAvl = 0;
  free(d->string);
}


/*
* Return the value of the dynamic string.
*/
char *DStringValue(DString *d)
{
  return d->string;
} /* DStringValue */
