/*
* Error handling function.
*/
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>   /* abort() */
#include <string.h>   /* strerror() */

int debug_flag = 0;
extern int errno;

void debug(char *function_name, char *format, ...)
{
  va_list ap;
  va_start(ap, format);

  if (debug_flag) {
    /* print out name of function causing error */
    (void) fprintf(stdout, "%s(): ", function_name);

    /* print out remainder of message */
    (void) vfprintf(stdout, format, ap);
  }

  va_end(ap);
} /* debug */


void fatal(char *function_name, char *format, ...)
{
  va_list ap;
  va_start(ap, format);

  /* print out name of function causing error */
  fprintf(stderr, "Fatal error in %s: ", function_name);

  if (errno > 0)
    fprintf(stderr, "%s: ", strerror(errno));

  /* print out remainder of message */
  vfprintf(stderr, format, ap);

  va_end(ap);
  abort;
} /* fatal */


void error(char *function_name, char *format, ...)
{
  va_list ap;
  va_start(ap, format);

  /* print out name of function causing error */
  fprintf(stderr, "Fatal error in %s: ", function_name);

  if (errno > 0)
    fprintf(stderr, "%s: ", strerror(errno));

  /* print out remainder of message */
  vfprintf(stderr, format, ap);

  va_end(ap);
} /* error */
