#include "parse_url.h"
#include "dstring.h"
#include <ctype.h>
#include <malloc.h>

static char *strdupn(char *string, int n)
{
  s = malloc(n+1);
  memcpy(s, string, n);
  s[n] = '\0';
  return s;
}

/*
* Parse a URL and normalize it (remove %xx).
*/
url_t parse_URL(char *url)
{
  url_t u;
  unsigned char *s;
  char *start;
  DString d;
  int c;
  int len = strlen(url);
  enum {U_scheme, U_host, U_port, U_path} state = U_scheme;

  DStringInit(&d);
  start = url;
  for (s = url; *s && (s - url) < len; s++) {
    switch(state) {
    case U_scheme:
      if (*s == ':') {
        u.scheme = strdupn(start, s - start);
        s += 2;
        state = U_host;
      }
      break;

    case U_host:
    case U_path:
      if (*s == '%') {
        sscanf(s, "%2d", &c);
        DStringAppend(&d, &c, 1);
        s += 2;
      }
      else if (state == U_host) {
        if (*s == ':') {
          start = s+1;
          state = U_port;
        }
        else if (*s == '/') {
          u.url = DStringValue(&d); 
          DSTringInit(&d);
          state = U_path;
        }
      }
      else DStringAppend(&d, s, 1);
      break;

    case U_port:
      if (!isdigit(*s)) {
        u.port = atoi(start);
        state = U_path;
      }
      break;
    }    
  }
  if (state == U_path) u.path = DStringValue(&d); 
  else u.host = DStringValue(&d);
  return u;
} /* parse_URL */
