#ifndef PARSER_H
#define PARSER_H

#include <time.h>

#define MAX_MIME_TYPE 32

/*
 * Each header consist of a name and a list of comma-separated elements. 
 * Each value has zero or more unnamed values and a set of parameters,
 * each again consisting of a name and a value. Parameter values may be
 * enclosed in quotation marks.
 */
typedef struct parser_parameter {    /* header parameters */
  char *attribute; 
  char *value;
  struct parser_parameter *next;
} parser_parameter_t;

typedef struct parser_element {
  char *element;                      /* The whole element */
  char *value;                        /* top-level value */  
  parser_parameter_t *parameters;       /* list of parameters */
  struct parser_element *next;
} parser_element_t;

typedef struct parser_header {          /* header */
  unsigned char *name;
  parser_element_t *elements;
  struct parser_header *next;
} parser_header_t;


/* RFC822 header extraction and parsing */
extern parser_header_t    *RFC822_Parse(char *header, int *length);
extern parser_header_t    *RFC822_Header(parser_header_t *, char *name);
extern char               *RFC822_Attribute(parser_header_t *h, int index,
                             char *attribute);
extern char               *RFC822_Attribute1(parser_parameter_t *plist,
                              char *attribute);
extern parser_parameter_t *RFC822_ParseHeader(char *element,
                              char **value, int all_parameter);
extern void                RFC822_ParameterFree(parser_parameter_t *plist);
extern void                RFC822_Free(parser_header_t *hlist);

/* Utility functions */
char *strdupn(char *s, int n);

#endif /* PARSER_H */
