/*
 * Structure that represents complete message.
 */
typedef struct parser_t {
  char *method;            /* Request: method */
  char *URI;               /* Request: request-URI */
  int status;              /* Response: response code */
  char *reason;            /* Response: reason phrase */
  char *protocol;          /* R/R: protocol name */
  char *version;           /* R/R: protocol version */
 
  parser_header_t *headers;  /* header list */
  unsigned char *body;     /* message body */
} parser_t;

typedef struct http_content_type {    /* message body protocol */
  char type[MAX_MIME_TYPE];
  char subtype[MAX_MIME_TYPE];
} http_content_type_t;

typedef struct http_agent {           /* client identification */
  char *type;
} http_agent_t;

typedef struct http_server {          /* server identification */
  char *type;
} http_server_t;


/* Functions to initialize and close parser. */
extern parser_t HTTP_Parse (char *);
extern void     HTTP_Free  (parser_t *);

/* HTTP request and status line information */
extern char * HTTP_Method   (parser_t *);
extern char * HTTP_URI      (parser_t *);
extern int    HTTP_Status   (parser_t *);
extern char * HTTP_Reason   (parser_t *);
extern char * HTTP_Protocol (parser_t *);
extern char * HTTP_Version  (parser_t *);

extern struct tm           RFC822_Date(parser_t *);

extern char               *HTTP_From          (parser_t *);
extern int                 HTTP_ContentLength (parser_t *);
extern http_content_type_t HTTP_ContentType   (parser_t *);
extern http_server_t       HTTP_Server        (parser_t *);
extern http_agent_t        HTTP_UserAgent     (parser_t *);
