#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>  /* inet_addr() */

#include "parser.h"
#include "parser_http.h"

/*
* RTSP header definitions.
*/
typedef struct rtsp_transport {       /* transport options */
  int port[2];
  int client_port[2];
  int server_port[2];
  int interleaved;
  int ssrc;
  int ttl;
  int layers;
  struct in_addr destination;
  boolean_t multicast;
  boolean_t compressed;
  boolean_t append;
  enum {P_RTP} protocol;
  enum {T_UDP,T_TCP} transport;
  enum {Pr_AVP} profile;
  int mode;
} rtsp_transport_t;

typedef struct rtsp_range {           /* range */
  double begin, end;                  /* beginning and ending time */
  double effective;                   /* when to play/record */
  boolean_t absolute;                   /* wallclock time or offset */
} rtsp_range_t;

typedef struct rtsp_session {         /* session id string */
  char *id;
  int timeout;
} rtsp_session_t;

typedef struct rtsp_timestamp {       /* range */
  double timestamp;
  double delay;
} rtsp_timestamp_t;

extern int                 RTSP_CSeq          (parser_t *);
extern rtsp_range_t        RTSP_Range         (parser_t *);
extern float               RTSP_Scale         (parser_t *);
extern rtsp_session_t      RTSP_Session       (parser_t *);
extern char               *RTSP_Subject       (parser_t *);
extern rtsp_timestamp_t    RTSP_Timestamp     (parser_t *);
extern rtsp_transport_t    RTSP_Transport     (parser_t *);

extern double strtoutc(char *utc);
extern double strtonpt(char *npt);

