#include <unistd.h>
#include <stdio.h>
#include "stream.h"

#define MAX_RESPONSE 4096

static char *m[699];

void ResponseInit(void)
{
  m[100] = "Continue";
  m[200] = "OK";
  m[201] = "Created";
  m[250] = "Low on Storage Space";
  m[300] = "Multiple Choices";
  m[301] = "Moved Permanently";
  m[302] = "Moved Temporarily";
  m[303] = "See Other";
  m[304] = "Not Modified";
  m[305] = "Use Proxy";
  m[400] = "Bad Request";
  m[401] = "Unauthorized";
  m[402] = "Payment Required";
  m[403] = "Forbidden";
  m[404] = "Not Found";
  m[405] = "Method Not Allowed";
  m[406] = "Not Acceptable";
  m[407] = "Proxy Authentication Required";
  m[408] = "Request Time-out";
  m[410] = "Gone";
  m[411] = "Length Required";
  m[412] = "Precondition Failed";
  m[413] = "Request Entity Too Large";
  m[414] = "Request-URI Too Large";
  m[415] = "Unsupported Media Type";
  m[451] = "Parameter Not Understood";
  m[452] = "Conference Not Found";
  m[453] = "Not Enough Bandwidth";
  m[454] = "Session Not Found";
  m[455] = "Method Not Valid in This State";
  m[456] = "Header Field Not Valid for Resource";
  m[457] = "Invalid Range";
  m[458] = "Parameter Is Read-Only";
  m[459] = "Aggregate operation not allowed";
  m[460] = "Only aggregate operation allowed";
  m[500] = "Internal Server Error";
  m[501] = "Not Implemented";
  m[502] = "Bad Gateway";
  m[503] = "Service Unavailable";
  m[504] = "Gateway Time-out";
  m[505] = "RTSP Version not supported";
} /* ResponseInit */


int Response(request_t *r, int status, char *reason)
{
  char msg[MAX_RESPONSE];
  char *s = msg;
  int length = 0;

  /* Invalid code. */
  if ((status > 600) || !m[status]) {
    error("Response", "invalid status %d", status);
    return -1;
  }

  /* Status line. */
  sprintf(s, "RTSP/%s %d %s\r\n",
    r->version,  status, reason ? reason : m[status]);
  length += strlen(s); s += length;

  /* Sequence number. */
  if (r->seq >= 0) {
    sprintf(s, "CSeq: %d\r\n", r->seq);
    length += strlen(s); s += length;
  }

  /* Session. */
  if (r->session >= 0) {
    sprintf(s, "Session: %d\r\n", r->session);
    length += strlen(s); s += length; 
  }
  sprintf(s, "Content-Length: 0\r\n");
  length += strlen(s); s += length;

  sprintf(s, "\r\n");

  if (r->type == SOCK_DGRAM) {
    status = sendto(r->socket, msg, length, 0,
      (struct sockaddr *)&r->sin, sizeof(r->sin));
  }
  else {
    status = write(r->socket, msg, length);
  }
  if (status < 0) {
    perror("Response");
    return -1;
  }
  return 0;
} /* Response */
