#include <pthread.h>
#include "parser_rtsp.h"
#include "rtp_api.h"

struct Stream;

typedef struct request_t {
  int type;                    /* SOCK_DGRAM or SOCK_STREAM */
  int socket;                  /* socket identifier for request */
  struct sockaddr_in sin;      /* origin of request (for UDP) */

  parser_t p;                  /* header for this request */
  char *URI;                   /* request URI */
  char *version;               /* request version */
  int seq;                     /* request sequence number */
  rtsp_transport_t transport;  /* transport parameters */
  rtsp_range_t range;          /* range parameters */
  int session;                 /* session identifier */
  struct Stream *stream;       /* stream this request belongs to */
  struct request_t *next;      /* next request in queue */ 
} request_t;


typedef struct Stream
{
  pthread_mutex_t lock;            /* lock this data structure */ 

  int session;                     /* stream session id */
  enum {S_INVALID, S_SETUP, S_PLAY, S_PAUSE, S_TEARDOWN} state;
  double npt;                      /* current normal play time */

  /* PLAY request queue */
  request_t *first, *last;

  /* RTP-related information */
  struct in_addr client_address;   /* network address of client */
  u_int32         pkt_size;    // number of bytes to be sent in packet
  u_int32         bps;         // bytes per sample
  int             encoding;    // audio encoding ID
  u_int32         IPaddress;   // 32 bit IPv4 address
  short int       port;        // port number
  int             tsinc;       // time synch value of packet
  short int       pti;         // payload type identifier
  u_int32         marker:1;    // marker bit
  int             socket[2];   // socket file descriptor
  u_int32         channels;    // number of interleaved channels 
  int             data_fd;     // file descriptor for audio file
  float           play_time;   // time into audio file
  float           play_start_time;// start of audio play time
  float           play_end_time;  // end of audio play time
  char            *cname;         // server e-mail address
  char            *name;          // actual user name
  context         cid;            // RTP context 
  int32           opaque;
  struct timeval  tp;
} Stream;

extern int Response(request_t *r, int status, char *reason);

extern Stream *StreamCreate(void);
extern int StreamFree(int n);
extern Stream *StreamGet(int n);

extern void debug(char *function_name, char *format, ...);
extern void fatal(char *function_name, char *format, ...);
extern void error(char *function_name, char *format, ...);
